@@ -17,8 +17,6 @@
  */
 
 #include "GxEQSlider.h"
-#include <gtk/gtkmain.h>
-#include <gtk/gtkprivate.h>
 
 #define P_(s) (s)   // FIXME -> gettext
 
@@ -45,8 +43,8 @@ static void gx_eq_slider_class_init(GxEQSliderClass *klass)
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_int("slider-width",P_("size of slider"),
-		                   P_("Height of movable part of vslider"),
-		                 0, 100, 5, GParamFlags(GTK_PARAM_READABLE)));
+				 P_("Height of movable part of vslider"),
+		                 0, 100, 5, GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 }
 
 static void gx_eq_slider_size_request (GtkWidget *widget, GtkRequisition *requisition)
@@ -64,9 +62,11 @@ static void gx_eq_slider_size_request (GtkWidget *widget, GtkRequisition *requis
 static void eq_slider_expose(
 	GtkWidget *widget, GdkRectangle *rect, gdouble sliderstate, GdkPixbuf *image)
 {
-	gdk_draw_pixbuf(GDK_DRAWABLE(widget->window), widget->style->fg_gc[0],
-	                image, 0, (gint)sliderstate, rect->x, rect->y,
-	                rect->width, rect->height, GDK_RGB_DITHER_NORMAL, 0, 0);
+	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
+	gdk_cairo_set_source_pixbuf (cr, image, rect->x, rect->y - (gint)sliderstate);
+	cairo_rectangle(cr, rect->x, rect->y, rect->width, rect->height);
+	cairo_fill(cr);
+	cairo_destroy(cr);
 }
 
 static gboolean gx_eq_slider_expose(GtkWidget *widget, GdkEventExpose *event)
