@@ -24,7 +24,6 @@
  */
 #include <math.h>
 #include <stdlib.h>
-#include <gtk/gtkprivate.h>
 #include "GxFastMeter.h"
 
 #ifndef max
@@ -77,41 +76,41 @@ void gx_fast_meter_class_init(GxFastMeterClass* klass)
 		gobject_class, PROP_HOLD, g_param_spec_int(
 			"hold", P_("Hold"),
 			P_("Count of cycles for which the peak value is held on display"),
-			0, 1000, 5, GParamFlags(GTK_PARAM_READWRITE)));
+			0, 1000, 5, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_DIMEN, g_param_spec_int(
 			"dimen", P_("Dimen"),
 			P_("Horizontal size of meter"),
-			0, 100, 20, GParamFlags(GTK_PARAM_READWRITE)));
+			0, 100, 20, GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_boxed("clr-bottom",P_("bottom color"),
 		                   P_("indicator color gradient: value at the bottom"),
 		                   GDK_TYPE_COLOR,
-		                   GParamFlags(GTK_PARAM_READABLE)));
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_boxed("clr-middle",P_("middle color"),
 		                   P_("indicator color gradient: value in the middle"),
 		                   GDK_TYPE_COLOR,
-		                   GParamFlags(GTK_PARAM_READABLE)));
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_boxed("clr-top",P_("top color"),
 		                   P_("indicator color gradient: value near the top"),
 		                   GDK_TYPE_COLOR,
-		                   GParamFlags(GTK_PARAM_READABLE)));
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_boxed("over",P_("clip warn color"),
 		                   P_("indicator color for values > 0 dbFS"),
 		                   GDK_TYPE_COLOR,
-		                   GParamFlags(GTK_PARAM_READABLE)));
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_int("dimen",P_("width of indicator"),
 		                   P_("width of (vertical) indicator"),
-		                 0, 100, 5, GParamFlags(GTK_PARAM_READABLE)));
+		                 0, 100, 5, GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 }
 
 /* ----- fast meter init ----- */
@@ -294,8 +293,6 @@ static gboolean gx_fast_meter_expose_event (GtkWidget* wd, GdkEventExpose* ev)
 {
 	GxFastMeter* fm = GX_FAST_METER(wd);
 	gint         top_of_meter;
-	GdkRectangle intersection;
-	GdkRectangle rect;
 
 	if (!fm->pixbuf) {
 		return FALSE;
@@ -306,41 +303,14 @@ static gboolean gx_fast_meter_expose_event (GtkWidget* wd, GdkEventExpose* ev)
 
 	/* reset the height & origin of the rect that needs to show the pixbuf */
 
-	rect.x = 0;
-	rect.y = 0;
-	rect.width  = width;
-	rect.height = height - top_of_meter;
-
-	if (gdk_rectangle_intersect (&rect, &ev->area, &intersection)) {
-		GdkWindow* window = GTK_WIDGET(fm)->window;/*gtk_widget_get_window(GTK_WIDGET(fm));*/
-		GtkStyle*  style  = gtk_widget_get_style (GTK_WIDGET(fm));
-
-		gdk_draw_rectangle(GDK_DRAWABLE(window),
-		                   style->black_gc,
-		                   TRUE,
-		                   intersection.x,
-		                   intersection.y,
-		                   intersection.width,
-		                   intersection.height);
-	}
-
-	rect.x = 0;
-	rect.y = height - top_of_meter;
-	rect.width  = width;
-	rect.height = height - rect.y;
-	if (gdk_rectangle_intersect(&rect, &ev->area, &intersection)) {
-		// draw the part of the meter image that we need.
-		// the area we draw is bounded "in reverse" (top->bottom)
-		GdkWindow*   window = GTK_WIDGET(fm)->window;/*gtk_widget_get_window(GTK_WIDGET(fm));*/
-		GtkStyle*    style  = gtk_widget_get_style (GTK_WIDGET(fm));
-		gdk_draw_pixbuf(GDK_DRAWABLE(window),
-		                style->fg_gc[GTK_WIDGET_STATE(GTK_WIDGET(fm))],
-		                fm->pixbuf,
-		                intersection.x, intersection.y,
-		                intersection.x, intersection.y,
-		                intersection.width, intersection.height,
-		                GDK_RGB_DITHER_NONE, 0, 0);
-	}
+	cairo_t *cr = gdk_cairo_create(GTK_WIDGET(fm)->window);
+	int h = height - top_of_meter;
+	cairo_rectangle(cr, 0, 0, width, h);
+	gdk_cairo_set_source_color(cr, &gtk_widget_get_style(GTK_WIDGET(fm))->black);
+	cairo_fill(cr);
+	gdk_cairo_set_source_pixbuf(cr, fm->pixbuf, 0, 0);
+	cairo_rectangle(cr, 0, h, width, height - h);
+	cairo_fill(cr);
 
 	// draw peak bar
 
@@ -351,21 +321,14 @@ static gboolean gx_fast_meter_expose_event (GtkWidget* wd, GdkEventExpose* ev)
 
 		fm->last_peak_rect.height = min(3, height - fm->last_peak_rect.y);
 
-		GdkWindow*   window = GTK_WIDGET(fm)->window;
-		GtkStyle*    style  = gtk_widget_get_style (GTK_WIDGET(fm));
-
-		gdk_draw_pixbuf(GDK_DRAWABLE(window),
-		                style->fg_gc[GTK_WIDGET_STATE(GTK_WIDGET(fm))],
-		                fm->pixbuf,
-		                0, fm->last_peak_rect.y,
-		                0, fm->last_peak_rect.y,
-		                width, fm->last_peak_rect.height,
-		                GDK_RGB_DITHER_NONE, 0, 0);
+		gdk_cairo_set_source_pixbuf(cr, fm->pixbuf, 0, 0);
+		cairo_rectangle(cr, 0, fm->last_peak_rect.y, width, fm->last_peak_rect.height);
+		cairo_fill(cr);
 	} else {
 		fm->last_peak_rect.width  = 0;
 		fm->last_peak_rect.height = 0;
 	}
-
+    cairo_destroy(cr);
 	return FALSE;
 }
 
