@@ -27,8 +27,7 @@
 #ifndef __GX_FAST_METER_H__
 #define __GX_FAST_METER_H__
 
-#include <gtk/gtkdrawingarea.h>
-#include "compat2-12.h"
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
