@@ -17,8 +17,6 @@
  */
 
 #include "GxHSlider.h"
-#include <gtk/gtkmain.h>
-#include <gtk/gtkprivate.h>
 
 #define P_(s) (s)   // FIXME -> gettext
 
@@ -50,7 +48,7 @@ static void gx_hslider_class_init(GxHSliderClass *klass)
 		widget_class,
 		g_param_spec_int("slider-width",P_("size of slider"),
 		                   P_("Width of movable part of vslider"),
-		                 0, 100, 20, GParamFlags(GTK_PARAM_READABLE)));
+		                 0, 100, 20, GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 }
 
 static void gx_hslider_size_request (GtkWidget *widget, GtkRequisition *requisition)
@@ -80,9 +78,11 @@ static void hslider_expose(
 			                rect->x, rect->y, rect->width, rect->height);
 		}
 	}
-	gdk_draw_pixbuf(GDK_DRAWABLE(widget->window), widget->style->fg_gc[0],
-	                image1, 0, 0, rect->x, rect->y,
-	                rect->width, rect->height, GDK_RGB_DITHER_NORMAL, 0, 0);
+	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
+	gdk_cairo_set_source_pixbuf (cr, image1, rect->x, rect->y);
+	cairo_rectangle(cr, rect->x, rect->y, rect->width, rect->height);
+	cairo_fill(cr);
+	cairo_destroy (cr);
 	g_object_unref(image1);
 }
 
