@@ -20,8 +20,6 @@
 #include "GxPaintBox.h"
 #include "GxGradient.h"
 #include "GxLevelSlider.h"
-#include <gtk/gtkprivate.h>
-#include <gtk/gtk.h>
 #include <cmath>
 #include <cstring>
 
@@ -33,6 +31,7 @@
 enum {
 	PROP_PAINT_FUNC = 1,
 	PROP_ICON_SET = 2,
+    PROP_FORCE_RELOAD = 3,
 };
 
 static void gx_paint_box_destroy(GtkObject *object);
@@ -49,6 +48,9 @@ G_DEFINE_TYPE(GxPaintBox, gx_paint_box, GTK_TYPE_BOX)
 #define get_main_image_id(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->main_image_id)
 #define get_widget_id(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->widget_id)
 #define get_widget_id2(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->widget_id2)
+#define get_widget_id3(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->widget_id3)
+#define get_cab_id(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->cab_id)
+#define get_amp_id(widget) (GX_PAINT_BOX_CLASS(GTK_OBJECT_GET_CLASS(widget))->amp_id)
 
 
 static void gx_paint_box_class_init (GxPaintBoxClass *klass)
@@ -65,20 +67,23 @@ static void gx_paint_box_class_init (GxPaintBoxClass *klass)
 	klass->main_image_id = "main_image";
     klass->widget_id = "gxplate";
     klass->widget_id2 = "gxplate2";
+    klass->widget_id3 = "gxplate3";
+    klass->cab_id = "texture_cab";
+    klass->amp_id = "amp_skin";
 	g_object_class_install_property(
 		gobject_class, PROP_PAINT_FUNC,
 		g_param_spec_string("paint-func",
 		                    P_("Paint Type"),
 		                    P_("Type of paint function for background"),
 		                    "",
-		                    GParamFlags(GTK_PARAM_READWRITE)));
+		                    GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property_parser(
 		GTK_WIDGET_CLASS(klass),
 		g_param_spec_boxed("skin-gradient",
 		                   P_("Skin color"),
 		                   P_("Color gradient defined as part of skin"),
 		                   GX_TYPE_GRADIENT,
-		                   GParamFlags(GTK_PARAM_READABLE)),
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)),
 		gx_parse_gradient);
 	gtk_widget_class_install_style_property_parser(
 		GTK_WIDGET_CLASS(klass),
@@ -86,7 +91,15 @@ static void gx_paint_box_class_init (GxPaintBoxClass *klass)
 		                   P_("Skin color"),
 		                   P_("Color gradient defined as part of skin"),
 		                   GX_TYPE_GRADIENT,
-		                   GParamFlags(GTK_PARAM_READABLE)),
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)),
+		gx_parse_gradient);
+	gtk_widget_class_install_style_property_parser(
+		GTK_WIDGET_CLASS(klass),
+		g_param_spec_boxed("rack-gradient",
+		                   P_("Skin color"),
+		                   P_("Color gradient defined as part of skin"),
+		                   GX_TYPE_GRADIENT,
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)),
 		gx_parse_gradient);
 	gtk_widget_class_install_style_property(
 		GTK_WIDGET_CLASS(klass),
@@ -94,43 +107,61 @@ static void gx_paint_box_class_init (GxPaintBoxClass *klass)
 		                    P_("Paint Type"),
 		                    P_("Type of paint function for background"),
 		                    NULL,
-		                    GParamFlags(GTK_PARAM_READABLE)));
+		                    GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class,PROP_ICON_SET,
 	    g_param_spec_int ("icon-set",
-						 P_("Icon Set"),
-						 P_("Type of Icon function for background"),
-						 0,
-						 G_MAXINT,
-						 0,
-						 GParamFlags(GTK_PARAM_READWRITE)));
+			      P_("Icon Set"),
+			      P_("Type of Icon function for background"),
+			      0,
+			      G_MAXINT,
+			      0,
+			      GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		GTK_WIDGET_CLASS(klass),
 		g_param_spec_int("icon-set",
 		                 P_("Icon Set"),
 		                 P_("Type of Icon function for background"),
 		                 0,
-						 G_MAXINT,
-						 0,
-		                 GParamFlags(GTK_PARAM_READABLE)));
+                         G_MAXINT,
+                         0,
+		                 GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
+	g_object_class_install_property(
+		gobject_class,PROP_FORCE_RELOAD,
+	    g_param_spec_int ("force-reload",
+			      P_("Force Reload"),
+			      P_("Reload Image for background"),
+			      0,
+			      G_MAXINT,
+			      0,
+			      GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
+	gtk_widget_class_install_style_property(
+		GTK_WIDGET_CLASS(klass),
+		g_param_spec_int("force-reload",
+		                 P_("Force Reload"),
+		                 P_("Reload Image for background"),
+		                 0,
+                         G_MAXINT,
+                         0,
+		                 GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		GTK_WIDGET_CLASS(klass),
 		g_param_spec_int("width",
 		                 P_("Width"),
 		                 P_("size.width request for paintbox"),
 		                 0,
-						 G_MAXINT,
-						 0,
-		                 GParamFlags(GTK_PARAM_READABLE)));
+				 G_MAXINT,
+				 0,
+		                 GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		GTK_WIDGET_CLASS(klass),
 		g_param_spec_int("height",
 		                 P_("Height"),
 		                 P_("size.height request for paintbox"),
 		                 0,
-						 G_MAXINT,
-						 0,
-		                 GParamFlags(GTK_PARAM_READABLE)));
+				 G_MAXINT,
+				 0,
+		                 GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 
 }
 
@@ -181,6 +212,7 @@ static void gx_paint_box_init (GxPaintBox *paint_box)
 	paint_box->gxh_image = NULL;
 	paint_box->gxr_image = NULL;
 	paint_box->icon_set = 0;
+    paint_box->force_reload = 0;
 }
 
 static void gx_paint_box_destroy(GtkObject *object)
@@ -190,12 +222,12 @@ static void gx_paint_box_destroy(GtkObject *object)
 		g_free(paint_box->paint_func);
 		paint_box->paint_func = NULL;
 	}
-	if (G_IS_OBJECT(paint_box->gxh_image)) {
-		g_object_unref(paint_box->gxh_image);
+	while (G_IS_OBJECT(paint_box->gxh_image)) {
+        g_object_unref(paint_box->gxh_image);
 	}
 	paint_box->gxh_image = NULL;
-	if (G_IS_OBJECT(paint_box->gxr_image)) {
-		g_object_unref(paint_box->gxr_image);
+	while (G_IS_OBJECT(paint_box->gxr_image)) {
+        g_object_unref(paint_box->gxr_image);
 	}
 	paint_box->gxr_image = NULL;
 	GTK_OBJECT_CLASS(gx_paint_box_parent_class)->destroy(object);
@@ -211,6 +243,13 @@ static gboolean gx_paint_box_expose(GtkWidget *widget, GdkEventExpose *event)
 	return FALSE;
 }
 
+static void set_reload(GxPaintBox *paint_box, int value)
+{
+	int spf;
+	gtk_widget_style_get(GTK_WIDGET(paint_box), "force-reload", &spf, NULL);
+	 paint_box->force_reload = spf;
+}
+
 static void set_icon(GxPaintBox *paint_box, int value)
 {
 	int spf;
@@ -229,6 +268,9 @@ static void gx_paint_box_set_property(
 	case PROP_ICON_SET:
 		set_icon(paint_box, g_value_get_int(value));
 		break;
+	case PROP_FORCE_RELOAD:
+		set_reload(paint_box, g_value_get_int(value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -245,6 +287,9 @@ static void gx_paint_box_get_property(
 	case PROP_ICON_SET:
 		g_value_set_int (value, GX_PAINT_BOX(object)->icon_set);
 		break;
+	case PROP_FORCE_RELOAD:
+		g_value_set_int (value, GX_PAINT_BOX(object)->force_reload);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -320,6 +365,31 @@ static void set_box_color(GtkWidget *wi, cairo_pattern_t *pat)
 	gx_gradient_free(grad);
 }
 
+// set cairo color related to the used skin
+static void set_rack_color(GtkWidget *wi, cairo_pattern_t *pat)
+{
+	GxGradient *grad;
+	gtk_widget_style_get(wi, "rack-gradient", &grad, NULL);
+	if (!grad) {
+		GdkColor *p1 = &wi->style->bg[GTK_STATE_NORMAL];
+		cairo_pattern_add_color_stop_rgba(
+			pat, 0, cairo_clr(p1->red), cairo_clr(p1->green),
+			cairo_clr(p1->blue), 0.8);
+		GdkColor *p2 = &wi->style->fg[GTK_STATE_NORMAL];
+		cairo_pattern_add_color_stop_rgba(
+			pat, 1, (cairo_clr(p1->red)+cairo_clr(p2->red))/2,
+			(cairo_clr(p1->green)+cairo_clr(p2->green))/2,
+			(cairo_clr(p1->blue)+cairo_clr(p2->blue))/2, 0.8);
+		return;
+	}
+	GSList *p;
+	for (p = grad->colors; p; p = g_slist_next(p)) {
+		GxGradientElement *el = (GxGradientElement*)p->data;
+		cairo_pattern_add_color_stop_rgba(pat, el->offset, el->red, el->green, el->blue, el->alpha);
+	}
+	gx_gradient_free(grad);
+}
+
 //----- paint boxes with cairo -----
 static void amp_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
@@ -538,7 +608,7 @@ static void led_expose(GtkWidget *wi, GdkEventExpose *ev)
 
 static void rectangle_skin_color_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
-	cairo_t *cr;
+    cairo_t *cr;
     cairo_pattern_t*pat;
 	/* create a cairo context */
 	cr = gdk_cairo_create(wi->window);
@@ -565,12 +635,28 @@ static void rectangle_skin_color_expose(GtkWidget *wi, GdkEventExpose *ev)
     
     int spf;
 	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
+    if (spf == 1000)  {
+        cairo_move_to  (cr, x0, y0 + radius);
+        cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+        cairo_line_to (cr, x1 - radius, y0);
+        cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+        cairo_line_to (cr, x1 , y1 - radius);
+        cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+        cairo_line_to (cr, x0 + radius, y1);
+        cairo_curve_to (cr, x0, y1, x0, y1, x0, y1- radius);
+        cairo_close_path (cr);
+        cairo_stroke (cr);
+        cairo_destroy(cr);
+        gdk_region_destroy (region);
+        return;
+    }
+
 
 	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
 	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
 	cairo_paint(cr);
 	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-    if(spf == 6 || spf == 7) {
+    if(spf >= 6 || spf <= 8) {
         GdkPixbuf * stock_image =
             gtk_widget_render_icon(wi,get_widget_id(wi),(GtkIconSize)-1,NULL);
     
@@ -599,7 +685,6 @@ static void rectangle_skin_color_expose(GtkWidget *wi, GdkEventExpose *ev)
         cairo_fill(cr);
         g_object_unref(stock_image);
         cairo_surface_destroy(s_image);
-        
     } else {
         cairo_move_to  (cr, x0, y0 + radius);
 	cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
@@ -641,6 +726,156 @@ static void rectangle_skin_color_expose(GtkWidget *wi, GdkEventExpose *ev)
 	gdk_region_destroy (region);
 }
 
+static void cabinet_expose(GtkWidget *wi, GdkEventExpose *ev)
+{
+    int spf;
+	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
+    if (spf == 1000)  return;
+	cairo_t *cr;
+    cairo_pattern_t*pat;
+	/* create a cairo context */
+	cr = gdk_cairo_create(wi->window);
+	GdkRegion *region;
+	region = gdk_region_rectangle (&wi->allocation);
+	gdk_region_intersect (region, ev->region);
+	gdk_cairo_region (cr, region);
+	cairo_clip (cr);
+
+	double x0      = wi->allocation.x+1;
+	double y0      = wi->allocation.y+1;
+	double rect_width  = wi->allocation.width-2;
+	double rect_height = wi->allocation.height-2;
+    
+    double radius = 45.;
+	if (rect_width<45) radius = rect_width;
+	else if (rect_height<45) radius = rect_height;
+	double x1,y1;
+
+	x1=x0+rect_width-2;
+	y1=y0+rect_height-2;
+    x0+=1;
+    y0+=1;
+    
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
+	//cairo_paint(cr);
+	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+        GdkPixbuf * stock_image =
+            gtk_widget_render_icon(wi,get_widget_id3(wi),(GtkIconSize)-1,NULL);
+    
+        guchar *pb_pixel = gdk_pixbuf_get_pixels (stock_image);
+        gint pixbuf_rowstride = gdk_pixbuf_get_rowstride (stock_image);
+        gint width = gdk_pixbuf_get_width (stock_image);
+        gint height = gdk_pixbuf_get_height (stock_image);
+        cairo_surface_t *s_image =
+            cairo_image_surface_create_for_data
+            (pb_pixel,CAIRO_FORMAT_RGB24 ,width, height,pixbuf_rowstride);
+
+        pat = cairo_pattern_create_for_surface(s_image);
+        cairo_set_source (cr, pat);
+        cairo_pattern_set_extend(cairo_get_source(cr), CAIRO_EXTEND_REPEAT);
+        
+        cairo_move_to  (cr, x0, y0 + radius);
+        cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+        cairo_line_to (cr, x1 - radius, y0);
+        cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+        cairo_line_to (cr, x1 , y1 - radius);
+        cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+        cairo_line_to (cr, x0 + radius, y1);
+        cairo_curve_to (cr, x0, y1, x0, y1, x0, y1- radius);
+        cairo_close_path (cr);
+        
+        cairo_fill(cr);
+        g_object_unref(stock_image);
+        cairo_surface_destroy(s_image);
+        
+    
+	pat = cairo_pattern_create_linear (x0, y0, x0, y0+rect_height);
+    //cairo_pattern_create_radial (-50, y0, 5,rect_width-10,  rect_height, 20.0);
+	set_rack_color(wi, pat);
+	cairo_set_source (cr, pat);
+    
+    
+    x1+=2;
+	y1+=2;
+    x0-=1;
+    y0-=1;
+	cairo_move_to  (cr, x0, y0 + radius);
+	cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+	cairo_line_to (cr, x1 - radius, y0);
+	cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+	cairo_line_to (cr, x1 , y1 - radius);
+	cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+	cairo_line_to (cr, x0 + radius, y1);
+	cairo_curve_to (cr, x0, y1, x0, y1, x0, y1- radius);
+	cairo_close_path (cr);
+    
+    
+	//cairo_rectangle (cr, x0+1,y0+1,rect_width-2,rect_height-1);
+	cairo_fill (cr);
+	cairo_pattern_destroy (pat);
+	cairo_destroy(cr);
+	gdk_region_destroy (region);
+}
+
+static void cab_expose(GtkWidget *wi, GdkEventExpose *ev)
+{
+	int spf;
+	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
+    if (spf == 1000)  return;
+    cairo_t *cr;
+    cairo_pattern_t*pat;
+	/* create a cairo context */
+	cr = gdk_cairo_create(wi->window);
+	GdkRegion *region;
+	region = gdk_region_rectangle (&wi->allocation);
+	gdk_region_intersect (region, ev->region);
+	gdk_cairo_region (cr, region);
+	cairo_clip (cr);
+
+	double x0      = wi->allocation.x+1;
+	double y0      = wi->allocation.y+1;
+	double rect_width  = wi->allocation.width-2;
+	double rect_height = wi->allocation.height-2;
+    
+    
+    
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
+	cairo_paint(cr);
+	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+    
+    GdkPixbuf * stock_image =
+        gtk_widget_render_icon(wi,get_cab_id(wi),(GtkIconSize)-1,NULL);
+
+    guchar *pb_pixel = gdk_pixbuf_get_pixels (stock_image);
+    gint pixbuf_rowstride = gdk_pixbuf_get_rowstride (stock_image);
+    gint width = gdk_pixbuf_get_width (stock_image);
+    gint height = gdk_pixbuf_get_height (stock_image);
+    cairo_surface_t *s_image =
+        cairo_image_surface_create_for_data
+        (pb_pixel,CAIRO_FORMAT_RGB24 ,width, height,pixbuf_rowstride);
+
+    pat = cairo_pattern_create_for_surface(s_image);
+    cairo_set_source (cr, pat);
+    cairo_pattern_set_extend(cairo_get_source(cr), CAIRO_EXTEND_REPEAT);
+    cairo_rectangle (cr, x0+1,y0+1,rect_width-2,rect_height-1);
+
+    cairo_fill(cr);
+    g_object_unref(stock_image);
+    cairo_surface_destroy(s_image);
+
+	pat = cairo_pattern_create_linear (x0, y0, x0, y0+rect_height);
+    set_rack_color(wi, pat);
+    
+	cairo_set_source (cr, pat);
+	cairo_rectangle (cr, x0+1,y0+1,rect_width-2,rect_height-1);
+	cairo_fill (cr);
+	cairo_pattern_destroy (pat);
+	cairo_destroy(cr);
+	gdk_region_destroy (region);
+}
+
 static void convolver_icon_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	cairo_t *cr;
@@ -938,7 +1173,7 @@ static void RackBox_expose(GtkWidget *wi, GdkEventExpose *ev)
     int spf;
 	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
     if(spf == 7) return;
-    
+    if(spf == 1000) return;
 	cairo_t *cr;
 	cairo_text_extents_t extents;
     cairo_pattern_t*pat;
@@ -958,10 +1193,14 @@ static void RackBox_expose(GtkWidget *wi, GdkEventExpose *ev)
 	double rect_height = wi->allocation.height;
 	double x,y;
 
-    if (spf == 6) {
-        GdkPixbuf * stock_image =
-            gtk_widget_render_icon(wi,get_widget_id2(wi),(GtkIconSize)-1,NULL);
-        
+    if (spf == 6 || spf == 8) {
+        if(strcmp(title,"oscilloscope")==0) return;
+        GdkPixbuf * stock_image;
+        if (spf == 6) {
+            stock_image =gtk_widget_render_icon(wi,get_widget_id2(wi),(GtkIconSize)-1,NULL);
+        } else {
+            stock_image =gtk_widget_render_icon(wi,get_widget_id3(wi),(GtkIconSize)-1,NULL);
+        }
         guchar *pb_pixel = gdk_pixbuf_get_pixels (stock_image);
         gint pixbuf_rowstride = gdk_pixbuf_get_rowstride (stock_image);
         gint width = gdk_pixbuf_get_width (stock_image);
@@ -1442,36 +1681,33 @@ static void lhfilter_expose(GtkWidget *wi, GdkEventExpose *ev)
 	double rect_height = wi->allocation.height-2;
 	GdkPixbuf  *stock_image, *frame;	
 	stock_image = gtk_widget_render_icon(wi,"gxhead_face",(GtkIconSize)-1,NULL);
-	
-    cairo_rectangle (cr, x0,y0,rect_width,rect_height);
-    cairo_set_line_width(cr, 3.0);
-    cairo_set_source_rgb (cr, 0., 0., 0.);
-    cairo_stroke (cr);
 
-    cairo_set_source_rgb(cr,  0.3, 0.3, 0.3);
-    cairo_set_line_width(cr, 2.0);
-    cairo_move_to(cr,x0+rect_width-3, y0+3);
-    cairo_line_to(cr, x0+rect_width-3, y0+rect_height-2);
-    cairo_line_to(cr, x0+2, y0+rect_height-2);
-    cairo_stroke(cr);
+	cairo_rectangle (cr, x0,y0,rect_width,rect_height);
+	cairo_set_line_width(cr, 3.0);
+	cairo_set_source_rgb (cr, 0., 0., 0.);
+	cairo_stroke (cr);
+
+	cairo_set_source_rgb(cr,  0.3, 0.3, 0.3);
+	cairo_set_line_width(cr, 2.0);
+	cairo_move_to(cr,x0+rect_width-3, y0+3);
+	cairo_line_to(cr, x0+rect_width-3, y0+rect_height-2);
+	cairo_line_to(cr, x0+2, y0+rect_height-2);
+	cairo_stroke(cr);
+
+	cairo_set_source_rgb(cr,  0.4, 0.4, 0.4);
+	cairo_set_line_width(cr, 2.0);
+	cairo_move_to(cr,x0+3, y0+rect_height-1);
+	cairo_line_to(cr, x0+3, y0+3);
+	cairo_line_to(cr, x0+rect_width-3, y0+3);
+	cairo_stroke(cr);
 
-    cairo_set_source_rgb(cr,  0.4, 0.4, 0.4);
-    cairo_set_line_width(cr, 2.0);
-    cairo_move_to(cr,x0+3, y0+rect_height-1);
-    cairo_line_to(cr, x0+3, y0+3);
-    cairo_line_to(cr, x0+rect_width-3, y0+3);
-    cairo_stroke(cr);
-    
- 
-	
 	frame = gdk_pixbuf_scale_simple(
 			stock_image, rect_width-8, rect_height-8, GDK_INTERP_NEAREST);
-	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                frame, 0, 0,
-	                x0+4, y0+4, rect_width-8,rect_height-8,
-	                GDK_RGB_DITHER_NORMAL, 0, 0);
-	
-    g_object_unref(stock_image);
+	gdk_cairo_set_source_pixbuf (cr, frame, x0+4, y0+4);
+	cairo_rectangle(cr, x0+4, y0+4, rect_width-8,rect_height-8);
+	cairo_fill(cr);
+
+	g_object_unref(stock_image);
 	g_object_unref(frame);
 	cairo_destroy(cr);
 	gdk_region_destroy (region);
@@ -1527,10 +1763,9 @@ static void tribal_box_expose(GtkWidget *wi, GdkEventExpose *ev)
 	cairo_pattern_destroy (pat);
 	cairo_destroy(cr);
 
-	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                _image, 0, 0,
-	                x0, y0, rect_width,rect_height,
-	                GDK_RGB_DITHER_NORMAL, 0, 0);
+	gdk_cairo_set_source_pixbuf (cr, _image, x0, y0);
+	cairo_rectangle(cr, x0, y0, rect_width,rect_height);
+	cairo_fill(cr);
 
 	g_object_unref(_image);
 	gdk_region_destroy (region);
@@ -1593,11 +1828,9 @@ static void filter_box_expose(GtkWidget *wi, GdkEventExpose *ev)
 
 	cairo_pattern_t*pat;
 
-	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                _image, 0, 0,
-	                x0, y0, rect_width,rect_height,
-	                GDK_RGB_DITHER_NORMAL, 0, 0);
-
+	gdk_cairo_set_source_pixbuf (cr, _image, x0, y0);
+	cairo_rectangle(cr, x0, y0, rect_width, rect_height);
+	cairo_fill(cr);
 	double radius = 38.;
 	if (rect_width<38) radius = rect_width;
 	else if (rect_height<38) radius = rect_height;
@@ -1830,6 +2063,50 @@ static void slooper_expose(GtkWidget *wi, GdkEventExpose *ev)
 	gdk_region_destroy (region);
 }
 
+static void line_expose(GtkWidget *wi, GdkEventExpose *ev)
+{
+	cairo_t *cr;
+	/* create a cairo context */
+	cr = gdk_cairo_create(wi->window);
+	GdkRegion *region;
+	region = gdk_region_rectangle (&wi->allocation);
+	gdk_region_intersect (region, ev->region);
+	gdk_cairo_region (cr, region);
+	cairo_clip (cr);
+
+	gint x0      = wi->allocation.x+3;
+	gint y0      = wi->allocation.y+3;
+	gint rect_width  = wi->allocation.width-6;
+	gint rect_height = wi->allocation.height-6;
+
+
+	double radius = 12.;
+	if (rect_width<12) radius = rect_width;
+	else if (rect_height<12) radius = rect_height;
+	double x1,y1;
+
+	x1=x0+rect_width;
+	y1=y0+rect_height;
+
+	cairo_move_to  (cr, x0, y0 + radius);
+	cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
+	cairo_line_to (cr, x1 - radius, y0);
+	cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
+	cairo_line_to (cr, x1 , y1 - radius);
+	cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
+	cairo_line_to (cr, x0 + radius, y1);
+	cairo_curve_to (cr, x0, y1, x0, y1, x0, y1- radius);
+	cairo_close_path (cr);
+    cairo_set_line_width(cr, 4.0);
+    cairo_set_source_rgb(cr, 0.2, 0.2, 0.2);
+    cairo_stroke_preserve(cr);
+    cairo_set_line_width(cr, 1.0);
+    cairo_set_source_rgb(cr, 0, 0, 0);
+    cairo_stroke(cr);
+	cairo_destroy(cr);
+	gdk_region_destroy (region);
+}
+
 static void gxhead_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
 	GxPaintBox *paintbox =  GX_PAINT_BOX(wi);
@@ -1844,12 +2121,46 @@ static void gxhead_expose(GtkWidget *wi, GdkEventExpose *ev)
 	int spf;
 
 	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
+    if (spf == 1000) return;
+    if (spf >= 7) {
+        rectangle_skin_color_expose(wi,ev);
+        line_expose(wi,ev);
+        if ( spf <10) {
+        paintbox->gxh_image = gtk_widget_render_icon(wi,"guitarix",(GtkIconSize)-1,NULL);
+        gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
+	                paintbox->gxh_image, 0, 0,
+	                x0+38, y0+20, 131,26,
+	                GDK_RGB_DITHER_NORMAL, 0, 0);
+        g_object_unref(paintbox->gxh_image);
+        paintbox->gxh_image = NULL;
+        }
+        GdkPixbuf  *stock_image = gtk_widget_render_icon(wi,"screw",(GtkIconSize)-1,NULL);
+        gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
+	                stock_image, 0, 0,
+	                x0+5, y0+5, 10,10,
+	                GDK_RGB_DITHER_NORMAL, 0, 0);
+        gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
+	                stock_image, 0, 0,
+	                x0+5, y0+rect_height-15, 10,10,
+	                GDK_RGB_DITHER_NORMAL, 0, 0);
+        gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
+	                stock_image, 0, 0,
+	                x0+rect_width-15, y0+rect_height-15, 10,10,
+	                GDK_RGB_DITHER_NORMAL, 0, 0);
+        gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
+	                stock_image, 0, 0,
+	                x0+rect_width-15, y0+5, 10,10,
+	                GDK_RGB_DITHER_NORMAL, 0, 0);
+        g_object_unref(stock_image);
+        return;
+    }
 
 	if (nf != spf || ne_w1 != rect_width*rect_height || !(GDK_IS_PIXBUF (paintbox-> gxh_image))) {
 		ne_w1 = rect_width*rect_height;
 		nf = spf;
 		if (G_IS_OBJECT(paintbox-> gxh_image)) {
 			g_object_unref(paintbox->gxh_image);
+            paintbox->gxh_image = NULL;
 		}
 		GdkPixbuf  *stock_image, *frame;
 		stock_image = gtk_widget_render_icon(wi,get_stock_id(wi),(GtkIconSize)-1,NULL);
@@ -1926,15 +2237,18 @@ static void gxhead_expose(GtkWidget *wi, GdkEventExpose *ev)
 		g_object_unref(frame);
 	}
 	// draw to display
-	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                paintbox->gxh_image, 0, 0,
-	                x0, y0, rect_width,rect_height,
-	                GDK_RGB_DITHER_NORMAL, 0, 0);
-	
+	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(wi));
+	gdk_cairo_set_source_pixbuf(cr, paintbox->gxh_image, x0, y0);
+	cairo_rectangle(cr, x0, y0, rect_width, rect_height);
+	cairo_fill(cr);
+	cairo_destroy(cr);
 }
 
 static void gxrack_expose(GtkWidget *wi, GdkEventExpose *ev)
 {
+    int spf;
+	gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
+    if (spf == 1000)  return;
 	gint rect_width  = wi->allocation.width-2;
 	gint rect_height = wi->allocation.height-3;
 	if (rect_width <= 0 || rect_height <= 0) {
@@ -2026,10 +2340,9 @@ static void gxrack_expose(GtkWidget *wi, GdkEventExpose *ev)
 	}
 	
 	// draw to display
-	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                paintbox->gxr_image, 0, 0,
-	                x0, y0, rect_width,rect_height,
-	                GDK_RGB_DITHER_NORMAL, 0, 0);
+	gdk_cairo_set_source_pixbuf (cr, paintbox->gxr_image, x0, y0);
+	cairo_rectangle(cr, x0, y0, rect_width, rect_height);
+	cairo_fill(cr);
 
 	// base 
 	x0      += 12;
@@ -2090,14 +2403,47 @@ static void main_expose(GtkWidget *wi, GdkEventExpose *ev)
 	
 	GdkPixbuf  *main_image = gtk_widget_render_icon(wi,get_main_image_id(wi),(GtkIconSize)-1,NULL);
 	
-	gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
-	                main_image, 0, 0,
+	gdk_cairo_set_source_pixbuf (cr, main_image, x0, y0);
+	cairo_rectangle(cr, x0, y0, w, h);
+	cairo_fill(cr);
+	g_object_unref(main_image);
+	cairo_destroy(cr);
+	gdk_region_destroy (region);   
+}
+
+static void amp_skin_expose(GtkWidget *wi, GdkEventExpose *ev)
+{
+    GdkRegion *region;
+	region = gdk_region_rectangle (&wi->allocation);
+	gdk_region_intersect (region, ev->region);
+    GxPaintBox *paintbox = GX_PAINT_BOX(wi);
+    gint x0      = wi->allocation.x;
+	gint y0      = wi->allocation.y;
+	gint w      = wi->allocation.width;
+	gint h      = wi->allocation.height;
+    static int spf, opf, rel = 0;
+    gtk_widget_style_get(GTK_WIDGET(wi), "icon-set", &spf, NULL);
+    gtk_widget_style_get(GTK_WIDGET(wi), "force-reload", &rel, NULL);
+    
+    static double ne_w = 0.;
+	if (rel || spf != opf || ne_w != w*h || !(GDK_IS_PIXBUF (paintbox-> gxr_image))) {
+		ne_w = w*h;
+        opf = spf;
+        while (G_IS_OBJECT(paintbox-> gxr_image)) {
+			g_object_unref(paintbox->gxr_image);
+		}
+        GdkPixbuf  *stock_image = gtk_widget_render_icon(
+            wi,get_amp_id(wi),(GtkIconSize)-1,NULL);
+        paintbox->gxr_image = gdk_pixbuf_scale_simple(
+			stock_image,wi->allocation.width ,wi->allocation.height , GDK_INTERP_NEAREST);
+        g_object_unref(stock_image);
+        //paintbox-> gxr_image = gtk_widget_render_icon(wi,get_amp_id(wi),(GtkIconSize)-1,NULL);
+    }
+    gdk_draw_pixbuf(GDK_DRAWABLE(wi->window), gdk_gc_new(GDK_DRAWABLE(wi->window)),
+	                paintbox-> gxr_image, 0, 0,
 	                x0, y0, w,h,
 	                GDK_RGB_DITHER_NORMAL, 0, 0);
-
-    g_object_unref(main_image);
-    cairo_destroy(cr);
-	gdk_region_destroy (region);   
+    gdk_region_destroy (region);  
 }
 
 static void level_meter_expose(GtkWidget *wi, GdkEventExpose *ev)
@@ -2226,6 +2572,12 @@ static void set_expose_func(GxPaintBox *paint_box, const gchar *paint_func)
 		paint_box->expose_func = main_expose;
 	} else if (strcmp(paint_func, "level_meter_expose") == 0) {
 	    paint_box->expose_func = level_meter_expose;
+	} else if (strcmp(paint_func, "cab_expose") == 0) {
+	    paint_box->expose_func = cab_expose;
+	} else if (strcmp(paint_func, "cabinet_expose") == 0) {
+	    paint_box->expose_func = cabinet_expose;
+	} else if (strcmp(paint_func, "amp_skin_expose") == 0) {
+	    paint_box->expose_func = amp_skin_expose;
 	} else {
 		paint_box->expose_func = 0;
 	}
