@@ -24,10 +24,6 @@
 #include "GxRegler.h"
 #include "GxControlParameter.h"
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
-#include <gtk/gtkprivate.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtkmarshal.h>
 
 #define gtk_widget_get_requisition(w, r) (*r = (w)->requisition)
 
@@ -48,6 +44,7 @@ struct _GxReglerPrivate
 enum {
   VALUE_ENTRY,
   FORMAT_VALUE,
+  INPUT_VALUE,
   LAST_SIGNAL
 };
 
@@ -345,25 +342,36 @@ static void gx_regler_class_init(GxReglerClass *klass)
 					  G_TYPE_STRING, 1,
 					  G_TYPE_DOUBLE);
 
+	signals[INPUT_VALUE] =
+		g_signal_new (I_("input-value"),
+					  G_TYPE_FROM_CLASS (gobject_class),
+					  G_SIGNAL_RUN_LAST,
+					  G_STRUCT_OFFSET (GxReglerClass, input_value),
+					  NULL, NULL,
+					  NULL,
+					  G_TYPE_INT, 2,
+					  G_TYPE_POINTER, //FIXME: GtkEntry*, gdouble*
+					  G_TYPE_POINTER);
+
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_boolean("show-value",
 		                     P_("show value"),
 		                     P_("display the value"),
 		                     TRUE,
-		                     GParamFlags(GTK_PARAM_READABLE)));
+		                     GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_int("value-spacing",P_("Value spacing"),
 		                 P_("Distance of value display"),
-		                 0, 100, 5, GParamFlags(GTK_PARAM_READABLE)));
+		                 0, 100, 5, GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_boxed("value-border",
 		                   P_("Value Spacing"),
 		                   P_("Extra space for value display"),
 		                   GTK_TYPE_BORDER,
-		                   GParamFlags(GTK_PARAM_READABLE)));
+		                   GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 
 	g_object_class_install_property(
 		gobject_class, PROP_SHOW_VALUE,
@@ -371,14 +379,14 @@ static void gx_regler_class_init(GxReglerClass *klass)
 		                     P_("show value"),
 		                     P_("display the value"),
 		                     TRUE,
-		                     GParamFlags(GTK_PARAM_READWRITE)));
+		                     GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_LABEL_REF,
 		g_param_spec_object("label-ref",
 		                    P_("Label ref"),
 		                    P_("GtkLabel for caption"),
 		                    GTK_TYPE_LABEL,
-		                    GParamFlags(GTK_PARAM_READWRITE)));
+		                    GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_VALUE_POSITION,
 		g_param_spec_enum("value-position",
@@ -386,21 +394,21 @@ static void gx_regler_class_init(GxReglerClass *klass)
 		                  P_("The position of the value display"),
 		                  GTK_TYPE_POSITION_TYPE,
 		                  GTK_POS_BOTTOM,
-		                  GParamFlags(GTK_PARAM_READWRITE)));
+		                  GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_VALUE_XALIGN,
 		g_param_spec_double("value-xalign",
 		                    P_("Value Alignment"),
 		                    P_("The horizontal position of the value (0..1)"),
 		                    0, 1, 0.5,
-		                    GParamFlags(GTK_PARAM_READWRITE)));
+		                    GParamFlags(G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_install_property(
 		gobject_class, PROP_DIGITS,
 		g_param_spec_int("digits",
 		                 P_("Digits"),
 		                 P_("Number of digits for display"),
 		                 0, 10, 1,
-		                 GParamFlags(GTK_PARAM_READABLE)));
+		                 GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 	g_object_class_override_property(gobject_class, PROP_VAR_ID, "var-id");
 
 	binding_set = gtk_binding_set_by_class(klass);
@@ -899,12 +907,11 @@ static gchar* _gx_regler_format_value(GxRegler *regler, gdouble value)
 	if (fmt) {
 		return fmt;
 	} else {
-		/* insert a LRM, to prevent -20 to come out as 20- in RTL locales */
 		int rd = GTK_RANGE(regler)->round_digits;
 		if (rd < 0) {
 			rd = 0;
 		}
-		return g_strdup_printf ("\342\200\216%0.*f", rd, value);
+		return g_strdup_printf ("%0.*f", rd, value);
 	}
 }
 
@@ -1104,6 +1111,9 @@ static gboolean spinner_button_press_event(
 	if (event->button == 3) {
 		return TRUE;
 	}
+	if (event->type == GDK_2BUTTON_PRESS) {
+	    return TRUE;
+	}
 	return FALSE;
 }
 
@@ -1164,7 +1174,22 @@ static gboolean map_check(
 }
 
 static void gx_regler_relay_value(GtkAdjustment *src, GtkAdjustment *dst) {
-    gtk_adjustment_set_value(dst, gtk_adjustment_get_value(src));
+	gtk_adjustment_set_value(dst, gtk_adjustment_get_value(src));
+}
+
+static gint gx_regler_spinner_input(GtkSpinButton *spin_button, gdouble *new_val, GxRegler *regler)
+{
+	gint ret;
+	g_signal_emit(regler, signals[INPUT_VALUE], 0, spin_button, new_val, &ret);
+	return ret;
+}
+
+static void gx_regler_spinner_output(GtkSpinButton *spinner, GxRegler *regler) {
+	gchar *fmt;
+	gdouble value = gtk_adjustment_get_value(gtk_spin_button_get_adjustment(spinner));
+	fmt = _gx_regler_format_value(regler, value);
+	gtk_entry_set_text (GTK_ENTRY(spinner), fmt);
+	g_free(fmt);
 }
 
 static gboolean gx_regler_value_entry(GxRegler *regler, GdkRectangle *rect, GdkEventButton *event)
@@ -1187,6 +1212,8 @@ static gboolean gx_regler_value_entry(GxRegler *regler, GdkRectangle *rect, GdkE
 	}
 	GtkWidget *spinner = gtk_spin_button_new(
 		src_adj, src_adj->step_increment, rd);
+	g_signal_connect(spinner, "output", G_CALLBACK(gx_regler_spinner_output), regler);
+	g_signal_connect(spinner, "input", G_CALLBACK(gx_regler_spinner_input), regler);
 	g_signal_connect(src_adj, "value-changed", G_CALLBACK(gx_regler_relay_value), dst_adj);
 	gtk_container_add(GTK_CONTAINER(dialog), spinner);
 
