@@ -17,8 +17,6 @@
  */
 
 #include "GxVSlider.h"
-#include <gtk/gtkmain.h>
-#include <gtk/gtkprivate.h>
 
 #define P_(s) (s)   // FIXME -> gettext
 
@@ -47,8 +45,8 @@ static void gx_vslider_class_init(GxVSliderClass *klass)
 	gtk_widget_class_install_style_property(
 		widget_class,
 		g_param_spec_int("slider-width",P_("size of slider"),
-		                   P_("Height of movable part of vslider"),
-		                 0, 100, 20, GParamFlags(GTK_PARAM_READABLE)));
+				 P_("Height of movable part of vslider"),
+		                 0, 100, 20, GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
 }
 
 static void gx_vslider_size_request (GtkWidget *widget, GtkRequisition *requisition)
@@ -73,15 +71,17 @@ static void vslider_expose(
 	gdk_pixbuf_copy_area(image, 0, rect->height, rect->width, slider_height,
 	                     image1, 0, rect->height-(int)sliderstate-slider_height);
 	if (has_focus) {
-		gdk_pixbuf_saturate_and_pixelate(image1, image1, sat, FALSE);
+		//gdk_pixbuf_saturate_and_pixelate(image1, image1, sat, FALSE);
 		if (paint_focus) {
 			gtk_paint_focus(widget->style, widget->window, GTK_STATE_NORMAL, NULL, widget, NULL,
 			                rect->x, rect->y, rect->width, rect->height);
 		}
 	}
-	gdk_draw_pixbuf(GDK_DRAWABLE(widget->window), widget->style->fg_gc[0],
-	                image1, 0, 0, rect->x, rect->y,
-	                rect->width, rect->height, GDK_RGB_DITHER_NORMAL, 0, 0);
+	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
+	gdk_cairo_set_source_pixbuf(cr, image1, rect->x, rect->y);
+	cairo_rectangle(cr, rect->x, rect->y, rect->width, rect->height);
+	cairo_fill(cr);
+	cairo_destroy(cr);
 	g_object_unref(image1);
 }
 
