@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <gtk/gtkiconfactory.h>
 #include "gxinit.h"
 
 #include "gximages.cc"
@@ -39,7 +38,7 @@ static image_entry image_data[] = {
 	{ "minitoggle_on", pb_minitoggle_on },
 	{ "hslider", pb_hslider },
 	{ "smallknob", pb_smallknob1 },
-	{ "smallknobr", pb_smallknob1 },
+	{ "smallknobr", pb_knob_medium },
 	{ "switchit_off", pb_switchit_off },
 	{ "switchit_on", pb_switchit_on },
 	{ "switch_off", pb_switch_off },
@@ -60,12 +59,20 @@ static image_entry image_data[] = {
 	{ "main_image", pb_gxhead },
     { "gxplate", pb_gxplate },
     { "gxplate2", pb_gxplate2 },
+    { "gxplate3", pb_gxplate3 },
+    { "portdisplay", pb_portdisplay },
+    { "texture_cab", pb_texture_cab },
+	{ "guitarix", pb_guitarix },
+    { "screw", pb_screw },
+    { "amp_skin", pb_amp21 },
 	{ NULL, NULL },
 };
 
 void gxw_init()
 {
+#if !GLIB_CHECK_VERSION(2, 36, 0) 
 	g_type_init();
+#endif
 	GtkIconFactory *factory = gtk_icon_factory_new();
 	for (image_entry *p = image_data; p->icon_name; p++) {
 		gtk_icon_factory_add(
