@@ -20,11 +20,11 @@
 part of guitarix, use  knobs with Gtk
 ******************************************************************************/
 
-#ifndef __GX_RADIO_BUTTON_H__
-#define __GX_RADIO_BUTTON_H__
+#ifndef __GX_INIT_H__
+#define __GX_INIT_H__
 
-#include <gtk/gtkradiobutton.h>
-#include "compat2-12.h"
+#include <gtk/gtk.h>
+#include <glib.h>
 
 G_BEGIN_DECLS
 
