@@ -23,6 +23,7 @@
 #include <gxwmm/controlparameter.h>
 #include <gxwmm/eqslider.h>
 #include <gxwmm/fastmeter.h>
+#include <gxwmm/levelslider.h>
 #include <gxwmm/hslider.h>
 #include <gxwmm/iredit.h>
 #include <gxwmm/knob.h>
@@ -33,12 +34,17 @@
 #include <gxwmm/regler.h>
 #include <gxwmm/selector.h>
 #include <gxwmm/smallknob.h>
+#include <gxwmm/smallknobr.h>
+#include <gxwmm/simplevaluedisplay.h>
 #include <gxwmm/switch.h>
 #include <gxwmm/toggleimage.h>
 #include <gxwmm/tuner.h>
 #include <gxwmm/racktuner.h>
 #include <gxwmm/vslider.h>
+#include <gxwmm/valuedisplay.h>
 #include <gxwmm/waveview.h>
 #include <gxwmm/wheel.h>
+#include <gxwmm/portdisplay.h>
+#include <gxwmm/init.h>
 
 #endif /* __GXWMM_H__ */
