@@ -25,6 +25,7 @@
  */
 
 #include "engine.h"
+#include "valve.h"
 
 #include "gx_faust_plugins.h"
 #include "../plugins/pluginlib.h"
@@ -41,6 +42,37 @@ static plugindef_creator builtin_crybaby_plugins[] = {
     0
 };
 
+static int load_crybaby_ui(const UiBuilder& builder, int format) {
+    if (format & UI_FORM_GLADE) {
+	builder.load_glade_file("crybaby_ui.glade");
+	return 0;
+    }
+    if (format & UI_FORM_STACK) {
+	builder.openHorizontalhideBox("");
+	builder.create_master_slider("crybaby.level", _("  level  "));
+	builder.closeBox();
+	builder.openHorizontalBox("");
+	{
+	    builder.insertSpacer();
+	    builder.create_selector("crybaby.autowah", 0);
+	    builder.insertSpacer();
+	    builder.insertSpacer();
+	    builder.openHorizontalTableBox("");
+	    {
+		builder.create_small_rackknobr("crybaby.wah", _("  wah   "));
+		builder.create_small_rackknob("crybaby.level", _("  level  "));
+		builder.create_small_rackknob("crybaby.wet_dry", _("  dry/wet  "));
+	    }
+	    builder.closeBox();
+	    builder.insertSpacer();
+	}
+	builder.closeBox();
+	return 0;
+    }
+    return -1;
+}
+
+
 static plugindef_creator builtin_tonestack_plugins[] = {
     gx_tonestacks::tonestack_default::plugin,
     gx_tonestacks::tonestack_bassman::plugin,
@@ -68,9 +100,37 @@ static plugindef_creator builtin_tonestack_plugins[] = {
     gx_tonestacks::tonestack_fender_default::plugin,
     gx_tonestacks::tonestack_fender_deville::plugin,
     gx_tonestacks::tonestack_gibsen::plugin,
+    gx_tonestacks::tonestack_engl::plugin,
     0
 };
 
+static int load_tonestack_ui(const UiBuilder& builder, int format) {
+    if (format & UI_FORM_GLADE) {
+	builder.load_glade_file("amp.tonestack_ui.glade");
+	return 0;
+    }
+    if (format & UI_FORM_STACK) {
+	builder.openHorizontalhideBox("");
+	builder.create_selector("amp.tonestack.select", 0);
+	builder.closeBox();
+	builder.openVerticalBox("");
+	{
+	    builder.openHorizontalBox("");
+	    {
+		builder.create_selector("amp.tonestack.select", 0);
+		builder.create_small_rackknob("amp.tonestack.Bass", _("bass"));
+		builder.create_small_rackknob("amp.tonestack.Middle", _("middle"));
+		builder.create_small_rackknob("amp.tonestack.Treble", _("treble"));
+	    }
+	    builder.closeBox();
+	}
+	builder.closeBox();
+	return 0;
+    }
+    return -1;
+}
+
+
 static plugindef_creator builtin_amp_plugins[] = {
     gx_amps::gxamp::plugin,
     gx_amps::gxamp3::plugin,
@@ -94,6 +154,9 @@ static plugindef_creator builtin_amp_plugins[] = {
     gx_amps::gxamp8::plugin,
     gx_amps::gxamp16::plugin,
     gx_amps::gxamp6::plugin,
+
+    gx_amps::gxnoamp::plugin, // keep last position (UI switches controls)
+
     0
 };
 
@@ -105,20 +168,23 @@ static const char* ampstack_groups[] = {
     0
 };
 
-GxEngine::GxEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups, const gx_system::CmdlineOptions& options)
+GxEngine::GxEngine(const string& plugin_dir, ParameterGroups& groups, const gx_system::CmdlineOptions& options)
     : ModuleSequencer(),
       resamp(),
+      plugin_changed(),
+      ladspaloader(options),
+      controller_map(),
       // ModuleSelector's
       crybaby(
-	  *this, ui, "crybaby", N_("Crybaby"), builtin_crybaby_plugins,
-	  "crybaby.autowah", _("select"), 0, PGN_POST_PRE),
+	  *this, "crybaby", N_("Crybaby"), N_("Guitar Effects"), builtin_crybaby_plugins,
+	  "crybaby.autowah", _("select"), load_crybaby_ui, 0, PGN_POST_PRE),
       tonestack(
-	  *this, ui, "amp.tonestack", N_("Tonestack"),
+	  *this, "amp.tonestack", N_("Tonestack"), N_("Tone control"),
 	  builtin_tonestack_plugins, "amp.tonestack.select",
-	  _("select"), 0, PGN_POST_PRE),
+	  _("select"), load_tonestack_ui, 0, PGN_POST_PRE),
       ampstack(
-	  *this, ui, "ampstack", _("Amp"), builtin_amp_plugins,
-	  "tube.select", _("select"), ampstack_groups),
+	  *this, "ampstack", _("Amp"), "", builtin_amp_plugins,
+	  "tube.select", _("select"), 0, ampstack_groups),
       // internal audio modules
       noisegate(),
       monomute(),
@@ -126,29 +192,43 @@ GxEngine::GxEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& g
       tuner(*this),
       midiaudiobuffer(tuner),
       maxlevel(),
-      oscilloscope(&ui, *this),
-      convolver(*this, sigc::mem_fun(stereo_chain, &StereoModuleChain::sync),
-		param, options.get_IR_pathlist(), options.get_sys_IR_dir()),
+      oscilloscope(*this),
+      mono_convolver(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), get_param()),
+      stereo_convolver(*this, sigc::mem_fun(stereo_chain, &StereoModuleChain::sync), get_param()),
       cabinet(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
+      preamp(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       contrast(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp) {
+    set_overload_interval(options.get_sporadic_overload());
+    if (!options.get_convolver_watchdog()) {
+	ov_disabled |= ov_Convolver;
+    }
+    if (!options.get_xrun_watchdog()) {
+	ov_disabled |= ov_XRun;
+    }
 #ifdef USE_MIDI_OUT
     tuner.set_dep_module(&midiaudiobuffer.plugin);
 #endif
 
     load_static_plugins();
-
     // loaded from shared libs
     if (!plugin_dir.empty()) {
 	pluginlist.load_from_path(plugin_dir, PLUGIN_POS_RACK);
     }
 
+    for (unsigned int i = 0; i < ladspaloader.size(); ++i) {
+	PluginDef *p = ladspaloader.create(i);
+	if (p) {
+	    pluginlist.add(p);
+	}
+    }
+
     // selector objects to switch "alternative" modules
     add_selector(ampstack);
     add_selector(crybaby);
     add_selector(tonestack);
     add_selector(tuner);
 
-    registerParameter(param, groups);
+    registerParameter(groups);
 
 #ifndef NDEBUG
     pluginlist.printlist();
@@ -202,14 +282,18 @@ void GxEngine::load_static_plugins() {
 
     // mono
     pl.add(gx_effects::low_high_pass::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::highbooster::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::selecteq::plugin(),        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&crybaby.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::gx_distortion::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::ts9sim::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::impulseresponse::plugin(), PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::compressor::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::expander::plugin(),        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::overdrive::plugin(),       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::echo::plugin(),            PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::delay::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&mono_convolver.plugin,                PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::freeverb::plugin(),        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&oscilloscope.plugin,                  PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::biquad::plugin(),          PLUGIN_POS_RACK, PGN_GUI);
@@ -217,11 +301,16 @@ void GxEngine::load_static_plugins() {
     pl.add(gx_effects::phaser_mono::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::chorus_mono::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::flanger_mono::plugin(),    PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::flanger_gx::plugin(),       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::gx_feedback::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::peak_eq::plugin(),         PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::digital_delay::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&tonestack.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&cabinet.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&preamp.plugin,                        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::abgate::plugin(),           PLUGIN_POS_RACK);
     pl.add(pluginlib::vibe::plugin_mono(),        PLUGIN_POS_RACK);
+    pl.add(pluginlib::mbc::plugin(),              PLUGIN_POS_RACK);
     // stereo
     pl.add(gx_effects::chorus::plugin(),          PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::flanger::plugin(),         PLUGIN_POS_RACK, PGN_GUI);
@@ -231,10 +320,80 @@ void GxEngine::load_static_plugins() {
     pl.add(gx_effects::moog::plugin(),            PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_amps::gx_ampmodul::plugin(),        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::tonecontroll::plugin(),    PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&convolver.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::digital_delay_st::plugin(),PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&stereo_convolver.plugin,              PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::stereoverb::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::zita_rev1::plugin(),        PLUGIN_POS_RACK);
     pl.add(pluginlib::vibe::plugin_stereo(),      PLUGIN_POS_RACK);
+    pl.add(pluginlib::mbcs::plugin(),             PLUGIN_POS_RACK);
+}
+
+static LadspaLoader::pluginarray::iterator find_plugin(LadspaLoader::pluginarray& ml, plugdesc *pl) {
+    for (LadspaLoader::pluginarray::iterator i = ml.begin(); i != ml.end(); ++i) {
+	if ((*i)->UniqueID == pl->UniqueID) {
+	    return i;
+	}
+    }
+    return ml.end();
+}
+
+void GxEngine::ladspaloader_update_plugins() {
+    // load plugindesc list
+    LadspaLoader::pluginarray ml;
+    ladspaloader.load(ml);
+    // look for removed and changed plugins
+    std::vector<PluginChange> pv;
+    for (LadspaLoader::pluginarray::iterator i = ladspaloader.begin(); i != ladspaloader.end(); ++i) {
+	Plugin *pl = pluginlist.lookup_plugin((*i)->id_str);
+	LadspaLoader::pluginarray::iterator j = find_plugin(ml, *i);
+	if (j == ml.end()) {
+	    pl->set_on_off(false);
+	    pv.push_back(PluginChange(pl, PluginChange::remove));
+	} else {
+	    ladspaloader.update_instance(pl->get_pdef(), *j);
+	    if (pl->get_pdef()->register_params) {
+		pmap.set_replace_mode(true);
+		gx_engine::ParamRegImpl preg(&pmap);
+		preg.plugin = pl->get_pdef();
+		pl->get_pdef()->register_params(preg);
+		pmap.set_replace_mode(false);
+	    }
+	    pv.push_back(
+		PluginChange(
+		    pl,
+		    ((*j)->category == (*i)->category ? PluginChange::update : PluginChange::update_category)));
+	}
+    }
+
+    // update engine for plugins to be removed
+    update_module_lists();
+    mono_chain.release();
+    stereo_chain.release();
+    // remove plugins
+    for (std::vector<PluginChange>::iterator i = pv.begin(); i != pv.end(); ++i) {
+	if (i->status == PluginChange::remove) {
+	    plugin_changed(i->pl, PluginChange::remove);
+	    pluginlist.unregisterPlugin(i->pl, pmap, get_group_table());
+	    pluginlist.delete_module(i->pl);
+	} else {
+	    plugin_changed(i->pl, i->status);
+	}
+    }
+    // add new plugins (engine)
+    for (LadspaLoader::pluginarray::const_iterator i = ml.begin(); i != ml.end(); ++i) {
+	if (ladspaloader.find((*i)->UniqueID) == ladspaloader.end()) {
+	    PluginDef *plugin = ladspaloader.create(*i);
+	    if (plugin) {
+		Plugin *p = pluginlist.add(plugin);
+		pluginlist.registerPlugin(p, get_param(), get_group_table());
+		plugin->set_samplerate(get_samplerate(), plugin);
+		plugin_changed(p, PluginChange::add);
+	    }
+	}
+    }
+    // update ladspaloader with new list
+    ladspaloader.set_plugins(ml);
+    plugin_changed(0, PluginChange::update);
 }
 
 } /* end of gx_engine namespace */
