@@ -54,7 +54,7 @@ void ProcessingChainBase::set_samplerate(int samplerate) {
     steps_up_dead = 0;
 }
 
-void ProcessingChainBase::set_stopped(bool v) {
+void __rt_func ProcessingChainBase::set_stopped(bool v) {
     stopped = v;
     if (v) {
 	post_rt_finished();  // in case someone is already waiting
@@ -78,10 +78,10 @@ bool ProcessingChainBase::wait_rt_finished() {
 	    continue;
 	}
 	if (errno == ETIMEDOUT) {
-	    gx_system::gx_print_warning("sem_timedwait", "timeout");
+	    gx_print_warning("sem_timedwait", "timeout");
 	    return false;
 	}
-	gx_system::gx_print_error("sem_timedwait", "unknown error");
+	gx_print_error("sem_timedwait", "unknown error");
 	break;
     }
     return true;
@@ -132,7 +132,7 @@ void ProcessingChainBase::start_ramp_down() {
     }
 }
 
-void ProcessingChainBase::try_set_ramp_mode(RampMode oldmode, RampMode newmode, int oldrv, int newrv) {
+void __rt_func ProcessingChainBase::try_set_ramp_mode(RampMode oldmode, RampMode newmode, int oldrv, int newrv) {
     if (oldmode != newmode) {
 	if (!gx_system::atomic_compare_and_exchange(&ramp_mode, oldmode, newmode)) {
 	    return;
@@ -166,13 +166,13 @@ bool lists_equal(const list<Plugin*>& p1, const list<Plugin*>& p2, bool *need_ra
 	}
 	if (*i1 != *i2) {
 	    ret = false;
-	    while ((*i1)->pdef->flags & PGN_SNOOP) {
+	    while ((*i1)->get_pdef()->flags & PGN_SNOOP) {
 		++i1;
 		if (i1 == p1.end()) {
 		    break;
 		}
 	    }
-	    while ((*i2)->pdef->flags & PGN_SNOOP) {
+	    while ((*i2)->get_pdef()->flags & PGN_SNOOP) {
 		++i2;
 		if (i2 == p2.end()) {
 		    break;
@@ -204,13 +204,13 @@ bool ProcessingChainBase::set_plugin_list(const list<Plugin*> &p) {
     typedef set<const char*, stringcomp> pchar_set;
     pchar_set new_ids;
     for (list<Plugin*>::const_iterator i = p.begin(); i != p.end(); ++i) {
-	new_ids.insert((*i)->pdef->id);
+	new_ids.insert((*i)->get_pdef()->id);
     }
     for (list<Plugin*>::const_iterator i = modules.begin(); i != modules.end(); ++i) {
-	if (!(*i)->pdef->activate_plugin) {
+	if (!(*i)->get_pdef()->activate_plugin) {
 	    continue;
 	}
-	pchar_set::iterator r = new_ids.find((*i)->pdef->id);
+	pchar_set::iterator r = new_ids.find((*i)->get_pdef()->id);
 	if (r == new_ids.end()) {
 	    to_release.push_back(*i);
 	}
@@ -221,7 +221,7 @@ bool ProcessingChainBase::set_plugin_list(const list<Plugin*> &p) {
 
 void ProcessingChainBase::clear_module_states() {
     for (list<Plugin*>::const_iterator p = modules.begin(); p != modules.end(); ++p) {
-	PluginDef* pd = (*p)->pdef;
+	PluginDef* pd = (*p)->get_pdef();
 	if (pd->activate_plugin) {
 	    pd->activate_plugin(true, pd);
 	} else if (pd->clear_state) {
@@ -233,7 +233,7 @@ void ProcessingChainBase::clear_module_states() {
 void ProcessingChainBase::release() {
     wait_latch();
     for (list<Plugin*>::const_iterator p = to_release.begin(); p != to_release.end(); ++p) {
-	(*p)->pdef->activate_plugin(false, (*p)->pdef);
+	(*p)->get_pdef()->activate_plugin(false, (*p)->get_pdef());
     }
     to_release.clear();
 }
@@ -252,7 +252,7 @@ void ProcessingChainBase::print_chain_state(const char *title) {
  ** MonoModuleChain, StereoModuleChain
  */
 
-void MonoModuleChain::process(int count, float *input, float *output) {
+void __rt_func MonoModuleChain::process(int count, float *input, float *output) {
     RampMode rm = get_ramp_mode();
     if (rm == ramp_mode_down_dead) {
 	memset(output, 0, count*sizeof(float));
@@ -312,7 +312,7 @@ void MonoModuleChain::process(int count, float *input, float *output) {
     try_set_ramp_mode(rm, rm1, rv, rv1);
 }
 
-void StereoModuleChain::process(int count, float *input1, float *input2, float *output1, float *output2) {
+void __rt_func StereoModuleChain::process(int count, float *input1, float *input2, float *output1, float *output2) {
     // run stereo rack
     RampMode rm = get_ramp_mode();
     if (rm == ramp_mode_down_dead) {
@@ -381,16 +381,15 @@ void StereoModuleChain::process(int count, float *input1, float *input2, float *
 
 
 /****************************************************************
- ** ModuleSelector
+ ** ModuleSelectorFromList
  */
 
 ModuleSelectorFromList::ModuleSelectorFromList(
-    EngineControl& seq_, gx_ui::GxUI& ui, const char* id_, const char* name_,
-    plugindef_creator plugins[], const char* select_id_,
-    const char* select_name_, const char** groups_, int flags_)
+    EngineControl& seq_, const char* id_, const char* name_,
+    const char* category_, plugindef_creator plugins[], const char* select_id_,
+    const char* select_name_, uiloader loader, const char** groups_, int flags_)
     : ModuleSelector(seq_),
       PluginDef(),
-      gx_ui::GxUiItemUInt(&ui, &selector),
       selector(0),
       select_id(select_id_),
       select_name(select_name_),
@@ -408,8 +407,10 @@ ModuleSelectorFromList::ModuleSelectorFromList(
     }
     id = id_;
     name = name_;
+    category = category_;
     groups = groups_;
     flags = flags_;
+    load_ui = loader;
     plugin = this;
 }
 
@@ -425,7 +426,9 @@ int ModuleSelectorFromList::register_parameter(const ParamReg &param) {
     }
     p[size].value_id = 0;
     p[size].value_label = 0;
-    param.registerUEnumVar(select_id, select_name, "S", "", p, &selector, 0);
+    param.registerIEnumVar(select_id, select_name, "S", "", p, &selector, 0);
+    seq.get_param()[select_id].signal_changed_int().connect(
+	sigc::hide(sigc::mem_fun(seq, &EngineControl::set_rack_changed)));
     return 0;
 }
 
@@ -434,22 +437,17 @@ int ModuleSelectorFromList::static_register(const ParamReg &param) {
 	->register_parameter(param);
 }
 
-void ModuleSelectorFromList::reflectZone() {
-    fCache = selector;
-    seq.set_rack_changed();
-}
-
 void ModuleSelectorFromList::set_module() {
-    if (current_plugin) {
-	current_plugin->on_off = false;
-    }
-    if (plugin.on_off) {
-	const char* id;
-	id = modules[selector]->id;
-	current_plugin = seq.pluginlist.lookup_plugin(id);
-	current_plugin->on_off = true;
-	current_plugin->effect_post_pre = plugin.effect_post_pre;
-    } else {
+    if (plugin.get_on_off()) {
+	Plugin *old = current_plugin;
+	current_plugin = seq.pluginlist.lookup_plugin(modules[selector]->id);
+	if (old && old != current_plugin) {
+	    old->set_on_off(false);
+	}
+	current_plugin->set_on_off(true);
+	current_plugin->copy_position(plugin);
+    } else if (current_plugin) {
+	current_plugin->set_on_off(false);
 	current_plugin = 0;
     }
 }
@@ -461,15 +459,15 @@ void ModuleSelectorFromList::set_module() {
 
 EngineControl::EngineControl()
     : selectors(),
-      rack_changed(true),
-      ui(),
+      rack_changed(),
+      pmap(),
       policy(),
       priority(),
       buffersize_change(),
       samplerate_change(),
       buffersize(0),
       samplerate(0),
-      pluginlist(ui,*this) {
+      pluginlist(*this) {
 }
 
 EngineControl::~EngineControl() {
@@ -479,9 +477,9 @@ void EngineControl::add_selector(ModuleSelector& sel) {
     selectors.push_back(&sel);
 }
 
-void EngineControl::registerParameter(ParamMap& param, ParameterGroups& groups)
+void EngineControl::registerParameter(ParameterGroups& groups)
 {
-    pluginlist.registerParameter(param, groups);
+    pluginlist.registerAllPlugins(pmap, groups);
 }
 
 void EngineControl::get_sched_priority(int &policy_, int &priority_, int prio_dim) { 
@@ -536,6 +534,14 @@ void EngineControl::init(unsigned int samplerate_, unsigned int buffersize_,
     }
 }
 
+void EngineControl::clear_rack_changed() {
+    rack_changed.disconnect();
+}
+
+bool EngineControl::get_rack_changed() {
+    return rack_changed.connected();
+}
+
 
 /****************************************************************
  ** ModuleSequencer
@@ -547,11 +553,19 @@ ModuleSequencer::ModuleSequencer()
       stateflags_mutex(),
       stateflags(SF_INITIALIZING),
       state_change(),
+      overload_detected(),
+      overload_reason(),
+      ov_disabled(0),
       mono_chain(),
       stereo_chain() {
+    overload_detected.connect(
+	sigc::mem_fun(this, &ModuleSequencer::check_overload));
 }
 
 ModuleSequencer::~ModuleSequencer() {
+    start_ramp_down();
+    wait_ramp_down_finished();
+    set_stateflag(SF_INITIALIZING);
 }
 
 void ModuleSequencer::start_ramp_up() {
@@ -575,6 +589,13 @@ bool ModuleSequencer::update_module_lists() {
     }
     if (prepare_module_lists()) {
 	commit_module_lists();
+	if (stateflags & SF_OVERLOAD) {
+	    // hack: jackd need some time for new load statistic
+	    Glib::signal_timeout().connect_once(
+		sigc::bind(
+		    sigc::mem_fun(this,&ModuleSequencer::clear_stateflag),
+		    SF_OVERLOAD), 1000);
+	}
 	return true;
     }
     return false;
@@ -586,12 +607,33 @@ void ModuleSequencer::set_samplerate(unsigned int samplerate) {
     EngineControl::set_samplerate(samplerate);
 }
 
+bool ModuleSequencer::check_module_lists() {
+    if (mono_chain.check_release()) {
+	mono_chain.release();
+    }
+    if (stereo_chain.check_release()) {
+	stereo_chain.release();
+    }
+    if (get_rack_changed()) {
+	update_module_lists();
+	return mono_chain.check_release() || stereo_chain.check_release();
+    }
+    return false;
+}
+
+void ModuleSequencer::set_rack_changed() {
+    if (rack_changed.connected()) {
+	return;
+    }
+    rack_changed = Glib::signal_idle().connect(
+	sigc::mem_fun(this, &ModuleSequencer::check_module_lists));
+}
+
 bool ModuleSequencer::prepare_module_lists() {
     for (list<ModuleSelector*>::iterator i = selectors.begin(); i != selectors.end(); ++i) {
 	(*i)->set_module();
     }
     list<Plugin*> modules;
-    rack_changed = false;
     pluginlist.ordered_mono_list(modules, audio_mode);
     bool ret_mono = mono_chain.set_plugin_list(modules);
     pluginlist.ordered_stereo_list(modules, audio_mode);
@@ -610,14 +652,14 @@ void ModuleSequencer::commit_module_lists() {
 	mono_chain.start_ramp_down();
 	mono_chain.wait_ramp_down_finished();
     }
-    mono_chain.commit(mono_chain.next_commit_needs_ramp);
+    mono_chain.commit(mono_chain.next_commit_needs_ramp, get_param());
     already_down =  (stereo_chain.get_ramp_mode() == ProcessingChainBase::ramp_mode_down_dead);
     bool stereoramp = stereo_chain.next_commit_needs_ramp && !already_down;
     if (stereoramp) {
 	stereo_chain.start_ramp_down();
 	stereo_chain.wait_ramp_down_finished();
     }
-    stereo_chain.commit(stereo_chain.next_commit_needs_ramp);
+    stereo_chain.commit(stereo_chain.next_commit_needs_ramp, get_param());
     if (monoramp) {
 	mono_chain.start_ramp_up();
 	mono_chain.next_commit_needs_ramp = false;
@@ -628,7 +670,37 @@ void ModuleSequencer::commit_module_lists() {
     }
 }
 
-void ModuleSequencer::set_stateflag(StateFlag flag) {
+int ModuleSequencer::sporadic_interval = 0;
+
+void __rt_func ModuleSequencer::overload(OverloadType tp, const char *reason) {
+    if (!(audio_mode & PGN_MODE_NORMAL)) {
+	return; // no overload message in mute/bypass modes
+    }
+    if ((tp & ov_disabled) == ov_XRun) {
+	return; // the xrun should show up in the log anyhow
+    }
+    bool ignore = false;
+    if ((tp & ov_disabled) == ov_Convolver) {
+	ignore = true;
+    }
+    if (sporadic_interval > 0 && !ignore && (tp & (ov_Convolver|ov_XRun))) {
+	static float last = -sporadic_interval;
+	timespec ts;
+	clock_gettime(CLOCK_MONOTONIC, &ts);
+	float now = ts.tv_sec + ts.tv_nsec * 1e-9;
+	if (now - last < sporadic_interval) { // max. 1 event every sporadic_interval seconds
+	    last = now;
+	    ignore = true;
+	}
+    }
+    if (!ignore) {
+	set_stateflag(SF_OVERLOAD);
+    }
+    gx_system::atomic_set(&overload_reason, reason);
+    overload_detected();
+}
+
+void __rt_func ModuleSequencer::set_stateflag(StateFlag flag) {
     if (stateflags & flag) {
 	return;
     }
@@ -641,7 +713,7 @@ void ModuleSequencer::set_stateflag(StateFlag flag) {
     stateflags |= flag;
 }
 
-void ModuleSequencer::clear_stateflag(StateFlag flag) {
+void __rt_func ModuleSequencer::clear_stateflag(StateFlag flag) {
     if (!(stateflags & flag)) {
 	return;
     }
@@ -654,6 +726,20 @@ void ModuleSequencer::clear_stateflag(StateFlag flag) {
     }
 }
 
+void ModuleSequencer::check_overload() {
+    if (stateflags & SF_OVERLOAD) {
+	set_state(kEngineBypass);
+	check_module_lists();
+	gx_print_error(
+	    "watchdog",
+	    boost::format(_("Overload (%s)")) % gx_system::atomic_get(overload_reason));
+    } else {
+	gx_print_error(
+	    "watchdog",
+	    boost::format(_("Overload ignored (%s)")) % gx_system::atomic_get(overload_reason));
+    }
+}
+
 void ModuleSequencer::set_state(GxEngineState state) {
     int newmode = PGN_MODE_MUTE;
     switch( state ) {
