@@ -50,7 +50,7 @@ NoiseGate::NoiseGate() {
     inputdef.mono_audio = inputlevel_compute;
     inputdef.register_params = noisegate_register;
 
-    inputlevel.pdef = &inputdef;
+    inputlevel.set_pdef(&inputdef);
 
     outputgate.version = PLUGINDEF_VERSION;
     outputgate.id = "noiseshut";
@@ -93,7 +93,7 @@ void NoiseGate::outputgate_compute(int count, float *input, float *output, Plugi
 
 int NoiseGate::outputgate_activate(bool start, PluginDef *pdef) {
     if (start) {
-	off = !inputlevel.on_off;
+	off = !inputlevel.get_on_off();
     }
     return 0;
 }
@@ -139,27 +139,15 @@ void GxJConvSettings::setFullIRPath(string name) {
     fIRFile= Glib::path_get_basename(name);
 }
 
-void GxJConvSettings::writeJSON(gx_system::JsonWriter& w,
-				const gx_system::PathList& search_path) {
+void GxJConvSettings::writeJSON(gx_system::JsonWriter& w) const {
     w.begin_object(true);
-    w.write_key("jconv.IRFile");
-    w.write(fIRFile, true);
-    w.write_key("jconv.IRDir");
-    string dir = fIRDir;
-    if (search_path.contains(dir)) {
-	dir = "";
-    }
-    w.write(dir, true);
-    w.write_key("jconv.Gain");
-    w.write(fGain, true);
-    w.write_key("jconv.GainCor");
-    w.write(fGainCor, true);
-    w.write_key("jconv.Offset");
-    w.write(fOffset, true);
-    w.write_key("jconv.Length");
-    w.write(fLength, true);
-    w.write_key("jconv.Delay");
-    w.write(fDelay, true);
+    w.write_kv("jconv.IRFile", fIRFile);
+    w.write_kv("jconv.IRDir", gx_system::get_options().get_IR_prefixmap().replace_path(fIRDir));
+    w.write_kv("jconv.Gain", fGain);
+    w.write_kv("jconv.GainCor", fGainCor);
+    w.write_kv("jconv.Offset", fOffset);
+    w.write_kv("jconv.Length", fLength);
+    w.write_kv("jconv.Delay", fDelay);
     w.write_key("jconv.gainline");
     w.begin_array();
     for (unsigned int i = 0; i < gainline.size(); i++) {
@@ -207,44 +195,33 @@ void GxJConvSettings::read_gainline(gx_system::JsonParser& jp) {
     jp.next(gx_system::JsonParser::end_array);
 }
 
-void GxJConvSettings::readJSON(gx_system::JsonParser& jp,
-			       const gx_system::PathList& search_path) {
+void GxJConvSettings::readJSON(gx_system::JsonParser& jp) {
     jp.next(gx_system::JsonParser::begin_object);
     do {
         jp.next(gx_system::JsonParser::value_key);
-        if (jp.current_value() == "jconv.IRFile") {
-            jp.next(gx_system::JsonParser::value_string);
-            fIRFile = jp.current_value();
-        } else if (jp.current_value() == "jconv.IRDir") {
-            jp.next(gx_system::JsonParser::value_string);
-            fIRDir = jp.current_value();
-        } else if (jp.current_value() == "jconv.Gain") {
-            jp.next(gx_system::JsonParser::value_number);
-            fGain = jp.current_value_float();
-        } else if (jp.current_value() == "jconv.GainCor") {
-            jp.next(gx_system::JsonParser::value_number);
-            fGainCor = jp.current_value_int();
-        } else if (jp.current_value() == "jconv.Offset") {
-            jp.next(gx_system::JsonParser::value_number);
-            fOffset = jp.current_value_int();
-        } else if (jp.current_value() == "jconv.Length") {
-            jp.next(gx_system::JsonParser::value_number);
-            fLength = jp.current_value_int();
-        } else if (jp.current_value() == "jconv.Delay") {
-            jp.next(gx_system::JsonParser::value_number);
-            fDelay = jp.current_value_int();
+	if (jp.read_kv("jconv.IRFile", fIRFile) ||
+	    jp.read_kv("jconv.IRDir", fIRDir) ||
+	    jp.read_kv("jconv.Gain", fGain) ||
+	    jp.read_kv("jconv.GainCor", fGainCor) ||
+	    jp.read_kv("jconv.Offset", fOffset) ||
+	    jp.read_kv("jconv.Length", fLength) ||
+	    jp.read_kv("jconv.Delay", fDelay)) {
         } else if (jp.current_value() == "jconv.gainline") {
             read_gainline(jp);
         } else if (jp.current_value() == "jconv.favorits") {
             jp.skip_object();
         } else {
-            gx_system::gx_print_warning("jconv settings", "unknown key: " + jp.current_value());
+            gx_print_warning("jconv settings", "unknown key: " + jp.current_value());
             jp.skip_object();
         }
     } while (jp.peek() == gx_system::JsonParser::value_key);
     jp.next(gx_system::JsonParser::end_object);
-    if (!fIRFile.empty() && fIRDir.empty()) {
-	search_path.find_dir(&fIRDir, fIRFile);
+    if (!fIRFile.empty()) {
+	if (fIRDir.empty()) {
+	    gx_system::get_options().get_IR_pathlist().find_dir(&fIRDir, fIRFile);
+	} else {
+	    fIRDir = gx_system::get_options().get_IR_prefixmap().replace_symbol(fIRDir);
+	}
     }
 }
 
@@ -253,14 +230,14 @@ void GxJConvSettings::readJSON(gx_system::JsonParser& jp,
  ** class JConvParameter
  */
 
-ParameterV<GxJConvSettings>::ParameterV(const string& id, ConvolverAdapter &conv_, GxJConvSettings *v)
+ParameterV<GxJConvSettings>::ParameterV(const string& id, gx_engine::ConvolverAdapter& conv, GxJConvSettings *v)
     : Parameter(id, "", tp_special, None, true, false),
       json_value(),
-      value(v ? v : new GxJConvSettings),
+      value(v),
       std_value(),
-      conv(conv_) {
-    own_var = !v;
-    std_value.setFullIRPath(Glib::build_filename(conv.get_sys_IR_dir(), "greathall.wav"));
+      value_storage(),
+      changed() {
+    std_value.setFullIRPath(gx_system::get_options().get_IR_prefixmap().replace_symbol("%S/greathall.wav"));
     std_value.fGainCor = true;
     std_value.fGain = 0.598717;
     const int ir_len = 112561;
@@ -277,13 +254,36 @@ JConvParameter *JConvParameter::insert_param(
 }
 
 JConvParameter::~ParameterV() {
-    if (own_var) {
-	delete value;
-    }
 }
 
-void *JConvParameter::zone() {
-    return value;
+JConvParameter::ParameterV(gx_system::JsonParser& jp)
+ : Parameter(jp_next(jp, "Parameter")),
+   searchpath(0),
+   pfx_conv(0),
+   json_value(),
+   value(&value_storage),
+   std_value() {
+    while (jp.peek() != gx_system::JsonParser::end_object) {
+	jp.next(gx_system::JsonParser::value_key);
+	if (jp.current_value() == "value") {
+	    value->readJSON(jp);
+	} else if (jp.current_value() == "std_value") {
+	    std_value.readJSON(jp);
+	} else {
+	    gx_print_warning(
+		"JConvParameter", Glib::ustring::compose("%1: unknown key: %2", _id, jp.current_value()));
+	    jp.skip_object();
+	}
+    }
+    jp.next(gx_system::JsonParser::end_object);
+}
+
+void JConvParameter::serializeJSON(gx_system::JsonWriter& jw) {
+    jw.begin_object();
+    jw.write_key("Parameter"); Parameter::serializeJSON(jw);
+    jw.write_key("value"); value->writeJSON(jw);
+    jw.write_key("std_value"); std_value.writeJSON(jw);
+    jw.end_object();
 }
 
 bool JConvParameter::on_off_value() {
@@ -291,21 +291,17 @@ bool JConvParameter::on_off_value() {
     return false;
 }
 
-void JConvParameter::set(float n, float high, float llimit, float ulimit) {
-    assert(false);
-}
-
 void JConvParameter::stdJSON_value() {
     json_value = std_value;
 }
 
-void JConvParameter::writeJSON(gx_system::JsonWriter& jw) {
+void JConvParameter::writeJSON(gx_system::JsonWriter& jw) const {
     jw.write_key(_id.c_str());
-    value->writeJSON(jw, conv.get_pathlist());
+    value->writeJSON(jw);
 }
 
 void JConvParameter::readJSON_value(gx_system::JsonParser& jp) {
-    json_value.readJSON(jp, conv.get_pathlist());
+    json_value.readJSON(jp);
 }
 
 bool JConvParameter::compareJSON_value() {
@@ -317,8 +313,7 @@ bool JConvParameter::set(const GxJConvSettings& val) const {
 	return false;
     }
     *value = val;
-    conv.restart();
-    conv.signal_settings_changed()();
+    changed(value);
     return true;
 }
 
@@ -332,31 +327,22 @@ void JConvParameter::setJSON_value() {
  */
 
 #include "faust/jconv_post.cc"
+#include "faust/jconv_post_mono.cc"
 
 ConvolverAdapter::ConvolverAdapter(
-    EngineControl& engine_, sigc::slot<void> sync_, ParamMap& param_,
-    const gx_system::PathList& pathlist_, const std::string& sys_ir_dir_)
+    EngineControl& engine_, sigc::slot<void> sync_, ParamMap& param_)
     : PluginDef(),
       conv(),
       activate_mutex(),
       engine(engine_),
       sync(sync_),
       param(param_),
-      pathlist(pathlist_),
-      sys_ir_dir(sys_ir_dir_),
       activated(false),
-      jc_post(),
-      settings_changed(),
       jcset(),
       jcp(0),
       plugin() {
     version = PLUGINDEF_VERSION;
-    id = "jconv";
-    name = N_("Convolver");
-    stereo_audio = convolver;
-    set_samplerate = convolver_init;
-    activate_plugin = activate;
-    register_params = convolver_register;
+    category = N_("Reverb");
     //FIXME: add clear_state
     plugin = this;
     engine.signal_buffersize_change().connect(
@@ -383,7 +369,7 @@ void ConvolverAdapter::change_buffersize(unsigned int size) {
 }
 
 void ConvolverAdapter::restart() {
-    if (!plugin.on_off) {
+    if (!plugin.get_on_off()) {
         return;
     }
     conv.set_not_runnable();
@@ -402,7 +388,7 @@ void ConvolverAdapter::restart() {
     int policy, priority;
     engine.get_sched_priority(policy, priority);
     if (!rc || !conv.start(policy, priority)) {
-        plugin.on_off = false;
+        plugin.set_on_off(false);
     }
 }
 
@@ -412,8 +398,8 @@ bool ConvolverAdapter::conv_start() {
     }
     string path = jcset.getFullIRPath();
     if (path.empty()) {
-        gx_system::gx_print_warning(_("convolver"), _("no impulseresponse file"));
-        plugin.on_off = false;
+        gx_print_warning(_("convolver"), _("no impulseresponse file"));
+        plugin.set_on_off(false);
         return false;
     }
     while (!conv.checkstate());
@@ -436,30 +422,57 @@ bool ConvolverAdapter::conv_start() {
     return conv.start(policy, priority);
 }
 
-void ConvolverAdapter::convolver(int count, float *input0, float *input1,
+
+/****************************************************************
+ ** class ConvolverStereoAdapter
+ */
+
+ConvolverStereoAdapter::ConvolverStereoAdapter(
+    EngineControl& engine_, sigc::slot<void> sync_, ParamMap& param_)
+    : ConvolverAdapter(engine_, sync_, param_) {
+    id = "jconv";
+    name = N_("Convolver");
+    register_params = convolver_register;
+    set_samplerate = convolver_init;
+    activate_plugin = activate;
+    stereo_audio = convolver;
+}
+
+ConvolverStereoAdapter::~ConvolverStereoAdapter() {
+}
+
+void ConvolverStereoAdapter::convolver(int count, float *input0, float *input1,
 				 float *output0, float *output1, PluginDef* plugin) {
-    ConvolverAdapter& self = *static_cast<ConvolverAdapter*>(plugin);
+    ConvolverStereoAdapter& self = *static_cast<ConvolverStereoAdapter*>(plugin);
     if (self.conv.is_runnable()) {
         float conv_out0[count];
         float conv_out1[count];
-        if (!self.conv.compute(count, output0, output1, conv_out0, conv_out1)) {
-            self.plugin.on_off = false;
-	    gx_system::gx_print_error("Convolver", "overload");
-        } else {
-            self.jc_post.compute(count, output0, output1,
-			    conv_out0, conv_out1, output0, output1);
+        if (self.conv.compute(count, input0, input1, conv_out0, conv_out1)) {
+            self.jc_post.compute(count, input0, input1,
+				 conv_out0, conv_out1, output0, output1);
+	    return;
         }
+	self.engine.overload(EngineControl::ov_Convolver, self.id);
+    }
+    if (input0 != output0) {
+	memcpy(output0, input0, count * sizeof(float));
+    }
+    if (input1 != output1) {
+	memcpy(output1, input1, count * sizeof(float));
     }
 }
 
-int ConvolverAdapter::convolver_register(const ParamReg& reg) {
-    ConvolverAdapter& self = *static_cast<ConvolverAdapter*>(reg.plugin);
+int ConvolverStereoAdapter::convolver_register(const ParamReg& reg) {
+    ConvolverStereoAdapter& self = *static_cast<ConvolverStereoAdapter*>(reg.plugin);
     self.jcp = JConvParameter::insert_param(self.param, "jconv.convolver", self, &self.jcset);
+    self.jcp->signal_changed().connect(
+	sigc::hide(
+	    sigc::mem_fun(self, &ConvolverStereoAdapter::restart)));
     return self.jc_post.register_par(reg);
 }
 
-void ConvolverAdapter::convolver_init(unsigned int samplingFreq, PluginDef *p) {
-    ConvolverAdapter& self = *static_cast<ConvolverAdapter*>(p);
+void ConvolverStereoAdapter::convolver_init(unsigned int samplingFreq, PluginDef *p) {
+    ConvolverStereoAdapter& self = *static_cast<ConvolverStereoAdapter*>(p);
     boost::mutex::scoped_lock lock(self.activate_mutex);
     if (self.activated) {
 	self.conv.stop_process();
@@ -475,8 +488,8 @@ void ConvolverAdapter::convolver_init(unsigned int samplingFreq, PluginDef *p) {
     }
 }
 
-int ConvolverAdapter::activate(bool start, PluginDef *p) {
-    ConvolverAdapter& self = *static_cast<ConvolverAdapter*>(p);
+int ConvolverStereoAdapter::activate(bool start, PluginDef *p) {
+    ConvolverStereoAdapter& self = *static_cast<ConvolverStereoAdapter*>(p);
     boost::mutex::scoped_lock lock(self.activate_mutex);
     if (start) {
 	if (self.activated && self.conv.is_runnable()) {
@@ -490,7 +503,7 @@ int ConvolverAdapter::activate(bool start, PluginDef *p) {
     self.activated = start;
     if (start) {
 	if (self.jc_post.activate(true) != 0) {
-	    gx_system::gx_print_error(_("convolver"), "jconv post activate error?!");
+	    gx_print_error(_("convolver"), "jconv post activate error?!");
 	    return -1;
 	}
 	if (!self.conv_start()) {
@@ -504,6 +517,86 @@ int ConvolverAdapter::activate(bool start, PluginDef *p) {
 }
 
 
+/****************************************************************
+ ** class ConvolverMonoAdapter
+ */
+
+ConvolverMonoAdapter::ConvolverMonoAdapter(
+    EngineControl& engine_, sigc::slot<void> sync_, ParamMap& param_)
+    : ConvolverAdapter(engine_, sync_, param_) {
+    id = "jconv_mono";
+    name = N_("Convolver");
+    register_params = convolver_register;
+    set_samplerate = convolver_init;
+    activate_plugin = activate;
+    mono_audio = convolver;
+}
+
+ConvolverMonoAdapter::~ConvolverMonoAdapter() {
+}
+
+void ConvolverMonoAdapter::convolver(int count, float *input, float *output, PluginDef* plugin) {
+    ConvolverMonoAdapter& self = *static_cast<ConvolverMonoAdapter*>(plugin);
+    if (self.conv.is_runnable()) {
+        float conv_out[count];
+        if (self.conv.compute(count, input, conv_out)) {
+            self.jc_post_mono.compute(count, output, conv_out, output);
+	    return;
+        }
+	self.engine.overload(EngineControl::ov_Convolver, self.id);
+    }
+    if (input != output) {
+	memcpy(output, input, count * sizeof(float));
+    }
+}
+
+int ConvolverMonoAdapter::convolver_register(const ParamReg& reg) {
+    ConvolverMonoAdapter& self = *static_cast<ConvolverMonoAdapter*>(reg.plugin);
+    self.jcp = JConvParameter::insert_param(self.param, "jconv_mono.convolver", self, &self.jcset);
+    self.jcp->signal_changed().connect(
+	sigc::hide(
+	    sigc::mem_fun(self, &ConvolverMonoAdapter::restart)));
+    return self.jc_post_mono.register_par(reg);;
+}
+
+void ConvolverMonoAdapter::convolver_init(unsigned int samplingFreq, PluginDef *p) {
+    ConvolverMonoAdapter& self = *static_cast<ConvolverMonoAdapter*>(p);
+    boost::mutex::scoped_lock lock(self.activate_mutex);
+    if (self.activated) {
+	self.conv.stop_process();
+	self.conv.set_samplerate(samplingFreq);
+	while (self.conv.is_runnable()) {
+	    self.conv.checkstate();
+	}
+	self.conv_start();
+    } else {
+	self.conv.set_samplerate(samplingFreq);
+    }
+}
+
+int ConvolverMonoAdapter::activate(bool start, PluginDef *p) {
+    ConvolverMonoAdapter& self = *static_cast<ConvolverMonoAdapter*>(p);
+    boost::mutex::scoped_lock lock(self.activate_mutex);
+    if (start) {
+	if (self.activated && self.conv.is_runnable()) {
+	    return 0;
+	}
+    } else {
+	if (!self.activated) {
+	    return 0;
+	}
+    }
+    self.activated = start;
+    if (start) {
+	if (!self.conv_start()) {
+	    return -1;
+	}
+    } else {
+	self.conv.stop_process();
+    }
+    return 0;
+}
+
 /****************************************************************
  ** class BaseConvolver
  */
@@ -551,7 +644,7 @@ void BaseConvolver::init(unsigned int samplingFreq, PluginDef *p) {
 }
 
 bool BaseConvolver::check_update_timeout() {
-    if (!activated || !plugin.on_off) {
+    if (!activated || !plugin.get_on_off()) {
 	return false;
     }
     check_update();
@@ -628,6 +721,8 @@ struct CabEntry {
     { "1x15",        N_("1x15"),        &static_cast<CabDesc&>(cab_data_1x15) },
     { "Mesa Boogie", N_("Mesa Boogie"), &static_cast<CabDesc&>(cab_data_mesa) },
     { "Briliant",    N_("Briliant"),    &static_cast<CabDesc&>(cab_data_briliant) },
+    { "Vitalize",     N_("Vitalize"),     &static_cast<CabDesc&>(cab_data_vitalize) },
+    { "Charisma",    N_("Charisma"),    &static_cast<CabDesc&>(cab_data_charisma) },
 };
 static const unsigned int cab_table_size = sizeof(cab_table) / sizeof(cab_table[0]);
 
@@ -642,6 +737,28 @@ static const float no_sum = 1e10;
 
 #include "faust/cabinet_impulse_former.cc"
 
+static int cab_load_ui(const UiBuilder& builder, int format) {
+    if (!(format & UI_FORM_STACK)) {
+	return -1;
+    }
+    builder.openHorizontalhideBox("");
+    builder.create_selector_no_caption("cab.select");
+    builder.closeBox();
+    builder.openVerticalBox("");
+    {
+	builder.openHorizontalBox("");
+	{
+	    builder.create_selector_no_caption("cab.select");
+	    builder.create_small_rackknob("cab.bass", "bass");
+	    builder.create_small_rackknob("cab.treble", "treble");
+	    builder.create_small_rackknobr("cab.Level", "level");
+	}
+	builder.closeBox();
+    }
+    builder.closeBox();
+    return 0;
+}
+
 CabinetConvolver::CabinetConvolver(EngineControl& engine, sigc::slot<void> sync, gx_resample::BufferResampler& resamp):
     BaseConvolver(engine, sync, resamp),
     current_cab(-1),
@@ -661,6 +778,8 @@ CabinetConvolver::CabinetConvolver(EngineControl& engine, sigc::slot<void> sync,
     cab_names[cab_table_size].value_label = 0;
     id = "cab";
     name = N_("Cabinet");
+    category = N_("Tone control");
+    load_ui = cab_load_ui;
     mono_audio = run_cab_conf;
     register_params = register_cab;
 }
@@ -697,6 +816,18 @@ bool CabinetConvolver::do_update() {
     return conv_start();
 }
 
+bool CabinetConvolver::do_only_update() {
+    CabDesc& cab = *getCabEntry(cabinet).data;
+    float cab_irdata_c[cab.ir_count];
+    impf.compute(cab.ir_count,cab.ir_data,cab_irdata_c);
+
+	if (!conv.update(cab.ir_count, cab_irdata_c, cab.ir_sr)) {
+	    return false;
+	}
+    update_sum();
+    return true;
+}
+
 bool CabinetConvolver::start(bool force) {
     if (force) {
 	current_cab = -1;
@@ -715,14 +846,13 @@ bool CabinetConvolver::start(bool force) {
 void CabinetConvolver::check_update() {
     if (cabinet_changed() || sum_changed()) {
 	do_update();
-    }
+    } 
 }
 
 void CabinetConvolver::run_cab_conf(int count, float *input0, float *output0, PluginDef *p) {
     CabinetConvolver& self = *static_cast<CabinetConvolver*>(p);
     if (!self.conv.compute(count, output0)) {
-	self.plugin.on_off = false;
-	gx_system::gx_print_error("Convolver", "cabinet overload");
+	self.engine.overload(EngineControl::ov_Convolver, "cab");
     }
 }
 
@@ -736,6 +866,180 @@ int CabinetConvolver::register_cab(const ParamReg& reg) {
     return 0;
 }
 
+/****************************************************************
+ ** class PreampConvolver
+ */
+
+struct PreDesc {
+    int ir_count;
+    int ir_sr;
+    float ir_data[];
+};
+
+template <int tab_size>
+struct PreDesc_imp {
+    int ir_count;
+    int ir_sr;
+    float ir_data[tab_size];
+    operator PreDesc&() { return *(PreDesc*)this; }
+};
+
+#include "gx_preamp_data.cc"
+
+struct PreEntry {
+    const char *value_id;
+    const char *value_label;
+    PreDesc *data;
+} pre_table[] = {
+    { "AC30",        N_("AC30"),        &static_cast<PreDesc&>(pre_data_ac30) },
+    { "Bassman",     N_("Bassman"),     &static_cast<PreDesc&>(pre_data_bassman) },
+    { "Tube",        N_("Tube"),        &static_cast<PreDesc&>(pre_data_tube) },
+    { "Fender",      N_("Fender"),      &static_cast<PreDesc&>(pre_data_fender) },
+    { "JCM800",      N_("JCM800"),      &static_cast<PreDesc&>(pre_data_jcm800) },
+    { "JTM45",       N_("JTM45"),       &static_cast<PreDesc&>(pre_data_jtm45) },
+    { "Mesa Boogie", N_("Mesa Boogie"), &static_cast<PreDesc&>(pre_data_mesaboogie) },
+    { "Boutique",    N_("Boutique"),    &static_cast<PreDesc&>(pre_data_boutique) },
+    { "Ampeg",       N_("Ampeg"),       &static_cast<PreDesc&>(pre_data_ampeg) },
+    { "Rectifier",   N_("Rectifier"),   &static_cast<PreDesc&>(pre_data_rectifier) },
+};
+static const unsigned int pre_table_size = sizeof(pre_table) / sizeof(pre_table[0]);
+
+static PreEntry& getPreEntry(unsigned int n) {
+    if (n >= pre_table_size) {
+	n = pre_table_size - 1;
+    }
+    return pre_table[n];
+}
+
+#include "faust/preamp_impulse_former.cc"
+
+static int pre_load_ui(const UiBuilder& builder, int format) {
+    if (!(format & UI_FORM_STACK)) {
+	return -1;
+    }
+    builder.openHorizontalhideBox("");
+    builder.create_selector_no_caption("pre.select");
+    builder.closeBox();
+    builder.openVerticalBox("");
+    {
+	builder.openHorizontalBox("");
+	{
+	    builder.create_selector_no_caption("pre.select");
+	    builder.create_small_rackknob("pre.bass", "bass");
+	    builder.create_small_rackknob("pre.treble", "treble");
+	    builder.create_small_rackknobr("pre.Level", "level");
+	}
+	builder.closeBox();
+    }
+    builder.closeBox();
+    return 0;
+}
+
+PreampConvolver::PreampConvolver(EngineControl& engine, sigc::slot<void> sync, gx_resample::BufferResampler& resamp):
+    BaseConvolver(engine, sync, resamp),
+    current_pre(-1),
+    level(0),
+    preamp(0),
+    bass(0),
+    treble(0),
+    sum(no_sum),
+    pre_names(new value_pair[pre_table_size+1]),
+    impf() {
+    for (unsigned int i = 0; i < pre_table_size; ++i) {
+	PreEntry& pre = getPreEntry(i);
+	pre_names[i].value_id = pre.value_id;
+	pre_names[i].value_label = pre.value_label;
+    }
+    pre_names[pre_table_size].value_id = 0;
+    pre_names[pre_table_size].value_label = 0;
+    id = "pre";
+    name = N_("Amp impulse");
+    category = N_("Tone control");
+    load_ui = pre_load_ui;
+    mono_audio = run_pre_conf;
+    register_params = register_pre;
+}
+
+PreampConvolver::~PreampConvolver() {
+    delete[] pre_names;
+}
+
+bool PreampConvolver::do_update() {
+    bool configure = preamp_changed();
+    if (conv.is_runnable()) {
+	conv.set_not_runnable();
+	sync();
+	conv.stop_process();
+    }
+    PreDesc& pre = *getPreEntry(preamp).data;
+    if (current_pre == -1) {
+	impf.init(pre.ir_sr);
+    }
+    float pre_irdata_c[pre.ir_count];
+    impf.compute(pre.ir_count,pre.ir_data,pre_irdata_c);
+    while (!conv.checkstate());
+    if (configure) {
+	if (!conv.configure(pre.ir_count, pre_irdata_c, pre.ir_sr)) {
+	    return false;
+	}
+    } else {
+	if (!conv.update(pre.ir_count, pre_irdata_c, pre.ir_sr)) {
+	    return false;
+	}
+    }
+    update_preamp();
+    update_sum();
+    return conv_start();
+}
+
+bool PreampConvolver::do_only_update() {
+    PreDesc& pre = *getPreEntry(preamp).data;
+    float pre_irdata_c[pre.ir_count];
+    impf.compute(pre.ir_count,pre.ir_data,pre_irdata_c);
+	if (!conv.update(pre.ir_count, pre_irdata_c, pre.ir_sr)) {
+	    return false;
+	}
+    update_sum();
+    return true;
+}
+
+bool PreampConvolver::start(bool force) {
+    if (force) {
+	current_pre = -1;
+    }
+    if (preamp_changed() || sum_changed()) {
+	return do_update();
+    } else {
+	while (!conv.checkstate());
+	if (!conv.is_runnable()) {
+	    return conv_start();
+	}
+	return true;
+    }
+}
+
+void PreampConvolver::check_update() {
+    if (preamp_changed() || sum_changed()) {
+	do_update();
+    }
+}
+
+void PreampConvolver::run_pre_conf(int count, float *input0, float *output0, PluginDef *p) {
+    PreampConvolver& self = *static_cast<PreampConvolver*>(p);
+    if (!self.conv.compute(count, output0)) {
+	self.engine.overload(EngineControl::ov_Convolver, "pre");
+    }
+}
+
+int PreampConvolver::register_pre(const ParamReg& reg) {
+    PreampConvolver& pre = *static_cast<PreampConvolver*>(reg.plugin);
+    reg.registerIEnumVar("pre.select", "select", "B", "", pre.pre_names, &pre.preamp, 0);
+    reg.registerVar("pre.Level", "",  "S", "", &pre.level,  1.0, 0.1, 2.1, 0.1);
+    reg.registerVar("pre.bass", "",   "S", "", &pre.bass,   0.0, -10.0, 10.0, 0.5);
+    reg.registerVar("pre.treble", "", "S", "", &pre.treble, 0.0, -10.0, 10.0, 0.5);
+    pre.impf.register_par(reg);
+    return 0;
+}
 
 /****************************************************************
  ** class ContrastConvolver
@@ -783,6 +1087,17 @@ bool ContrastConvolver::do_update() {
     return conv_start();
 }
 
+bool ContrastConvolver::do_only_update() {
+    float contrast_irdata_c[contrast_ir_desc.ir_count];
+    presl.compute(contrast_ir_desc.ir_count,contrast_ir_desc.ir_data,contrast_irdata_c);
+    
+	if (!conv.update(contrast_ir_desc.ir_count, contrast_irdata_c, contrast_ir_desc.ir_sr)) {
+	    return false;
+	}
+    update_sum();
+    return true;
+}
+
 bool ContrastConvolver::start(bool force) {
     if (force) {
 	sum = no_sum;
@@ -814,8 +1129,7 @@ int ContrastConvolver::register_con(const ParamReg& reg) {
 void ContrastConvolver::run_contrast(int count, float *input0, float *output0, PluginDef *p) {
     ContrastConvolver& self = *static_cast<ContrastConvolver*>(p);
     if (!self.conv.compute(count, output0)) {
-	self.plugin.on_off = false;
-	gx_system::gx_print_error("Convolver", "presence overload");
+	self.engine.overload(EngineControl::ov_Convolver, "contrast");
     }
 }
 
