@@ -48,6 +48,53 @@ string GxJack::get_default_instancename() {
     return default_jack_instancename;
 }
 
+
+/****************************************************************
+ ** rt_watchdog
+ */
+
+static unsigned int rt_watchdog_counter;
+
+#ifndef SCHED_IDLE
+#define SCHED_IDLE SCHED_OTHER  // non-linux systems
+#endif
+
+static void *rt_watchdog_run(void *p) {
+    struct sched_param  spar;
+    spar.sched_priority = 0;
+    pthread_setschedparam(pthread_self(), SCHED_IDLE, &spar);
+    while (true) {
+	gx_system::atomic_set(&rt_watchdog_counter, 0);
+	usleep(1000000);
+    }
+    return NULL;
+}
+
+static int rt_watchdog_limit = 0;
+
+static void rt_watchdog_start() {
+    if (rt_watchdog_limit > 0) {
+	pthread_attr_t      attr;
+	pthread_attr_init(&attr);
+	pthread_t pthr;
+	if (pthread_create(&pthr, &attr, rt_watchdog_run, 0)) {
+	    gx_print_error("watchdog", _("can't create thread"));
+	}
+	pthread_attr_destroy(&attr);
+    }
+}
+
+static inline bool rt_watchdog_check_alive(unsigned int bs, unsigned int sr) {
+    if (rt_watchdog_limit > 0) {
+	if (gx_system::atomic_get(rt_watchdog_counter) > rt_watchdog_limit*(2*sr)/bs) {
+	    return false;
+	}
+	gx_system::atomic_inc(&rt_watchdog_counter);
+    }
+    return true;
+}
+
+
 /****************************************************************
  ** GxJack ctor, dtor
  */
@@ -70,27 +117,36 @@ GxJack::GxJack(gx_engine::GxEngine& engine_)
       client_instance(),
       jack_sr(),
       jack_bs(),
+      xrun(),
       last_xrun(0),
+      xrun_msg_blocked(false),
       ports(),
       client(0),
       client_insert(0),
       client_name(),
       client_insert_name(),
-      xrun(),
       session(),
       session_ins(),
       shutdown(),
       connection() {
     connection_queue.new_data.connect(sigc::mem_fun(*this, &GxJack::fetch_connection_data));
     client_change_rt.connect(client_change);
-    gx_system::GxExit::get_instance().signal_exit().connect(
+    GxExit::get_instance().signal_exit().connect(
 	sigc::mem_fun(*this, &GxJack::cleanup_slot));
+    xrun.connect(sigc::mem_fun(this, &GxJack::report_xrun));
 }
 
 GxJack::~GxJack() {
     gx_jack_cleanup();
 }
 
+void GxJack::rt_watchdog_set_limit(int limit) {
+    rt_watchdog_limit = limit;
+    if (limit > 0) {
+	rt_watchdog_start();
+    }
+}
+
 
 /****************************************************************
  ** load state, save state
@@ -116,7 +172,7 @@ void GxJack::read_connections(gx_system::JsonParser& jp) {
         } else if (jp.current_value() == "insert_in") {
             i = &ports.insert_in.conn;
         } else {
-	    gx_system::gx_print_warning(
+	    gx_print_warning(
 		_("recall state"),
 		_("unknown jack ports section: ") + jp.current_value());
             jp.skip_object();
@@ -174,10 +230,9 @@ void GxJack::write_connections(gx_system::JsonWriter& w) {
  */
 
 // ----- pop up a dialog for starting jack
-bool GxJack::gx_jack_init(bool startserver, int wait_after_connect) {
+bool GxJack::gx_jack_init(bool startserver, int wait_after_connect, const gx_system::CmdlineOptions& opt) {
     AVOIDDENORMALS();
     int jackopt = (startserver ? JackNullOption : JackNoStartServer);
-    gx_system::CmdlineOptions& opt = gx_system::get_options();
     client_instance = opt.get_jack_instancename();
     if (client_instance.empty()) {
 	client_instance = get_default_instancename();
@@ -185,6 +240,8 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect) {
 	jackopt |= JackUseExactName;
     }
 
+    std::string ServerName = opt.get_jack_servername();
+
     set_jack_down(false);
     set_jack_exit(true);
     engine.set_stateflag(gx_engine::GxEngine::SF_INITIALIZING);
@@ -201,10 +258,20 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect) {
 	    client_name.c_str(), JackOptions(jackopt | JackSessionID),
 	    &jackstat, opt.get_jack_uuid().c_str());
     } else {
+        if (ServerName.empty()) {
         client = jack_client_open(client_name.c_str(), JackOptions(jackopt), &jackstat);
+        } else {
+        client = jack_client_open(client_name.c_str(), JackOptions(jackopt | JackServerName),
+        &jackstat, ServerName.c_str());
+        }
     }
 #else
+    if (ServerName.empty()) {
     client = jack_client_open(client_name.c_str(), JackOptions(jackopt), &jackstat);
+    } else {
+    client = jack_client_open(client_name.c_str(), JackOptions(jackopt | JackServerName),
+    &jackstat, ServerName.c_str());
+    }
 #endif
     // ----- only start the insert gxjack.client when the amp gxjack.client is true
     if (client) {
@@ -224,14 +291,28 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect) {
 		JackOptions(jackopt | JackSessionID | JackUseExactName),
 		&jackstat, opt.get_jack_uuid2().c_str());
         } else {
+            if (ServerName.empty()) {
             client_insert = jack_client_open(
 		client_insert_name.c_str(),
-		JackOptions(jackopt | JackUseExactName), &jackstat);
+		JackOptions(jackopt | JackUseExactName ), &jackstat);
+        } else {
+            client_insert = jack_client_open(
+		client_insert_name.c_str(),
+		JackOptions(jackopt | JackUseExactName | JackServerName),
+        &jackstat, ServerName.c_str());
+        }
         }
 #else
+        if (ServerName.empty()) {
         client_insert = jack_client_open(
 	    client_insert_name.c_str(),
 	    JackOptions(jackopt | JackUseExactName), &jackstat);
+        } else {
+        client_insert = jack_client_open(
+	    client_insert_name.c_str(),
+	    JackOptions(jackopt | JackUseExactName | JackServerName),
+        &jackstat, ServerName.c_str());
+        }
 #endif
 	if (!client_insert) {
 	    jack_client_close(client);
@@ -242,12 +323,12 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect) {
     if (!client) {
 	if (!(jackstat & JackServerFailed)) {
 	    if ((jackstat & JackServerError) && (jackopt & JackUseExactName)) {
-		gx_system::gx_print_error(
+		gx_print_error(
 		    _("Jack Init"),
 		    boost::format(_("can't get requested jack instance name '%1%'"))
 		    % client_instance);
 	    } else {
-		gx_system::gx_print_error(
+		gx_print_error(
 		    _("Jack Init"),
 		    _("unknown jack server communication error"));
 	    }
@@ -262,21 +343,21 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect) {
 	usleep(wait_after_connect);
     }
     jack_sr = jack_get_sample_rate(client); // jack sample rate
-    gx_system::gx_print_info(
+    gx_print_info(
 	_("Jack init"),
 	boost::format(_("The jack sample rate is %1%/sec")) % jack_sr);
 
     jack_bs = jack_get_buffer_size(client); // jack buffer size
-    gx_system::gx_print_info(
+    gx_print_info(
 	_("Jack init"),
 	boost::format(_("The jack buffer size is %1%/frames ... "))
 	% jack_bs);
 
     gx_jack_callbacks();
     client_change(); // might load port connection definitions
-    if (opt.get_jack_uuid().empty()) {
+    if (opt.get_jack_uuid().empty() && !opt.get_jack_noconnect()) {
 	// when not loaded by session manager
-	gx_jack_init_port_connection();
+	gx_jack_init_port_connection(opt);
     }
     set_jack_exit(false);
 
@@ -284,15 +365,26 @@ bool GxJack::gx_jack_init(bool startserver, int wait_after_connect) {
 }
 
 void GxJack::cleanup_slot(bool otherthread) {
-    if (!client || is_jack_down()) {
-	return;
+    if (!otherthread) {
+	gx_jack_cleanup();
+    } else {
+	// called from other thread. Since most cleanup functions are
+	// not thread safe, just do minimal jack cleanup
+	if (client) {
+	    if (!is_jack_down()) {
+		engine.start_ramp_down();
+		engine.wait_ramp_down_finished();
+	    }
+	    jack_deactivate(client);
+	    jack_client_close(client);
+	    client = 0;
+	}
+	if (client_insert) {
+	    jack_deactivate(client_insert);
+	    jack_client_close(client_insert);
+	    client_insert = 0;
+	}
     }
-    engine.start_ramp_down();
-    engine.wait_ramp_down_finished();
-    jack_client_close(client);
-    client = 0;
-    jack_client_close(client_insert);
-    client_insert = 0;
 }
 
 // -----Function that cleans the jack stuff on shutdown
@@ -323,15 +415,15 @@ void GxJack::gx_jack_cleanup() {
 }
 
 // ---- Jack server connection / disconnection
-bool GxJack::gx_jack_connection(bool connect, bool startserver, int wait_after_connect) {
+bool GxJack::gx_jack_connection(bool connect, bool startserver, int wait_after_connect, const gx_system::CmdlineOptions& opt) {
     if (connect) {
 	if (client) {
 	    return true;
 	}
-	if (!gx_jack_init(startserver, wait_after_connect)) {
+	if (!gx_jack_init(startserver, wait_after_connect, opt)) {
 	    return false;
 	}
-	engine.check_module_lists();
+	engine.set_rack_changed();
 	engine.clear_stateflag(gx_engine::GxEngine::SF_INITIALIZING);
     } else {
 	if (!client) {
@@ -374,8 +466,7 @@ std::string GxJack::replace_clientvar(const std::string& s) {
 }
 
 // ----- connect ports if we know them
-void GxJack::gx_jack_init_port_connection() {
-    gx_system::CmdlineOptions& opt = gx_system::get_options();
+void GxJack::gx_jack_init_port_connection(const gx_system::CmdlineOptions& opt) {
     // set autoconnect capture to user capture port
     if (!opt.get_jack_input().empty()) {
         jack_connect(client, opt.get_jack_input().c_str(),
@@ -508,12 +599,12 @@ void GxJack::gx_jack_callbacks() {
     jack_set_process_callback(client, gx_jack_process, this);
     jack_set_process_callback(client_insert, gx_jack_insert_process, this);
     if (jack_activate(client) != 0) {
-        gx_system::gx_print_fatal(
+        gx_print_fatal(
 	    _("Jack Activation"),
 	    string(_("Can't activate JACK gx_amp client")));
     }
     if (jack_activate(client_insert) != 0) {
-        gx_system::gx_print_fatal(_("Jack Activation"),
+        gx_print_fatal(_("Jack Activation"),
                        string(_("Can't activate JACK gx_amp_fx client")));
     }
 }
@@ -539,12 +630,21 @@ static inline float *get_float_buf(jack_port_t *port, jack_nframes_t nframes) {
     return static_cast<float *>(jack_port_get_buffer(port, nframes));
 }
 
+inline void GxJack::check_overload() {
+    if (!rt_watchdog_check_alive(jack_bs, jack_sr)) {
+	engine.overload(gx_engine::EngineControl::ov_User, "watchdog thread");
+    }
+}
+
 // ----- main jack process method gx_amp, mono -> mono
 // RT process thread
-int GxJack::gx_jack_process(jack_nframes_t nframes, void *arg) {
+int __rt_func GxJack::gx_jack_process(jack_nframes_t nframes, void *arg) {
     gx_system::measure_start();
     GxJack& self = *static_cast<GxJack*>(arg);
     if (!self.is_jack_exit()) {
+	if (!self.engine.mono_chain.is_stopped()) {
+	    self.check_overload();
+	}
         // gx_head DSP computing
 	self.engine.mono_chain.process(
 	    nframes,
@@ -553,7 +653,7 @@ int GxJack::gx_jack_process(jack_nframes_t nframes, void *arg) {
 
         // midi input processing
 	if (self.ports.midi_input.port) {
-	    gx_engine::controller_map.compute_midi_in(
+	    self.engine.controller_map.compute_midi_in(
 		jack_port_get_buffer(self.ports.midi_input.port, nframes));
 	}
     }
@@ -564,10 +664,13 @@ int GxJack::gx_jack_process(jack_nframes_t nframes, void *arg) {
 
 // ----- main jack process method, gx_fx_amp, mono -> stereo
 // RT process_insert thread
-int GxJack::gx_jack_insert_process(jack_nframes_t nframes, void *arg) {
+int __rt_func GxJack::gx_jack_insert_process(jack_nframes_t nframes, void *arg) {
     GxJack& self = *static_cast<GxJack*>(arg);
     gx_system::measure_cont();
     if (!self.is_jack_exit()) {
+	if (!self.engine.stereo_chain.is_stopped()) {
+	    self.check_overload();
+	}
         // gx_head DSP computing
 	float *ibuf = get_float_buf(self.ports.insert_in.port, nframes);
 	self.engine.stereo_chain.process(
@@ -592,7 +695,7 @@ PortConnRing::PortConnRing()
       new_data(),
       portchange() {
     if (!ring) {
-	gx_system::gx_print_fatal(
+	gx_print_fatal(
 	    _("Jack init"), _("can't get memory for ringbuffer"));
     }
     jack_ringbuffer_mlock(ring);
@@ -753,6 +856,22 @@ void GxJack::shutdown_callback_client_insert(void *arg) {
     self.gx_jack_shutdown_callback();
 }
 
+void GxJack::report_xrun_clear() {
+    xrun_msg_blocked = false;
+}
+
+void GxJack::report_xrun() {
+    if (xrun_msg_blocked) {
+	return;
+    }
+    xrun_msg_blocked = true;
+    Glib::signal_timeout().connect_once(
+	sigc::mem_fun(this, &GxJack::report_xrun_clear), 100);
+    gx_print_warning(
+	_("Jack XRun"),
+	(boost::format(_(" delay of at least %1% microsecs")) % last_xrun).str());
+}
+
 // ---- jack xrun callback
 int GxJack::gx_jack_xrun_callback(void* arg) {
     GxJack& self = *static_cast<GxJack*>(arg);
@@ -760,6 +879,9 @@ int GxJack::gx_jack_xrun_callback(void* arg) {
 	return 0;
     }
     self.last_xrun = jack_get_xrun_delayed_usecs(self.client);
+    if (!self.engine.mono_chain.is_stopped()) {
+	self.engine.overload(gx_engine::EngineControl::ov_XRun, "xrun");
+    }
     self.xrun();
     return 0;
 }
@@ -811,7 +933,7 @@ string GxJack::get_uuid_insert() {
 	    client_insert, client_insert_name.c_str());
     } else {
 	assert(false);
-	gx_system::gx_print_error(_("session save"), _("can't get client uuid"));
+	gx_print_error(_("session save"), _("can't get client uuid"));
 	return "";
     }
     string ret(uuid);
@@ -823,7 +945,7 @@ void GxJack::gx_jack_session_callback(jack_session_event_t *event, void *arg) {
     GxJack& self = *static_cast<GxJack*>(arg);
     jack_session_event_t *np = 0;
     if (!gx_system::atomic_compare_and_exchange(&self.session_event, np, event)) {
-	gx_system::gx_print_error("jack","last session not cleared");
+	gx_print_error("jack","last session not cleared");
 	return;
     }
     self.session();
@@ -833,7 +955,7 @@ void GxJack::gx_jack_session_callback_ins(jack_session_event_t *event, void *arg
     GxJack& self = *static_cast<GxJack*>(arg);
     jack_session_event_t *np = 0;
     if (!gx_system::atomic_compare_and_exchange(&self.session_event_ins, np, event)) {
-	gx_system::gx_print_error("jack","last session not cleared");
+	gx_print_error("jack","last session not cleared");
 	return;
     }
     self.session_ins();
