@@ -46,21 +46,30 @@ bool check_mtime(const std::string& filename, time_t& mtime) {
  ** JsonWriter
  */
 
-JsonWriter::JsonWriter(ostream *o)
+JsonWriter::JsonWriter(ostream *o, bool enable_newlines)
     : os(o),
       first(true),
-      deferred_nl(false),
+      deferred_nl(enable_newlines ? 0 : -1),
       indent("") {}
 
 JsonWriter::~JsonWriter() {
     close();
 }
 
+void JsonWriter::reset() {
+    os->flush();
+    first = true;
+    if (deferred_nl == 1) {
+	deferred_nl = 0;
+    }
+    indent.clear();
+}
+
 void JsonWriter::close() {
     if (is_closed()) {
 	return;
     }
-    if (deferred_nl) {
+    if (deferred_nl == 1) {
 	*os << endl;
     }
     os = 0;
@@ -69,7 +78,7 @@ void JsonWriter::close() {
 inline void JsonWriter::komma() {
     if (first)
         first = false;
-    else if (!deferred_nl)
+    else if (deferred_nl == 0)
         *os << ", ";
     else
         *os << ",";
@@ -79,7 +88,7 @@ inline void JsonWriter::komma() {
 inline void JsonWriter::space() {
     if (first)
         first = false;
-    else if (!deferred_nl)
+    else if (deferred_nl == 0)
         *os << " ";
     flush();
 }
@@ -94,15 +103,24 @@ inline void JsonWriter::iminus() {
     }
 }
 
+template<class T> static inline T fp_sanitize(T v) {
+    switch (fpclassify(v)) {
+    case FP_NAN: assert(false); return 1e50;
+    case FP_INFINITE: assert(false); return (v < 0 ? -1e50 : 1e50);
+    case FP_SUBNORMAL: return 0;
+    }
+    return v;
+}
+
 void JsonWriter::write(float v, bool nl) {
     komma();
-    *os << v;
+    *os << fp_sanitize(v);
     snl(nl);
 }
 
 void JsonWriter::write(double v, bool nl) {
     komma();
-    *os << v;
+    *os << fp_sanitize(v);
     snl(nl);
 }
 
@@ -125,20 +143,24 @@ void JsonWriter::write_lit(const string& s, bool nl) {
 }
 
 void JsonWriter::write(const char* p, bool nl) {
-    komma();
-    *os << '"';
-    while (*p) {
-        switch (*p) {
-        case '\\': case '"': *os << '\\'; break;
-        case '\b': *os << '\\'; *os << 'b'; continue;       // NOLINT
-        case '\f': *os << '\\'; *os << 'f'; continue;       // NOLINT
-        case '\n': *os << '\\'; *os << 'n'; continue;       // NOLINT
-        case '\r': *os << '\\'; *os << 'r'; continue;       // NOLINT
-        case '\t': *os << '\\'; *os << 't'; continue;       // NOLINT
-        }
-        *os << *p++;
+    if (p) {
+	komma();
+	*os << '"';
+	for (; *p; p++) {
+	    switch (*p) {
+	    case '\\': case '"': *os << '\\'; break;
+	    case '\b': *os << '\\'; *os << 'b'; continue;       // NOLINT
+	    case '\f': *os << '\\'; *os << 'f'; continue;       // NOLINT
+	    case '\n': *os << '\\'; *os << 'n'; continue;       // NOLINT
+	    case '\r': *os << '\\'; *os << 'r'; continue;       // NOLINT
+	    case '\t': *os << '\\'; *os << 't'; continue;       // NOLINT
+	    }
+	    *os << *p;
+	}
+	*os << '"';
+    } else {
+	write_null();
     }
-    *os << '"';
     snl(nl);
 }
 
@@ -188,7 +210,7 @@ void JsonWriter::write_key(const string& p, bool nl) {
 
 // called before output of next element
 void JsonWriter::flush() {
-    if (deferred_nl) {
+    if (deferred_nl == 1) {
         *os << endl;
         deferred_nl = false;
         *os << indent;
@@ -196,26 +218,56 @@ void JsonWriter::flush() {
 }
 
 
+/****************************************************************
+ ** class JsonStringWriter
+ */
+
+void JsonStringWriter::send_notify_begin(const char *method) {
+    begin_object();
+    write_key("jsonrpc");
+    write("2.0");
+    write_key("method");
+    write(method);
+    write_key("params");
+    begin_array();
+}
+
+void JsonStringWriter::send_notify_end() {
+    end_array();
+    end_object();
+}
+
+
 /****************************************************************
  ** JsonParser
  */
 
-const char* JsonParser::token_names[] = {
-    "no_token", "end_token", "begin_object", "end_object",
-    "begin_array", "end_array", "value_string", "value_number",
-    "value_key" };
+JsonException::JsonException(const Glib::ustring& desc) {
+    what_str = "Json parse error: " + desc;
+}
 
-JsonException::JsonException(const char* desc) {
-    what_str = string("Json parse error: ") + desc;
+void JsonParser::reset() {
+    depth = 0;
+    cur_tok = no_token;
+    str.clear();
+    nl = false;
+    next_depth = 0;
+    next_tok = no_token;
+    next_str.clear();
+    next_pos = 0;
 }
 
 JsonParser::JsonParser(istream* i)
     : is(i),
-    depth(0),
-    cur_tok(no_token),
-    nl(false),
-    next_depth(0),
-    next_tok(no_token) {}
+      depth(0),
+      cur_tok(no_token),
+      str(),
+      nl(false),
+      next_depth(0),
+      next_tok(no_token),
+      next_str(),
+      next_pos(0) {
+}
 
 JsonParser::~JsonParser() {
     close();
@@ -228,11 +280,31 @@ void JsonParser::close() {
     is = 0;
 }
 
+const char* JsonParser::get_token_name(token tok) {
+    switch (tok) {
+    case no_token: return "no_token";
+    case end_token: return "end_token";
+    case begin_object: return "begin_object";
+    case end_object: return "end_object";
+    case begin_array: return "begin_array";
+    case end_array: return "end_array";
+    case value_string: return "value_string";
+    case value_number: return "value_number";
+    case value_key: return "value_key";
+    case value_null: return "value_null";
+    case value_bool: return "value_bool";
+    case value_false: return "value_false";
+    case value_true: return "value_true";
+    default: assert(false); return 0;
+    }
+}
+
 void JsonParser::throw_unexpected(token expect) {
     ostringstream b;
-    b << "unexpected token: " << token_names[cur_tok]
-      << " (expected: " << token_names[expect] << ")"
+    b << "unexpected token: " << get_token_name(cur_tok)
+      << " (expected: " << get_token_name(expect) << ")"
       << endl;
+    //cerr << b.str() << endl; assert(false);
     throw JsonException(b.str().c_str());
 }
 
@@ -267,7 +339,7 @@ const char* JsonParser::readcode() {
     for (int i = 0; i < 4; i++) {
         int n = is->get();
         if (!is->good())
-            throw JsonException("eof");
+            throw JsonExceptionEOF("eof");
         if ('0' <= n && n <= '9')
             n = n - '0';
         else
@@ -324,6 +396,40 @@ string JsonParser::readnumber(char c) {
     return "";
 }
 
+JsonParser::token JsonParser::read_value_token(char c) {
+    ostringstream os("");
+    do {
+        os << c;
+        c = is->peek();
+	if (c < 'a' || c > 'z') {
+	    break;
+        }
+        is->get(c);
+    } while (is->good());
+    next_str = os.str();
+    if (next_str == "null") {
+	return value_null;
+    }
+    if (next_str == "true") {
+	return value_true;
+    }
+    if (next_str == "false") {
+	return value_false;
+    }
+    // sanitize bad input
+    if (next_str == "nan" && next_str == "inf") {
+	assert(false);
+	next_str = "1e50";
+	return value_number;
+    }
+    if (next_str == "-inf") {
+	assert(false);
+	next_str = "-1e50";
+	return value_number;
+    }
+    return no_token;
+}
+
 void JsonParser::read_next() {
     if (next_tok == end_token)
         return;
@@ -337,7 +443,7 @@ void JsonParser::read_next() {
         do {
             is->get(c);
             if (!is->good())
-                throw JsonException("eof");
+                throw JsonExceptionEOF("eof");
             if (c == '\n')
                 nl = true;
         } while (c == ' ' || c == '\t' || c == '\r' || c == '\n');
@@ -357,7 +463,7 @@ void JsonParser::read_next() {
             next_str = readstring();
             *is >> c;
             if (!is->good())
-                throw JsonException("eof");
+                throw JsonExceptionEOF("eof");
             if (c == ':') {
                 next_tok = value_key;
             } else {
@@ -373,7 +479,11 @@ void JsonParser::read_next() {
             break;
 
         default:
-            throw JsonException("bad token");
+	    next_tok = read_value_token(c);
+	    if (next_tok == no_token) {
+		throw JsonException("bad token");
+	    }
+	    break;
         }
         break;
     }
@@ -394,6 +504,66 @@ JsonParser::token JsonParser::next(token expect) {
     return cur_tok;
 }
 
+bool JsonParser::read_kv(const char *key, float& v) {
+    if (str == key) {
+	next(value_number);
+	v = current_value_float();
+	return true;
+    } else {
+	return false;
+    }
+}
+
+bool JsonParser::read_kv(const char *key, double& v) {
+    if (str == key) {
+	next(value_number);
+	v = current_value_double();
+	return true;
+    } else {
+	return false;
+    }
+}
+
+bool JsonParser::read_kv(const char *key, int& i) {
+    if (str == key) {
+	next(value_number);
+	i = current_value_int();
+	return true;
+    } else {
+	return false;
+    }
+}
+
+bool JsonParser::read_kv(const char *key, unsigned int& i) {
+    if (str == key) {
+	next(value_number);
+	i = current_value_uint();
+	return true;
+    } else {
+	return false;
+    }
+}
+
+bool JsonParser::read_kv(const char *key, string& s) {
+    if (str == key) {
+	next(value_string);
+	s = current_value();
+	return true;
+    } else {
+	return false;
+    }
+}
+
+bool JsonParser::read_kv(const char *key, Glib::ustring& s) {
+    if (str == key) {
+	next(value_string);
+	s = current_value();
+	return true;
+    } else {
+	return false;
+    }
+}
+
 void JsonParser::set_streampos(streampos pos) {
     is->seekg(pos);
     depth = 0;
@@ -443,6 +613,16 @@ void JsonParser::skip_object() {
     } while (curdepth != depth);
 }
 
+JsonSubParser::JsonSubParser(JsonParser& jp, streampos pos)
+    : JsonParser() {
+    set_stream(jp.get_stream());
+    position = get_stream()->tellg();
+    set_streampos(pos);
+}
+
+JsonSubParser::~JsonSubParser() {
+    get_stream()->seekg(position);
+}
 
 /****************************************************************
  ** class SettingsFileHeader
@@ -559,6 +739,12 @@ JsonParser *StateFile::create_reader() {
     return jp;
 }
 
+void StateFile::set_filename(const string& fn) {
+    filename = fn;
+    delete is;
+    is = 0;
+}
+
 void StateFile::ensure_is_current() {
     if (filename.empty() || !mtime) {
 	return;
@@ -684,10 +870,87 @@ PresetFile::PresetFile()
       flags() {
 }
 
+void PresetFile::readJSON_remote(JsonParser& jp) {
+    entries.clear();
+    flags = 0;
+    name = "";
+    tp = PresetFile::PRESET_FILE;
+    jp.next(JsonParser::begin_object);
+    while (jp.peek() != JsonParser::end_object) {
+	jp.next(JsonParser::value_key);
+	if (jp.current_value() == "name") {
+	    jp.next(JsonParser::value_string);
+	    name = jp.current_value();
+	} else if (jp.current_value() == "type") {
+	    jp.next(JsonParser::value_string);
+	    if (jp.current_value() == "scratch") {
+		tp = PresetFile::PRESET_SCRATCH;
+	    } else if (jp.current_value() == "factory") {
+		tp = PresetFile::PRESET_FACTORY;
+	    } else if (jp.current_value() == "file") {
+		tp = PresetFile::PRESET_FILE;
+	    }
+	} else if (jp.current_value() == "mutable") {
+	    jp.skip_object();
+	} else if (jp.current_value() == "flag_invalid") {
+	    flags |= PRESET_FLAG_INVALID;
+	} else if (jp.current_value() == "flag_readonly") {
+	    flags |= PRESET_FLAG_READONLY;
+	} else if (jp.current_value() == "flag_versiondiff") {
+	    flags |= PRESET_FLAG_VERSIONDIFF;
+	} else if (jp.current_value() == "presets") {
+	    jp.next(JsonParser::begin_array);
+	    while (jp.peek() != JsonParser::end_array) {
+		jp.next(JsonParser::value_string);
+		entries.push_back(Position(jp.current_value(), 0));
+	    }
+	    jp.next(JsonParser::end_array);
+	} else {
+	    gx_print_warning(
+		"PresetFile", Glib::ustring::compose("%1: unknown remote key: %2", name, jp.current_value()));
+	}
+    }
+    jp.next(JsonParser::end_object);
+}
+
+void PresetFile::writeJSON_remote(gx_system::JsonWriter& jw) {
+    jw.begin_object();
+    jw.write_key("name");
+    jw.write(name);
+    jw.write_key("mutable");
+    jw.write(is_mutable());
+    jw.write_key("type");
+    switch (tp) {
+    case gx_system::PresetFile::PRESET_SCRATCH: jw.write("scratch"); break;
+    case gx_system::PresetFile::PRESET_FACTORY: jw.write("factory"); break;
+    case gx_system::PresetFile::PRESET_FILE: jw.write("file"); break;
+    default: jw.write("unknown"); break;
+    }
+    if (flags & gx_system::PRESET_FLAG_INVALID) {
+	jw.write_key("flag_invalid");
+	jw.write(1);
+    }
+    if (flags & gx_system::PRESET_FLAG_READONLY) {
+	jw.write_key("flag_readonly");
+	jw.write(1);
+    }
+    if (flags & gx_system::PRESET_FLAG_VERSIONDIFF) {
+	jw.write_key("flag_versiondiff");
+	jw.write(1);
+    }
+    jw.write_key("presets");
+    jw.begin_array();
+    for (int i = 0; i < size(); i++) {
+	jw.write(entries[i].name);
+    }
+    jw.end_array();
+    jw.end_object();
+}
+
 bool PresetFile::set_factory(const Glib::ustring& name_, const std::string& path) {
     check_mtime(path, mtime);
     if (mtime == 0) {
-	gx_system::gx_print_error(
+	gx_print_error(
 	    _("open factory preset"),
 	    boost::format(_("couldn't open %1%")) % path);
 	return false;
@@ -704,7 +967,7 @@ bool PresetFile::fail() {
     try {
 	reopen();
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
 	return true;
     }
     return is->fail();
@@ -734,7 +997,7 @@ bool PresetFile::create_file(const Glib::ustring& name_, const std::string& path
 	header.set_to_current();
 	check_mtime(path, mtime);
     } else {
-	gx_system::gx_print_error(
+	gx_print_error(
 	    _("create preset bank"),
 	    boost::format(_("couldn't create %1%")) % path);
     }
@@ -763,14 +1026,14 @@ bool PresetFile::readJSON(const std::string& dirpath, JsonParser &jp, bool *mtim
     if (!check_mtime(filename, mtime)) {
 	*mtime_diff = true;
 	if (mtime == 0) {
-	    gx_system::gx_print_error(filename.c_str(), _("not found"));
+	    gx_print_error(filename.c_str(), _("not found"));
 	    return false;
 	}
 	try {
 	    open();
 	} catch (gx_system::JsonException& e) {
 	    set_flag(PRESET_FLAG_INVALID, true);
-	    gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	    gx_print_error(filename.c_str(), _("parse error"));
 	    return false;
 	}
 	set_flag(PRESET_FLAG_INVALID, false);
@@ -826,7 +1089,7 @@ int PresetFile::size() {
     try {
 	reopen();
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
     }
     return entries.size();
 }
@@ -1082,7 +1345,7 @@ PresetFile::iterator PresetFile::begin() {
     try {
 	reopen();
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
     }
     return entries.begin();
 }
@@ -1104,6 +1367,20 @@ PresetBanks::~PresetBanks() {
     }
 }
 
+void PresetBanks::readJSON_remote(gx_system::JsonParser& jp) {
+    for (iterator i = begin(); i != end(); ++i) {
+	delete *i;
+    }
+    banklist.clear();
+    jp.next(gx_system::JsonParser::begin_array);
+    while (jp.peek() != gx_system::JsonParser::end_array) {
+	gx_system::PresetFile *pf = new gx_system::PresetFile;
+	pf->readJSON_remote(jp);
+	banklist.push_back(pf);
+    }
+    jp.next(gx_system::JsonParser::end_array);
+}
+
 bool PresetBanks::check_reparse() {
     if (check_mtime(filepath, mtime)) {
 	bool reload = false;
@@ -1302,6 +1579,9 @@ void PresetBanks::collect_lost_banks(const char* scratchpad_name, const char* sc
 
 
 void PresetBanks::save() {
+    if (filepath.empty()) { //FIXME remote operation hack
+	return;
+    }
     std::string tmpfile = filepath + "_tmp";
     ofstream os(tmpfile.c_str());
     gx_system::JsonWriter jw(&os);
@@ -1332,7 +1612,7 @@ void PresetBanks::save() {
 void PresetBanks::parse_factory_list(const std::string& path) {
     ifstream is(Glib::build_filename(path, "dirlist.js").c_str());
     if (is.fail()) {
-	gx_system::gx_print_error(_("Presets"), _("factory preset list not found"));
+	gx_print_error(_("Presets"), _("factory preset list not found"));
 	return;
     }
     gx_system::JsonParser jp(&is);
@@ -1354,7 +1634,7 @@ void PresetBanks::parse_factory_list(const std::string& path) {
 		}
 	    } catch (gx_system::JsonException& e) {
 		delete f;
-		gx_system::gx_print_error(fname.c_str(), _("not found or parse error"));
+		gx_print_error(fname.c_str(), _("not found or parse error"));
 	    }
 	    f = 0;
 	    jp.next(gx_system::JsonParser::end_array);
@@ -1371,7 +1651,7 @@ void PresetBanks::parse_factory_list(const std::string& path) {
 void PresetBanks::parse_bank_list(bl_type::iterator pos) {
     ifstream is(filepath.c_str());
     if (is.fail()) {
-	gx_system::gx_print_error(
+	gx_print_error(
 	    _("Presets"), boost::format(_("banks not found: '%1%'")) % filepath);
 	return;
     }
@@ -1393,7 +1673,7 @@ void PresetBanks::parse_bank_list(bl_type::iterator pos) {
 	jp.next(gx_system::JsonParser::end_token);
     } catch (gx_system::JsonException& e) {
 	delete f;
-	gx_system::gx_print_error(filepath.c_str(), _("parse error"));
+	gx_print_error(filepath.c_str(), _("parse error"));
     }
     jp.close();
     is.close();
@@ -1570,9 +1850,6 @@ void GxSettingsBase::load_preset(PresetFile* pf, const Glib::ustring& name) {
     seq.start_ramp_down();
     bool modules_changed = loadsetting(pf, name);
     seq.start_ramp_up();
-    in_load = true;
-    gx_ui::GxUI::updateAllGuis();
-    in_load = false;
     // if no modules changed either there was no change (then
     // rack_changed should not be set anyhow) or the modules
     // could not be installed because jack is not initialized.
@@ -1589,9 +1866,6 @@ void GxSettingsBase::loadstate() {
     seq.start_ramp_down();
     bool modules_changed = loadsetting(0, current_name);
     seq.start_ramp_up();
-    in_load = true;
-    gx_ui::GxUI::updateAllGuis();
-    in_load = false;
     if (modules_changed) { // see comment in load_preset()
 	seq.clear_rack_changed();
     }
@@ -1605,15 +1879,17 @@ void GxSettingsBase::set_source_to_state() {
 }
 
 void GxSettingsBase::save_to_state(bool preserve_preset) {
-    gx_system::gx_print_info("write state",boost::format("%2% [%1%]")
+    gx_print_info("write state",boost::format("%2% [%1%]")
 			     % preserve_preset % statefile.get_filename());
     JsonWriter *jw = statefile.create_writer(&preserve_preset);
     state_io->write_state(*jw, preserve_preset);
     delete jw;
+#if 0
     if (!preserve_preset && setting_is_preset()) {
 	set_source_to_state();
 	presetlist_changed();
     }
+#endif
 }
 
 void GxSettingsBase::append(PresetFile& pf, const Glib::ustring& src, PresetFile& pftgt, const Glib::ustring& name) {
@@ -1771,6 +2047,16 @@ bool GxSettingsBase::rename_bank(const Glib::ustring& oldname, const Glib::ustri
     return true;
 }
 
+bool GxSettingsBase::remove_bank(const Glib::ustring& bank) {
+    if (!banks.remove(bank)) {
+	return false;
+    }
+    if (bank == current_bank) {
+	set_source_to_state();
+    }
+    return true;
+}
+
 bool GxSettingsBase::rename_preset(PresetFile& pf, const Glib::ustring& oldname, const Glib::ustring& newname) {
     if (!pf.rename(oldname, newname)) {
 	return false;
