@@ -37,7 +37,8 @@ namespace gx_preset {
 PresetIO::PresetIO(gx_engine::MidiControllerList& mctrl_,
 		   gx_engine::ConvolverAdapter& cvr_,
 		   gx_engine::ParamMap& param_,
-		   const gx_system::CmdlineOptions& opt_)
+		   gx_system::CmdlineOptions& opt_,
+		   UnitRacks& rack_units_)
     : gx_system::AbstractPresetIO(),
       mctrl(mctrl_),
       convolver(cvr_),
@@ -45,7 +46,7 @@ PresetIO::PresetIO(gx_engine::MidiControllerList& mctrl_,
       opt(opt_),
       plist(),
       m(0),
-      jcset(0) {
+      rack_units(rack_units_) {
 }
 
 PresetIO::~PresetIO() {
@@ -56,18 +57,21 @@ void PresetIO::clear() {
     plist.clear();
     delete m;
     m = 0;
-    delete jcset;
-    jcset = 0;
 }
 
 bool PresetIO::midi_in_preset() {
-    return param["system.midi_in_preset"].getSwitch().get();
+    const char *i = "system.midi_in_preset";
+    if (param.hasId(i)) {
+	return param[i].getBool().get_value();
+    } else {
+	return false;
+    }
 }
 
 void PresetIO::read_preset(gx_system::JsonParser &jp, const gx_system::SettingsFileHeader& head) {
     clear();
     for (gx_engine::ParamMap::iterator i = param.begin(); i != param.end(); ++i) {
- 	if (i->second->isInPreset()) {
+ 	if (i->second->isInPreset() && i->second->isSavable()) {
 	    i->second->stdJSON_value();
 	    plist.push_back(i->second);
 	}
@@ -97,6 +101,47 @@ static std::string replaced_id(const std::string& s) {
 	{ "beat_detector.atack_gain", "midi_out.beat_detector.atack_gain" },
 	{ "beat_detector.beat_gain", "midi_out.beat_detector.beat_gain" },
 	{ "beat_detector.midi_gain", "midi_out.beat_detector.midi_gain" },
+	// begin ui.*
+	// ui.<name> -> ui.<id> (rack unit visible)
+	{ "ui.3 Band EQ", "ui.tonemodul" },
+	{ "ui.BiQuad Filter", "ui.biquad" },
+	{ "ui.Cabinet", "ui.cab" },
+    { "ui.Preamp", "ui.pre" },
+	{ "ui.Chorus", "ui.chorus" },
+	{ "ui.Chorus Mono", "ui.chorus_mono" },
+	{ "ui.Compressor", "ui.compressor" },
+	{ "ui.Convolver", "ui.jconv" },
+	{ "ui.Crybaby", "ui.crybaby" },
+	{ "ui.Delay", "ui.delay" },
+	{ "ui.Echo", "ui.echo" },
+	{ "ui.Feedback", "ui.feedback" },
+	{ "ui.Flanger", "ui.flanger" },
+	{ "ui.Flanger Mono", "ui.flanger_mono" },
+	{ "ui.Freeverb", "ui.freeverb" },
+	{ "ui.ImpulseResponse", "ui.IR" },
+	{ "ui.Midi Out", "ui.midi_out" },
+	{ "ui.Moog Filter", "ui.moog" },
+	{ "ui.Multi Band Distortion", "ui.gx_distortion" },
+	{ "ui.Oscilloscope", "ui.oscilloscope" },
+	{ "ui.Overdrive", "ui.overdrive" },
+	{ "ui.Phaser", "ui.phaser" },
+	{ "ui.Phaser Mono", "ui.phaser_mono" },
+	{ "ui.Postamp", "ui.ampmodul" },
+	{ "ui.Scaleable EQ", "ui.eqs" },
+	{ "ui.Stereo Delay", "ui.stereodelay" },
+	{ "ui.Stereo Echo", "ui.stereoecho" },
+	{ "ui.Stereo Verb", "ui.stereoverb" },
+	{ "ui.Tonestack", "ui.amp.tonestack" },
+	{ "ui.Tremolo", "ui.tremolo" },
+	{ "ui.Vibe", "ui.univibe" },
+	{ "ui.Vibe Mono", "ui.univibe_mono" },
+	{ "ui.Zita Rev1", "ui.zita_rev1" },
+	{ "ui.abGate", "ui.abgate" },
+	{ "ui.low high pass", "ui.low_highpass" },
+	{ "Rev.Rocket.s_h", "rev_rocket.s_h"},
+	{ "Rev.Rocket.position", "rev_rocket.position"},
+	{ "Rev.Rocket.pp", "rev_rocket.pp"},
+	{ "ui.Rev.Rocket", "ui.rev_rocket"},
 	{0}
     };
     for (const char *(*p)[2] = old_new; (*p)[0]; ++p) {
@@ -107,15 +152,202 @@ static std::string replaced_id(const std::string& s) {
     return "";
 }
 
+class UnitPositionID: public UnitPosition {
+public:
+    std::string id;
+    int weight;
+    UnitPositionID(const string& id_, const UnitPosition& u);
+    bool operator<(const UnitPositionID& v) const { return weight < v.weight; }
+};
+
+UnitPositionID::UnitPositionID(const string& id_, const UnitPosition& u)
+  : UnitPosition(u),
+    id(id_),
+    weight(position - 1000 * pp) {
+}
+
+void UnitsCollector::get_list(std::vector<std::string>& l, bool stereo, gx_engine::ParamMap& param) {
+    std::vector<UnitPositionID> v;
+    for (std::map<std::string,UnitPosition>::iterator i = m.begin(); i != m.end(); ++i) {
+	if (i->first == "jconv" && i->second.position < 0) {
+	    i->second.position = 99; // very old presets
+	    gx_system::JsonStringParser jp;
+	    jp.get_ostream() << i->second.position;
+	    jp.start_parser();
+	    param[i->first+".position"].readJSON_value(jp);
+	}
+	if (i->first == "cab") {
+	    if (i->second.position < 0) {
+		i->second.position = 98; // very old presets
+		gx_system::JsonStringParser jp;
+		jp.get_ostream() << i->second.position;
+		jp.start_parser();
+		param[i->first+".position"].readJSON_value(jp);
+	    }
+	    if (i->second.pp < 0) {
+		i->second.pp = 0;
+		gx_system::JsonStringParser jp;
+		jp.get_ostream() << i->second.pp;
+		jp.start_parser();
+		param[i->first+".pp"].readJSON_value(jp);
+	    }
+	}
+	if (i->second.position >= 0 && i->second.show) {
+	    if ((stereo && i->second.pp < 0) || (!stereo && i->second.pp >= 0)) {
+		if (!i->second.visible) { // make sure ui.XX is set for old presets
+		    i->second.visible = true;
+		    gx_system::JsonStringParser jp;
+		    jp.put('1');
+		    jp.start_parser();
+		    param["ui."+i->first].readJSON_value(jp);
+		}
+		v.push_back(UnitPositionID(i->first, i->second));
+	    }
+	}
+    }
+    l.clear();
+    std::sort(v.begin(), v.end());
+    int pp = 1;
+    for (std::vector<UnitPositionID>::iterator j = v.begin(); j != v.end(); ++j) {
+	if (!stereo && j->pp != pp) {
+	    pp = j->pp;
+	    l.push_back("ampstack");
+	}
+	l.push_back(j->id);
+    }
+    if (!stereo && pp) {
+	l.push_back("ampstack");
+    }
+    /*
+    cerr << "SL";
+    for (std::vector<std::string>::iterator ii = l.begin(); ii != l.end(); ++ii) {
+	cerr << " '" << *ii << "'";
+    }
+    cerr << endl;
+    */
+}
+
+bool PresetIO::convert_old(gx_system::JsonParser &jp) {
+    const std::string& s = jp.current_value();
+    if (s == "system.mainwin_x") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.mainwin_x = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.mainwin_y") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.mainwin_y = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.mainwin_height") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.mainwin_height = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.mainwin_rack_height") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.window_height = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.preset_window_height") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.preset_window_height = jp.current_value_int();
+	return true;
+    }
+    if (s == "oscilloscope.bufferset") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.mul_buffer = jp.current_value_int();
+	return true;
+    }
+    if (s == "ui.skin_name") {
+	jp.next(gx_system::JsonParser::value_string);
+	opt.skin_name = jp.current_value();
+	return true;
+    }
+    if (s == "ui.latency_nowarn") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.no_warn_latency = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.order_rack_h") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.system_order_rack_h = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.show_value") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.system_show_value = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.show_tooltips") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.system_show_tooltips = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.animations") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.system_animations = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.show_presets") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.system_show_presets = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.show_toolbar") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.system_show_toolbar = jp.current_value_int();
+	return true;
+    }
+    if (s == "system.show_rack") {
+	jp.next(gx_system::JsonParser::value_number);
+	opt.system_show_rack = jp.current_value_int();
+	return true;
+    }
+
+    return false;
+}
+
+static inline bool endswith(const std::string& s, int n, const char *t) {
+    return s.compare(std::max<int>(0, s.size()-n), n, t) == 0;
+}
+
+static inline bool startswith(const std::string& s, int n, const char *t) {
+    return s.compare(0, n, t) == 0;
+}
+
+void PresetIO::collectRackOrder(gx_engine::Parameter *p, gx_system::JsonParser &jp, UnitsCollector& u) {
+    const std::string& s = p->id();
+    if (startswith(s, 3, "ui.")) {
+	if (jp.current_value_int()) {
+	    std::string ss = s.substr(3);
+	    u.set_visible(ss, true);
+	    u.set_show(ss, true);
+	}
+    } else if (endswith(s, 7, ".on_off")) {
+	if (jp.current_value_int()) {
+	    u.set_show(s.substr(0, s.size()-7), true);
+	}
+    } else if (endswith(s, 9, ".position")) {
+	u.set_position(s.substr(0, s.size()-9), jp.current_value_int());
+    } else if (endswith(s, 3, ".pp")) {
+	u.set_pp(s.substr(0, s.size()-3), (jp.current_value() == "pre" ? 1 : 0));
+    }
+}
+
 void PresetIO::read_parameters(gx_system::JsonParser &jp, bool preset) {
+    UnitsCollector u;
     jp.next(gx_system::JsonParser::begin_object);
     do {
         jp.next(gx_system::JsonParser::value_key);
         gx_engine::Parameter *p;
         if (!param.hasId(jp.current_value())) {
+	    if (convert_old(jp)) {
+		continue;
+	    }
 	    std::string s = replaced_id(jp.current_value());
 	    if (s.empty()) {
-		gx_system::gx_print_warning(
+		gx_print_warning(
 		    _("recall settings"),
 		    _("unknown parameter: ")+jp.current_value());
 		jp.skip_object();
@@ -126,27 +358,53 @@ void PresetIO::read_parameters(gx_system::JsonParser &jp, bool preset) {
 	    p = &param[jp.current_value()];
 	}
         if (!preset and p->isInPreset()) {
-            gx_system::gx_print_warning(
+            gx_print_warning(
 		_("recall settings"),
 		_("preset-parameter ")+p->id()+_(" in settings"));
             jp.skip_object();
             continue;
         } else if (preset and !p->isInPreset()) {
-            gx_system::gx_print_warning(
+            gx_print_warning(
 		_("recall settings"),
 		_("non preset-parameter ")+p->id()+_(" in preset"));
             jp.skip_object();
             continue;
-        }
-        p->readJSON_value(jp);
+        } else if (!p->isSavable()) {
+            gx_print_warning(
+		_("recall settings"),
+		_("non savable parameter ")+p->id()+_(" in settings"));
+            jp.skip_object();
+            continue;
+	}
+	p->readJSON_value(jp);
+	collectRackOrder(p, jp, u);
     } while (jp.peek() == gx_system::JsonParser::value_key);
     jp.next(gx_system::JsonParser::end_object);
+    u.get_list(rack_units.mono, false, param);
+    u.get_list(rack_units.stereo, true, param);
 }
 
 void PresetIO::write_parameters(gx_system::JsonWriter &w, bool preset) {
     w.begin_object(true);
+#if 0
+    int n = 0;
+    int pp = 1;
+    for (std::vector<std::string>::iterator j = mono_rack_units.begin(); j != mono_rack_units.end(); ++j) {
+	if (*j == "ampstack") {
+	    n = 1;
+	    pp = 0;
+	}
+	w.write_key("ui."+*j);
+	w.write(1);
+	w.write_key(*j+".pp");
+	w.write(pp);
+	w.write_key(*j+".position");
+	w.write(n);
+	n++;
+    }
+#endif
     for (gx_engine::ParamMap::iterator i = param.begin();
-                                   i != param.end(); ++i) {
+	 i != param.end(); ++i) {
         gx_engine::Parameter *param = i->second;
 	if (!param->isSavable()) {
 	    continue;
@@ -169,13 +427,12 @@ void PresetIO::read_intern(gx_system::JsonParser &jp, bool *has_midi, const gx_s
         jp.next(gx_system::JsonParser::value_key);
         if (jp.current_value() == "engine") {
             read_parameters(jp, true);
-        } else if (jp.current_value() == "jconv") {
-	    jcset = new gx_engine::GxJConvSettings();
-	    jcset->readJSON(jp, opt.get_IR_pathlist());
+        } else if (jp.current_value() == "jconv") { // for backwards compatibility
+	    dynamic_cast<gx_engine::JConvParameter*>(&param["jconv.convolver"])->readJSON_value(jp);
         } else if (jp.current_value() == "midi_controller") {
             if (use_midi) {
-                m = gx_engine::MidiControllerList::create_controller_array();
-                mctrl.readJSON(jp, param, *m);
+                m = new gx_engine::ControllerArray();
+                m->readJSON(jp, param);
                 if (has_midi) {
                     *has_midi = true;
                 }
@@ -183,7 +440,7 @@ void PresetIO::read_intern(gx_system::JsonParser &jp, bool *has_midi, const gx_s
                 jp.skip_object();
             }
         } else {
-            gx_system::gx_print_warning(
+            gx_print_warning(
 		_("recall settings"),
 		_("unknown preset section: ") + jp.current_value());
 	    jp.skip_object();
@@ -194,9 +451,6 @@ void PresetIO::read_intern(gx_system::JsonParser &jp, bool *has_midi, const gx_s
 }
 
 void PresetIO::commit_preset() {
-    if (jcset) {
-	convolver.set(*jcset);
-    }
     for (gx_engine::paramlist::iterator i = plist.begin(); i != plist.end(); ++i) {
         (*i)->setJSON_value();
     }
@@ -204,7 +458,7 @@ void PresetIO::commit_preset() {
         mctrl.set_controller_array(*m);
     }
     clear();
-    gx_engine::controller_map.update_from_controllers();
+    mctrl.update_from_controllers();
 }
 
 void PresetIO::write_intern(gx_system::JsonWriter &w, bool write_midi) {
@@ -240,8 +494,8 @@ void PresetIO::copy_preset(gx_system::JsonParser &jp, const gx_system::SettingsF
 
 StateIO::StateIO(gx_engine::MidiControllerList& mctrl, gx_engine::ConvolverAdapter& cvr,
 		 gx_engine::ParamMap& param, gx_engine::MidiStandardControllers& mstdctr,
-		 gx_jack::GxJack& jack_, const gx_system::CmdlineOptions& opt_)
-    : PresetIO(mctrl, cvr, param, opt_),
+		 gx_jack::GxJack& jack_, gx_system::CmdlineOptions& opt_, UnitRacks& rack_units)
+    : PresetIO(mctrl, cvr, param, opt_, rack_units),
       midi_std_control(mstdctr),
       jack(jack_) {
 }
@@ -252,8 +506,10 @@ StateIO::~StateIO() {
 void StateIO::read_state(gx_system::JsonParser &jp, const gx_system::SettingsFileHeader& head) {
     clear();
     for (gx_engine::ParamMap::iterator i = param.begin(); i != param.end(); ++i) {
-	i->second->stdJSON_value();
-	plist.push_back(i->second);
+	if (i->second->isSavable()) {
+	    i->second->stdJSON_value();
+	    plist.push_back(i->second);
+	}
     }
     do {
 	jp.next(gx_system::JsonParser::value_string);
@@ -262,14 +518,14 @@ void StateIO::read_state(gx_system::JsonParser &jp, const gx_system::SettingsFil
 	} else if (jp.current_value() == "current_preset") {
 	    read_intern(jp, 0, head);
 	} else if (jp.current_value() == "midi_controller") {
-	    m = gx_engine::MidiControllerList::create_controller_array();
-	    mctrl.readJSON(jp, param, *m);
+	    m = new gx_engine::ControllerArray();
+	    m->readJSON(jp, param);
 	} else if (jp.current_value() == "midi_ctrl_names") {
 	    midi_std_control.readJSON(jp);
 	} else if (jp.current_value() == "jack_connections") {
 	    jack.read_connections(jp);
 	} else {
-	    gx_system::gx_print_warning(
+	    gx_print_warning(
 		_("recall settings"),
 		_("unknown section: ") + jp.current_value());
 	    jp.skip_object();
@@ -286,10 +542,10 @@ void StateIO::write_state(gx_system::JsonWriter &jw, bool no_preset) {
     write_parameters(jw, false);
 
     jw.write("midi_controller");
-    gx_engine::controller_map.writeJSON(jw);
+    mctrl.writeJSON(jw);
 
     jw.write("midi_ctrl_names");
-    gx_engine::midi_std_ctr.writeJSON(jw);
+    midi_std_control.writeJSON(jw);
 
     if (!no_preset) {
 	jw.write("current_preset");
@@ -307,12 +563,9 @@ void StateIO::write_state(gx_system::JsonWriter &jw, bool no_preset) {
  ** class PluginPresetList
  */
 
-PluginPresetList::PluginPresetList(const std::string& fname, gx_engine::ParamMap& pmap_)
-    : Glib::Object(), filename(fname), pmap(pmap_), is(), jp(&is) {
-}
-
-Glib::RefPtr<PluginPresetList> PluginPresetList::create(const std::string& fname, gx_engine::ParamMap& pmap) {
-    return Glib::RefPtr<PluginPresetList>(new PluginPresetList(fname, pmap));
+PluginPresetList::PluginPresetList(const std::string& fname, gx_engine::ParamMap& pmap_,
+				   gx_engine::MidiControllerList& mctrl_)
+    : filename(fname), pmap(pmap_), mctrl(mctrl_), is(), jp(&is) {
 }
 
 bool PluginPresetList::start() {
@@ -330,7 +583,7 @@ bool PluginPresetList::start() {
 	}
 	jp.next(gx_system::JsonParser::value_number);
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
 	return false;
     }
     return true;
@@ -353,7 +606,7 @@ bool PluginPresetList::next(Glib::ustring& name, bool *is_set) {
 	    while (jp.peek() != gx_system::JsonParser::end_object) {
 		jp.next(gx_system::JsonParser::value_key);
 		if (!pmap.hasId(jp.current_value())) {
-		    gx_system::gx_print_warning(
+		    gx_print_warning(
 			_("recall plugin settings"),
 			_("unknown parameter: ")+jp.current_value());
 		    jp.skip_object();
@@ -370,7 +623,7 @@ bool PluginPresetList::next(Glib::ustring& name, bool *is_set) {
 	    jp.skip_object();
 	}
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
 	return false;
     }
     return true;
@@ -402,10 +655,10 @@ void PluginPresetList::set(const Glib::ustring& name) {
 	jp.next(gx_system::JsonParser::end_array);
 	jp.next(gx_system::JsonParser::end_token);
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
 	return;
     }
-    gx_engine::controller_map.remove_controlled_parameters(plist, 0);
+    mctrl.remove_controlled_parameters(plist, 0);
     for (gx_engine::paramlist::iterator i = plist.begin(); i != plist.end(); ++i) {
         (*i)->setJSON_value();
     }
@@ -413,15 +666,32 @@ void PluginPresetList::set(const Glib::ustring& name) {
 
 static const int GX_PLUGIN_VERSION = 1;
 
-void PluginPresetList::write_values(gx_system::JsonWriter& jw, std::string id) {
-    //FIXME
+static inline bool compare_groups(const std::string& id, const char **groups) { //FIXME (is copy from gx_paramtable.cpp
+    if (!groups) {
+	return false;
+    }
+    for (const char **g = groups; *g; g += 2) {
+	const char *p = *g;
+	if ((*p) != '.') {
+	    continue;
+	}
+	p++;
+	int n = strlen(p);
+	if (strncmp(id.c_str(), p, n) == 0 && id[n] == '.') {
+	    return true;
+	}
+    }
+    return false;
+}
+
+void PluginPresetList::write_values(gx_system::JsonWriter& jw, std::string id, const char **groups) {
     id += ".";
     string on_off = id + "on_off";
     string pp = id + "pp";
     std::string position = id + "position";
     jw.begin_object(true);
     for (gx_engine::ParamMap::iterator i = pmap.begin(); i != pmap.end(); ++i) {
-	if (i->first.compare(0, id.size(), id) == 0) {
+	if (i->first.compare(0, id.size(), id) == 0 || compare_groups(i->first, groups)) {
 	    if (i->second->isInPreset()) {
 		if (i->first != on_off && i->first != pp && i->first != position) {
 		    i->second->writeJSON(jw);
@@ -433,7 +703,7 @@ void PluginPresetList::write_values(gx_system::JsonWriter& jw, std::string id) {
     jw.end_object(true);
 }
 
-void PluginPresetList::save(const Glib::ustring& name, const std::string& id) {
+void PluginPresetList::save(const Glib::ustring& name, const std::string& id, const char **groups) {
     try {
 	std::string tmpfile(filename + "_tmp");
 	ofstream os(tmpfile.c_str());
@@ -448,7 +718,7 @@ void PluginPresetList::save(const Glib::ustring& name, const std::string& id) {
 		jw.write(jp.current_value());
 		if (jp.current_value() == name) {
 		    found = true;
-		    write_values(jw, id);
+		    write_values(jw, id, groups);
 		    jp.skip_object();
 		} else {
 		    jp.copy_object(jw);
@@ -457,24 +727,24 @@ void PluginPresetList::save(const Glib::ustring& name, const std::string& id) {
 	}
 	if (!found) {
 	    jw.write(name);
-	    write_values(jw, id);
+	    write_values(jw, id, groups);
 	}
 	jw.end_array(true);
 	jw.close();
 	os.close();
 	if (!os.good()) {
-	    gx_system::gx_print_error(_("save plugin preset"),
+	    gx_print_error(_("save plugin preset"),
 				      boost::format(_("couldn't write %1%")) % tmpfile);
 	    return;
 	}
 	int rc = rename(tmpfile.c_str(), filename.c_str());
 	if (rc != 0) {
-	    gx_system::gx_print_error(_("save plugin preset"),
+	    gx_print_error(_("save plugin preset"),
 				      boost::format(_("couldn't rename %1% to %2%"))
 				      % tmpfile % filename);
 	}
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
     }
 }
 
@@ -503,18 +773,18 @@ void PluginPresetList::remove(const Glib::ustring& name) {
 	jw.close();
 	os.close();
 	if (!os.good()) {
-	    gx_system::gx_print_error(_("save plugin preset"),
+	    gx_print_error(_("save plugin preset"),
 				      boost::format(_("couldn't write %1%")) % tmpfile);
 	    return;
 	}
 	int rc = rename(tmpfile.c_str(), filename.c_str());
 	if (rc != 0) {
-	    gx_system::gx_print_error(_("save plugin preset"),
+	    gx_print_error(_("save plugin preset"),
 				      boost::format(_("couldn't rename %1% to %2%"))
 				      % tmpfile % filename);
 	}
     } catch (gx_system::JsonException& e) {
-	gx_system::gx_print_error(filename.c_str(), _("parse error"));
+	gx_print_error(filename.c_str(), _("parse error"));
     }
 }
 
@@ -529,16 +799,17 @@ static const char *statename_postfix = "_rc";
 static const char *bank_list = "banklist.js";
 
 GxSettings::GxSettings(gx_system::CmdlineOptions& opt, gx_jack::GxJack& jack_, gx_engine::ConvolverAdapter& cvr,
-		       gx_engine::MidiStandardControllers& mstdctr, gx_engine::MidiControllerList& mctrl,
-		       gx_engine::ModuleSequencer& seq_, gx_engine::ParamMap& param_)
+		       gx_engine::MidiStandardControllers& mstdctr, gx_engine::MidiControllerList& mctrl_,
+		       gx_engine::ModuleSequencer& seq_)
     : sigc::trackable(),
       GxSettingsBase(seq_),
-      param(param_),
-      preset_io(mctrl, cvr, param, opt),
-      state_io(mctrl, cvr, param, mstdctr, jack_, opt),
+      param(seq_.get_param()),
+      preset_io(mctrl_, cvr, param, opt, rack_units),
+      state_io(mctrl_, cvr, param, mstdctr, jack_, opt, rack_units),
       state_loaded(false),
       no_autosave(false),
       jack(jack_),
+      mctrl(mctrl_),
       options(opt),
       preset_parameter(*param.reg_string("system.current_preset", "?", &current_name, "")),
       bank_parameter(*param.reg_string("system.current_bank", "?", &current_bank, "")) {
@@ -547,7 +818,7 @@ GxSettings::GxSettings(gx_system::CmdlineOptions& opt, gx_jack::GxJack& jack_, g
     banks.parse(opt.get_preset_filepath(bank_list), opt.get_preset_dir(), opt.get_factory_dir(),
 		scratchpad_name, scratchpad_file);
     instance = this;
-    gx_system::GxExit::get_instance().signal_exit().connect(
+    GxExit::get_instance().signal_exit().connect(
 	sigc::mem_fun(*this, &GxSettings::exit_handler));
     jack.signal_client_change().connect(
 	sigc::mem_fun(*this, &GxSettings::jack_client_changed));
@@ -625,12 +896,12 @@ string GxSettings::make_state_filename() {
 bool GxSettings::check_create_config_dir(const Glib::ustring& dir) {
     if (access((Glib::build_filename(dir, ".")).c_str(), R_OK|W_OK|X_OK) != 0) {
 	if (errno != ENOENT) {
-	    throw gx_system::GxFatalError(
+	    throw GxFatalError(
 		boost::format(_("no read/write access in guitarix config dir '%1%'"))
 		% dir);
 	}
 	if (mkdir(dir.c_str(), 0777) != 0) {
-	    throw gx_system::GxFatalError(
+	    throw GxFatalError(
 		boost::format(_("can't create guitarix config dir '%1%'"))
 		% dir);
 	}
@@ -643,6 +914,127 @@ void GxSettings::create_default_scratch_preset() {
     save(*banks.get_file(scratchpad_name), "livebuffer1");
 }
 
+static inline std::vector<std::string>::iterator find_unit(std::vector<std::string>& r, const std::string& unit) {
+    std::vector<std::string>::iterator i = r.begin();
+    for (; i != r.end(); ++i) {
+	if (unit == *i) {
+	    break;
+	}
+    }
+    return i;
+}
+
+bool GxSettings::remove_rack_unit(const std::string& unit, bool stereo) {
+    std::vector<std::string>& r = stereo ? rack_units.stereo : rack_units.mono;
+    std::vector<std::string>::iterator i = find_unit(r, unit);
+    if (i != r.end()) {
+	r.erase(i);
+	return true;
+    }
+    return false;
+}
+
+void GxSettings::insert_rack_unit(const std::string& unit, const std::string& before, bool stereo) {
+    std::vector<std::string>& r = stereo ? rack_units.stereo : rack_units.mono;
+    remove_rack_unit(unit, stereo);
+    if (before.empty()) {
+	r.push_back(unit);
+    } else {
+	r.insert(find_unit(r, before), unit);
+    }
+}
+
+Glib::RefPtr<Gio::File> GxSettings::uri_to_name_filename(const Glib::ustring& uri, Glib::ustring& name, std::string& filename) {
+    Glib::RefPtr<Gio::File> rem = Gio::File::create_for_uri(uri);
+    filename = rem->get_basename();
+    banks.strip_preset_postfix(filename);
+    name = banks.decode_filename(filename);
+    banks.make_valid_utf8(name);
+    banks.make_bank_unique(name, &filename);
+    return rem;
+}
+
+gx_system::PresetFile* GxSettings::bank_insert_uri(const Glib::ustring& uri, bool move) {
+    Glib::ustring name;
+    std::string filename;
+    Glib::RefPtr<Gio::File> rem = uri_to_name_filename(uri, name, filename);
+    Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path(filename);
+    try {
+	rem->copy(dest);
+    } catch (Gio::Error& e) {
+	gx_print_error(e.what().c_str(), _("can't copy to config dir"));
+	return 0;
+    }
+    gx_system::PresetFile *f = new gx_system::PresetFile();
+    if (f->open_file(name, filename, gx_system::PresetFile::PRESET_FILE, 0)) {
+	banks.insert(f);
+    } else {
+	delete f;
+	try {
+	    dest->remove();
+	} catch (Gio::Error& e) {
+	    gx_print_error(e.what().c_str(), _("can't remove copied file!?"));
+	}
+	return 0;
+    }
+    if (move) {
+	try {
+	    rem->remove();
+	} catch (Gio::Error& e) {
+	    gx_print_error(e.what().c_str(), _("can't move; file has been copied"));
+	}
+    }
+    return f;
+}
+
+gx_system::PresetFile* GxSettings::bank_insert_content(const Glib::ustring& uri, const std::string content) {
+    Glib::ustring name;
+    std::string filename;
+    uri_to_name_filename(uri, name, filename);
+    Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path(filename);
+    try {
+	Glib::RefPtr<Gio::FileOutputStream> s = dest->create_file();
+	s->write(content);
+	s->close();
+    } catch (Gio::Error& e) {
+	gx_print_error(e.what().c_str(), _("can't bank"));
+	return 0;
+    }
+    gx_system::PresetFile *f = new gx_system::PresetFile();
+    if (f->open_file(name, filename, gx_system::PresetFile::PRESET_FILE, 0)) {
+	banks.insert(f);
+    } else {
+	delete f;
+	try {
+	    dest->remove();
+	} catch (Gio::Error& e) {
+	    gx_print_error(e.what().c_str(), _("can't remove copied file!?"));
+	}
+	return 0;
+    }
+    return f;
+}
+
+gx_system::PresetFile *GxSettings::bank_insert_new(const Glib::ustring& name) {
+    Glib::ustring newname = name;
+    std::string newfile;
+    banks.make_bank_unique(newname, &newfile);
+    gx_system::PresetFile *f = new gx_system::PresetFile();
+    if (f->create_file(newname, newfile, gx_system::PresetFile::PRESET_FILE, 0)) {
+	banks.insert(f);
+	return f;
+    } else {
+	delete f;
+	return 0;
+    }
+}
+
+bool GxSettings::rename_bank(const Glib::ustring& oldname, Glib::ustring& newname) {
+    std::string newfile;
+    banks.make_bank_unique(newname, &newfile);
+    return GxSettingsBase::rename_bank(oldname, newname, newfile);
+}
+
 //static
 bool GxSettings::check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_new_preset) {
     bool copied_from_old = false;
@@ -652,6 +1044,8 @@ bool GxSettings::check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_n
 	check_create_config_dir(opt.get_preset_dir());
 	check_create_config_dir(opt.get_plugin_dir());
 	check_create_config_dir(opt.get_pluginpreset_dir());
+	check_create_config_dir(opt.get_user_IR_dir());
+	check_create_config_dir(opt.get_temp_dir());
 	std::string fname = gx_jack::GxJack::get_default_instancename() + statename_postfix;
 	if (access(Glib::build_filename(opt.get_old_user_dir(), fname).c_str(), R_OK) == 0) {
 	    copied_from_old = true;
@@ -660,7 +1054,7 @@ bool GxSettings::check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_n
 	    try {
 		f->copy(Gio::File::create_for_path(opt.get_user_filepath(fname)));
 	    } catch (Gio::Error& e) {
-		gx_system::gx_print_error(e.what().c_str(), _("can't copy to new config dir"));
+		gx_print_error(e.what().c_str(), _("can't copy to new config dir"));
 	    }
 	}
 	fname = Glib::build_filename(
@@ -672,7 +1066,7 @@ bool GxSettings::check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_n
 	    try {
 		f->copy(Gio::File::create_for_path(oldpreset));
 	    } catch (Gio::Error& e) {
-		gx_system::gx_print_error(e.what().c_str(), _("can't copy to new config preset dir"));
+		gx_print_error(e.what().c_str(), _("can't copy to new config preset dir"));
 		oldpreset = "";
 	    }
 	}
@@ -680,11 +1074,13 @@ bool GxSettings::check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_n
 	check_create_config_dir(opt.get_preset_dir());
 	check_create_config_dir(opt.get_plugin_dir());
 	check_create_config_dir(opt.get_pluginpreset_dir());
+	check_create_config_dir(opt.get_user_IR_dir());
+	check_create_config_dir(opt.get_temp_dir());
     }
     std::string fname = opt.get_preset_filepath(scratchpad_file);
     if (access(fname.c_str(), R_OK) != 0) {
 	if (!gx_system::SettingsFileHeader::make_empty_settingsfile(fname)) {
-	    throw gx_system::GxFatalError(
+	    throw GxFatalError(
 		boost::format(_("can't create file in '%1%' !!??")) % opt.get_preset_dir());
 	}
 	*need_new_preset = true;
@@ -693,7 +1089,7 @@ bool GxSettings::check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_n
     if (access(fname.c_str(), R_OK) != 0) {
 	ofstream f(fname.c_str());
 	if (!f.good()) {
-	    throw gx_system::GxFatalError(
+	    throw GxFatalError(
 		boost::format(_("can't create '%1%' in directory '%2%'"))
 		% bank_list % opt.get_preset_dir());
 	}
@@ -714,6 +1110,21 @@ bool GxSettings::check_settings_dir(gx_system::CmdlineOptions& opt, bool *need_n
 
 void GxSettings::loadstate() {
     GxSettingsBase::loadstate();
+#if 0
+    /* This following code is commented out because
+    **
+    ** o its debatable which of state file or scratch preset should have
+    **   priority in case both differ because 2 different instantiations
+    **   accessed the same scratch preset but different statefiles.
+    **
+    ** o if the scratch preset has priority, the autosave function would
+    **   have to save to statefile *and* scratch preset (in case of the
+    **   current preset being a scratch preset).
+    **
+    ** As an aside: load_preset() would have to be changed to not save
+    ** to the scratch preset before loading in the case that this code
+    ** is activated.
+    */
     if (setting_is_preset()) {
 	gx_system::PresetFile *pf = banks.get_file(current_bank);
 	if (pf && pf->get_type() == gx_system::PresetFile::PRESET_SCRATCH) {
@@ -724,16 +1135,41 @@ void GxSettings::loadstate() {
 	    load_preset(pf, current_name);
 	}
     }
+#endif
     state_loaded = true;
 }
 
-Glib::RefPtr<PluginPresetList> GxSettings::load_plugin_preset_list(const Glib::ustring& id, bool factory) const {
-    if (factory) {
-	return PluginPresetList::create(options.get_factory_filepath(id), param);
-    } else {
-	return PluginPresetList::create(options.get_pluginpreset_filepath(id), param);
+void GxSettings::add_plugin_preset_list(gx_preset::PluginPresetList& l,
+					UnitPresetList &presetnames) {
+    if (l.start()) {
+	Glib::ustring name;
+	bool is_set;
+	while (l.next(name, &is_set)) {
+	    presetnames.push_back(PluginPresetEntry(name, is_set));
+	}
     }
 }
 
+void GxSettings::plugin_preset_list_load(const PluginDef *pdef, UnitPresetList &presetnames) {
+    PluginPresetList user(options.get_pluginpreset_filepath(pdef->id, false), param, mctrl);
+    add_plugin_preset_list(user, presetnames);
+    presetnames.push_back(PluginPresetEntry("", false));
+    PluginPresetList factory(options.get_pluginpreset_filepath(pdef->id, true), param, mctrl);
+    add_plugin_preset_list(factory, presetnames);
+}
+
+void GxSettings::plugin_preset_list_set(const PluginDef *pdef, bool factory, const Glib::ustring& name) {
+    PluginPresetList(options.get_pluginpreset_filepath(pdef->id, factory), param, mctrl).set(name);
+}
+
+void GxSettings::plugin_preset_list_save(const PluginDef *pdef, const Glib::ustring& name) {
+    PluginPresetList(options.get_pluginpreset_filepath(pdef->id, false), param, mctrl).save(name, pdef->id, pdef->groups);
+}
+
+void GxSettings::plugin_preset_list_remove(const PluginDef *pdef, const Glib::ustring& name) {
+    PluginPresetList(options.get_pluginpreset_filepath(pdef->id, false), param, mctrl).remove(name);
+}
+
+
 /* ----------------------------------------------------------------*/
 } /* end of gx_preset namespace */
