@@ -165,8 +165,7 @@ void add_time_measurement() {
 
 
 /****************************************************************
- ** CmdlineOptions
- ** command line options
+ ** class SkinHandling
  */
 
 void SkinHandling::set_styledir(const string& style_dir) {
@@ -196,12 +195,55 @@ void SkinHandling::set_styledir(const string& style_dir) {
 }
 
 bool SkinHandling::is_in_list(const string& name) {
-    for (vector<string>::iterator i = skin_list.begin(); i != skin_list.end(); ++i) {
-	if (*i == name) {
-	    return true;
+    return index(name) < skin_list.size();
+}
+
+unsigned int SkinHandling::index(const Glib::ustring& name) {
+    unsigned int i = 0;
+    for (; i < skin_list.size(); ++i) {
+	if (skin_list[i] == name) {
+	    break;
+	}
+    }
+    return i;
+}
+
+const Glib::ustring& SkinHandling::operator[](unsigned int idx) {
+    if (idx < skin_list.size()) {
+	return skin_list[idx];
+    } else {
+	return empty;
+    }
+}
+
+
+/****************************************************************
+ ** class PathList
+ */
+
+PathList::PathList(const char *env_name): dirs() {
+    if (!env_name) {
+	return;
+    }
+    const char *p = getenv(env_name);
+    if (!p) {
+	return;
+    }
+    while (true) {
+	const char *q = strchr(p, ':');
+	if (q) {
+	    int n = q - p;
+	    if (n) {
+		add(std::string(p, n));
+	    }
+	    p = q+1;
+	} else {
+	    if (*p) {
+		add(p);
+	    }
+	    break;
 	}
     }
-    return false;
 }
 
 bool PathList::contains(const string& d) const {
@@ -229,7 +271,152 @@ bool PathList::find_dir(std::string* d, const std::string& filename) const {
     return false;
 }
 
-CmdlineOptions *CmdlineOptions::instance = 0;
+
+/****************************************************************
+ ** class PrefixConverter
+ */
+
+void PrefixConverter::add(char s, const std::string& d) {
+    assert(s != '%');
+    dirs[s] = (d[d.size()-1] == '/' ? d.substr(0,d.size()-1) : d);
+}
+
+std::string PrefixConverter::replace_symbol(const std::string& dir) const {
+    if (dir.size() < 2 || dir[0] != '%') {
+	return dir;
+    }
+    symbol_path_map::const_iterator i = dirs.find(dir[1]);
+    if (i != dirs.end()) {
+	return Glib::build_filename(i->second, dir.substr(2));
+    }
+    if (dir.compare(0, 2, "%%")) {
+	return dir.substr(1);
+    }
+    return dir;
+}
+
+std::string PrefixConverter::replace_path(const std::string& dir) const {
+    for (symbol_path_map::const_iterator i = dirs.begin(); i != dirs.end(); ++i) {
+	size_t n = i->second.size();
+	if (dir.compare(0, n, i->second) == 0) {
+	    std::string tail = dir.substr(n);
+	    if (Glib::build_filename(i->second, tail) == dir) {
+		std::string sym = "%";
+		sym.push_back(i->first);
+		return sym + tail;
+	    }
+	}
+    }
+    if (dir.size() < 2 || dir[0] != '%') {
+	return dir;
+    }
+    return "%" + dir;
+}
+
+
+/*****************************************************************
+ ** class DirectoryListing
+ */
+
+IRFileListing::IRFileListing(const std::string& path) {
+    Glib::RefPtr<Gio::File> file = Gio::File::create_for_path(path);
+    if (file->query_exists()) {
+        Glib::RefPtr<Gio::FileEnumerator> child_enumeration =
+              file->enumerate_children(G_FILE_ATTRIBUTE_STANDARD_NAME
+				       "," G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME
+				       "," G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE);
+        Glib::RefPtr<Gio::FileInfo> file_info;
+        while ((file_info = child_enumeration->next_file()) != 0) {
+	    if (file_info->get_attribute_string(G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE) == "audio/x-wav") {
+		listing.push_back(
+		    FileName(
+			file_info->get_attribute_byte_string(G_FILE_ATTRIBUTE_STANDARD_NAME),
+			file_info->get_attribute_string(G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME)));
+	    }
+        }
+    } else {
+        gx_print_error(
+	    "jconvolver",
+	    boost::format(_("Error reading file path %1%")) % path);
+    }
+}
+
+static void list_subdirs(const Glib::RefPtr<Gio::File>& file, std::vector<FileName>& dirs, const Glib::ustring& prefix) {
+    Glib::RefPtr<Gio::FileEnumerator> child_enumeration =
+	file->enumerate_children(G_FILE_ATTRIBUTE_STANDARD_NAME
+				 "," G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME);
+    Glib::RefPtr<Gio::FileInfo> file_info;
+    while ((file_info = child_enumeration->next_file()) != 0) {
+	if (file_info->get_file_type() == Gio::FILE_TYPE_DIRECTORY) {
+	    Glib::RefPtr<Gio::File> child = file->get_child(
+		file_info->get_attribute_byte_string(G_FILE_ATTRIBUTE_STANDARD_NAME));
+	    dirs.push_back(
+		FileName(
+		    child->get_path(),
+		    prefix+file_info->get_attribute_string(G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME)));
+	    list_subdirs(child, dirs, prefix+"  ");
+	}
+    }
+}
+
+void list_subdirs(PathList pl, std::vector<FileName>& dirs) {
+    for (PathList::iterator i = pl.begin(); i != pl.end(); ++i) {
+	std::string fn = (*i)->get_path();
+	dirs.push_back(FileName(fn, fn));
+	list_subdirs(*i, dirs, "  ");
+    }
+}
+
+
+/****************************************************************
+ ** class BasicOptions
+ */
+
+BasicOptions *BasicOptions::instance = 0;
+
+BasicOptions::BasicOptions()
+    : user_dir(),
+      user_IR_dir(),
+      sys_IR_dir(GX_SOUND_DIR),
+      IR_pathlist(),
+      IR_prefixmap(),
+    builder_dir(GX_BUILDER_DIR) {
+    user_dir = Glib::build_filename(Glib::get_user_config_dir(), "guitarix");
+    user_IR_dir = Glib::build_filename(user_dir, "IR");
+
+    make_ending_slash(user_dir);
+    make_ending_slash(user_IR_dir);
+    make_ending_slash(sys_IR_dir);
+    make_ending_slash(builder_dir);
+
+    // for legacy presets
+    IR_pathlist.add(get_user_IR_dir());
+    IR_pathlist.add(get_sys_IR_dir());
+
+    // for current presets
+    IR_prefixmap.add('U', get_user_IR_dir());
+    IR_prefixmap.add('S', get_sys_IR_dir());
+
+    instance = this;
+}
+
+BasicOptions::~BasicOptions() {
+    instance = 0;
+}
+
+void BasicOptions::make_ending_slash(string& dirpath) {
+    if (dirpath.empty()) {
+	return;
+    }
+    if (dirpath[dirpath.size()-1] != '/') {
+	dirpath += "/";
+    }
+}
+
+
+/****************************************************************
+ ** class CmdlineOptions
+ */
 
 static inline const char *shellvar(const char *name) {
     const char *p = getenv(name);
@@ -240,9 +427,11 @@ static inline const char *shellvar(const char *name) {
 #define TCLR2(s) TCLR(s), s
 
 CmdlineOptions::CmdlineOptions()
-    : main_group("",""),
+    : BasicOptions(),
+      main_group("",""),
       optgroup_style("style", TCLR2("GTK style configuration options")),
       optgroup_jack("jack", TCLR2("JACK configuration options")),
+      optgroup_overload("overload", TCLR2("Switch to bypass mode on overload condition")),
       optgroup_file("file", TCLR2("File options")),
       optgroup_debug("debug", TCLR2("Debug options")),
       version(false), clear(false),
@@ -251,33 +440,57 @@ CmdlineOptions::CmdlineOptions()
       jack_outputs(),
       jack_uuid(),
       jack_uuid2(),
+      jack_noconnect(false),
+      jack_servername(),
       load_file(shellvar("GUITARIX_LOAD_FILE")),
-      builder_dir(GX_BUILDER_DIR),
       style_dir(GX_STYLE_DIR),
       factory_dir(GX_FACTORY_DIR),
       pixmap_dir(GX_PIXMAPS_DIR),
-      user_dir(),
       old_user_dir(),
       preset_dir(),
       pluginpreset_dir(),
+      temp_dir(),
       plugin_dir(),
-      sys_IR_dir(GX_SOUND_DIR),
-      IR_pathlist(),
       rcset(shellvar("GUITARIX_RC_STYLE")),
+      nogui(false),
+      rpcport(RPCPORT_DEFAULT),
+      rpcaddress(),
+      onlygui(false),
+      sporadic_overload(0),
+      idle_thread_timeout(0),
+      convolver_watchdog(true),
+      xrun_watchdog(false),
       lterminal(false),
+      a_save(false),
+      auto_save(false),
 #ifndef NDEBUG
       dump_parameter(false),
 #endif
-      skin(style_dir) {
+      skin(style_dir),
+      mainwin_x(-1),
+      mainwin_y(-1),
+      mainwin_height(-1),
+      window_height(600),
+      preset_window_height(220),
+      mul_buffer(1),
+      skin_name("gx7-blues"),
+      no_warn_latency(false),
+      system_order_rack_h(false),
+      system_show_value(false),
+      system_show_tooltips(true),
+      system_animations(true),
+      system_show_presets(false),
+      system_show_toolbar(false),
+      system_show_rack(false) {
     const char* home = getenv("HOME");
     if (!home) {
 	throw GxFatalError(_("no HOME environment variable"));
     }
     old_user_dir = string(home) + "/.gx_head/";
-    user_dir = Glib::build_filename(Glib::get_user_config_dir(), "guitarix");
-    plugin_dir = Glib::build_filename(user_dir, "plugins");
-    preset_dir = Glib::build_filename(user_dir, "banks");
-    pluginpreset_dir = Glib::build_filename(user_dir, "pluginpresets");
+    plugin_dir = Glib::build_filename(get_user_dir(), "plugins");
+    preset_dir = Glib::build_filename(get_user_dir(), "banks");
+    pluginpreset_dir = Glib::build_filename(get_user_dir(), "pluginpresets");
+    temp_dir = Glib::build_filename(get_user_dir(), "temp");
     const char *tmp = getenv("GUITARIX2JACK_OUTPUTS1");
     if (tmp && *tmp) {
 	jack_outputs.push_back(tmp);
@@ -287,19 +500,46 @@ CmdlineOptions::CmdlineOptions()
 	jack_outputs.push_back(tmp);
     }
 
+    read_ui_vars();
+
     // ---- parse command line arguments
     set_summary(
         "All parameters are optional. Examples:\n"
         "\tguitarix\n"
         "\tguitarix -r gx4-black -i system:capture_3\n"
         "\tguitarix -c -o system:playback_1 -o system:playback_2");
+
+    // main group
     Glib::OptionEntry opt_version;
     opt_version.set_short_name('v');
     opt_version.set_long_name("version");
     opt_version.set_description("Print version string and exit");
+    Glib::OptionEntry opt_nogui;
+    opt_nogui.set_short_name('N');
+    opt_nogui.set_long_name("nogui");
+    opt_nogui.set_description("start without GUI");
+    Glib::OptionEntry opt_rpcport;
+    opt_rpcport.set_short_name('p');
+    opt_rpcport.set_long_name("rpcport");
+    opt_rpcport.set_description("start a JSON-RPC server listening on port PORT");
+    opt_rpcport.set_arg_description("PORT");
+    Glib::OptionEntry opt_rpchost;
+    opt_rpchost.set_short_name('H');
+    opt_rpchost.set_long_name("rpchost");
+    opt_rpchost.set_description("set hostname to connect to");
+    opt_rpchost.set_arg_description("HOSTNAME");
+    Glib::OptionEntry opt_onlygui;
+    opt_onlygui.set_short_name('G');
+    opt_onlygui.set_long_name("onlygui");
+    opt_onlygui.set_description("start only GUI");
     main_group.add_entry(opt_version, version);
+    main_group.add_entry(opt_nogui, nogui);
+    main_group.add_entry(opt_rpcport, rpcport);
+    main_group.add_entry(opt_rpchost, rpcaddress);
+    main_group.add_entry(opt_onlygui, onlygui);
     set_main_group(main_group);
 
+    // style options
     Glib::OptionEntry opt_clear;
     opt_clear.set_short_name('c');
     opt_clear.set_long_name("clear");
@@ -312,7 +552,7 @@ CmdlineOptions::CmdlineOptions()
     optgroup_style.add_entry(opt_clear, clear);
     optgroup_style.add_entry(opt_rcset, rcset);
 
-    // JACK options: input and output ports
+    // JACK options
     Glib::OptionEntry opt_jack_input;
     opt_jack_input.set_short_name('i');
     opt_jack_input.set_long_name("jack-input");
@@ -328,6 +568,10 @@ CmdlineOptions::CmdlineOptions()
     opt_jack_midi.set_long_name("jack-midi");
     opt_jack_midi.set_description("Guitarix JACK midi control");
     opt_jack_midi.set_arg_description("PORT");
+    Glib::OptionEntry opt_jack_noconnect;
+    opt_jack_noconnect.set_short_name('J');
+    opt_jack_noconnect.set_long_name("jack-no-conect");
+    opt_jack_noconnect.set_description("dissable self-connect JACK ports");
     Glib::OptionEntry opt_jack_instance;
     opt_jack_instance.set_short_name('n');
     opt_jack_instance.set_long_name("name");
@@ -337,19 +581,56 @@ CmdlineOptions::CmdlineOptions()
     opt_jack_uuid.set_short_name('U');
     opt_jack_uuid.set_long_name("jack-uuid");
     opt_jack_uuid.set_description("JackSession ID");
+    opt_jack_uuid.set_arg_description("UUID");
     Glib::OptionEntry opt_jack_uuid2;
     opt_jack_uuid2.set_short_name('A');
     opt_jack_uuid2.set_long_name("jack-uuid2");
     opt_jack_uuid2.set_description("JackSession ID");
+    opt_jack_uuid2.set_arg_description("UUID2");
+    Glib::OptionEntry opt_jack_servername;
+    opt_jack_servername.set_short_name('s');
+    opt_jack_servername.set_long_name("server-name");
+    opt_jack_servername.set_description("JACK server name to connect to");
+    opt_jack_servername.set_arg_description("NAME");
     optgroup_jack.add_entry(opt_jack_input, jack_input);
     optgroup_jack.add_entry(opt_jack_output, jack_outputs);
     optgroup_jack.add_entry(opt_jack_midi, jack_midi);
+    optgroup_jack.add_entry(opt_jack_noconnect, jack_noconnect);
     optgroup_jack.add_entry(opt_jack_instance, jack_instance);
     optgroup_jack.add_entry(opt_jack_uuid, jack_uuid);
     optgroup_jack.add_entry(opt_jack_uuid2, jack_uuid2);
+    optgroup_jack.add_entry(opt_jack_servername, jack_servername);
+
+    // Engine overload options
+    Glib::OptionEntry opt_watchdog_idle;
+    opt_watchdog_idle.set_short_name('I');
+    opt_watchdog_idle.set_long_name("idle-timeout");
+    opt_watchdog_idle.set_description(
+	"starved idle thread probe (default: disabled)");
+    opt_watchdog_idle.set_arg_description("SECONDS");
+    Glib::OptionEntry opt_watchdog_convolver;
+    opt_watchdog_convolver.set_short_name('C');
+    opt_watchdog_convolver.set_long_name("no-convolver-overload");
+    opt_watchdog_convolver.set_description(
+	"disable overload on convolver missed deadline");
+    opt_watchdog_convolver.set_flags(Glib::OptionEntry::FLAG_REVERSE);
+    Glib::OptionEntry opt_watchdog_xrun;
+    opt_watchdog_xrun.set_short_name('X');
+    opt_watchdog_xrun.set_long_name("xrun-overload");
+    opt_watchdog_xrun.set_description(
+	"JACK xrun (default: false)");
+    Glib::OptionEntry opt_sporadic_overload;
+    opt_sporadic_overload.set_short_name('S');
+    opt_sporadic_overload.set_long_name("sporadic");
+    opt_sporadic_overload.set_description(
+	"allow single overload events per interval (default: disabled)");
+    opt_sporadic_overload.set_arg_description("SECONDS");
+    optgroup_overload.add_entry(opt_watchdog_idle, idle_thread_timeout);
+    optgroup_overload.add_entry(opt_watchdog_convolver, convolver_watchdog);
+    optgroup_overload.add_entry(opt_watchdog_xrun, xrun_watchdog);
+    optgroup_overload.add_entry(opt_sporadic_overload, sporadic_overload);
 
     // FILE options
-
     Glib::OptionEntry opt_load_file;
     opt_load_file.set_short_name('f');
     opt_load_file.set_long_name("load-file");
@@ -362,6 +643,16 @@ CmdlineOptions::CmdlineOptions()
     opt_plugin_dir.set_description(_("directory with guitarix plugins (.so files)"));
     opt_plugin_dir.set_arg_description("DIR");
     optgroup_file.add_entry_filename(opt_plugin_dir, plugin_dir);
+    Glib::OptionEntry opt_save_on_exit;
+    opt_save_on_exit.set_short_name('K');
+    opt_save_on_exit.set_long_name("disable-save-on-exit");
+    opt_save_on_exit.set_description(_("disable auto save to state file when quit"));
+    optgroup_file.add_entry(opt_save_on_exit, a_save);
+    Glib::OptionEntry opt_auto_save;
+    opt_auto_save.set_short_name('a');
+    opt_auto_save.set_long_name("auto-save");
+    opt_auto_save.set_description(_("enable auto save (only in server mode)"));
+    optgroup_file.add_entry(opt_auto_save, auto_save);
 
     // DEBUG options
     Glib::OptionEntry opt_builder_dir;
@@ -392,14 +683,109 @@ CmdlineOptions::CmdlineOptions()
     // collecting all option groups
     add_group(optgroup_style);
     add_group(optgroup_jack);
+    add_group(optgroup_overload);
     add_group(optgroup_file);
     add_group(optgroup_debug);
-
-    instance = this;
 }
 
 CmdlineOptions::~CmdlineOptions() {
-    instance = 0;
+    write_ui_vars();
+}
+
+void CmdlineOptions::read_ui_vars() {
+    ifstream i(Glib::build_filename(get_user_dir(), "ui_rc").c_str());
+    if (i.fail()) {
+	return;
+    }
+    JsonParser jp(&i);
+    try {
+	jp.next(JsonParser::begin_object);
+	while (jp.peek() != JsonParser::end_object) {
+	    jp.next(JsonParser::value_key);
+	    if (jp.current_value() == "system.mainwin_x") {
+		jp.next(JsonParser::value_number);
+		mainwin_x = jp.current_value_int();
+	    } else if (jp.current_value() == "system.mainwin_y") {
+		jp.next(JsonParser::value_number);
+		mainwin_y = jp.current_value_int();
+	    } else if (jp.current_value() == "system.mainwin_height") {
+		jp.next(JsonParser::value_number);
+		mainwin_height = jp.current_value_int();
+	    } else if (jp.current_value() == "system.mainwin_rack_height") {
+		jp.next(JsonParser::value_number);
+		window_height = jp.current_value_int();
+	    } else if (jp.current_value() == "system.preset_window_height") {
+		jp.next(JsonParser::value_number);
+		preset_window_height = jp.current_value_int();
+	    } else if (jp.current_value() == "system.mul_buffer") {
+		jp.next(JsonParser::value_number);
+		mul_buffer = jp.current_value_int();
+	    } else if (jp.current_value() == "ui.skin_name") {
+		jp.next(JsonParser::value_string);
+		skin_name = jp.current_value();
+	    } else if (jp.current_value() == "ui.latency_nowarn") {
+		jp.next(JsonParser::value_number);
+		no_warn_latency = jp.current_value_int();
+	    } else if (jp.current_value() == "system.order_rack_h") {
+		jp.next(JsonParser::value_number);
+		system_order_rack_h = jp.current_value_int();
+	    } else if (jp.current_value() == "system.show_value") {
+		jp.next(JsonParser::value_number);
+		system_show_value = jp.current_value_int();
+	    } else if (jp.current_value() == "system.show_tooltips") {
+		jp.next(JsonParser::value_number);
+		system_show_tooltips = jp.current_value_int();
+	    } else if (jp.current_value() == "system.animations") {
+		jp.next(JsonParser::value_number);
+		system_animations = jp.current_value_int();
+	    } else if (jp.current_value() == "system.show_presets") {
+		jp.next(JsonParser::value_number);
+		system_show_presets = jp.current_value_int();
+	    } else if (jp.current_value() == "system.show_toolbar") {
+		jp.next(JsonParser::value_number);
+		system_show_toolbar = jp.current_value_int();
+	    } else if (jp.current_value() == "system.show_rack") {
+		jp.next(JsonParser::value_number);
+		system_show_rack = jp.current_value_int();
+	    }
+	}
+	jp.next(JsonParser::end_object);
+	jp.close();
+    } catch (JsonException) {
+	gx_print_warning("main", "can't read/parse ui_rc");
+    }
+    i.close();
+}
+
+void CmdlineOptions::write_ui_vars() {
+    ofstream o(Glib::build_filename(get_user_dir(), "ui_rc").c_str());
+    if (o.fail()) {
+	return;
+    }
+    JsonWriter jw(&o);
+    try {
+	jw.begin_object(true);
+	jw.write_kv("system.mainwin_x", mainwin_x);
+	jw.write_kv("system.mainwin_y", mainwin_y);
+	jw.write_kv("system.mainwin_height", mainwin_height);
+	jw.write_kv("system.mainwin_rack_height", window_height);
+	jw.write_kv("system.preset_window_height", preset_window_height);
+	jw.write_kv("system.mul_buffer", mul_buffer);
+	jw.write_kv("ui.skin_name", skin_name);
+	jw.write_kv("ui.latency_nowarn", no_warn_latency);
+	jw.write_kv("system.order_rack_h", system_order_rack_h);
+	jw.write_kv("system.show_value", system_show_value);
+	jw.write_kv("system.show_tooltips", system_show_tooltips);
+	jw.write_kv("system.animations", system_animations);
+	jw.write_kv("system.show_presets", system_show_presets);
+	jw.write_kv("system.show_toolbar", system_show_toolbar);
+	jw.write_kv("system.show_rack", system_show_rack);
+	jw.end_object(true);
+	jw.close();
+    } catch (JsonException) {
+	gx_print_warning("main", "can't write ui_rc");
+    }
+    o.close();
 }
 
 Glib::ustring CmdlineOptions::get_jack_output(unsigned int n) const {
@@ -413,14 +799,13 @@ string CmdlineOptions::get_opskin() {
     // initialize number of skins. We just count the number of rc files
     unsigned int n = skin.skin_list.size();
     if (n < 1) {
-        gx_print_error(_("main"), string(_("number of skins is 0")));
-	GxExit::get_instance().exit_program();
+        gx_print_fatal(_("main"), string(_("number of skins is 0")));
     }
 
     // GTK options: rc style (aka skin)
     string opskin("Style to use");
 
-    vector<string>::iterator it;
+    vector<Glib::ustring>::iterator it;
 
     for (it = skin.skin_list.begin(); it != skin.skin_list.end(); ++it) {
         opskin += ", '" + *it + "'";
@@ -428,12 +813,12 @@ string CmdlineOptions::get_opskin() {
     return opskin;
 }
 
-static void log_terminal(const string& msg, GxMsgType tp, bool plugged) {
+static void log_terminal(const string& msg, GxLogger::MsgType tp, bool plugged) {
     const char *t;
     switch (tp) {
-    case kInfo:    t = "I"; break;
-    case kWarning: t = "W"; break;
-    case kError:   t = "E"; break;
+    case GxLogger::kInfo:    t = "I"; break;
+    case GxLogger::kWarning: t = "W"; break;
+    case GxLogger::kError:   t = "E"; break;
     default:       t = "?"; break;
     }
     if (!plugged) {
@@ -441,15 +826,6 @@ static void log_terminal(const string& msg, GxMsgType tp, bool plugged) {
     }
 }
 
-void CmdlineOptions::make_ending_slash(string& dirpath) {
-    if (dirpath.empty()) {
-	return;
-    }
-    if (dirpath[dirpath.size()-1] != '/') {
-	dirpath += "/";
-    }
-}
-
 void CmdlineOptions::process(int argc, char** argv) {
     path_to_program = Gio::File::create_for_path(argv[0])->get_path();
     if (version) {
@@ -462,7 +838,7 @@ void CmdlineOptions::process(int argc, char** argv) {
     }
 #ifdef NDEBUG
     if (argc > 1) {
-	throw gx_system::GxFatalError(
+	throw GxFatalError(
 	    string("unknown argument on command line: ")+argv[1]);
     }
 #endif
@@ -472,31 +848,34 @@ void CmdlineOptions::process(int argc, char** argv) {
 	    _("-c and -r cannot be used together"));
     }
     if (lterminal) {
-	Logger::get_logger().signal_message().connect(
+	GxLogger::get_logger().signal_message().connect(
 	    sigc::ptr_fun(log_terminal));
+	if (nogui) {
+	    GxLogger::get_logger().unplug_queue();
+	}
     }
 
     make_ending_slash(builder_dir);
     make_ending_slash(style_dir);
     make_ending_slash(factory_dir);
     make_ending_slash(pixmap_dir);
-    make_ending_slash(user_dir);
     make_ending_slash(preset_dir);
     make_ending_slash(pluginpreset_dir);
+    make_ending_slash(temp_dir);
     make_ending_slash(plugin_dir);
-    make_ending_slash(sys_IR_dir);
-
-    IR_pathlist.add(get_user_dir());
-    IR_pathlist.add(get_sys_IR_dir());
 
     skin.set_styledir(style_dir);
-    if (!rcset.empty() && !skin.is_in_list(rcset)) {
-	throw Glib::OptionError(
-	    Glib::OptionError::BAD_VALUE,
-	    (boost::format(_("invalid style '%1%' on command line"))
-	     % rcset).str());
+    if (!rcset.empty()) {
+	if (skin.is_in_list(rcset)) {
+	    skin_name = rcset;
+	} else {
+	    throw Glib::OptionError(
+		Glib::OptionError::BAD_VALUE,
+		(boost::format(_("invalid style '%1%' on command line"))
+		 % rcset).str());
+	}
     }
-    if (jack_outputs.size() >= 2) {
+    if (jack_outputs.size() > 2) {
 	gx_print_warning(
 	    _("main"),
 	    _("Warning --> provided more than 2 output ports, ignoring extra ports"));
@@ -504,189 +883,6 @@ void CmdlineOptions::process(int argc, char** argv) {
 }
 
 
-/****************************************************************
- ** Logging
- */
-
-Logger::Logger()
-    : trackable(),
-      msglist(),
-      msgmutex(),
-      got_new_msg(),
-      ui_thread(),
-      handlers(),
-      queue_all_msgs(true) {
-}
-
-static class LoggerGuard {
-private:
-    Logger *logger_instance;
-public:
-    LoggerGuard() : logger_instance(0) {}
-    ~LoggerGuard() { destroy(); }
-    void create() { logger_instance = new Logger; }
-    void destroy() { if (logger_instance) { delete logger_instance; logger_instance = 0; }}
-    Logger *get() { return logger_instance; }
-} logger_guard;
-
-Logger& Logger::get_logger() {
-    Logger *l = logger_guard.get();
-    if (!l) {
-	logger_guard.create();
-	l = logger_guard.get();
-    }
-    return *l;
-}
-
-void Logger::destroy() {
-    logger_guard.destroy();
-}
-
-Logger::~Logger() {
-    delete got_new_msg;
-}
-
-void Logger::set_ui_thread() {
-    if (ui_thread) {
-	assert(pthread_equal(pthread_self(), ui_thread));
-    } else {
-	got_new_msg = new Glib::Dispatcher;
-	ui_thread = pthread_self();
-	got_new_msg->connect(mem_fun(*this, &Logger::write_queued));
-    }
-}
-
-Logger::msg_signal& Logger::signal_message() {
-    set_ui_thread();
-    return handlers;
-}
-
-void Logger::unplug_queue() {
-    if (!queue_all_msgs) {
-	return;
-    }
-    queue_all_msgs = false;
-    write_queued();
-}
-
-void Logger::write_queued() {
-    if (handlers.empty()) {
-	return;
-    }
-
-    // quick copy list
-    msgmutex.lock();
-    list<logmsg> l = msglist;
-    if (!queue_all_msgs) {
-	msglist.clear();
-    }
-    msgmutex.unlock();
-
-    // feed throught the handler(s)
-    for (list<logmsg>::iterator i = l.begin(); i != l.end(); ++i) {
-	if (queue_all_msgs) {
-	    if (!i->plugged) {
-		handlers(i->msg, i->msgtype, i->plugged);
-		i->plugged = true;
-	    }
-	} else {
-	    handlers(i->msg, i->msgtype, i->plugged);
-	}
-    }
-}
-
-string Logger::format(const char* func, const string& msg) {
-    // timestamp
-    time_t now;
-    time(&now);
-    struct tm *tm_now = localtime(&now);
-    ostringstream msgbuf;
-    msgbuf << "[" << setfill('0')
-           << setw(2) << tm_now->tm_hour << ":"
-           << setw(2) << tm_now->tm_min  << ":"
-           << setw(2) << tm_now->tm_sec  << "]"
-           << "  " << func << "  ***  " << msg;
-    return msgbuf.str();
-}
-
-void Logger::print(const char* func, const string& msg, GxMsgType msgtype) {
-    string m = format(func, msg);
-    if (handlers.empty() || !(pthread_equal(pthread_self(), ui_thread))) {
-	boost::mutex::scoped_lock lock(msgmutex);
-	// defer output
-        msglist.push_back(logmsg(m, msgtype, false));
-	if (!handlers.empty() && msglist.size() == 1) {
-	    (*got_new_msg)();
-	}
-    } else {
-	write_queued();
-	handlers(m, msgtype, false);
-	if (queue_all_msgs) {
-	    msglist.push_back(logmsg(m, msgtype, true));
-	}
-    }
-}
-
-/*
-** utility logger functions
-*/
-
-// ---- log message handler
-void gx_print_logmsg(const char* func, const string& msg, GxMsgType msgtype) {
-    Logger::get_logger().print(func, msg, msgtype);
-}
-
-// warning
-void gx_print_warning(const char* func, const string& msg) {
-    gx_print_logmsg(func, msg, kWarning);
-}
-
-// error
-void gx_print_error(const char* func, const string& msg) {
-    gx_print_logmsg(func, msg, kError);
-}
-
-GxFatalError::~GxFatalError() throw() {
-}
-
-// fatal error
-// - do not use before Gtk::Main() ctor
-// - do not use when main loop is blocked (modal dialog or something)
-//
-void gx_print_fatal(const char* func, const string& msg) {
-    string msgbuf = string(_("fatal system error: ")) + func + "  ***  " + msg + "\n";
-    GxExit::get_instance().fatal_msg(msgbuf);
-}
-
-// info
-void gx_print_info(const char* func, const string& msg) {
-    gx_print_logmsg(func, msg, kInfo);
-}
-
-
-/****************************************************************
- ** class GxExit
- */
-
-GxExit::GxExit(): exit_sig(), ui_thread() {}
-
-GxExit::~GxExit() {}
-
-void GxExit::exit_program(string msg, int errcode) {
-    exit_sig(pthread_equal(pthread_self(), ui_thread));
-    if (msg.empty()) {
-	msg = "** guitarix exit **";
-    }
-    cerr << msg << endl;
-    _exit(errcode);
-}
-
-GxExit& GxExit::get_instance() {
-    static GxExit instance;
-    return instance;
-}
-
-
 /****************************************************************
  ** misc functions
  */
@@ -714,4 +910,16 @@ int gx_system_call(const string& cmd,
     return rc;
 }
 
+void strip(Glib::ustring& s) {
+    size_t n = s.find_first_not_of(' ');
+    if (n == Glib::ustring::npos) {
+	s.erase();
+	return;
+    }
+    if (n != 0) {
+	s.erase(0, n);
+    }
+    s.erase(s.find_last_not_of(' ')+1);
+}
+
 } /* end of gx_system namespace */
