@@ -42,10 +42,10 @@ bool PresetStore::row_draggable_vfunc(const TreeModel::Path& path) const {
 }
 
 
-PresetWindow::PresetWindow(Glib::RefPtr<gx_gui::GxBuilder> bld, gx_preset::GxSettings& gx_settings_,
+PresetWindow::PresetWindow(Glib::RefPtr<gx_gui::GxBuilder> bld, gx_engine::GxMachineBase& machine_,
 			   const gx_system::CmdlineOptions& options_, GxActions& actions_)
     : sigc::trackable(),
-      gx_settings(gx_settings_),
+      machine(machine_),
       actions(actions_),
       in_edit(false),
       edit_iter(),
@@ -65,8 +65,7 @@ PresetWindow::PresetWindow(Glib::RefPtr<gx_gui::GxBuilder> bld, gx_preset::GxSet
       vpaned_target(),
       options(options_),
       in_current_preset(false),
-      on_map_conn(),
-      reload_on_change_conn()
+      on_map_conn()
       /* widget pointers not initialized */ {
     load_widget_pointers(bld);
     actions.new_bank = Gtk::Action::create("NewBank");
@@ -116,6 +115,8 @@ PresetWindow::PresetWindow(Glib::RefPtr<gx_gui::GxBuilder> bld, gx_preset::GxSet
 	sigc::mem_fun(*this, &PresetWindow::on_bank_drag_data_received));
     bank_treeview->signal_drag_data_get().connect(
 	sigc::mem_fun(*this, &PresetWindow::on_bank_drag_data_get));
+    bank_treeview->signal_drag_begin().connect(
+	sigc::hide(sigc::mem_fun(machine, &gx_engine::GxMachineBase::bank_drag_begin)));
     Glib::RefPtr<Gtk::TreeSelection> sel = bank_treeview->get_selection();
     sel->set_mode(Gtk::SELECTION_BROWSE);
     sel->signal_changed().connect(sigc::mem_fun(*this, &PresetWindow::on_bank_changed));
@@ -142,15 +143,20 @@ PresetWindow::PresetWindow(Glib::RefPtr<gx_gui::GxBuilder> bld, gx_preset::GxSet
 	sigc::bind(
 	    sigc::mem_fun(*this, &PresetWindow::on_editing_started),
 	    Glib::RefPtr<Gtk::TreeModel>::cast_static(pstore)));
-    preset_treeview->get_column(0)->set_cell_data_func(*preset_cellrenderer, sigc::mem_fun(*this, &PresetWindow::text_func));
-    banks_combobox->signal_changed().connect(sigc::mem_fun(*this, &PresetWindow::on_preset_combo_changed));
+    preset_treeview->get_column(0)->set_cell_data_func(
+	*preset_cellrenderer, sigc::mem_fun(*this, &PresetWindow::text_func));
+    banks_combobox->signal_changed().connect(
+	sigc::mem_fun(*this, &PresetWindow::on_preset_combo_changed));
     presets_target_treeview->get_selection()->set_mode(Gtk::SELECTION_NONE);
     std::vector<Gtk::TargetEntry> listTargets3;
-    listTargets3.push_back(Gtk::TargetEntry("application/x-guitarix-preset", Gtk::TARGET_SAME_APP, 0));
+    listTargets3.push_back(
+	Gtk::TargetEntry("application/x-guitarix-preset", Gtk::TARGET_SAME_APP, 0));
     presets_target_treeview->enable_model_drag_dest(listTargets3, Gdk::ACTION_COPY);
-    presets_target_treeview->signal_drag_motion().connect(sigc::mem_fun(*this, &PresetWindow::on_target_drag_motion), false);
-    presets_target_treeview->signal_drag_data_received().connect_notify(sigc::mem_fun(*this, &PresetWindow::target_drag_data_received));
-    reload_on_change_conn = gx_settings.signal_selection_changed().connect(
+    presets_target_treeview->signal_drag_motion().connect(
+	sigc::mem_fun(*this, &PresetWindow::on_target_drag_motion), false);
+    presets_target_treeview->signal_drag_data_received().connect_notify(
+	sigc::mem_fun(*this, &PresetWindow::target_drag_data_received));
+    machine.signal_selection_changed().connect(
 	sigc::mem_fun(*this, &PresetWindow::on_selection_changed));
 }
 
@@ -158,10 +164,22 @@ PresetWindow::~PresetWindow() {
 }
 
 void PresetWindow::on_selection_changed() {
+    Gtk::TreeIter it = bank_treeview->get_selection()->get_selected();
+    if (it && it->get_value(bank_col.name) == machine.get_current_bank()) {
+	it = preset_treeview->get_selection()->get_selected();
+	if (it && it->get_value(pstore->col.name) == machine.get_current_name()) {
+	    Glib::RefPtr<Gdk::Window> w = bank_treeview->get_window();
+	    if (w) { // might not yet be initialized
+		gdk_window_invalidate_rect(w->gobj(), 0, true);
+		gdk_window_invalidate_rect(preset_treeview->get_window()->gobj(), 0, true);
+	    }
+	    return;
+	}
+    }
     set_presets();
     bool s = false;
-    if (gx_settings.setting_is_preset()) {
-	gx_system::PresetFile *pf = gx_settings.get_current_bank_file();
+    if (machine.setting_is_preset()) {
+	gx_system::PresetFileGui *pf = machine.get_current_bank_file();
 	if (pf && pf->is_mutable()) {
 	    s = true;
 	}
@@ -214,7 +232,7 @@ bool PresetWindow::on_bank_query_tooltip(int x, int y, bool kb_tooltip, Glib::Re
     if (nm.empty()) {
 	return false;
     }
-    gx_system::PresetFile *f = gx_settings.banks.get_file(nm);
+    gx_system::PresetFileGui *f = machine.get_bank_file(nm);
     if (col == bank_treeview->get_column(0) || col == bank_treeview->get_column(1)) {
 	if (tp == gx_system::PresetFile::PRESET_FILE) {
 	    if (f->get_flags() & gx_system::PRESET_FLAG_INVALID) {
@@ -290,8 +308,8 @@ void PresetWindow::on_bank_drag_data_get(const Glib::RefPtr<Gdk::DragContext>& c
     bank_treeview->get_cursor(path, focus_column);
     Glib::RefPtr<Gio::File> f =
 	Gio::File::create_for_path(
-	    gx_settings.banks.get_file(
-		bank_treeview->get_model()->get_iter(path)->get_value(bank_col.name))->get_filename());
+	    machine.bank_get_filename(
+		bank_treeview->get_model()->get_iter(path)->get_value(bank_col.name)));
     if (info == TEXT_TARGETS) {
 	selection.set_text(f->get_path());
     } else {
@@ -314,43 +332,13 @@ void PresetWindow::on_bank_drag_data_received(const Glib::RefPtr<Gdk::DragContex
     std::vector<Glib::ustring> uris = data.get_uris();
     Glib::RefPtr<Gtk::ListStore> ls = Glib::RefPtr<Gtk::ListStore>::cast_dynamic(bank_treeview->get_model());
     for (std::vector<Glib::ustring>::iterator i = uris.begin(); i != uris.end(); ++i) {
-	Glib::RefPtr<Gio::File> rem = Gio::File::create_for_uri(*i);
-	std::string filename = rem->get_basename();
-	gx_settings.banks.strip_preset_postfix(filename);
-	Glib::ustring name = gx_settings.banks.decode_filename(filename);
-	gx_settings.banks.make_valid_utf8(name);
-	gx_settings.banks.make_bank_unique(name, &filename);
-	Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path(filename);
-	try {
-	    rem->copy(dest);
-	} catch (Gio::Error& e) {
-	    gx_system::gx_print_error(e.what().c_str(), _("can't copy to config dir"));
-	    is_move = false;
-	    continue;
-	}
-	gx_system::PresetFile *f = new gx_system::PresetFile();
-	if (f->open_file(name, filename, gx_system::PresetFile::PRESET_FILE, 0)) {
-	    gx_settings.banks.insert(f);
+	gx_system::PresetFileGui *f = machine.bank_insert_uri(*i, is_move);
+	if (f) {
 	    Gtk::TreeIter i = ls->prepend();
 	    set_row_for_presetfile(i,f);
 	    bank_treeview->set_cursor(ls->get_path(i));
 	    bank_treeview->get_selection()->select(i);
 	    success = true;
-	} else {
-	    delete f;
-	    try {
-		dest->remove();
-	    } catch (Gio::Error& e) {
-		gx_system::gx_print_error(e.what().c_str(), _("can't remove copied file!?"));
-	    }
-	    is_move = false;
-	}
-	if (is_move) {
-	    try {
-		rem->remove();
-	    } catch (Gio::Error& e) {
-		gx_system::gx_print_error(e.what().c_str(), _("can't move; file has been copied"));
-	    }
 	}
     }
     context->drag_finish(success, false, timestamp);
@@ -374,7 +362,7 @@ void PresetWindow::target_drag_data_received(const Glib::RefPtr<Gdk::DragContext
 	presets_target_treeview->signal_drag_data_received().emission_stop();
 	return;
     }
-    gx_system::PresetFile& fl = *gx_settings.banks.get_file(bank);
+    gx_system::PresetFileGui& fl = *machine.get_bank_file(bank);
     Glib::RefPtr<Gtk::ListStore> ls = Glib::RefPtr<Gtk::ListStore>::cast_dynamic(presets_target_treeview->get_model());
     Glib::ustring srcnm = data.get_data_as_string();
     Glib::ustring dstnm = srcnm;
@@ -384,20 +372,24 @@ void PresetWindow::target_drag_data_received(const Glib::RefPtr<Gdk::DragContext
 	n += 1;
     }
     Glib::ustring src_bank = get_current_bank();
-    gx_system::PresetFile& pf = *gx_settings.banks.get_file(src_bank);
+    gx_system::PresetFileGui& pf = *machine.bank_get_file(src_bank);
+    if (src_bank == bank) {
+        gx_print_error("preset", "can't copy inside the same bank");
+        return;
+    }
     Gtk::TreeModel::Path pt;
     Gtk::TreeViewDropPosition dst;
     if (!presets_target_treeview->get_dest_row_at_pos(x, y, pt, dst)) {
 	ls->append()->set_value(target_col.name, dstnm);
-	gx_settings.append(pf, srcnm, fl, dstnm);
+	machine.pf_append(pf, srcnm, fl, dstnm);
     } else {
 	Gtk::TreeIter it = ls->get_iter(pt);
 	if (dst == Gtk::TREE_VIEW_DROP_BEFORE || dst == Gtk::TREE_VIEW_DROP_INTO_OR_BEFORE) {
 	    ls->insert(it)->set_value(target_col.name, dstnm);
-	    gx_settings.insert_before(pf, srcnm, fl, it->get_value(target_col.name), dstnm);
+	    machine.pf_insert_before(pf, srcnm, fl, it->get_value(target_col.name), dstnm);
 	} else { // gtk.TREE_VIEW_DROP_INTO_OR_AFTER, gtk.TREE_VIEW_DROP_AFTER
 	    ls->insert_after(it)->set_value(target_col.name, dstnm);
-	    gx_settings.insert_after(pf, srcnm, fl, it->get_value(target_col.name), dstnm);
+	    machine.pf_insert_after(pf, srcnm, fl, it->get_value(target_col.name), dstnm);
 	}
     }
     if (context->get_action() == Gdk::ACTION_MOVE) {
@@ -407,7 +399,7 @@ void PresetWindow::target_drag_data_received(const Glib::RefPtr<Gdk::DragContext
 	preset_row_del_conn.block();
 	pstore->erase(pstore->get_iter(pt));
 	preset_row_del_conn.unblock();
-	gx_settings.erase_preset(pf, srcnm);
+	machine.erase_preset(pf, srcnm);
     }
     if (src_bank == bank) {
 	on_bank_changed();
@@ -424,7 +416,7 @@ bool PresetWindow::on_target_drag_motion(const Glib::RefPtr<Gdk::DragContext>& c
     Gtk::TreeIter it = bank_treeview->get_selection()->get_selected();
     int tp = it->get_value(bank_col.tp);
     Glib::ustring nm = it->get_value(bank_col.name);
-    if ((tp != gx_system::PresetFile::PRESET_SCRATCH && tp != gx_system::PresetFile::PRESET_FILE) || gx_settings.banks.get_file(nm)->get_flags() ||
+    if ((tp != gx_system::PresetFile::PRESET_SCRATCH && tp != gx_system::PresetFile::PRESET_FILE) || machine.get_bank_file(nm)->get_flags() ||
 	get_combo_selection() == nm) {
 	context->drag_status(Gdk::ACTION_COPY, timestamp);
     }
@@ -437,7 +429,7 @@ void PresetWindow::reload_combo() {
     ls->clear();
     int n = 0;
     int nn = -1;
-    for (gx_system::PresetBanks::iterator i = gx_settings.banks.begin(); i != gx_settings.banks.end(); ++i) {
+    for (gx_engine::bank_iterator i = machine.bank_begin(); i != machine.bank_end(); ++i) {
 	int tp = i->get_type();
 	if (tp != gx_system::PresetFile::PRESET_FILE && tp != gx_system::PresetFile::PRESET_SCRATCH) {
 	    continue;
@@ -462,7 +454,7 @@ void PresetWindow::on_preset_combo_changed() {
     if (nm.empty()) {
 	return;
     }
-    gx_system::PresetFile& f = *gx_settings.banks.get_file(nm);
+    gx_system::PresetFileGui& f = *machine.get_bank_file(nm);
     for (gx_system::PresetFile::iterator i = f.begin(); i != f.end(); ++i) {
 	ls->append()->set_value(bank_col.name, i->name);
     }
@@ -535,13 +527,13 @@ void PresetWindow::highlight_current_bank(Gtk::CellRenderer *cell, const Gtk::Tr
 	return;
     }
     Gtk::CellRendererText *tc = dynamic_cast<Gtk::CellRendererText*>(cell);
-    if (gx_settings.setting_is_preset() && t == gx_settings.get_current_bank()) {
+    if (machine.setting_is_preset() && t == machine.get_current_bank()) {
 	tc->property_foreground().set_value("#f00");
     } else{
 	tc->property_foreground_set().set_value(false);
     }
-    int n = gx_settings.banks.size() - *bank_treeview->get_model()->get_path(iter).begin();
-    if (!(gx_settings.banks.get_file(t)->get_type() & gx_system::PresetFile::PRESET_FACTORY)) {
+    int n = machine.bank_size() - *bank_treeview->get_model()->get_path(iter).begin();
+    if (!(machine.get_bank_file(t)->get_type() & gx_system::PresetFile::PRESET_FACTORY)) {
 	n -= 1;
     }
     if (n > 26) {
@@ -586,27 +578,25 @@ bool PresetWindow::on_bank_button_release(GdkEventButton *ev) {
     int tp = it->get_value(bank_col.tp);
     if (col == bank_treeview->get_column(0)) {
 	if (tp == gx_system::PresetFile::PRESET_SCRATCH || tp == gx_system::PresetFile::PRESET_FILE) {
-	    int flags = gx_settings.banks.get_file(nm)->get_flags();
-	    gx_system::PresetFile *f = gx_settings.banks.get_file(nm);
+	    int flags = machine.get_bank_file(nm)->get_flags();
+	    gx_system::PresetFileGui *f = machine.get_bank_file(nm);
 	    if (flags == 0 && tp == gx_system::PresetFile::PRESET_FILE) {
 		/*if (run_message_dialog(*bank_treeview, "set bank " + nm + " to readonly?")) {*/
-		f->set_flag(gx_system::PRESET_FLAG_READONLY, true);
-		gx_settings.banks.save();
+		machine.bank_set_flag(f, gx_system::PRESET_FLAG_READONLY, true);
 		reload_banks(nm);
 	    } else if (flags == gx_system::PRESET_FLAG_VERSIONDIFF) {
 		if (run_message_dialog(*bank_treeview, "convert bank " + nm + " to new version?")) {
-		    if (gx_settings.convert_preset(*f)) {
+		    if (machine.convert_preset(*f)) {
 			reload_banks(nm);
 		    }
 		}
 	    } else if (flags == gx_system::PRESET_FLAG_READONLY) {
 		/*if (run_message_dialog(*bank_treeview, "set bank " + nm + " to read/write?")) {*/
-		f->set_flag(gx_system::PRESET_FLAG_READONLY, false);
-		gx_settings.banks.save();
+		machine.bank_set_flag(f, gx_system::PRESET_FLAG_READONLY, false);
 		reload_banks(nm);
 	    } else if (flags == (gx_system::PRESET_FLAG_READONLY | gx_system::PRESET_FLAG_VERSIONDIFF)) {
 		if (run_message_dialog(*bank_treeview, "convert readonly bank " + nm + " to new version?")) {
-		    if (gx_settings.convert_preset(*f)) {
+		    if (machine.convert_preset(*f)) {
 			reload_banks(nm);
 		    }
 		}
@@ -615,14 +605,14 @@ bool PresetWindow::on_bank_button_release(GdkEventButton *ev) {
 			*bank_treeview, "delete damaged bank " + nm + "?"
 			" Export it before deleting and ask in the"
 			" guitarix online forum if you want to try to repair it!")) {
-		    gx_settings.banks.remove(nm);
+		    machine.bank_remove(nm);
 		    reload_banks(nm);
 		}
 	    }
 	}
 	return false;
     }
-    if (tp != gx_system::PresetFile::PRESET_FILE || gx_settings.banks.get_file(nm)->get_flags()) {
+    if (tp != gx_system::PresetFile::PRESET_FILE || machine.get_bank_file(nm)->get_flags()) {
 	return false;
     }
     if (col == bank_treeview->get_column(2)) {
@@ -635,10 +625,9 @@ bool PresetWindow::on_bank_button_release(GdkEventButton *ev) {
 	    bank_row_del_conn.block();
 	    ls->erase(it);
 	    bank_row_del_conn.unblock();
-	    gx_settings.banks.remove(nm);
+	    machine.bank_remove(nm);
 	    reload_combo();
-	    if (nm == gx_settings.get_current_bank()) {
-		gx_settings.set_source_to_state();
+	    if (!machine.setting_is_preset()) { // if current bank was removed
 		actions.save_changes->set_sensitive(false);
 	    }
 	}
@@ -646,50 +635,33 @@ bool PresetWindow::on_bank_button_release(GdkEventButton *ev) {
     return false;
 }
 
-static void strip(Glib::ustring& s) {
-    size_t n = s.find_first_not_of(' ');
-    if (n == Glib::ustring::npos) {
-	s.erase();
-	return;
-    }
-    if (n != 0) {
-	s.erase(0, n);
-    }
-    s.erase(s.find_last_not_of(' ')+1);
-}
-
 void PresetWindow::on_bank_edited(const Glib::ustring& path, const Glib::ustring& newtext, Gtk::TreeView* w) {
     Gtk::TreeIter sel = w->get_model()->get_iter(path);
     Glib::ustring oldname = sel->get_value(bank_col.name);
     Glib::ustring newname = newtext;
-    strip(newname);
+    gx_system::strip(newname);
     if (newname.empty() || newname == oldname) {
 	Gtk::TreeViewColumn *p = w->get_column(1);
 	reset_edit(*p);
 	return;
     }
     Glib::RefPtr<Gtk::ListStore> ls = Glib::RefPtr<Gtk::ListStore>::cast_dynamic(banks_combobox->get_model());
-    std::string newfile;
-    gx_settings.banks.make_bank_unique(newname, &newfile);
     if (edit_iter) {
-	ls->prepend()->set_value(target_col.name, newname);
-	edit_iter = ls->children().end();
-	gx_system::PresetFile *f = new gx_system::PresetFile();
-	if (f->create_file(newname, newfile, gx_system::PresetFile::PRESET_FILE, 0)) {
-	    gx_settings.banks.insert(f);
-	    set_row_for_presetfile(sel,f);
-	} else {
-	    delete f;
+	gx_system::PresetFileGui *f = machine.bank_insert_new(newname);
+	if (f) {
+	    ls->prepend()->set_value(target_col.name, f->get_name());
+	    edit_iter = ls->children().end();
+	    set_row_for_presetfile(sel, f);
+	    w->get_selection()->select(sel);
 	}
-	w->get_selection()->select(sel);
     } else {
+	machine.rename_bank(oldname, newname);
 	Gtk::TreeNodeChildren ch = ls->children();
 	for (Gtk::TreeIter it = ch.begin(); it != ch.end(); ++it) {
 	    if (it->get_value(bank_col.name) == oldname) {
 		it->set_value(bank_col.name, newname);
 	    }
 	}
-	gx_settings.rename_bank(oldname, newname, newfile);
 	w->get_model()->get_iter(path)->set_value(bank_col.name, newname);
     }
     Gtk::TreeViewColumn *p = w->get_column(1);
@@ -750,6 +722,9 @@ bool PresetWindow::on_bank_drag_motion(const Glib::RefPtr<Gdk::DragContext>& con
 
 void PresetWindow::on_bank_changed() {
     preset_row_del_conn.block();
+    if (in_edit) {
+    pstore->prepend();
+    }
     pstore->clear();
     preset_row_del_conn.unblock();
     Gtk::TreeIter it = bank_treeview->get_selection()->get_selected();
@@ -759,9 +734,9 @@ void PresetWindow::on_bank_changed() {
     }
     Glib::ustring nm = it->get_value(bank_col.name);
     preset_title->set_text(nm);
-    in_current_preset = (nm == gx_settings.get_current_bank());
+    in_current_preset = (nm == machine.get_current_bank());
     Gtk::TreeIter i;
-    gx_system::PresetFile& ll = *gx_settings.banks.get_file(nm);
+    gx_system::PresetFileGui& ll = *machine.get_bank_file(nm);
     if ((ll.get_flags() & gx_system::PRESET_FLAG_VERSIONDIFF) ||
 	((ll.get_flags() & gx_system::PRESET_FLAG_READONLY) && !actions.organize->get_active())) {
 	preset_treeview->unset_rows_drag_source();
@@ -781,7 +756,7 @@ void PresetWindow::on_bank_changed() {
 	    i->set_value(pstore->col.edit_pb, pb_edit);
 	    i->set_value(pstore->col.del_pb, pb_del);
 	}
-	if (in_current_preset && s->name == gx_settings.get_current_name()) {
+	if (in_current_preset && s->name == machine.get_current_name()) {
 	    if (preset_treeview->get_mapped()) {
 		preset_treeview->scroll_to_row(pstore->get_path(i));
 	    }
@@ -792,7 +767,7 @@ void PresetWindow::on_bank_changed() {
     }
 }
 
-void PresetWindow::set_row_for_presetfile(Gtk::TreeIter i, gx_system::PresetFile *f) {
+void PresetWindow::set_row_for_presetfile(Gtk::TreeIter i, gx_system::PresetFileGui *f) {
     i->set_value(bank_col.name, f->get_name());
     if (f->get_flags() & gx_system::PRESET_FLAG_INVALID) {
 	i->set_value(bank_col.type_pb, pb_del);
@@ -823,7 +798,7 @@ void PresetWindow::reload_banks(const Glib::ustring& sel_bank) {
     bank_row_del_conn.unblock();
     Gtk::TreeIter i;
     int in_factory = false;
-    for (gx_system::PresetBanks::iterator v = gx_settings.banks.begin(); v != gx_settings.banks.end(); ++v) {
+    for (gx_engine::bank_iterator v = machine.bank_begin(); v != machine.bank_end(); ++v) {
 	if (!in_factory && v->get_type() == gx_system::PresetFile::PRESET_FACTORY) {
 	    i = ls->append();
 	    i->set_value(bank_col.tp, static_cast<int>(gx_system::PresetFile::PRESET_SEP));
@@ -839,7 +814,7 @@ void PresetWindow::reload_banks(const Glib::ustring& sel_bank) {
 }
 
 void PresetWindow::set_presets() {
-    reload_banks(gx_settings.get_current_bank());
+    reload_banks(machine.get_current_bank());
 }
 
 void PresetWindow::on_bank_reordered(const Gtk::TreeModel::Path& path) {
@@ -853,7 +828,7 @@ void PresetWindow::on_bank_reordered(const Gtk::TreeModel::Path& path) {
 	    l.push_back(nm);
 	}
     }
-    gx_settings.banks.reorder(l);
+    machine.bank_reorder(l);
 }
 
 /*
@@ -882,7 +857,7 @@ void PresetWindow::text_func(Gtk::CellRenderer *cell, const Gtk::TreeModel::iter
     }
     cell->set_property("text", t);
     Gtk::CellRendererText *tc = dynamic_cast<Gtk::CellRendererText*>(cell);
-    if (in_current_preset && gx_settings.setting_is_preset() && val == gx_settings.get_current_name()) {
+    if (in_current_preset && machine.setting_is_preset() && val == machine.get_current_name()) {
 	tc->property_foreground().set_value("#f00");
     } else{
 	tc->property_foreground_set().set_value(false);
@@ -899,12 +874,12 @@ bool PresetWindow::on_preset_button_release(GdkEventButton *ev) {
     Gtk::TreeModel::Path path;
     Gtk::TreeViewColumn *focus_column;
     preset_treeview->get_cursor(path, focus_column);
-    if (col != focus_column || pt != path) {
+    if (col != focus_column || !path || pt != path) {
 	return false;
     }
     Gtk::TreeIter bank_iter = get_current_bank_iter();
     int tp = bank_iter->get_value(bank_col.tp);
-    if ((tp != gx_system::PresetFile::PRESET_SCRATCH && tp != gx_system::PresetFile::PRESET_FILE) || gx_settings.banks.get_file(bank_iter->get_value(bank_col.name))->get_flags()) {
+    if ((tp != gx_system::PresetFile::PRESET_SCRATCH && tp != gx_system::PresetFile::PRESET_FILE) || machine.get_bank_file(bank_iter->get_value(bank_col.name))->get_flags()) {
 	return false;
     }
     if (col == preset_treeview->get_column(1)) {
@@ -919,7 +894,7 @@ bool PresetWindow::on_preset_button_release(GdkEventButton *ev) {
 	    preset_row_del_conn.block();
 	    pstore->erase(pstore->get_iter(pt));
 	    preset_row_del_conn.unblock();
-	    gx_settings.erase_preset(*gx_settings.banks.get_file(bank_iter->get_value(bank_col.name)), nm);
+	    machine.erase_preset(*machine.get_bank_file(bank_iter->get_value(bank_col.name)), nm);
 	    reload_target();
 	}
     }
@@ -930,12 +905,12 @@ void PresetWindow::on_preset_edited(const Glib::ustring& path, const Glib::ustri
     Gtk::TreeIter it = pstore->get_iter(path);
     Glib::ustring oldname = it->get_value(pstore->col.name);
     Glib::ustring newname = newtext;
-    strip(newname);
+    gx_system::strip(newname);
     if (newname.empty() || newname == oldname) {
 	reset_edit(*preset_treeview->get_column(0));
 	return;
     }
-    gx_system::PresetFile& fl = *gx_settings.banks.get_file(get_current_bank());
+    gx_system::PresetFileGui& fl = *machine.get_bank_file(get_current_bank());
     Glib::ustring t = newname;
     int n = 1;
     while (fl.has_entry(newname)) {
@@ -947,18 +922,18 @@ void PresetWindow::on_preset_edited(const Glib::ustring& path, const Glib::ustri
     it->set_value(pstore->col.del_pb, pb_del);
     if (oldname.empty()) {
         // check if current preset is scratch and needs to be saved
-        if (!gx_settings.get_current_bank().empty()) {
-	    gx_system::PresetFile *cpf = gx_settings.banks.get_file(gx_settings.get_current_bank());
-	    if (cpf && cpf->has_entry(gx_settings.get_current_name())) {
+        if (!machine.get_current_bank().empty()) {
+	    gx_system::PresetFileGui *cpf = machine.get_bank_file(machine.get_current_bank());
+	    if (cpf && cpf->has_entry(machine.get_current_name())) {
 	        if (cpf->get_type() == gx_system::PresetFile::PRESET_SCRATCH && cpf->is_mutable()) {
-		    gx_settings.save(*cpf, gx_settings.get_current_name());
+		    machine.pf_save(*cpf, machine.get_current_name());
 		}
 	    }
 	}
 	pstore->append();
-	gx_settings.save(fl, newname);
+	machine.pf_save(fl, newname);
     } else {
-	gx_settings.rename_preset(fl, oldname, newname);
+	machine.rename_preset(fl, oldname, newname);
     }
     reload_target();
     reset_edit(*preset_treeview->get_column(0));
@@ -981,7 +956,7 @@ void PresetWindow::on_cursor_changed() {
 bool PresetWindow::on_preset_drag_motion(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, guint timestamp) {
     if (Gtk::Widget::drag_get_source_widget(context) == preset_treeview) {
 	Gtk::TreeIter it = bank_treeview->get_selection()->get_selected();
-	bool ro = it && (gx_settings.banks.get_file(it->get_value(bank_col.name))->get_flags() & gx_system::PRESET_FLAG_READONLY);
+	bool ro = it && (machine.get_bank_file(it->get_value(bank_col.name))->get_flags() & gx_system::PRESET_FLAG_READONLY);
 	if (!ro) {
 	    preset_treeview->on_drag_motion(context, x, y, timestamp);
 	    Gtk::TreeModel::Path pt;
@@ -1010,7 +985,7 @@ void PresetWindow::on_preset_reordered(const Gtk::TreeModel::Path& path) {
 	    l.push_back(s);
 	}
     }
-    gx_settings.reorder_preset(*gx_settings.banks.get_file(get_current_bank()), l);
+    machine.reorder_preset(*machine.get_bank_file(get_current_bank()), l);
     reload_target();
 }
 
@@ -1039,8 +1014,8 @@ void PresetWindow::on_organize() {
 	sel->set_mode(Gtk::SELECTION_BROWSE);
 	banks_combobox->hide();
 	presets_target_scrolledbox->hide();
-	if (gx_settings.setting_is_preset()) {
-	    if (gx_settings.banks.get_file(gx_settings.get_current_bank())->is_mutable()) {
+	if (machine.setting_is_preset()) {
+	    if (machine.get_bank_file(machine.get_current_bank())->is_mutable()) {
 		actions.save_changes->set_sensitive(true);
 	    }
 	}
@@ -1099,37 +1074,35 @@ void PresetWindow::on_preset_changed() {
     }
     name = it->get_value(pstore->col.name);
     bool is_scratch = false;
-    gx_system::PresetFile *cpf = 0;
-    if (!gx_settings.get_current_bank().empty()) {
-	cpf = gx_settings.banks.get_file(gx_settings.get_current_bank());
-	if (cpf && cpf->has_entry(gx_settings.get_current_name())) {
+    gx_system::PresetFileGui *cpf = 0;
+    if (!machine.get_current_bank().empty()) {
+	cpf = machine.get_bank_file(machine.get_current_bank());
+	if (cpf && cpf->has_entry(machine.get_current_name())) {
 	    is_scratch = (cpf->get_type() == gx_system::PresetFile::PRESET_SCRATCH);
 	}
     }
     if (is_scratch) {
-	if (bank == gx_settings.get_current_bank() && name == gx_settings.get_current_name()) {
-	    gx_settings.save(*cpf, gx_settings.get_current_name());
+	if (bank == machine.get_current_bank() && name == machine.get_current_name()) {
+	    machine.pf_save(*cpf, machine.get_current_name());
 	    // no reload necessary
 	    return;
 	}
     }
     in_current_preset = true;
-    cpf = gx_settings.banks.get_file(bank);
-    reload_on_change_conn.block();
-    gx_settings.load_preset(cpf, name);
-    reload_on_change_conn.unblock();
+    cpf = machine.get_bank_file(bank);
+    machine.load_preset(cpf, name);
     actions.save_changes->set_sensitive(cpf && cpf->is_mutable());
 }
 
 void PresetWindow::on_preset_save() {
-    if (!gx_settings.setting_is_preset()) {
+    if (!machine.setting_is_preset()) {
 	return;
     }
-    gx_system::PresetFile *pf = gx_settings.banks.get_file(gx_settings.get_current_bank());
+    gx_system::PresetFileGui *pf = machine.get_bank_file(machine.get_current_bank());
     if (!pf->is_mutable()) {
 	return;
     }
-    gx_settings.save(*pf, gx_settings.get_current_name());
+    machine.pf_save(*pf, machine.get_current_name());
 }
 
 void PresetWindow::display_paned(bool show_preset, int paned_child_height) {
@@ -1160,7 +1133,7 @@ void PresetWindow::display_paned(bool show_preset, int paned_child_height) {
     }
     Gtk::TreeNodeChildren ch = pstore->children();
     for (it = ch.begin(); it != ch.end(); ++it) {
-	if (it->get_value(pstore->col.name) == gx_settings.get_current_name()) {
+	if (it->get_value(pstore->col.name) == machine.get_current_name()) {
 	    Glib::signal_idle().connect_once(
 		sigc::bind(
 		    sigc::mem_fun1(*preset_treeview, &MyTreeView::scroll_to_row),
@@ -1181,10 +1154,10 @@ void PresetWindow::on_preset_select(bool v, bool animated, int paned_child_heigh
     bool is_mapped = main_vpaned->get_toplevel()->get_mapped();
     bool rack_visible = actions.show_rack->get_active();
     if (v) {
-	if (gx_settings.banks.check_reparse()) {
+	if (machine.bank_check_reparse()) {
 	    set_presets();
 	} else if (!get_current_bank_iter()) {
-	    Glib::ustring bank = gx_settings.get_current_bank();
+	    Glib::ustring bank = machine.get_current_bank();
 	    if (!bank.empty()) {
 		reload_banks(bank);
 	    }
