@@ -23,251 +23,6 @@
 
 #include <guitarix.h>
 
-/****************************************************************
- ** class TunerSwitcher
- */
-
-static const int no_note = 1000;
-static const int bad_note = 1002;
-
-inline bool is_no_note(float n) {
-    return abs(n - no_note) < 1;
-}
-
-TunerSwitcher::TunerSwitcher(Liveplay &lp_)
-    : lp(lp_),
-      switcher_conn(),
-      timeout_conn(),
-      current_note(),
-      state(normal_mode),
-      old_engine_state(),
-      new_engine_state(),
-      old_tuner_active(),
-      new_tuner_active(),
-      last_bank_idx(),
-      last_preset_idx() {
-}
-
-bool TunerSwitcher::display_bank_key(int idx) {
-    last_bank_idx = lp.gx_settings.banks.size() - idx - 1;
-    Glib::ustring bank = lp.gx_settings.banks.get_name(last_bank_idx);
-    if (bank.empty()) {
-	lp.display("--", "--");
-	return false;
-    }
-    lp.display(bank, "");
-    return true;
-}
-
-bool TunerSwitcher::display_preset_key(int idx) {
-    last_preset_idx = idx;
-    Glib::ustring bank = lp.gx_settings.banks.get_name(last_bank_idx);
-    if (bank.empty()) {
-	lp.display("??", gx_system::to_string(idx+1));
-	return false;
-    }
-    gx_system::PresetFile *f = lp.gx_settings.banks.get_file(bank);
-    if (idx >= f->size()) {
-	lp.display(bank, gx_system::to_string(idx+1));
-	return false;
-    }
-    lp.display(bank, f->get_name(idx));
-    return true;
-}
-
-void TunerSwitcher::try_load_preset() {
-    if (state == wait_stop) {
-	switch (last_bank_idx) {
-	case mute_on:    new_engine_state = gx_engine::kEngineOff;    break;
-	case mute_off:   new_engine_state = gx_engine::kEngineOn;     break;
-	case bypass_on:  new_engine_state = gx_engine::kEngineBypass; break;
-	case bypass_off: new_engine_state = gx_engine::kEngineOn;     break;
-	case tuner_on:   new_tuner_active = true;  break;
-	case tuner_off:  new_tuner_active = false; break;
-	default:
-	    Glib::ustring bank = lp.gx_settings.banks.get_name(last_bank_idx);
-	    if (!bank.empty()) {
-		gx_system::PresetFile *f = lp.gx_settings.banks.get_file(bank);
-		if (last_preset_idx < f->size()) {
-		    Glib::ustring preset = f->get_name(last_preset_idx);
-		    if (preset != lp.gx_settings.get_current_name() || bank != lp.gx_settings.get_current_bank()) {
-			lp.gx_settings.load_preset(f, preset);
-			return;
-		    }
-		}
-	    }
-	    break;
-	}
-    }
-    lp.on_selection_changed();
-}
-
-void TunerSwitcher::set_state(SwitcherState newstate) {
-    if (state == newstate) {
-	return;
-    }
-    state = newstate;
-    bool sens = true;
-    Gtk::StateType st = Gtk::STATE_NORMAL;
-    switch (state) {
-    case normal_mode:
-	break;
-    case wait_start:
-	sens = false;
-	st = Gtk::STATE_SELECTED;
-	break;
-    case listening:
-	st = Gtk::STATE_SELECTED;
-	break;
-    case wait_stop:
-	st = Gtk::STATE_PRELIGHT;
-	break;
-    default:
-	assert(false);
-	break;
-    }
-    lp.liveplay_bank->set_sensitive(sens);
-    lp.liveplay_bank->set_state(st);
-    lp.liveplay_preset->set_sensitive(sens);
-    lp.liveplay_preset->set_state(st);
-}
-
-bool TunerSwitcher::on_note_timeout() {
-    if (-24 <= current_note && current_note < -10) {
-	set_state(listening);
-	display_bank_key(current_note - (-24));
-    } else if (current_note >= -10 && current_note <= 7) {
-	if (display_preset_key(current_note - (-10))) {
-	    set_state(wait_stop);
-	} else {
-	    set_state(listening);
-	}
-    } else if (current_note == -25) {
-	if (old_engine_state != gx_engine::kEngineOff) {
-	    lp.display("", _("MUTE"));
-	    last_bank_idx = mute_on;
-	} else {
-	    lp.display("", _("UNMUTE"));
-	    last_bank_idx = mute_off;
-	}
-	set_state(wait_stop);
-    } else if (current_note == -26) {
-	if (old_engine_state != gx_engine::kEngineBypass) {
-	    lp.display("", _("BYPASS"));
-	    last_bank_idx = bypass_on;
-	} else {
-	    lp.display("", _("BYPASS OFF"));
-	    last_bank_idx = bypass_off;
-	}
-	set_state(wait_stop);
-    } else if (current_note < 26) {
-	if (!old_tuner_active) {
-	    lp.display("", _("TUNER ON"));
-	    last_bank_idx = tuner_on;
-	} else {
-	    lp.display("", _("TUNER OFF"));
-	    last_bank_idx = tuner_off;
-	}
-	set_state(wait_stop);
-    }
-    return false;
-}
-
-bool TunerSwitcher::on_state_timeout() {
-    if (state == wait_start) {
-	set_state(listening);
-	current_note = no_note;
-	if (lp.gx_settings.setting_is_preset()) {
-	    last_bank_idx = lp.gx_settings.banks.get_index(lp.gx_settings.get_current_bank());
-	    last_preset_idx = lp.gx_settings.get_current_bank_file()->get_index(lp.gx_settings.get_current_name());
-	} else {
-	    last_bank_idx = last_preset_idx = 0; //FIXME
-	}
-    } else {
-	assert(state == wait_stop);
-	try_load_preset();
-	set_active(false);
-    }
-    return false;
-}
-
-void TunerSwitcher::on_tuner_freq_changed() {
-    const float precision = 0.3;
-    float note = lp.engine.tuner.get_note();
-    if (state == wait_start) {
-	if (is_no_note(note)) {
-	    if (!timeout_conn.connected()) {
-		current_note = no_note;
-		timeout_conn = Glib::signal_timeout().connect(
-		    sigc::mem_fun(this, &TunerSwitcher::on_state_timeout),
-		    40);
-	    }
-	} else {
-	    timeout_conn.disconnect();
-	}
-	return;
-    }
-    if (abs(current_note - note) < precision) {
-	return;
-    }
-    if (state == wait_stop) {
-	if (is_no_note(note)) {
-	    if (!is_no_note(current_note)) {
-		timeout_conn.disconnect();
-	    }
-	    if (!timeout_conn.connected()) {
-		current_note = no_note;
-		timeout_conn = Glib::signal_timeout().connect(
-		    sigc::mem_fun(this, &TunerSwitcher::on_state_timeout),
-		    40);
-	    }
-	    return;
-	}
-    }
-    timeout_conn.disconnect();
-    float n = round(note);
-    if (abs(note - n) < precision) {
-	current_note = n;
-	if (!is_no_note(current_note)) {
-	    timeout_conn = Glib::signal_timeout().connect(
-		sigc::mem_fun(this, &TunerSwitcher::on_note_timeout),
-		40);
-	}
-    } else {
-	current_note = bad_note;
-    }
-}
-
-void TunerSwitcher::set_active(bool v) {
-    if (get_active() == v) {
-	return;
-    }
-    if (v) {
-	bool running = lp.engine.tuner.plugin.on_off;
-	lp.engine.tuner.used_for_switching(true);
-	state = wait_start;
-	new_engine_state = old_engine_state = lp.engine.get_state();
-	lp.engine.set_state(gx_engine::kEngineOff);
-	new_tuner_active = old_tuner_active = lp.actions.livetuner->get_active();
-	lp.actions.livetuner->set_active(false);
-	lp.liveplay_preset->set_sensitive(false);
-	switcher_conn = lp.engine.tuner.signal_freq_changed().connect(
-	    sigc::mem_fun(this, &TunerSwitcher::on_tuner_freq_changed));
-	if (running) {
-	    on_tuner_freq_changed();
-	}
-    } else {
-	switcher_conn.disconnect();
-	timeout_conn.disconnect();
-	lp.engine.tuner.used_for_switching(false);
-	set_state(normal_mode);
-	lp.engine.set_state(new_engine_state);
-	lp.actions.livetuner->set_active(new_tuner_active);
-	lp.on_selection_changed();
-    }
-}
-
-
 /****************************************************************
  ** class KeySwitcher
  */
@@ -305,10 +60,10 @@ bool KeySwitcher::display_selected_bank() {
 
 bool KeySwitcher::display_current() {
     last_bank_key.clear();
-    if (!gx_settings.setting_is_preset()) {
+    if (!machine.setting_is_preset()) {
 	display("----","");
     } else {
-	display(gx_settings.get_current_bank(), gx_settings.get_current_name());
+	display(machine.get_current_bank(), machine.get_current_name());
     }
     return false;
 }
@@ -317,25 +72,25 @@ bool KeySwitcher::process_preset_key(int idx) {
     key_timeout.disconnect();
     Glib::ustring bank = last_bank_key;
     if (bank.empty()) {
-	if (!gx_settings.setting_is_preset()) {
+	if (!machine.setting_is_preset()) {
 	    display_empty("??", gx_system::to_string(idx+1));
 	    return false;
 	}
-	bank = gx_settings.get_current_bank();
+	bank = machine.get_current_bank();
     }
-    gx_system::PresetFile *f = gx_settings.banks.get_file(bank);
+    gx_system::PresetFileGui *f = machine.get_bank_file(bank);
     if (idx >= f->size()) {
 	display_empty(bank, gx_system::to_string(idx+1)+"?");
 	return false;
     } else {
-	gx_settings.load_preset(f, f->get_name(idx));
+	machine.load_preset(f, f->get_name(idx));
 	return true;
     }
 }
 
 bool KeySwitcher::process_bank_key(int idx) {
     key_timeout.disconnect();
-    Glib::ustring bank = gx_settings.banks.get_name(gx_settings.banks.size() - idx - 1);
+    Glib::ustring bank = machine.get_bank_name(machine.bank_size() - idx - 1);
     if (bank.empty()) {
 	display_empty("--", "--");
 	return false;
@@ -386,7 +141,7 @@ bool Liveplay::on_keyboard_preset_select(GtkAccelGroup *accel_group, GObject *ac
 
 bool Liveplay::on_keyboard_toggle_mute(GtkAccelGroup *accel_group, GObject *acceleratable,
 				       guint keyval, GdkModifierType modifier, Liveplay& self) {
-    self.engine.set_state(self.engine.get_state() == gx_engine::kEngineOff ?
+    self.machine.set_state(self.machine.get_state() == gx_engine::kEngineOff ?
 		     gx_engine::kEngineOn
 		     : gx_engine::kEngineOff);
     return true;
@@ -394,7 +149,7 @@ bool Liveplay::on_keyboard_toggle_mute(GtkAccelGroup *accel_group, GObject *acce
 
 bool Liveplay::on_keyboard_toggle_bypass(GtkAccelGroup *accel_group, GObject *acceleratable,
 				       guint keyval, GdkModifierType modifier, Liveplay& self) {
-    self.engine.set_state(self.engine.get_state() == gx_engine::kEngineBypass ?
+    self.machine.set_state(self.machine.get_state() == gx_engine::kEngineBypass ?
 		     gx_engine::kEngineOn
 		     : gx_engine::kEngineBypass);
     return true;
@@ -402,18 +157,34 @@ bool Liveplay::on_keyboard_toggle_bypass(GtkAccelGroup *accel_group, GObject *ac
 
 bool Liveplay::on_keyboard_mode_switch(GtkAccelGroup *accel_group, GObject *acceleratable,
 				       guint keyval, GdkModifierType modifier, Liveplay& self) {
-    self.tuner_switcher.toggle();
+    self.machine.tuner_switcher_toggle(self.actions.livetuner->get_active());
     return true;
 }
 
-void Liveplay::on_switcher_toggled(bool v) {
-    if (false) {
-	tuner_switcher.set_active(v);
-    } else {
-	if (v) {
-	    tuner_switcher.toggle();
-	}
+void Liveplay::set_display_state(TunerSwitcher::SwitcherState newstate) {
+    bool sens = true;
+    Gtk::StateType st = Gtk::STATE_NORMAL;
+    switch (newstate) {
+    case TunerSwitcher::normal_mode:
+	break;
+    case TunerSwitcher::wait_start:
+	sens = false;
+	st = Gtk::STATE_SELECTED;
+	break;
+    case TunerSwitcher::listening:
+	st = Gtk::STATE_SELECTED;
+	break;
+    case TunerSwitcher::wait_stop:
+	st = Gtk::STATE_PRELIGHT;
+	break;
+    default:
+	assert(false);
+	break;
     }
+    liveplay_bank->set_sensitive(sens);
+    liveplay_bank->set_state(st);
+    liveplay_preset->set_sensitive(sens);
+    liveplay_preset->set_state(st);
 }
 
 bool Liveplay::on_keyboard_arrows(GtkAccelGroup *accel_group, GObject *acceleratable,
@@ -466,24 +237,20 @@ bool MyPaintBox::on_expose_event(GdkEventExpose *event) {
 
 
 Liveplay::Liveplay(
-    const gx_system::CmdlineOptions& options, gx_engine::GxEngine& engine_, gx_preset::GxSettings& gx_settings_,
+    const gx_system::CmdlineOptions& options, gx_engine::GxMachineBase& machine_,
     const std::string& fname, const GxActions& actions_)
-    : ui(),
-      bld(),
-      engine(engine_),
-      gx_settings(gx_settings_),
+    : bld(),
+      machine(machine_),
       actions(actions_),
       use_composite(),
       brightness_adj(),
       background_adj(),
-      keyswitch(gx_settings_, sigc::mem_fun(this, &Liveplay::display)),
+      keyswitch(machine_, sigc::mem_fun(this, &Liveplay::display)),
       midi_conn(),
       window(),
-      tuner_switcher(*this),
-      switcher_signal(&ui, &gx_engine::parameter_map["ui.live_play_switcher"].getBool().get_value()), //FIXME
       mouse_hide_conn() {
     const char *id_list[] = {"LivePlay", 0};
-    bld = gx_gui::GxBuilder::create_from_file(fname, &ui, id_list);
+    bld = gx_gui::GxBuilder::create_from_file(fname, &machine, id_list);
     bld->get_toplevel("LivePlay", window);
 
     bld->find_widget("liveplay_bank", liveplay_bank);
@@ -505,13 +272,29 @@ Liveplay::Liveplay(
 	sigc::bind(
 	    sigc::ptr_fun(MyPaintBox::create_from_builder),
 	    background_adj));
-
-    Glib::RefPtr<Gdk::Pixbuf> pb = Gdk::Pixbuf::create_from_file(
-	options.get_style_filepath("bypass.svg"), 300, 150);
-    bypass_image->set(pb);
-    pb = Gdk::Pixbuf::create_from_file(
-	options.get_style_filepath("mute.svg"), 300, 150);
-    mute_image->set(pb);
+    Glib::RefPtr<Gdk::Pixbuf> pb;
+    try {
+      pb = Gdk::Pixbuf::create_from_file(
+	  options.get_style_filepath("bypass.svg"), 300, 150);
+      bypass_image->set(pb);
+    } catch (const Glib::FileError& ex) {
+        gx_print_error("liveplay", ex.what());
+    } catch (const Gdk::PixbufError& ex) {
+        gx_print_error("liveplay", ex.what());
+    } catch(...) {
+        gx_print_error("liveplay", "failed to load pixmap bypass.svg");
+    }
+    try {
+      pb = Gdk::Pixbuf::create_from_file(
+	  options.get_style_filepath("mute.svg"), 300, 150);
+      mute_image->set(pb);
+    } catch (const Glib::FileError& ex) {
+        gx_print_error("liveplay", ex.what());
+    } catch (const Gdk::PixbufError& ex) {
+        gx_print_error("liveplay", ex.what());
+    } catch(...) {
+        gx_print_error("liveplay", "failed to load pixmap mute.svg");
+    }   
     use_composite = window->get_display()->supports_composite();
     if (use_composite) {
 	brightness_adj->signal_value_changed().connect(sigc::mem_fun(this, &Liveplay::on_brightness_changed));
@@ -541,11 +324,6 @@ Liveplay::Liveplay(
     cl = g_cclosure_new(G_CALLBACK(do_action), (gpointer)(actions.live_play->gobj()), 0);
     gtk_accel_group_connect(ag->gobj(), GDK_KEY_Escape, (GdkModifierType)0, (GtkAccelFlags)0, cl);
 
-    actions.group->add(
-	actions.livetuner,
-	sigc::compose(
-	    sigc::mem_fun(this, &Liveplay::display_tuner),
-	    sigc::mem_fun(actions.livetuner.operator->(), &Gtk::ToggleAction::get_active)));
     cl = g_cclosure_new(G_CALLBACK(do_action), (gpointer)(actions.livetuner->gobj()), 0);
     gtk_accel_group_connect(ag->gobj(), GDK_KEY_Return, (GdkModifierType)0, (GtkAccelFlags)0, cl);
 
@@ -582,16 +360,21 @@ Liveplay::Liveplay(
 
     cl = g_cclosure_new(G_CALLBACK(on_keyboard_mode_switch), (gpointer)this, 0);
     gtk_accel_group_connect(ag->gobj(), GDK_KEY_space, (GdkModifierType)0, (GtkAccelFlags)0, cl);
-    switcher_signal.changed.connect(sigc::mem_fun(this, &Liveplay::on_switcher_toggled));
-    
+    machine.tuner_switcher_signal_display().connect(
+	sigc::mem_fun(this, &Liveplay::display));
+    machine.tuner_switcher_signal_set_state().connect(
+	sigc::mem_fun(this, &Liveplay::set_display_state));
+    machine.tuner_switcher_signal_selection_done().connect(
+	sigc::mem_fun(this, &Liveplay::on_selection_done));
+
     window->add_accel_group(ag);
 
-    engine.signal_state_change().connect(
+    machine.signal_state_change().connect(
 	sigc::mem_fun(this, &Liveplay::on_engine_state_change));
-    gx_settings.signal_selection_changed().connect(
+    machine.signal_selection_changed().connect(
 	sigc::mem_fun(this, &Liveplay::on_selection_changed));
 
-    on_engine_state_change(engine.get_state());
+    on_engine_state_change(machine.get_state());
     on_selection_changed();
 }
 
@@ -626,28 +409,39 @@ void Liveplay::on_engine_state_change(gx_engine::GxEngineState state) {
     }
 }
 
-void Liveplay::on_selection_changed() {
+void Liveplay::on_selection_done(bool v) {
     keyswitch.deactivate();
-    if (!gx_settings.setting_is_preset()) {
+    if (!machine.setting_is_preset()) {
 	display("----","");
     } else {
-	display(gx_settings.get_current_bank(), gx_settings.get_current_name());
+	display(machine.get_current_bank(), machine.get_current_name());
+    }
+    actions.livetuner->set_active(v);
+}
+
+void Liveplay::on_selection_changed() {
+    keyswitch.deactivate();
+    if (!machine.setting_is_preset()) {
+	display("----","");
+    } else {
+	display(machine.get_current_bank(), machine.get_current_name());
     }
 }
 
 void Liveplay::on_live_play(Glib::RefPtr<Gtk::ToggleAction> act) {
     if (act->get_active()) {
 	window->fullscreen();
-	midi_conn = gx_engine::controller_map.signal_changed().connect(
+	midi_conn = machine.signal_midi_changed().connect(
 	    sigc::mem_fun(this, &Liveplay::add_midi_elements));
 	add_midi_elements();
 	window->show();
     } else {
 	midi_conn.disconnect();
 	keyswitch.deactivate();
-	tuner_switcher.set_active(false);
+	machine.tuner_switcher_deactivate();
 	window->hide();
     }
+    actions.livetuner->toggled();
 }
 
 bool Liveplay::window_expose_event(GdkEventExpose *event) {
@@ -680,14 +474,7 @@ void Liveplay::on_background_changed() {
 }
 
 void Liveplay::display_tuner(bool v) {
-    //tuner->set_sensitive(v);
-    Gtk::Widget *p = tuner->get_parent();
-    if (!v) {
-	Gtk::Requisition r;
-	p->size_request(r);
-	p->set_size_request(r.width, r.height);
-	p->queue_draw(); // bug in GxRackTuner? sometimes still visible when tuner was active
-    }
+    tuner->set_sensitive(v);
     tuner->set_visible(v);
 }
 
@@ -695,46 +482,37 @@ void Liveplay::on_realize() {
     liveplay_canvas->get_window()->set_composited(true);
 }
 
-class MidiControllerDisplay: public Gtk::ProgressBar, public gx_ui::GxUiItem {
+class MidiControllerDisplay: public Gtk::ProgressBar {
 private:
-    gx_ui::GxUI& ui;
-    unsigned int ctr;
-    const gx_engine::midi_controller_list& controller;
+    int ctr;
 private:
-    virtual void reflectZone();
-    virtual bool hasChanged();
+    void midi_value_changed(int ctr, int val);
 public:
-    MidiControllerDisplay(gx_ui::GxUI& ui, unsigned int n, const gx_engine::midi_controller_list& ctrl, const Glib::ustring& name);
+    MidiControllerDisplay(gx_engine::GxMachineBase& machine, unsigned int n, const gx_engine::midi_controller_list& ctrl, const Glib::ustring& name);
     ~MidiControllerDisplay();
 };
 
-MidiControllerDisplay::MidiControllerDisplay(gx_ui::GxUI& ui_, unsigned int n, const gx_engine::midi_controller_list& ctrl, const Glib::ustring& name)
-    : Gtk::ProgressBar(), gx_ui::GxUiItem(), ui(ui_), ctr(n), controller(ctrl) {
-    ui.registerZone(gx_engine::MidiControllerList::get_midi_control_zone(ctr), this);
+MidiControllerDisplay::MidiControllerDisplay(gx_engine::GxMachineBase& machine, unsigned int n, const gx_engine::midi_controller_list& ctrl, const Glib::ustring& name)
+    : Gtk::ProgressBar(), ctr(n) {
+    machine.signal_midi_value_changed().connect(
+	sigc::mem_fun(this, &MidiControllerDisplay::midi_value_changed));
     set_size_request(300, 50);
     set_text(name);
-    reflectZone();
+    machine.request_midi_value_update();
 }
 
 MidiControllerDisplay::~MidiControllerDisplay() {
-    ui.unregisterZone(gx_engine::MidiControllerList::get_midi_control_zone(ctr), this);
 }
 
-bool MidiControllerDisplay::hasChanged() {
-    int v = gx_engine::MidiControllerList::get_last_midi_control_value(ctr);
-    if (v >= 0) {
-	return get_fraction() != v / 127.0;
+void MidiControllerDisplay::midi_value_changed(int c, int v) {
+    if (c != ctr) {
+	return;
     }
-    if (get_sensitive()) {
-	set_sensitive(false);
-    }
-    return false;
-}
-
-void MidiControllerDisplay::reflectZone() {
-    int v = gx_engine::MidiControllerList::get_last_midi_control_value(ctr);
     if (v >= 0) {
+	set_sensitive(true);
 	set_fraction(v / 127.0);
+    } else {
+	set_sensitive(false);
     }
 }
 
@@ -745,8 +523,8 @@ void Liveplay::add_midi_elements() {
     int left_max = 3;
     Gtk::Table::TableList& tl = midictrl_table->children();
     tl.erase(tl.begin(), tl.end());
-    for (int i = 0; i < gx_engine::controller_map.size(); i++) {
-        gx_engine::midi_controller_list& cl = gx_engine::controller_map[i];
+    for (int i = 0; i < machine.midi_size(); i++) {
+        gx_engine::midi_controller_list& cl = machine.midi_get(i);
 	if (cl.empty()) {
 	    continue;
 	}
@@ -759,7 +537,7 @@ void Liveplay::add_midi_elements() {
 	    v += Glib::ustring::compose(" %1/%2", p.l_group(), p.l_name());
 	}
 	midictrl_table->attach(
-	    *manage(new MidiControllerDisplay(ui, i, cl, v)),
+	    *manage(new MidiControllerDisplay(machine, i, cl, v)),
 	    left, left+1, top, top+1, Gtk::AttachOptions(0));
 	top += 1;
 	if (top >= top_max) {
