@@ -41,71 +41,56 @@
  **
  */
 
-PluginUI::PluginUI(MainWindow& main_, const gx_engine::PluginList& pl, const char *name,
-		   const Glib::ustring& fname_, const Glib::ustring& tooltip_)
-    : GxUiItem(), merge_id(0), action(), plugin(pl.lookup_plugin(name)), fname(fname_),
-      tooltip(tooltip_), shortname(), icon(), group(), toolitem(), main(main_), rackbox(),
-      hidden(false), compressed(false) {
-    gx_ui::GxUI& ui = main.get_ui();
-    ui.registerZone(&plugin->box_visible, this);
-    ui.registerZone(&plugin->position, this);
-    ui.registerZone(&plugin->effect_post_pre, this);
-    plugin->pdef->flags |= gx_engine::PGNI_UI_REG;
+PluginUI::PluginUI(MainWindow& main_, const char *name,
+		   const Glib::ustring& tooltip_)
+    : merge_id(0),
+      action(),
+      plugin(main_.get_machine().pluginlist_lookup_plugin(name)),
+      tooltip(tooltip_),
+      shortname(),
+      icon(),
+      group(),
+      toolitem(),
+      main(main_),
+      rackbox(),
+      hidden(false) {
+    if (plugin->get_pdef()->description && tooltip.empty()) {
+	tooltip = plugin->get_pdef()->description;
+    }
+    plugin->get_pdef()->flags |= gx_engine::PGNI_UI_REG;
 }
 
 PluginUI::~PluginUI() {
-    plugin->pdef->flags &= ~gx_engine::PGNI_UI_REG;
-    gx_ui::GxUI& ui = main.get_ui();
-    ui.unregisterZone(&plugin->box_visible, this);
-    ui.unregisterZone(&plugin->position, this);
-    ui.unregisterZone(&plugin->effect_post_pre, this);
+    delete rackbox;
+    if (toolitem) {
+	if (group) {
+	    group->remove(*toolitem);
+	}
+	delete toolitem;
+    }
+    plugin->get_pdef()->flags &= ~gx_engine::PGNI_UI_REG;
+}
+
+void PluginUI::unset_ui_merge_id(Glib::RefPtr<Gtk::UIManager> uimanager) {
+    if (merge_id) {
+	uimanager->remove_ui(merge_id);
+	merge_id = 0;
+    }
 }
 
 void PluginUI::on_plugin_preset_popup() {
-    main.plugin_preset_popup(get_id());
+    main.plugin_preset_popup(plugin->get_pdef());
 }
 
-bool PluginUI::is_registered(gx_engine::PluginList& pl, const char *name) {
-    return pl.lookup_plugin(name)->pdef->flags & gx_engine::PGNI_UI_REG;
+bool PluginUI::is_registered(gx_engine::GxMachineBase& m, const char *name) {
+    return m.pluginlist_lookup_plugin(name)->get_pdef()->flags & gx_engine::PGNI_UI_REG;
 }
 
-bool PluginUI::hasChanged() {
-    if (plugin->on_off) {
-	// when loading from old preset files make sure plugins
-	// are visible if they are switched on
-	plugin->box_visible = true;
-    }
-    if (plugin->box_visible != is_displayed()) {
-	return true;
-    }
-    if (!plugin->box_visible) {
-	return false;
-    }
-    if (plugin->plug_visible != rackbox->get_plug_visible()) {
-	return true;
-    }
-    if (main.is_loading()) {
-	return false;
-    }
-    return rackbox->hasOrderDiff();
-}
-
-void PluginUI::reflectZone() {
-    if (!hasChanged()) {
-	return;
-    }
-    if (plugin->box_visible != is_displayed()) {
-	if (action) {
-	    action->set_active(plugin->box_visible);
-	}
-	display(plugin->box_visible, false);
-    }
-    if (plugin->box_visible) {
-	if (plugin->plug_visible != rackbox->get_plug_visible()) {
-	    rackbox->swtch(plugin->plug_visible);
-	}
-	if (!main.is_loading() && rackbox->hasOrderDiff()) {
-	    rackbox->get_parent()->check_order();
+void PluginUI::compress(bool state) {
+    plugin->set_plug_visible(state);
+    if (rackbox) {
+	if (rackbox->can_compress()) {
+	    rackbox->swtch(state);
 	}
     }
 }
@@ -117,8 +102,8 @@ void PluginUI::set_action(Glib::RefPtr<Gtk::ToggleAction>& act)
 }
 
 void PluginUI::on_action_toggled() {
-    if (plugin->box_visible != is_displayed()) {
-	return; // call triggered by reflectZone
+    if (rackbox && action->get_active() == rackbox->get_box_visible()) {
+	return;
     }
     if (action->get_active()) {
 	display_new();
@@ -127,162 +112,116 @@ void PluginUI::on_action_toggled() {
     }
 }
 
+void PluginUI::hide(bool animate) {
+    plugin->set_on_off(false);
+    if (rackbox) {
+	rackbox->display(false, animate);
+	main.add_icon(get_id());
+    }
+}
+
+void PluginUI::show(bool animate) {
+    if (!rackbox) {
+	rackbox = main.add_rackbox(*this, plugin->get_plug_visible(), -1, animate);
+	set_active(true);
+    } else {
+	rackbox->display(true, animate);
+    }
+    if (hidden) {
+	rackbox->hide();
+    }
+    main.hide_effect(get_id());
+}
+
 void PluginUI::display(bool v, bool animate) {
     // this function hides the rackbox. It could also destroy it (or
     // some other function could do it, e.g. when unloading a module),
     // but currently there are too many memory leaks in the stack based
     // builder.
-    plugin->box_visible = v;
+    plugin->set_box_visible(v);
     if (v) {
-	if (!rackbox) {
-	    rackbox = main.add_rackbox(*this, compressed, -1, animate);
-	} else {
-	    rackbox->display(true, animate);
-	}
-	if (hidden) {
-	    rackbox->hide();
-	}
-	main.hide_effect(get_id());
+	main.get_machine().insert_rack_unit(get_id(), "", get_type());
+	show(animate);
     } else {
-	if (rackbox) {
-	    rackbox->display(false, animate);
-	    main.add_icon(get_id());
-	}
+	main.get_machine().remove_rack_unit(get_id(), get_type());
+	hide(animate);
     }
 }
 
 void PluginUI::display_new(bool unordered) {
-    plugin->plug_visible = false;
+    plugin->set_plug_visible(false);
+    if (rackbox) {
+	rackbox->swtch(false);
+    }
     display(true, true);
     if (!unordered) {
 	rackbox->get_parent()->reorder(get_id(), -1);
     }
 }
 
+void PluginUI::update_rackbox() {
+    if (!rackbox) {
+	return;
+    }
+    if (plugin->get_box_visible()) {
+	RackContainer::rackbox_list l = rackbox->get_parent()->get_children();
+	int n = 0;
+	for (RackContainer::rackbox_list::iterator i = l.begin(); i != l.end(); ++i, ++n)
+	    if (*i == rackbox) {
+		break;
+	    }
+	delete rackbox;
+	rackbox = main.add_rackbox(*this, plugin->get_plug_visible(), n, false);
+    } else {
+	delete rackbox;
+	rackbox = 0;
+    }
+}
+
+bool plugins_by_name_less(PluginUI *a, PluginUI *b) {
+    int res = a->get_type() - b->get_type();
+    if (res == 0) {
+	gchar *an = g_utf8_casefold(a->get_shortname(), 1);
+	gchar *bn = g_utf8_casefold(b->get_shortname(), 1);
+	res = g_utf8_collate(an, bn);
+	g_free(an);
+	g_free(bn);
+    }
+    return res < 0;
+}
+
+
 /****************************************************************
  ** class PluginDict
  */
 
-PluginDict::~PluginDict() {
+void PluginDict::add(PluginUI *p) {
+    insert(pair<std::string, PluginUI*>(p->get_id(), p));
+}
+
+void PluginDict::remove(PluginUI *p) {
+    std::map<std::string, PluginUI*>::iterator i = find(p->get_id());
+    assert(i != end());
+    erase(i);
+}
+
+void PluginDict::cleanup() {
     for (std::map<std::string, PluginUI*>::iterator i = begin(); i != end(); ++i) {
 	delete i->second;
     }
+    clear();
 }
 
-
-/****************************************************************
- ** class StackBoxBuilderNew
- */
-
-StackBoxBuilderNew::StackBoxBuilderNew(
-    gx_engine::GxEngine& engine_, gx_preset::GxSettings& gx_settings_,
-    Gxw::WaveView &fWaveView_, Gtk::Label &convolver_filename_label_, gx_ui::GxUI& ui_,
-    Glib::RefPtr<Gdk::Pixbuf> window_icon)
-    : StackBoxBuilder(fTop, fBox, engine_, gx_settings_, fMode, fWaveView_,
-		      convolver_filename_label_, ui_, window_icon),
-      engine(engine_), gx_settings(gx_settings_), fWaveView(fWaveView_),
-      convolver_filename_label(convolver_filename_label_), widget() {
+PluginDict::~PluginDict() {
+    cleanup();
 }
 
-void StackBoxBuilderNew::openMonoRackBox(const char* label, int* posit, const char *id_on_off, const char *id_pre_post, const char *id_dialog) {
+void PluginDict::compress(bool state) {
+    for (std::map<std::string, PluginUI*>::iterator i = begin(); i != end(); ++i) {
+       i->second->compress(state);
+   }
 }
 
-void StackBoxBuilderNew::closeMonoRackBox() {
-}
-
-void StackBoxBuilderNew::openStereoRackBox(const char* label, int* posit, const char *id_on_off, const char *id_dialog) {
-}
-
-void StackBoxBuilderNew::closeStereoRackBox() {
-}
-
-void StackBoxBuilderNew::openVerticalMidiBox(const char* label) {
-    openVerticalBox(label);
-}
-
-void StackBoxBuilderNew::prepare() {
-    fTop = 0;
-    widget = new Gtk::HBox();
-    widget->show();
-    fBox[fTop] = GTK_WIDGET(widget->gobj());
-    fMode[fTop] = kBoxMode;
-}
-
-#ifndef NDEBUG
-void wnotify(gpointer data, GObject *where_the_object_was) {
-    printf("WN %p %p\n", where_the_object_was, data);
-}
-
-// check if object will be finalized
-void trace_finalize(Glib::Object *o, int n) {
-    g_object_weak_ref(o->gobj(), wnotify, (gpointer)n);
-}
-#endif
-
-void StackBoxBuilderNew::fetch(Gtk::Widget*& mainbox, Gtk::Widget*& minibox) {
-    assert(fTop == 0);
-    mainbox = widget;
-    Glib::ListHandle<Gtk::Widget*> l = widget->get_children();
-    if (l.size() == 2) {
-	Glib::ListHandle<Gtk::Widget*>::iterator i = l.begin();
-	minibox = new Gtk::VBox();
-	minibox->show();
-	(*i)->show();
-	(*i)->reference(); //FIXME can't unmanage widget, reparent unrefs
-	(*i)->reparent(*minibox);
-    } else {
-	minibox = 0;
-    }
-}
-
-void StackBoxBuilderNew::get_box(const std::string& name, Gtk::Widget*& mainbox, Gtk::Widget*& minibox) {
-    struct {
-	const char *name;
-	void (StackBoxBuilder::*func)();
-    } mapping[] = {
-	// mono
-	//{ "ampdetail", &StackBoxBuilder::make_rackbox_ampdetail },
-	{ "overdrive", &StackBoxBuilder::make_rackbox_overdrive },
-	{ "echo", &StackBoxBuilder::make_rackbox_echo },
-	{ "delay", &StackBoxBuilder::make_rackbox_delay },
-	{ "freeverb", &StackBoxBuilder::make_rackbox_freeverb },
-	{ "oscilloscope", &StackBoxBuilder::make_rackbox_oscilloscope },
-	{ "low_highpass", &StackBoxBuilder::make_rackbox_low_highpass },
-	{ "eqs", &StackBoxBuilder::make_rackbox_eqs },
-	//{ "crybaby", &StackBoxBuilder::make_rackbox_crybaby },
-	//{ "gx_distortion", &StackBoxBuilder::make_rackbox_gx_distortion },
-	{ "IR", &StackBoxBuilder::make_rackbox_IR },
-	{ "compressor", &StackBoxBuilder::make_rackbox_compressor },
-	{ "biquad", &StackBoxBuilder::make_rackbox_biquad },
-	//{ "tremolo", &StackBoxBuilder::make_rackbox_tremolo },
-	{ "phaser_mono", &StackBoxBuilder::make_rackbox_phaser_mono },
-	{ "chorus_mono", &StackBoxBuilder::make_rackbox_chorus_mono },
-	{ "flanger_mono", &StackBoxBuilder::make_rackbox_flanger_mono },
-	{ "feedback", &StackBoxBuilder::make_rackbox_feedback },
-	//{ "amp.tonestack", &StackBoxBuilder::make_rackbox_amp_tonestack },
-	{ "cab", &StackBoxBuilder::make_rackbox_cab },
-	{ "midi_out", &StackBoxBuilder::make_rackbox_midi_out },
-	// stereo
-	{ "chorus", &StackBoxBuilder::make_rackbox_chorus },
-	{ "flanger", &StackBoxBuilder::make_rackbox_flanger },
-	{ "phaser", &StackBoxBuilder::make_rackbox_phaser },
-	{ "stereodelay", &StackBoxBuilder::make_rackbox_stereodelay },
-	{ "stereoecho", &StackBoxBuilder::make_rackbox_stereoecho },
-	{ "moog", &StackBoxBuilder::make_rackbox_moog },
-	{ "ampmodul", &StackBoxBuilder::make_rackbox_ampmodul },
-	{ "tonemodul", &StackBoxBuilder::make_rackbox_tonemodul },
-	{ "jconv", &StackBoxBuilder::make_rackbox_jconv },
-	{ "stereoverb", &StackBoxBuilder::make_rackbox_stereoverb }
-    };
-    mainbox = minibox = 0;
-    for (unsigned int i = 0; i < sizeof(mapping) / sizeof(mapping[0]); ++i) {
-	if (name == mapping[i].name) {
-	    prepare();
-	    (this->*mapping[i].func)();
-	    fetch(mainbox, minibox);
-	}
-    }
-}
 
 /****************************************************************
  ** class DragIcon
@@ -441,9 +380,9 @@ MiniRackBox::MiniRackBox(RackBox& rb, gx_system::CmdlineOptions& options)
       mb_delete_button(),
       preset_button(),
       on_off_switch("minitoggle"),
-      toggle_on_off(rb.main.get_ui(), &on_off_switch, &rb.plugin.plugin->on_off) {
+      toggle_on_off(rb.main.get_machine(), &on_off_switch, rb.plugin.plugin->id_on_off()) {
     if (strcmp(rb.plugin.get_id(), "ampstack") != 0) { // FIXME
-	gx_gui::connect_midi_controller(GTK_WIDGET(on_off_switch.gobj()), &rb.plugin.plugin->on_off);
+	gx_gui::connect_midi_controller(&on_off_switch, rb.plugin.plugin->id_on_off().c_str(), rb.main.get_machine());
     }
     if (!szg_label) {
 	szg_label = Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
@@ -480,7 +419,7 @@ MiniRackBox::MiniRackBox(RackBox& rb, gx_system::CmdlineOptions& options)
     al->set_padding(1, 0, 4, 4);
     pack_end(*manage(al), Gtk::PACK_SHRINK);
     box->pack_end(*manage(rb.wrap_bar(8)), Gtk::PACK_SHRINK);
-    if (!(rb.plugin.plugin->pdef->flags & PGN_NO_PRESETS)) {
+    if (!(rb.plugin.plugin->get_pdef()->flags & PGN_NO_PRESETS)) {
 	preset_button = rb.make_preset_button();
 	box->pack_end(*manage(preset_button), Gtk::PACK_SHRINK);
     }
@@ -497,16 +436,18 @@ void MiniRackBox::set_config_mode(bool mode) {
     evbox.set_above_child(mode);
     if (mode) {
 	mconbox.hide();
-    if (preset_button)
-	preset_button->hide();
+	if (preset_button) {
+	    preset_button->hide();
+	}
 	mb_expand_button->hide();
 	if (mb_delete_button) {
 	    mb_delete_button->show();
 	}
     } else {
 	mconbox.show();
-    if (preset_button)
-	preset_button->show();
+	if (preset_button) {
+	    preset_button->show();
+	}
 	mb_expand_button->show();
 	if (mb_delete_button) {
 	    mb_delete_button->hide();
@@ -611,34 +552,39 @@ public:
 class PluginPresetListWindow: public Gtk::Window {
 private:
     Glib::RefPtr<TextListStore> textliststore;
-    Glib::RefPtr<gx_preset::PluginPresetList> presetlist;
+    PluginPresetPopup& presetlist;
     //
     Gtk::TreeView *treeview;
     Gtk::Button *removebutton;
+    using Gtk::Window::on_remove;
     void on_remove();
     void on_selection_changed();
     virtual bool on_key_press_event(GdkEventKey *event);
-    static PluginPresetListWindow* create_from_builder(BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, Glib::RefPtr<gx_preset::PluginPresetList> l);
-    PluginPresetListWindow(BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, Glib::RefPtr<gx_preset::PluginPresetList> l);
+    static PluginPresetListWindow* create_from_builder(
+	BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, PluginPresetPopup& p);
+    PluginPresetListWindow(BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, PluginPresetPopup& p);
 public:
     ~PluginPresetListWindow();
-    static PluginPresetListWindow *create(const gx_system::CmdlineOptions& options, Glib::RefPtr<gx_preset::PluginPresetList> l);
+    static PluginPresetListWindow *create(const gx_system::CmdlineOptions& options, PluginPresetPopup& p);
     void run();
 };
 
-PluginPresetListWindow *PluginPresetListWindow::create_from_builder(BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, Glib::RefPtr<gx_preset::PluginPresetList> l) {
-    return new PluginPresetListWindow(cobject, bld, l);
+PluginPresetListWindow *PluginPresetListWindow::create_from_builder(
+    BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, PluginPresetPopup& p) {
+    return new PluginPresetListWindow(cobject, bld, p);
 }
 
 PluginPresetListWindow::~PluginPresetListWindow() {
 }
 
-PluginPresetListWindow *PluginPresetListWindow::create(const gx_system::CmdlineOptions& options, Glib::RefPtr<gx_preset::PluginPresetList> l) {
-    Glib::RefPtr<gx_gui::GxBuilder> bld = gx_gui::GxBuilder::create_from_file(options.get_builder_filepath("pluginpreset_listwindow.glade"));
+PluginPresetListWindow *PluginPresetListWindow::create(
+    const gx_system::CmdlineOptions& options, PluginPresetPopup& p) {
+    Glib::RefPtr<gx_gui::GxBuilder> bld = gx_gui::GxBuilder::create_from_file(
+	options.get_builder_filepath("pluginpreset_listwindow.glade"));
     PluginPresetListWindow *w;
     bld->get_toplevel_derived(
 	"PluginPresetListWindow", w,
-	sigc::bind(sigc::ptr_fun(PluginPresetListWindow::create_from_builder), bld, l));
+	sigc::bind(sigc::ptr_fun(PluginPresetListWindow::create_from_builder), bld, sigc::ref(p)));
     return w;
 }
 
@@ -653,30 +599,34 @@ bool PluginPresetListWindow::on_key_press_event(GdkEventKey *event) {
 void PluginPresetListWindow::on_remove() {
     Gtk::TreeIter it = treeview->get_selection()->get_selected();
     if (it) {
-	presetlist->remove(it->get_value(textliststore->col.name));
+	presetlist.get_machine().plugin_preset_list_remove(
+	    presetlist.get_pdef(), it->get_value(textliststore->col.name));
 	textliststore->erase(it);
     }
 }
 
-PluginPresetListWindow::PluginPresetListWindow(BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, Glib::RefPtr<gx_preset::PluginPresetList> l)
-    : Gtk::Window(cobject), textliststore(TextListStore::create()), presetlist(l) {
+PluginPresetListWindow::PluginPresetListWindow(
+    BaseObjectType* cobject, Glib::RefPtr<gx_gui::GxBuilder> bld, PluginPresetPopup& p)
+    : Gtk::Window(cobject),
+      textliststore(TextListStore::create()),
+      presetlist(p) {
     Gtk::Button *b;
     bld->find_widget("closebutton", b);
     b->signal_clicked().connect(
 	sigc::mem_fun(*this, &PluginPresetListWindow::hide));
     bld->find_widget("removebutton", removebutton);
     removebutton->signal_clicked().connect(
-	sigc::mem_fun(*this, &PluginPresetListWindow::on_remove));
+	sigc::mem_fun0(*this, &PluginPresetListWindow::on_remove));
     bld->find_widget("treeview", treeview);
-    if (l->start()) {
-	Glib::ustring name;
-	while (l->next(name)) {
-	    textliststore->append()->set_value(textliststore->col.name, name);
+    for (gx_preset::UnitPresetList::const_iterator i = presetlist.begin(); i != presetlist.end(); ++i) {
+	if (i->name.empty()) {
+	    break;
 	}
+	textliststore->append()->set_value(textliststore->col.name, i->name);
     }
     treeview->set_model(textliststore);
+    removebutton->set_sensitive(false);
     Glib::RefPtr<Gtk::TreeSelection> sel = treeview->get_selection();
-    sel->select(textliststore->children().begin());
     sel->signal_changed().connect(
 	sigc::mem_fun(*this, &PluginPresetListWindow::on_selection_changed));
 }
@@ -693,44 +643,51 @@ void PluginPresetListWindow::run() {
 ** PluginPresetPopup
 */
 
-void PluginPresetPopup::set_plugin_preset(Glib::RefPtr<gx_preset::PluginPresetList> l, Glib::ustring name) {
-    l->set(name);
+void PluginPresetPopup::set_plugin_preset(bool factory, const Glib::ustring& name) {
+    machine.plugin_preset_list_set(pdef, factory, name);
 }
 
 void PluginPresetPopup::set_plugin_std_preset() {
-    gx_settings.get_param().reset_unit(id);
+    machine.reset_unit(pdef);
 }
 
-void PluginPresetPopup::save_plugin_preset(Glib::RefPtr<gx_preset::PluginPresetList> l) {
-    InputWindow *w = InputWindow::create(gx_settings.get_options(), save_name_default);
+void PluginPresetPopup::save_plugin_preset() {
+    InputWindow *w = InputWindow::create(machine.get_options(), save_name_default);
     w->run();
     if (!w->get_name().empty()) {
-	l->save(w->get_name(), id);
+	machine.plugin_preset_list_save(pdef, w->get_name());
     }
     delete w;
 }
 
-void PluginPresetPopup::remove_plugin_preset(Glib::RefPtr<gx_preset::PluginPresetList> l) {
-    PluginPresetListWindow *w = PluginPresetListWindow::create(gx_settings.get_options(), l);
+void PluginPresetPopup::remove_plugin_preset() {
+    PluginPresetListWindow *w = PluginPresetListWindow::create(machine.get_options(), *this);
     w->run();
     delete w;
 }
 
-bool PluginPresetPopup::add_plugin_preset_list(Glib::RefPtr<gx_preset::PluginPresetList> l) {
-    if (!l->start()) {
-	return false;
-    }
+bool PluginPresetPopup::add_plugin_preset_list(bool *found) {
+    *found = false;
     bool found_presets = false;
-    Glib::ustring name;
-    bool is_set;
-    while (l->next(name, &is_set)) {
-	found_presets = true;
-	Gtk::CheckMenuItem *c = new Gtk::CheckMenuItem(name);
-	if (is_set) {
+    bool factory = false;
+    for (gx_preset::UnitPresetList::iterator i = presetnames.begin(); i != presetnames.end(); ++i) {
+	if (i->name.empty()) {
+	    factory = true;
+	    if (found_presets) {
+		append(*manage(new Gtk::SeparatorMenuItem()));
+		*found = true;
+		found_presets = false;
+	    }
+	    continue;
+	} else {
+	    found_presets = true;
+	}
+	Gtk::CheckMenuItem *c = new Gtk::CheckMenuItem(i->name);
+	if (i->is_set) {
 	    c->set_active(true);
 	}
 	c->signal_activate().connect(
-	    sigc::bind(sigc::ptr_fun(set_plugin_preset), l, name));
+	    sigc::bind(sigc::mem_fun(this, &PluginPresetPopup::set_plugin_preset), factory, i->name));
 	append(*manage(c));
     }
     return found_presets;
@@ -748,17 +705,18 @@ void PluginPresetPopup::on_selection_done() {
 	    this));
 }
 
-PluginPresetPopup::PluginPresetPopup(const std::string& id_, const gx_preset::GxSettings& gx_settings_,
+PluginPresetPopup::PluginPresetPopup(const PluginDef *pdef_, gx_engine::GxMachineBase& machine_,
 				     const Glib::ustring& save_name_default_)
-  : Gtk::Menu(), id(id_), gx_settings(gx_settings_), save_name_default(save_name_default_) {
-    Glib::RefPtr<gx_preset::PluginPresetList> l = gx_settings.load_plugin_preset_list(id, false);
-    bool found_presets = add_plugin_preset_list(l);
-    if (found_presets) {
-	append(*manage(new Gtk::SeparatorMenuItem()));
-    }
-    if (!add_plugin_preset_list(gx_settings.load_plugin_preset_list(id, true))) {
+  : Gtk::Menu(),
+    pdef(pdef_),
+    machine(machine_),
+    save_name_default(save_name_default_),
+    presetnames() {
+    machine.plugin_preset_list_load(pdef, presetnames);
+    bool found_presets;
+    if (!add_plugin_preset_list(&found_presets)) {
 	Gtk::CheckMenuItem *c = new Gtk::CheckMenuItem(_("standard"));
-	if (gx_settings.get_param().unit_has_std_values(id)) {
+	if (machine.parameter_unit_has_std_values(pdef)) {
 	    c->set_active(true);
 	}
 	c->signal_activate().connect(
@@ -769,12 +727,12 @@ PluginPresetPopup::PluginPresetPopup(const std::string& id_, const gx_preset::Gx
     Gtk::MenuItem *mi = new Gtk::MenuItem(_("save..."));
     append(*manage(mi));
     mi->signal_activate().connect(
-	sigc::bind(sigc::mem_fun(this, &PluginPresetPopup::save_plugin_preset),l));
+	sigc::mem_fun(this, &PluginPresetPopup::save_plugin_preset));
     if (found_presets) {
 	mi = new Gtk::MenuItem(_("remove..."));
 	append(*manage(mi));
 	mi->signal_activate().connect(
-	    sigc::bind(sigc::mem_fun(this, &PluginPresetPopup::remove_plugin_preset),l));
+	    sigc::mem_fun(this, &PluginPresetPopup::remove_plugin_preset));
     }
     show_all();
     popup(1, gtk_get_current_event_time());
@@ -796,15 +754,7 @@ void RackBox::set_paintbox(Gxw::PaintBox& pb, PluginType tp) {
 }
 
 Gtk::Widget *RackBox::make_label(const PluginUI& plugin, gx_system::CmdlineOptions& options, bool useshort) {
-    const char *effect_name;
-    if (useshort) {
-	effect_name = plugin.shortname;
-	if (!effect_name) {
-	    effect_name = plugin.get_name();
-	}
-    } else {
-	effect_name = plugin.get_name();
-    }
+    const char *effect_name = useshort ? plugin.get_shortname() : plugin.get_name();
     Gtk::Label *effect_label = new Gtk::Label(effect_name);
     effect_label->set_alignment(0, 0.5);
     effect_label->set_name("rack_effect_label");
@@ -883,7 +833,7 @@ Gtk::Widget *RackBox::create_drag_widget(const PluginUI& plugin, gx_system::Cmdl
 	pb->property_paint_func().set_value("zac_expose");
     }
     Gxw::Switch *swtch = new Gxw::Switch("minitoggle");
-    swtch->set_active(plugin.plugin->on_off);
+    swtch->set_active(plugin.plugin->get_on_off());
 #ifdef USE_SZG
     RackBox::szg->add_widget(*swtch);
 #else
@@ -904,10 +854,9 @@ Gtk::Widget *RackBox::create_drag_widget(const PluginUI& plugin, gx_system::Cmdl
 }
 
 void RackBox::display(bool v, bool animate) {
-    if (box_visible == v) {
-	return;
-    }
+    assert(box_visible != v);
     box_visible = v;
+    plugin.set_active(v);
     if (v) {
 	if (animate) {
 	    animate_insert();
@@ -916,7 +865,6 @@ void RackBox::display(bool v, bool animate) {
 	}
 	get_parent()->increment();
     } else {
-	plugin.plugin->on_off = false;
 	if (animate) {
 	    animate_remove();
 	} else {
@@ -930,10 +878,10 @@ RackBox::RackBox(PluginUI& plugin_, MainWindow& tl, Gtk::Widget* bare)
     : Gtk::VBox(), plugin(plugin_), main(tl), config_mode(false), anim_tag(),
       compress(true), delete_button(true), mbox(Gtk::ORIENTATION_HORIZONTAL), minibox(0),
       fbox(0), target(), anim_height(0), anim_step(), drag_icon(), target_height(0),
-      box(Gtk::ORIENTATION_HORIZONTAL, 2), box_visible(true), position(), effect_post_pre(), on_off_switch("switchit"),
-      toggle_on_off(tl.get_ui(), &on_off_switch, &plugin.plugin->on_off) {
+      box(Gtk::ORIENTATION_HORIZONTAL, 2), box_visible(true), on_off_switch("switchit"),
+      toggle_on_off(tl.get_machine(), &on_off_switch, plugin.plugin->id_on_off()) {
     if (strcmp(plugin.get_id(), "ampstack") != 0) { // FIXME
-	gx_gui::connect_midi_controller(GTK_WIDGET(on_off_switch.gobj()), &plugin.plugin->on_off);
+	gx_gui::connect_midi_controller(&on_off_switch, plugin.plugin->id_on_off().c_str(), main.get_machine());
     }
 #ifdef USE_SZG
     if (!szg) {
@@ -1071,7 +1019,7 @@ void RackBox::on_my_drag_end(const Glib::RefPtr<Gdk::DragContext>& context) {
 	delete drag_icon;
 	drag_icon = 0;
     }
-    if (box_visible) {
+    if (plugin.plugin->get_box_visible()) {
 	animate_insert();
     }
 }
@@ -1094,8 +1042,7 @@ void RackBox::set_visibility(bool v) {
 }
 
 void RackBox::swtch(bool mini) {
-    plugin.plugin->plug_visible = mini;
-    plugin.compressed = mini;
+    plugin.plugin->set_plug_visible(mini);
     if (!config_mode) {
 	if (mini) {
 	    vis_switch(*fbox, mbox);
@@ -1108,7 +1055,7 @@ void RackBox::swtch(bool mini) {
 void RackBox::set_config_mode(bool mode) {
     config_mode = mode;
     minibox->set_config_mode(mode);
-    if (!get_plug_visible() || !can_compress()) {
+    if (!can_compress() || !get_plug_visible()) {
 	if (mode) {
 	    vis_switch(*fbox, mbox);
 	} else {
@@ -1118,6 +1065,13 @@ void RackBox::set_config_mode(bool mode) {
     enable_drag(mode);
 }
 
+void RackBox::setOrder(int pos, int post_pre) {
+    plugin.plugin->set_position(pos);
+    if (plugin.get_type() == PLUGIN_TYPE_MONO) {
+	plugin.plugin->set_effect_post_pre(post_pre);
+    }
+}
+
 void RackBox::do_expand() {
     swtch(false);
     Glib::signal_idle().connect_once(
@@ -1151,7 +1105,7 @@ Gtk::Button *RackBox::make_expand_button(bool expand) {
 }
 
 Gtk::Button *RackBox::make_preset_button() {
-    Gtk::Button *p = new Gtk::Button("p");
+    Gtk::Button *p = new Gtk::Button(C_("Preset", "p"));
     p->set_can_default(false);
     p->set_can_focus(false);
     p->set_size_request(18,18);
@@ -1160,24 +1114,32 @@ Gtk::Button *RackBox::make_preset_button() {
     return p;
 }
 
-void RackBox::pack(Gtk::Widget *main, Gtk::Widget *mini) {
+void RackBox::pack(Gtk::Widget *main, Gtk::Widget *mini, const Glib::RefPtr<Gtk::SizeGroup>& szg) {
     if (!main) {
 	return;
     }
     box.pack_start(*manage(main));
     minibox->pack(mini);
+    szg->add_widget(*fbox);
+    szg->add_widget(mbox);
 }
 
 Gtk::HBox *RackBox::make_full_box(gx_system::CmdlineOptions& options) {
     Gtk::HBox *bx = new Gtk::HBox();
-    bx->pack_start(*manage(switcher_vbox(options)), Gtk::PACK_SHRINK);
+    Gtk::VBox *vx = new Gtk::VBox();
+    bx->pack_start(*manage(vx));
+    Gtk::Widget *effect_label = make_label(plugin, options, false);
+    vx->pack_start(*manage(effect_label), Gtk::PACK_SHRINK);
+    Gtk::HBox *bx2 = new Gtk::HBox();
+    vx->pack_start(*manage(bx2));
+    bx2->pack_start(*manage(switcher_vbox(options)), Gtk::PACK_SHRINK);
     box.set_name(plugin.get_id());
     box.set_border_width(4);
     box.property_paint_func().set_value("RackBox_expose");
-    bx->pack_start(box);
+    bx2->pack_start(box);
     Gtk::VBox *vbox = new Gtk::VBox();
     vbox->pack_start(*manage(make_expand_button(false)), Gtk::PACK_SHRINK);
-    if (!(plugin.plugin->pdef->flags & PGN_NO_PRESETS)) {
+    if (!(plugin.plugin->get_pdef()->flags & PGN_NO_PRESETS)) {
 	vbox->pack_start(*manage(make_preset_button()), Gtk::PACK_EXPAND_PADDING);
     }
     Gtk::Alignment *al = new Gtk::Alignment(0.0, 0.0, 0.0, 0.7);
@@ -1191,15 +1153,14 @@ Gtk::HBox *RackBox::make_full_box(gx_system::CmdlineOptions& options) {
 
 Gtk::VBox *RackBox::switcher_vbox(gx_system::CmdlineOptions& options) {
     Gtk::VBox *vbox = new Gtk::VBox();
-    Gtk::Widget *effect_label = make_label(plugin, options, true);
-    vbox->pack_start(*manage(effect_label), Gtk::PACK_SHRINK);
+    
     Gtk::HBox *hbox = new Gtk::HBox();
     vbox->pack_start(*manage(hbox));
     Gtk::HBox *hbox2 = new Gtk::HBox();
     hbox->pack_start(*manage(hbox2), Gtk::PACK_SHRINK);
     Gtk::VBox *vbox2 = new Gtk::VBox();
     hbox2->pack_start(*manage(vbox2));
-    hbox2->pack_start(*manage(wrap_bar(4,13)), Gtk::PACK_SHRINK);
+    hbox2->pack_start(*manage(wrap_bar(4,4)), Gtk::PACK_SHRINK);
 #ifdef USE_SZG
     szg->add_widget(&on_off_switch);
 #endif
@@ -1223,7 +1184,6 @@ RackContainer::RackContainer(PluginType tp_, MainWindow& main_)
       config_mode(false),
       in_drag(-2),
       child_count(0),
-      switch_level(1),
       targets(),
       othertargets(),
       highlight_connection(),
@@ -1250,11 +1210,19 @@ RackContainer::RackContainer(PluginType tp_, MainWindow& main_)
     listTargets.push_back(Gtk::TargetEntry("application/x-guitarix-stereo-s", Gtk::TARGET_SAME_APP, 4));
     listTargets.push_back(Gtk::TargetEntry("application/x-gtk-tool-palette-item-stereo", Gtk::TARGET_SAME_APP, 5));
     drag_dest_set(listTargets, Gtk::DEST_DEFAULT_DROP, Gdk::ACTION_MOVE);
+    main.get_machine().signal_rack_unit_order_changed().connect(
+	sigc::mem_fun(this, &RackContainer::unit_order_changed));
     signal_remove().connect(sigc::mem_fun(*this, &RackContainer::on_my_remove));
     set_size_request(-1, min_containersize);
     show_all();
 }
 
+void RackContainer::unit_order_changed(bool stereo) {
+    if (stereo == (tp == PLUGIN_TYPE_STEREO)) {
+	check_order();
+    }
+}
+
 bool RackContainer::drag_highlight_expose(GdkEventExpose *event, int y0) {
     if (!is_drawable()) {
 	return false;
@@ -1331,9 +1299,7 @@ void RackContainer::find_index(int x, int y, int* len, int *ypos) {
 
 void RackContainer::on_my_remove(Gtk::Widget *ch) {
     decrement();
-    if (!main.is_loading()) {
-	renumber();
-    }
+    renumber();
 }
 
 bool RackContainer::check_targets(const std::vector<std::string>& tgts1, const std::vector<std::string>& tgts2) {
@@ -1495,26 +1461,10 @@ void RackContainer::hide_entries() {
     }
 }
 
-void RackContainer::reorder(const std::string& name, int pos) {
+void RackContainer::reorder(const std::string& name, unsigned int pos) {
     std::vector<RackBox*> l = get_children();
-    int i = 0;
-    RackBox *r = 0;
-    for (std::vector<RackBox*>::iterator v = l.begin(); v != l.end(); ++v) {
-	if ((*v)->get_id() == name) {
-	    if (pos > i) {
-		pos -= 1;
-	    }
-	    r = *v;
-	    break;
-	}
-	++i;
-    }
-    assert(r);
-    if (!r) {
-	return;
-    }
-    reorder_child(*r, pos);
-    renumber();
+    main.get_machine().insert_rack_unit(name, ((pos >= l.size()) ? "" : l[pos]->get_id()), tp);
+    check_order();
 }
 
 void RackContainer::on_add(Widget *ch) {
@@ -1527,9 +1477,6 @@ void RackContainer::add(RackBox& r, int pos) {
     if (config_mode) {
 	r.set_config_mode(true);
     }
-    if (main.is_loading()) {
-	return;
-    }
     reorder_child(r, pos);
     renumber();
 }
@@ -1557,57 +1504,54 @@ void RackContainer::set_config_mode(bool mode) {
     }
 }
 
-void RackContainer::compress_all() {
-    for (PluginDict::iterator i = main.plugins_begin(); i != main.plugins_end(); ++i) {
-	i->second->plugin->plug_visible = i->second->compressed = true;
-	RackBox *r = i->second->rackbox;
-	if (r) {
-	    if (r->can_compress()) {
-		r->swtch(true);
-	    }
-	}
-    }
-}
-
-void RackContainer::expand_all() {
-    for (PluginDict::iterator i = main.plugins_begin(); i != main.plugins_end(); ++i) {
-	i->second->plugin->plug_visible = i->second->compressed = false;
-	RackBox *r = i->second->rackbox;
-	if (r) {
-	    if (r->can_compress()) {
-		r->swtch(false);
-	    }
-	}
-    }
-}
-
-bool rackboxes_less(RackBox *a, RackBox *b) {
-    return a->position_weight() < b->position_weight();
-}
-
 void RackContainer::check_order() {
-    int last_weight = -1;
+    const std::vector<std::string>& ol = main.get_machine().get_rack_unit_order(tp);
     bool in_order = true;
+    std::set<std::string> unit_set(ol.begin(), ol.end());
     rackbox_list l = get_children();
+    std::vector<std::string>::const_iterator oi = ol.begin();
     for (rackbox_list::iterator c = l.begin(); c != l.end(); ++c) {
-	if (in_order) {
-	    int w = (*c)->position_weight();
-	    if (w <= last_weight) {
-		in_order = false;
+	if (!(*c)->get_box_visible()) {
+	    continue;
+	}
+	if (unit_set.find((*c)->get_id()) == unit_set.end()) {
+	    main.get_plugin((*c)->get_id())->hide(false);
+	    continue;
+	}
+	if (!in_order) {
+	    continue;
+	}
+	if (oi == ol.end()) {
+	    in_order = false;
+	    continue;
+	}
+	if (*oi != (*c)->get_id()) {
+	    in_order = false;
+	}
+	++oi;
+    }
+    if (oi != ol.end()) {
+	in_order = false;
+    }
+    if (in_order) {
+	return;
+    }
+    int n = 0;
+    for (std::vector<std::string>::const_iterator oi = ol.begin(); oi != ol.end(); ++oi) {
+	PluginUI *p = main.get_plugin(*oi);
+	if (!p->rackbox) {
+	    p->show(false);
+	} else {
+	    if (!p->rackbox->get_box_visible()) {
+		p->rackbox->display(true, false);
+		if (p->hidden) {
+		    p->rackbox->hide();
+		}
 	    }
-	    last_weight = w;
 	}
-	(*c)->storeOrder();
-    }
-    if (!in_order) {
-	std::vector<RackBox*> ol = get_children();
-	std::sort(ol.begin(), ol.end(), rackboxes_less);
-	int n = 0;
-	for (std::vector<RackBox*>::iterator i = ol.begin(); i != ol.end(); ++i) {
-	    reorder_child(**i, n++);
-	}
-	renumber();
+	reorder_child(*p->rackbox, n++);
     }
+    renumber();
 }
 
 void RackContainer::renumber() {
