@@ -35,9 +35,9 @@ namespace gx_engine {
  ** class ModuleSelectorFromList
  */
 
-class ModuleSelectorFromList: public ModuleSelector, private PluginDef, private gx_ui::GxUiItemUInt {
+class ModuleSelectorFromList: public ModuleSelector, private PluginDef {
 private:
-    unsigned int selector;
+    int selector;
     const char* select_id;
     const char* select_name;
     Plugin* current_plugin;
@@ -45,13 +45,12 @@ private:
     unsigned int size;
     static int static_register(const ParamReg& reg);
     int register_parameter(const ParamReg& reg);
-    void reflectZone();
 public:
     Plugin plugin;
     ModuleSelectorFromList(
-	EngineControl& seq, gx_ui::GxUI& ui, const char* id, const char* name,
-	plugindef_creator module_ids[],	const char* select_id,
-	const char* select_name, const char** groups = 0, int flags = 0);
+	EngineControl& seq, const char* id, const char* name,
+	const char *category, plugindef_creator module_ids[], const char* select_id,
+	const char* select_name, uiloader loader, const char** groups = 0, int flags = 0);
     ~ModuleSelectorFromList();
     void set_module();
 };
@@ -61,11 +60,20 @@ public:
  ** class GxEngine
  */
 
+struct PluginChange {
+    Plugin *pl;
+    enum pc { remove, update, update_category, add } status;
+    PluginChange(Plugin *pl_, pc status_): pl(pl_), status(status_) {}
+};
+
 class GxEngine: public ModuleSequencer {
 private:
     gx_resample::BufferResampler resamp;
+    sigc::signal<void,Plugin*,PluginChange::pc> plugin_changed;
+    LadspaLoader ladspaloader;
     void load_static_plugins();
 public:
+    MidiControllerList controller_map;
     // ModuleSelector's
     ModuleSelectorFromList crybaby;
     ModuleSelectorFromList tonestack;
@@ -78,13 +86,18 @@ public:
     MidiAudioBuffer midiaudiobuffer;
     MaxLevel maxlevel;
     OscilloscopeAdapter oscilloscope;
-    ConvolverAdapter convolver;
+    ConvolverMonoAdapter mono_convolver;
+    ConvolverStereoAdapter stereo_convolver;
     CabinetConvolver cabinet;
+    PreampConvolver preamp;
     ContrastConvolver contrast;
+    //
 public:
-    GxEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups, const gx_system::CmdlineOptions& options);
+    GxEngine(const string& plugin_dir, ParameterGroups& groups, const gx_system::CmdlineOptions& options);
     ~GxEngine();
     void set_jack(gx_jack::GxJack *jack) { midiaudiobuffer.set_jack(jack); }
+    void ladspaloader_update_plugins();
+    sigc::signal<void,Plugin*,PluginChange::pc>& signal_plugin_changed() { return plugin_changed; }
 };
 
 /* ------------------------------------------------------------------- */
