@@ -46,6 +46,7 @@ declare(gxamp8)
 declare(gxamp16)
 declare(gxamp6)
 declare(gxamp18)
+declare(gxnoamp)
 declare(gx_ampmodul)
 }
 
@@ -76,6 +77,7 @@ declare(tonestack_fender_blues)
 declare(tonestack_fender_default)
 declare(tonestack_fender_deville)
 declare(tonestack_gibsen)
+declare(tonestack_engl)
 }
 
 namespace gx_effects {
@@ -84,6 +86,7 @@ declare(autowah)
 declare(noise_shaper)
 declare(softclip)
 declare(bassbooster)
+declare(highbooster)
 declare(gx_ampout)
 declare(gxfeed)
 declare(gx_outputlevel)
@@ -92,6 +95,7 @@ declare(selecteq)
 declare(gx_distortion)
 declare(impulseresponse)
 declare(compressor)
+declare(expander)
 declare(overdrive)
 declare(echo)
 declare(delay)
@@ -110,6 +114,9 @@ declare(stereoecho)
 declare(moog)
 declare(tonecontroll)
 declare(stereoverb)
+declare(peak_eq)
+declare(digital_delay)
+declare(digital_delay_st)
 }
 
 declare(balance)
