@@ -137,15 +137,21 @@ class GxJack: public sigc::trackable {
     string              client_instance;
     jack_nframes_t      jack_sr;   // jack sample rate
     jack_nframes_t      jack_bs;   // jack buffer size
+    Glib::Dispatcher    xrun;
     float               last_xrun;
+    bool                xrun_msg_blocked;
+    void report_xrun_clear();
+    void report_xrun();
     void write_jack_port_connections(
 	gx_system::JsonWriter& w, const char *key, const PortConnection& pc, bool replace=false);
     std::string make_clientvar(const std::string& s);
     std::string replace_clientvar(const std::string& s);
-    bool                gx_jack_init(bool startserver, int wait_after_connect);
-    void                gx_jack_init_port_connection();
+    bool                gx_jack_init(bool startserver, int wait_after_connect,
+				     const gx_system::CmdlineOptions& opt);
+    void                gx_jack_init_port_connection(const gx_system::CmdlineOptions& opt);
     void                gx_jack_callbacks();
     void                gx_jack_cleanup();
+    inline void         check_overload();
 
  public:
     JackPorts           ports;
@@ -166,7 +172,8 @@ public:
     void                set_jack_down(bool v) { jack_is_down = v; }
     void                set_jack_exit(bool v) { jack_is_exit = v; }
 
-    bool                gx_jack_connection(bool connect, bool startserver, int wait_after_connect);
+    bool                gx_jack_connection(bool connect, bool startserver,
+					   int wait_after_connect, const gx_system::CmdlineOptions& opt);
     float               get_last_xrun() { return last_xrun; }
     void*               get_midi_buffer(jack_nframes_t nframes);
 
@@ -176,7 +183,6 @@ public:
     const string&       get_instancename() { return client_instance; }
     string              client_name;
     string              client_insert_name;
-    Glib::Dispatcher    xrun;
     Glib::Dispatcher    session;
     Glib::Dispatcher    session_ins;
     Glib::Dispatcher    shutdown;
@@ -188,7 +194,7 @@ public:
     Glib::Dispatcher&   signal_portchange() { return connection_queue.portchange; }
     Glib::Dispatcher&   signal_buffersize_change() { return buffersize_change; }
     void                send_connection_changes(bool v) { connection_queue.set_send(v); }
-
+    static void         rt_watchdog_set_limit(int limit);
 #ifdef HAVE_JACK_SESSION
     jack_session_event_t *get_last_session_event() {
 	return gx_system::atomic_get(session_event);
@@ -199,6 +205,7 @@ public:
     int                 return_last_session_event();
     int                 return_last_session_event_ins();
     string              get_uuid_insert();
+    gx_engine::GxEngine& get_engine() { return engine; }
 #endif
 };
 
