@@ -22,99 +22,29 @@
  * ----------------------------------------------------------------------------
  */
 
-template <class Param>
-class UiToggleAction: public Gtk::ToggleAction, public gx_ui::GxUiItem {
+template <class T>
+class UiToggleAction: public Gtk::ToggleAction {
 private:
-    Param& param;
-    gx_ui::GxUI& ui;
-    virtual void on_toggled();
-    virtual void reflectZone();
-    virtual bool hasChanged();
+    gx_engine::GxMachineBase& machine;
+    std::string id;
+    void on_toggled();
 protected:
     UiToggleAction(
-	gx_ui::GxUI& ui_, Param& para, const Glib::ustring& name, const Glib::ustring& icon_name,
+	gx_engine::GxMachineBase& machine_, const std::string& id, const Glib::ustring& name, const Glib::ustring& icon_name,
 	const Glib::ustring& label=Glib::ustring(), const Glib::ustring& tooltip=Glib::ustring(),
 	bool is_active=false);
     ~UiToggleAction();
 public:
     static Glib::RefPtr<UiToggleAction> create(
-	gx_ui::GxUI& ui, Param& para, const Glib::ustring& name, const Glib::ustring& label=Glib::ustring(),
+	gx_engine::GxMachineBase& machine, const std::string& id, const Glib::ustring& name, const Glib::ustring& label=Glib::ustring(),
 	const Glib::ustring& tooltip=Glib::ustring(), bool is_active=false) {
 	return Glib::RefPtr<UiToggleAction>(
-	    new UiToggleAction(ui, para, name, Glib::ustring(), label, tooltip, is_active));
+	    new UiToggleAction(machine, id, name, Glib::ustring(), label, tooltip, is_active));
     }
 };
 
-typedef UiToggleAction<gx_engine::SwitchParameter> UiSwitchToggleAction;
-typedef UiToggleAction<gx_engine::BoolParameter> UiBoolToggleAction;
-
-template <class Param>
-class UiRadioAction: public Gtk::RadioAction, public gx_ui::GxUiItem {
-private:
-    Param& param;
-    gx_ui::GxUI& ui;
-    virtual void on_changed(const Glib::RefPtr<Gtk::RadioAction>& act);
-    virtual void reflectZone();
-    virtual bool hasChanged();
-protected:
-    UiRadioAction(
-	gx_ui::GxUI& ui_, Param& para, Gtk::RadioButtonGroup& group, const Glib::ustring& name, const Glib::ustring& icon_name,
-	const Glib::ustring& label=Glib::ustring(), const Glib::ustring& tooltip=Glib::ustring());
-    ~UiRadioAction();
-public:
-    static Glib::RefPtr<UiRadioAction> create(
-	gx_ui::GxUI& ui, Param& para, Gtk::RadioButtonGroup& group, const Glib::ustring& name,
-	const Glib::ustring& label=Glib::ustring(), const Glib::ustring& tooltip=Glib::ustring()) {
-	return Glib::RefPtr<UiRadioAction>(
-	    new UiRadioAction(ui, para, group, name, Glib::ustring(), label, tooltip));
-    }
-};
-
-typedef UiRadioAction<gx_engine::SwitchParameter> UiSwitchRadioAction;
-
-
-/****************************************************************
- ** class TunerSwitcher
- */
-
-class Liveplay;
-
-class TunerSwitcher: public sigc::trackable {
-private:
-    enum SwitcherState { normal_mode, wait_start, listening, wait_stop };
-    enum SwitcherActions {
-	mute_on    = -1,
-	mute_off   = -2,
-	bypass_on  = -3,
-	bypass_off = -4,
-	tuner_on   = -5,
-	tuner_off  = -6,
-    };
-    Liveplay& lp;
-    sigc::connection switcher_conn;
-    sigc::connection timeout_conn;
-    int current_note;
-    SwitcherState state;
-    gx_engine::GxEngineState old_engine_state;
-    gx_engine::GxEngineState new_engine_state;
-    bool old_tuner_active;
-    bool new_tuner_active;
-    int last_bank_idx;
-    int last_preset_idx;
-private:
-    void set_state(SwitcherState newstate);
-    bool on_note_timeout();
-    bool on_state_timeout();
-    void on_tuner_freq_changed();
-    void try_load_preset();
-    bool display_bank_key(int idx);
-    bool display_preset_key(int idx);
-public:
-    TunerSwitcher(Liveplay& lp);
-    bool get_active() { return switcher_conn.connected(); }
-    void set_active(bool v);
-    void toggle() { set_active(!get_active()); }
-};
+typedef UiToggleAction<bool> UiBoolToggleAction;
+typedef UiBoolToggleAction UiSwitchToggleAction;
 
 
 /****************************************************************
@@ -123,7 +53,7 @@ public:
 
 class KeySwitcher: public sigc::trackable {
 private:
-    gx_preset::GxSettings& gx_settings;
+    gx_engine::GxMachineBase& machine;
     sigc::slot<void, const Glib::ustring&, const Glib::ustring&> display;
     sigc::connection key_timeout;
     Glib::ustring last_bank_key;
@@ -132,9 +62,9 @@ private:
     bool display_selected_bank();
     bool display_current();
 public:
-    KeySwitcher(gx_preset::GxSettings& gx_settings_,
+    KeySwitcher(gx_engine::GxMachineBase& machine_,
 		sigc::slot<void, const Glib::ustring&, const Glib::ustring&> display_)
-	: gx_settings(gx_settings_), display(display_) {}
+	: machine(machine_), display(display_) {}
     bool process_bank_key(int idx);
     bool process_preset_key(int idx);
     void display_key_error();
@@ -148,10 +78,8 @@ public:
 
 class Liveplay: public sigc::trackable {
 private:
-    gx_ui::GxUI ui;
     Glib::RefPtr<gx_gui::GxBuilder> bld;
-    gx_engine::GxEngine &engine;
-    gx_preset::GxSettings& gx_settings;
+    gx_engine::GxMachineBase &machine;
     const GxActions& actions;
     bool use_composite;
     Gtk::Adjustment *brightness_adj;
@@ -159,8 +87,6 @@ private:
     KeySwitcher keyswitch;
     sigc::connection midi_conn;
     Gtk::Window *window;
-    TunerSwitcher tuner_switcher;
-    gx_ui::UiSignal<bool> switcher_signal;
     sigc::connection mouse_hide_conn;
     //
     Gtk::Image *bypass_image;
@@ -183,6 +109,7 @@ private:
     bool window_expose_event(GdkEventExpose* event);
     void on_realize();
     void on_engine_state_change(gx_engine::GxEngineState state);
+    void on_selection_done(bool v);
     void on_selection_changed();
     static bool do_action(
 	GtkAccelGroup *accel_group, GObject *acceleratable,
@@ -202,12 +129,11 @@ private:
     static bool on_keyboard_mode_switch(
 	GtkAccelGroup *accel_group, GObject *acceleratable,
 	guint keyval, GdkModifierType modifier, Liveplay& self);
-    void on_switcher_toggled(bool v);
     bool pointer_motion(GdkEventMotion* event);
     void display(const Glib::ustring& bank, const Glib::ustring& preset);
-    friend class TunerSwitcher;
+    void set_display_state(TunerSwitcher::SwitcherState s);
 public:
-    Liveplay(const gx_system::CmdlineOptions& options, gx_engine::GxEngine& engine, gx_preset::GxSettings& gx_settings,
+    Liveplay(const gx_system::CmdlineOptions& options, gx_engine::GxMachineBase& machine,
 	     const std::string& fname, const GxActions& actions);
     ~Liveplay();
     void on_live_play(Glib::RefPtr<Gtk::ToggleAction> act);
@@ -223,21 +149,13 @@ public:
 class MainWindow;
 class RackBox;
 
-enum PluginType {
-    PLUGIN_TYPE_MONO,
-    PLUGIN_TYPE_STEREO,
-};
-
-class PluginUI: public gx_ui::GxUiItem {
+class PluginUI: public sigc::trackable {
 private:
-    virtual void reflectZone();
-    virtual bool hasChanged();
     Gtk::UIManager::ui_merge_id merge_id;
     Glib::RefPtr<Gtk::ToggleAction> action;
     void on_action_toggled();
 public:
     gx_engine::Plugin *plugin;
-    Glib::ustring fname;
     Glib::ustring tooltip;
     const char *shortname;
 
@@ -249,41 +167,63 @@ public:
     MainWindow& main;
     RackBox *rackbox;
     bool hidden;
-    bool compressed;
 
-    PluginUI(MainWindow& main, const gx_engine::PluginList& pl, const char* id_,
-	     const Glib::ustring& fname_="", const Glib::ustring& tooltip_="");
+    PluginUI(MainWindow& main, const char* id_,
+	     const Glib::ustring& tooltip_="");
     virtual ~PluginUI();
     PluginType get_type() const {
-	return (plugin->pdef->flags & PGN_STEREO) ? PLUGIN_TYPE_STEREO : PLUGIN_TYPE_MONO;
+	return (plugin->get_pdef()->flags & PGN_STEREO) ? PLUGIN_TYPE_STEREO : PLUGIN_TYPE_MONO;
     }
-    const char *get_id() const { return plugin->pdef->id; }
-    const char *get_name() const { return plugin->pdef->name; }
+    const char *get_id() const { return plugin->get_pdef()->id; }
+    const char *get_name() const { return plugin->get_pdef()->name; }
     void display(bool v, bool animate);
     void display_new(bool unordered = false);
-    inline bool is_displayed();
     void set_ui_merge_id(Gtk::UIManager::ui_merge_id id) { merge_id = id; }
+    void unset_ui_merge_id(Glib::RefPtr<Gtk::UIManager> uimanager);
     void set_action(Glib::RefPtr<Gtk::ToggleAction>& act);
-    static bool is_registered(gx_engine::PluginList& pl, const char *name);
+    void set_active(bool v) { if (action) action->set_active(v); }
+    Glib::RefPtr<Gtk::ToggleAction> get_action() { return action; }
+    static bool is_registered(gx_engine::GxMachineBase& m, const char *name);
     virtual void on_plugin_preset_popup();
+    inline const char *get_category() {
+	const char *cat = plugin->get_pdef()->category;
+	return (cat && *cat) ? cat : N_("External");
+    }
+    inline const char *get_shortname() const {
+	const char *name = shortname;
+	if (!name) {
+	    name = plugin->get_pdef()->shortname;
+	}
+	if (!name || !*name) {
+	    name = get_name();
+	}
+	return name;
+    }
+    void update_rackbox();
+    void compress(bool state);
+    void hide(bool animate);
+    void show(bool animate);
+    friend bool plugins_by_name_less(PluginUI *a, PluginUI *b);
 };
 
+bool plugins_by_name_less(PluginUI *a, PluginUI *b);
 
 /****************************************************************
  ** class PluginDict
  */
 
 class PluginDict: private std::map<std::string, PluginUI*> {
-private:
-    gx_ui::GxUI& ui;
 public:
     typedef std::map<std::string, PluginUI*>::iterator iterator;
-    PluginDict(gx_ui::GxUI& ui_): std::map<std::string, PluginUI*>(), ui(ui_) {}
+    PluginDict(): std::map<std::string, PluginUI*>() {}
     ~PluginDict();
-    void add(PluginUI *p) { insert(pair<std::string, PluginUI*>(p->get_id(), p)); }
+    void cleanup();
+    void add(PluginUI *p);
+    void remove(PluginUI *p);
     PluginUI *operator[](const std::string& s) { return find(s)->second; }
     using std::map<std::string, PluginUI*>::begin;
     using std::map<std::string, PluginUI*>::end;
+    void compress(bool state);
 };
 
 
@@ -335,14 +275,12 @@ private:
     DragIcon *drag_icon;
     int target_height;
     Gxw::PaintBox box;
-    int box_visible;
-    int position;
-    unsigned int effect_post_pre;
+    bool box_visible;
     Gxw::Switch on_off_switch;
     gx_gui::uiToggle<bool> toggle_on_off;
 private:
     static void set_paintbox(Gxw::PaintBox& pb, PluginType tp);
-    static Gtk::Widget *make_label(const PluginUI& plugin, gx_system::CmdlineOptions& options, bool useshort=false);
+    static Gtk::Widget *make_label(const PluginUI& plugin, gx_system::CmdlineOptions& options, bool useshort=true);
     static Gtk::Widget *make_bar(int left=4, int right=4, bool sens=false);
     Gtk::Widget *wrap_bar(int left=4, int right=4, bool sens=false);
     void init_dnd();
@@ -363,7 +301,6 @@ private:
     Gtk::VBox *switcher_vbox(gx_system::CmdlineOptions& options);
     bool has_delete() const { return delete_button; }
     void do_expand();
-    friend bool rackboxes_less(RackBox *a, RackBox *b);
 public:
     RackBox(PluginUI& plugin, MainWindow& main, Gtk::Widget* bare=0);
     static Gtk::Widget *create_drag_widget(const PluginUI& plugin, gx_system::CmdlineOptions& options);
@@ -373,19 +310,13 @@ public:
     const char *get_id() const { return plugin.get_id(); }
     void set_config_mode(bool mode);
     void swtch(bool mini);
-    void pack(Gtk::Widget *mainbox, Gtk::Widget *minibox);
+    void pack(Gtk::Widget *mainbox, Gtk::Widget *minibox, const Glib::RefPtr<Gtk::SizeGroup>& szg);
     void animate_insert();
     static Gtk::Widget *create_icon_widget(const PluginUI& plugin, gx_system::CmdlineOptions& options);
-    void setOrder(int pos, unsigned int post_pre) {
-	position = plugin.plugin->position = pos;
-	effect_post_pre = plugin.plugin->effect_post_pre = post_pre;
-    }
-    void storeOrder() { position = plugin.plugin->position; effect_post_pre = plugin.plugin->effect_post_pre; }
-    bool hasOrderDiff() { return plugin.plugin->position != position || plugin.plugin->effect_post_pre != effect_post_pre; }
-    int position_weight() { return plugin.plugin->position_weight(); }
-    bool is_displayed() { return box_visible; }
+    void setOrder(int pos, int post_pre);
     void display(bool v, bool animate);
-    bool get_plug_visible() { return plugin.compressed; }
+    bool get_plug_visible() { return plugin.plugin->get_plug_visible(); }
+    bool get_box_visible() { return box_visible; }
 };
 
 class MiniRackBox: public Gtk::HBox {
@@ -408,10 +339,6 @@ public:
     void pack(Gtk::Widget *w);
 };
 
-inline bool PluginUI::is_displayed() {
-    return rackbox && rackbox->is_displayed();
-}
-
 /****************************************************************
  ** class RackContainer
  */
@@ -423,12 +350,12 @@ private:
     bool config_mode;
     int in_drag;
     int child_count;
-    int switch_level;
     std::vector<std::string> targets;
     std::vector<std::string> othertargets;
     sigc::connection highlight_connection;
     sigc::connection autoscroll_connection;
 private:
+    using Gtk::VBox::add;
     bool drag_highlight_expose(GdkEventExpose *event, int y0);
     void find_index(int x, int y, int* len, int *ypos);
     void on_my_remove(Gtk::Widget*);
@@ -437,6 +364,7 @@ private:
     virtual void on_drag_leave(const Glib::RefPtr<Gdk::DragContext>& context, guint timestamp);
     virtual void on_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& data, guint info, guint timestamp);
     virtual void on_add(Widget* ch);
+    void unit_order_changed(bool stereo);
     void renumber();
     bool scroll_timeout();
     bool scrollother_timeout();
@@ -457,51 +385,18 @@ public:
     inline bool check_if_animate(const RackBox& rackbox);
     void show_entries();
     void hide_entries();
-    void compress_all();
-    void expand_all();
     void set_config_mode(bool mode);
     bool empty() const { return child_count == 0; }
     void add(RackBox& r, int pos=-1);
     void check_order();
     void ensure_visible(RackBox& child);
-    void reorder(const std::string& name, int pos);
+    void reorder(const std::string& name, unsigned int pos);
     void increment();
     void decrement();
     inline void resize_finished();
 };
 
 
-/****************************************************************
- ** class StackBoxBuilderNew
- */
-
-class StackBoxBuilderNew: public gx_gui::StackBoxBuilder {
-private:
-    int                  fTop;
-    GtkWidget*           fBox[stackSize];
-    int                  fMode[stackSize];
-    gx_engine::GxEngine& engine;
-    gx_preset::GxSettings& gx_settings;
-    Gxw::WaveView&       fWaveView;
-    Gtk::Label&          convolver_filename_label;
-    Gtk::HBox           *widget;
-private:
-    virtual void openMonoRackBox(const char* label, int* posit, const char *id_on_off, const char *id_pre_post, const char *id_dialog);
-    virtual void closeMonoRackBox();
-    virtual void openStereoRackBox(const char* label, int* posit, const char *id_on_off, const char *id_dialog);
-    virtual void closeStereoRackBox();
-    virtual void openVerticalMidiBox(const char* label = "");
-public:
-    StackBoxBuilderNew(gx_engine::GxEngine& engine_, gx_preset::GxSettings& gx_settings_,
-		       Gxw::WaveView &fWaveView_, Gtk::Label &convolver_filename_label_, gx_ui::GxUI& ui,
-		       Glib::RefPtr<Gdk::Pixbuf> window_icon);
-    void set_accelgroup(Glib::RefPtr<Gtk::AccelGroup> accels_) { accels = accels_; }
-    void get_box(const std::string& name, Gtk::Widget*& mainbox, Gtk::Widget*& minibox);
-    void prepare();
-    void fetch(Gtk::Widget*& mainbox, Gtk::Widget*& minibox);
-};
-
-
 /****************************************************************
  ** class TextLoggingBox
  */
@@ -517,14 +412,13 @@ private:
     Gtk::HButtonBox buttonbox;
     Gtk::ScrolledWindow scrollbox;
     Gtk::TextView tbox;
-    static tab_table tagdefs[gx_system::kMessageTypeCount];
-    Glib::RefPtr<Gtk::TextTag> tags[gx_system::kMessageTypeCount];
+    static tab_table tagdefs[GxLogger::kMessageTypeCount];
+    Glib::RefPtr<Gtk::TextTag> tags[GxLogger::kMessageTypeCount];
     int highest_unseen_msg_level;
     sigc::signal<void> msg_level_changed;
 private:
-    bool on_delete_event();
     virtual bool on_key_press_event(GdkEventKey *event);
-    void show_msg(string msgbuf, gx_system::GxMsgType msgtype, bool plugged);
+    void show_msg(string msgbuf, GxLogger::MsgType msgtype, bool plugged);
     virtual void on_show();
     virtual void on_hide();
 public:
@@ -568,36 +462,42 @@ public:
 
 class PluginPresetPopup: public Gtk::Menu {
 private:
-    const std::string id;
-    const gx_preset::GxSettings& gx_settings;
+    const PluginDef *pdef;
+    gx_engine::GxMachineBase& machine;
     const Glib::ustring save_name_default;
+    gx_preset::UnitPresetList presetnames;
     void set_plugin_std_preset();
-    static void set_plugin_preset(Glib::RefPtr<gx_preset::PluginPresetList> l, Glib::ustring name);
-    bool add_plugin_preset_list(Glib::RefPtr<gx_preset::PluginPresetList> l);
-    void save_plugin_preset(Glib::RefPtr<gx_preset::PluginPresetList> l);
-    void remove_plugin_preset(Glib::RefPtr<gx_preset::PluginPresetList> l);
+    void set_plugin_preset(bool factory, const Glib::ustring& name);
+    bool add_plugin_preset_list(bool *found);
+    void save_plugin_preset();
+    void remove_plugin_preset();
 protected:
     virtual void on_selection_done();
 public:
-    PluginPresetPopup(const std::string& id, const gx_preset::GxSettings& gx_settings,
+    PluginPresetPopup(const PluginDef *pdef, gx_engine::GxMachineBase& machine,
 		      const Glib::ustring& save_name_default = "");
+    gx_preset::UnitPresetList::const_iterator begin() { return presetnames.begin(); }
+    gx_preset::UnitPresetList::const_iterator end() { return presetnames.end(); }
+    const PluginDef *get_pdef() { return pdef; }
+    gx_engine::GxMachineBase& get_machine() { return machine; }
 };
 
 
 /****************************************************************
  ** GxUiRadioMenu
- ** adds the values of an UEnumParameter as Gtk::RadioMenuItem's
+ ** adds the values of an EnumParameter as Gtk::RadioMenuItem's
  ** to a Gtk::MenuShell
  */
 
-class GxUiRadioMenu: public gx_ui::GxUiItemUInt {
+class GxUiRadioMenu {
 private:
+    gx_engine::GxMachineBase& machine;
+    const std::string id;
     Glib::RefPtr<Gtk::RadioAction> action;
-    gx_engine::UIntParameter& param;
-    virtual void reflectZone();
+    void set_value(unsigned int v);
     void on_changed(Glib::RefPtr<Gtk::RadioAction> act);
 public:
-    GxUiRadioMenu(gx_ui::GxUI* ui, gx_engine::UIntParameter& param);
+    GxUiRadioMenu(gx_engine::GxMachineBase& machine, const std::string& id);
     void setup(const Glib::ustring& prefix, const Glib::ustring& postfix,
 	       Glib::RefPtr<Gtk::UIManager>& uimanager, Glib::RefPtr<Gtk::ActionGroup>& actiongroup);
 };
@@ -612,12 +512,16 @@ struct GxActions {
     Glib::RefPtr<Gtk::Action> compress;
     Glib::RefPtr<Gtk::Action> expand;
     Glib::RefPtr<Gtk::Action> jack_latency_menu;
+    Glib::RefPtr<Gtk::Action> osc_buffer_menu;
     Glib::RefPtr<Gtk::Action> jackstartup;
+    Glib::RefPtr<Gtk::Action> loadladspa;
 
     Glib::RefPtr<Gtk::ToggleAction> rack_config;
     Glib::RefPtr<Gtk::ToggleAction> live_play;
     Glib::RefPtr<Gtk::ToggleAction> engine_mute;
+    sigc::connection engine_mute_conn;
     Glib::RefPtr<Gtk::ToggleAction> engine_bypass;
+    sigc::connection engine_bypass_conn;
     Glib::RefPtr<Gtk::ToggleAction> jackserverconnection;
     Glib::RefPtr<Gtk::ToggleAction> jackports;
     Glib::RefPtr<Gtk::ToggleAction> midicontroller;
@@ -625,20 +529,21 @@ struct GxActions {
     Glib::RefPtr<Gtk::ToggleAction> loggingbox;
     Glib::RefPtr<Gtk::ToggleAction> animations;
 
-    Glib::RefPtr<UiSwitchToggleAction> show_plugin_bar;
-    Glib::RefPtr<UiSwitchToggleAction> presets;
-    Glib::RefPtr<UiSwitchToggleAction> show_rack;
+    Glib::RefPtr<Gtk::ToggleAction> show_plugin_bar;
+    Glib::RefPtr<Gtk::ToggleAction> presets;
+    Glib::RefPtr<Gtk::ToggleAction> show_rack;
     Glib::RefPtr<UiBoolToggleAction> tuner;
     Glib::RefPtr<UiBoolToggleAction> livetuner;
     Glib::RefPtr<UiBoolToggleAction> midi_out;
     Glib::RefPtr<UiBoolToggleAction> midi_out_plug;
-    Glib::RefPtr<UiSwitchToggleAction> show_values;
-    Glib::RefPtr<UiSwitchToggleAction> tooltips;
+    Glib::RefPtr<Gtk::ToggleAction> show_values;
+    Glib::RefPtr<Gtk::ToggleAction> tooltips;
     Glib::RefPtr<UiSwitchToggleAction> midi_in_presets;
-    Glib::RefPtr<UiSwitchToggleAction> rackh;
+    Glib::RefPtr<Gtk::ToggleAction> rackh;
 
     Glib::RefPtr<Gtk::RadioAction> skin;
     Glib::RefPtr<Gtk::RadioAction> latency;
+    Glib::RefPtr<Gtk::RadioAction> osc_buffer_size;
 
     // preset window
     Glib::RefPtr<Gtk::Action> new_bank;
@@ -648,13 +553,9 @@ struct GxActions {
 
 class MainWindow: public sigc::trackable {
 private:
-    gx_ui::GxUI ui;
+    gx_system::CmdlineOptions& options;
+    gx_engine::GxMachineBase&  machine;
     Glib::RefPtr<gx_gui::GxBuilder> bld;
-    static int mainwin_x;
-    static int mainwin_y;
-    static int mainwin_height;
-    static int window_height;
-    static int preset_window_height;
     Freezer freezer;
     PluginDict plugin_dict;
     int oldpos;
@@ -669,21 +570,14 @@ private:
     Gtk::UIManager::ui_merge_id preset_list_merge_id;
     Glib::RefPtr<Gtk::ActionGroup> preset_list_actiongroup;
     Glib::RefPtr<Gtk::UIManager> uimanager;
-    gx_system::CmdlineOptions& options;
-    gx_engine::ParamMap&  pmap;
-    gx_engine::GxEngine&  engine;
-    gx_jack::GxJack       jack;
-    gx_preset::GxSettings gx_settings;
     Liveplay *live_play;
     PresetWindow *preset_window;
     Gxw::WaveView fWaveView;
     Gtk::Label convolver_filename_label;
+    Gtk::Label convolver_mono_filename_label;
     Glib::RefPtr<Gdk::Pixbuf> gx_head_icon;
-    StackBoxBuilderNew boxbuilder;
+    gx_gui::StackBoxBuilder boxbuilder;
     gx_portmap::PortMapWindow* portmap_window;
-    static int skin;
-    static bool no_warn_latency;
-    gx_ui::UiSignal<int> skin_changed;
     gx_gui::SelectJackControlPgm *select_jack_control;
     TextLoggingBox fLoggingWindow;
     GxUiRadioMenu amp_radio_menu;
@@ -695,15 +589,15 @@ private:
     Glib::RefPtr<Gdk::Pixbuf> pixbuf_log_grey;
     Glib::RefPtr<Gdk::Pixbuf> pixbuf_log_yellow;
     Glib::RefPtr<Gdk::Pixbuf> pixbuf_log_red;
-    gx_ui::UiSignal<bool> mute_changed;
-    gx_ui::UiSignal<bool> ampdetail_sh;
-    gx_gui::ReportXrun report_xrun;
     bool in_session;
     Glib::RefPtr<Gtk::StatusIcon> status_icon;
     Glib::RefPtr<Gdk::Pixbuf> gx_head_midi;
     Glib::RefPtr<Gdk::Pixbuf> gx_head_warn;
     GxActions actions;
     KeySwitcher keyswitch;
+    std::map<Glib::ustring, Gtk::ToolItemGroup*> groupmap;
+    ladspa::PluginDisplay *ladspalist_window;
+    Glib::RefPtr<Gtk::SizeGroup> szg_rack_units;
 
     // Widget pointers
     Gxw::PaintBox *tunerbox;
@@ -776,9 +670,11 @@ private:
     void on_expand_all();
     void on_show_plugin_bar();
     void on_rack_configuration();
+    void set_tooltips();
+    void set_animations();
     void move_widget(Gtk::Widget& w, Gtk::Box& b1, Gtk::Box& b2);
     int rackbox_stacked_vertical() const;
-    static void change_expand(Gtk::Widget& w, bool value);
+    void change_expand(Gtk::Widget& w, bool value);
     void on_dir_changed();
     void on_configure_event(GdkEventConfigure *ev);
     void clear_box(Gtk::Container& box);
@@ -796,8 +692,7 @@ private:
     bool on_ti_button_press(GdkEventButton *ev, const char *effect_id);
     void on_tp_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& data, int info, int timestamp);
     void fill_pluginlist();
-    void make_icons();
-    bool update_all_gui();
+    void make_icons(bool force=true);
     void jack_connection();
     void on_miditable_toggle();
     void on_portmap_activate();
@@ -812,24 +707,28 @@ private:
     void user_disable_latency_warn(Gtk::CheckButton* disable_warn);
     int gx_wait_latency_warn();
     void set_latency();
+    void set_osc_size();
+    void add_osc_size_menu();
+    void change_osc_buffer(Glib::RefPtr<Gtk::RadioAction> action);
     void on_select_jack_control();
+    void on_load_ladspa();
     void delete_select_jack_control();
     void on_log_activate();
     void do_program_change(int pgm);
     void on_engine_toggled();
     void on_engine_state_change(gx_engine::GxEngineState state);
-    void set_new_skin(unsigned int idx);
+    void set_new_skin(const Glib::ustring& skin_name);
     void set_tuning(Gxw::RackTuner& tuner);
     void setup_tuner(Gxw::RackTuner& tuner);
     bool on_toggle_mute(GdkEventButton* ev);
     void on_msg_level_changed();
-    void on_ampdetail_switch(bool compress);
+    void on_ampdetail_switch(bool compress, bool setparam);
     void on_show_oscilloscope(bool v);
     void set_waveview_buffer(unsigned int size);
     void on_oscilloscope_post_pre(int post_pre);
     int on_oscilloscope_activate(bool start);
     bool on_refresh_oscilloscope();
-    bool refresh_meter_level();
+    bool refresh_meter_level(float falloff);
     bool survive_jack_shutdown();
     void gx_jack_is_down();
     void jack_session_event();
@@ -837,7 +736,7 @@ private:
     void set_in_session();
     void hide_extended_settings();
     void systray_menu(guint button, guint32 activate_time);
-    void overload_status_changed();
+    void overload_status_changed(gx_engine::MidiAudioBuffer::Load l);
     bool on_window_state_changed(GdkEventWindowState* event);
     bool on_meter_button_release(GdkEventButton* ev);
     void show_selected_preset();
@@ -850,9 +749,18 @@ private:
     void on_show_midi_out();
     void on_show_midi_out_plug();
     void on_midi_out_channel_toggled(Gtk::RadioButton *rb, Gtk::Container *c);
+    void on_livetuner_toggled();
+    Gtk::ToolItemGroup *add_plugin_category(const char *cat, bool collapse = true);
+    Glib::ustring add_plugin_menu_entry(PluginUI *pui);
+    void register_plugin(PluginUI *pui);
+    void on_ladspa_finished(bool reload, bool quit);
+    bool delete_ladspalist_window();
+    bool on_quit();
+    void amp_controls_visible(Gtk::Range *rr);
+    void on_plugin_changed(gx_engine::Plugin *pl, gx_engine::PluginChange::pc c);
 public:
-    MainWindow(gx_engine::GxEngine& engine, gx_system::CmdlineOptions& options,
-	       gx_engine::ParamMap& pmap, Gtk::Window *splash);
+    MainWindow(gx_engine::GxMachineBase& machine, gx_system::CmdlineOptions& options,
+	       Gtk::Window *splash, const Glib::ustring& title);
     ~MainWindow();
     void hide_effect(const std::string& name);
     RackContainer& get_monorackcontainer() { return monorackcontainer; }
@@ -863,19 +771,17 @@ public:
     PluginDict::iterator plugins_end() { return plugin_dict.end(); }
     void run();
     gx_system::CmdlineOptions& get_options() { return options; }
-    gx_ui::GxUI& get_ui() { return ui; }
-    void plugin_preset_popup(const std::string& id);
-    void plugin_preset_popup(const std::string& id, const Glib::ustring& name);
-    gx_engine::ParamMap& get_parametermap() { return pmap; }
-    gx_engine::GxEngine& get_engine() { return engine; }
-    bool is_loading() { return gx_settings.is_loading(); }
-    void add_plugin(std::vector<PluginUI*> *p, const char *id, const Glib::ustring& fname_="", const Glib::ustring& tooltip_="");
+    void plugin_preset_popup(const PluginDef *pdef);
+    void plugin_preset_popup(const PluginDef *pdef, const Glib::ustring& name);
+    gx_engine::GxMachineBase& get_machine() { return machine; }
+    void add_plugin(std::vector<PluginUI*>& p, const char *id, const Glib::ustring& tooltip_="");
     void set_rackbox_expansion();
     double stop_at_stereo_bottom(double off, double step_size, double pagesize);
     double stop_at_mono_top(double off, double step_size);
     bool use_animations() { return actions.animations->get_active(); }
-    void create_default_scratch_preset() { gx_settings.create_default_scratch_preset(); }
+    void create_default_scratch_preset() { machine.create_default_scratch_preset(); }
     void resize_finished(RackContainer *ch);
+    void update_width();
 };
 
 inline bool RackContainer::check_if_animate(const RackBox& rackbox) { return main.use_animations(); }
