@@ -52,7 +52,7 @@ class ParameterGroups {
     map<string, bool> used;
     void group_exists(const string& id);
     void group_is_new(const string& id);
-    friend string param_group(const string& id, bool nowarn);
+    friend string param_group(const string& group_id, bool nowarn);
 #endif
 
  public:
@@ -61,13 +61,21 @@ class ParameterGroups {
 
     inline string get(const string& id) { return groups[id]; }
     inline string operator[](const string& id) {
-            debug_check(group_exists, id);
-            return groups[id];
-        }
+	debug_check(group_exists, id);
+	return groups[id];
+    }
     inline void insert(const string& id, const string& group) {
-            debug_check(group_is_new, id);
-            groups.insert(pair<string, string>(id, group));
-        }
+	debug_check(group_is_new, id);
+	groups.insert(pair<string, string>(id, group));
+    }
+    inline void erase(const string& id) {
+#ifndef NDEBUG // avoid unused variable warning in release mode
+	size_t n = groups.erase(id);
+	assert(n == 1);
+#else
+	groups.erase(id);
+#endif
+    }
 #ifndef NDEBUG
     void dump();
 #endif
@@ -75,7 +83,7 @@ class ParameterGroups {
 
 
 ParameterGroups& get_group_table();
-string param_group(const string& id, bool nowarn = false);
+string param_group(const string& group_id, bool nowarn = false);
 
 /****************************************************************
  ** Parameter
@@ -85,14 +93,11 @@ template <class T> class ParameterV;
 
 typedef ParameterV<float> FloatParameter;
 typedef ParameterV<int> IntParameter;
-typedef ParameterV<unsigned int> UIntParameter;
 typedef ParameterV<bool> BoolParameter;
 typedef ParameterV<Glib::ustring> StringParameter;
 
 class FloatEnumParameter;
 class EnumParameter;
-class UEnumParameter;
-class SwitchParameter;
 class FileParameter;
 
 /****************************************************************/
@@ -100,33 +105,46 @@ class FileParameter;
 class Parameter: boost::noncopyable {
 public:
     enum ctrl_type { None, Continuous, Switch, Enum };
+private:
+    virtual bool midi_set(float n, float high, float llimit, float ulimit); //RT
+    virtual void trigger_changed();
+    friend class MidiController;
 protected:
-    enum value_type { tp_float, tp_int, tp_uint, tp_bool, tp_switch, tp_file, tp_string, tp_special };
+    enum value_type { tp_float, tp_int, tp_bool, tp_file, tp_string, tp_special };
+    enum display_flags { dtp_normal, dtp_log = 1 };
     string _id;
     string _name, _group, _desc;
     enum value_type v_type : 3;
-    enum ctrl_type c_type : 3;
-    bool save_in_preset : 1;
-    bool controllable : 1;
-    bool own_var : 1;
-    bool do_not_save : 1;
-    bool used : 1; // debug
+    enum ctrl_type c_type  : 3;
+    unsigned int d_flags   : 2;
+    bool save_in_preset    : 1;
+    bool controllable      : 1;
+    bool do_not_save       : 1;
+    bool blocked           : 1;
+    bool used              : 1; // debug
 protected:
     void range_warning(float value, float lower, float upper);
+    static gx_system::JsonParser& jp_next(gx_system::JsonParser& jp, const char *key);
 public:
+    inline std::string group_id() const { return _id.substr(0, _id.find_last_of(".")); }
     Parameter(const string& id, const string& name, value_type vtp, ctrl_type ctp, bool preset,
               bool ctrl):
+	boost::noncopyable(),
         _id(id),
         _name(name),
-        _group(param_group(id)),
+        _group(param_group(group_id())),
+        _desc(),
         v_type(vtp),
         c_type(ctp),
+	d_flags(0),
         save_in_preset(preset),
         controllable(ctrl),
-	own_var(false),
 	do_not_save(false),
+	blocked(false),
         used(false) {}
+    Parameter(gx_system::JsonParser& jp);
     virtual ~Parameter();
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
 
 #ifndef NDEBUG
     bool isUsed() const { return used; }
@@ -136,11 +154,10 @@ public:
     void dump(gx_system::JsonWriter *jw);
 #endif
 
+    const char *get_typename() const;
     bool isFloat() const { return v_type == tp_float; }
     bool isInt() const { return v_type == tp_int; }
-    bool isUInt() const { return v_type == tp_uint; }
     bool isBool() const { return v_type == tp_bool; }
-    bool isSwitch() const { return v_type == tp_switch; }
     bool isFile() const { return v_type == tp_file; }
     bool isString() const { return v_type == tp_string; }
     ctrl_type getControlType() const { return c_type; }
@@ -156,12 +173,14 @@ public:
     const string& desc() const { return _desc; }
     void set_desc(const string& desc) { _desc = desc; }
     string l_desc() const { return gettext(_desc.c_str()); }
+    void set_log_display() { d_flags |= dtp_log; }
+    bool is_log_display() { return d_flags & dtp_log; }
+    void set_blocked(bool v) { blocked = v; }
+    bool get_blocked() { return blocked; }
     bool operator==(const Parameter& p) const { return &p == this; }
-    virtual void *zone() = 0;
     virtual void stdJSON_value() = 0;
-    virtual void set(float n, float high, float llimit, float ulimit) = 0;
-    virtual bool on_off_value() = 0;
-    virtual void writeJSON(gx_system::JsonWriter& jw) = 0;
+    virtual bool on_off_value() = 0; //RT
+    virtual void writeJSON(gx_system::JsonWriter& jw) const = 0;
     virtual void readJSON_value(gx_system::JsonParser& jp) = 0;
     virtual void setJSON_value() = 0;
     virtual bool compareJSON_value() = 0;
@@ -176,12 +195,13 @@ public:
     FloatParameter& getFloat();
     IntParameter& getInt();
     EnumParameter& getEnum();
-    UIntParameter& getUInt();
-    UEnumParameter& getUEnum();
     BoolParameter& getBool();
-    SwitchParameter& getSwitch();
     FileParameter &getFile();
     StringParameter &getString();
+    sigc::signal<void, float>& signal_changed_float();
+    sigc::signal<void, int>& signal_changed_int();
+    sigc::signal<void, bool>& signal_changed_bool();
+    sigc::signal<void, const Glib::ustring&>& signal_changed_string();
 };
 
 #ifndef NDEBUG
@@ -201,20 +221,24 @@ class ParameterV: public Parameter {
 
 template<>
 class ParameterV<float>: public Parameter {
+private:
+    virtual bool midi_set(float n, float high, float llimit, float ulimit); //RT
+    virtual void trigger_changed();
 protected:
     float json_value;
     float *value;
-public:
     float std_value;
     float lower, upper, step;
-    void set(float val) const { *value = min(max(val, lower), upper); }
-    float &get_value() const { return *value; }
+    sigc::signal<void, float> changed;
+    float value_storage;
+    friend class ParamRegImpl;
+public:
+    bool set(float val) const;
+    float get_value() const { return *value; }
     void convert_from_range(float low, float up);
-    virtual void *zone();
     virtual void stdJSON_value();
     virtual bool on_off_value();
-    virtual void set(float n, float high, float llimit, float ulimit);
-    virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual void writeJSON(gx_system::JsonWriter& jw) const;
     virtual void readJSON_value(gx_system::JsonParser& jp);
     virtual bool compareJSON_value();
     virtual void setJSON_value();
@@ -222,51 +246,59 @@ public:
     virtual float getLowerAsFloat() const;
     virtual float getUpperAsFloat() const;
     virtual float getStepAsFloat() const;
+    virtual float idx_from_id(string v_id);
     ParameterV(const string& id, const string& name, ctrl_type ctp, bool preset,
-	       float *v, float sv, float lv, float uv, float tv, bool ctrl):
+	       float *v, float sv, float lv, float uv, float tv, bool ctrl, bool no_init):
 	Parameter(id, name, tp_float, ctp, preset, ctrl),
-	value(v ? v : new float()), std_value(sv),lower(lv),upper(uv),step(tv) {
-	own_var = !v;
-	*value = sv;
+	value(v ? v : &value_storage), std_value(sv),lower(lv),upper(uv),step(tv) {
+	set(no_init ? *value : sv);
     }
 #ifndef NDEBUG
     friend void compare_parameter(const char* title, Parameter* p1,
 				  Parameter* p2, bool all);
 #endif
     ~ParameterV();
+    ParameterV(gx_system::JsonParser& jp);
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
+    sigc::signal<void, float>& signal_changed() { return changed; }
 };
 
-
 /****************************************************************/
 
 class FloatEnumParameter: public FloatParameter {
- private:
+protected:
     const value_pair* value_names;
- public:
-    virtual void writeJSON(gx_system::JsonWriter& jw);
+    FloatEnumParameter(gx_system::JsonParser& jp): FloatParameter(jp_next(jp, "FloatParameter")), value_names(0) {}
+public:
+    virtual void writeJSON(gx_system::JsonWriter& jw) const;
     virtual void readJSON_value(gx_system::JsonParser& jp);
     virtual const value_pair *getValueNames() const;
     FloatEnumParameter(const string& id, const string& name, const value_pair* vn, bool preset, float *v,
-                       int sv, bool ctrl);
+                       int sv, int low, bool ctrl, bool no_init);
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
+    virtual float idx_from_id(string v_id);
 };
 
 /****************************************************************/
 
 template<>
 class ParameterV<int>: public Parameter {
+private:
+    virtual bool midi_set(float n, float high, float llimit, float ulimit); //RT
+    virtual void trigger_changed();
 protected:
     int json_value;
     int *value;
-public:
     int std_value;
     int lower, upper;
-    void set(int val) const { *value = min(max(val, lower), upper); }
-    int &get_value() const { return *value; }
-    virtual void *zone();
+    sigc::signal<void, int> changed;
+    int value_storage;
+public:
+    bool set(int val) const;
+    int get_value() const { return *value; }
     virtual void stdJSON_value();
     virtual bool on_off_value();
-    virtual void set(float n, float high, float llimit, float ulimit);
-    virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual void writeJSON(gx_system::JsonWriter& jw) const;
     virtual void readJSON_value(gx_system::JsonParser& jp);
     virtual bool compareJSON_value();
     virtual void setJSON_value();
@@ -277,74 +309,30 @@ public:
     ParameterV(const string& id, const string& name, ctrl_type ctp, bool preset,
 	       int *v, int sv, int lv, int uv, bool ctrl):
 	Parameter(id, name, tp_int, ctp, preset, ctrl),
-        value(v ? v : new int()), std_value(sv), lower(lv), upper(uv) {
-	own_var = !v;
+        value(v ? v : &value_storage), std_value(sv), lower(lv), upper(uv) {
 	*value = sv;
     }
     ~ParameterV();
+    ParameterV(gx_system::JsonParser& jp);
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
+    sigc::signal<void, int>& signal_changed() { return changed; }
 };
 
 /****************************************************************/
 
 class EnumParameter: public IntParameter {
- private:
+protected:
     const value_pair* value_names;
- public:
-    virtual void writeJSON(gx_system::JsonWriter& jw);
+    EnumParameter(gx_system::JsonParser& jp): IntParameter(jp_next(jp, "IntParameter")), value_names(0) {}
+public:
+    virtual void writeJSON(gx_system::JsonWriter& jw) const;
     virtual void readJSON_value(gx_system::JsonParser& jp);
     virtual const value_pair *getValueNames() const;
     virtual int idx_from_id(string v_id);
     const value_pair& get_pair() { return getValueNames()[get_value()]; }
     EnumParameter(const string& id, const string& name, const value_pair* vn, bool preset, int *v,
                   int sv, bool ctrl);
-};
-
-/****************************************************************/
-
-template<>
-class ParameterV<unsigned int>: public Parameter {
-protected:
-    unsigned int json_value;
-    unsigned int *value;
-public:
-    unsigned int std_value;
-    unsigned int lower, upper;
-    void set(unsigned int val) const { *value = min(max(val, lower), upper); }
-    unsigned int &get_value() const { return *value; }
-    virtual void *zone();
-    virtual void stdJSON_value();
-    virtual bool on_off_value();
-    virtual void set(float n, float high, float llimit, float ulimit);
-    virtual void writeJSON(gx_system::JsonWriter& jw);
-    virtual void readJSON_value(gx_system::JsonParser& jp);
-    virtual bool compareJSON_value();
-    virtual void setJSON_value();
-    virtual bool hasRange() const;
-    virtual float getLowerAsFloat() const;
-    virtual float getUpperAsFloat() const;
-    ParameterV(const string& id, const string& name, ctrl_type ctp, bool preset,
-	       unsigned int *v, unsigned int sv, unsigned int lv,
-	       unsigned int uv, bool ctrl):
-        Parameter(id, name, tp_uint, ctp, preset, ctrl),
-        value(v ? v : new unsigned int()), std_value(sv), lower(lv), upper(uv) {
-	own_var = !v;
-	*value = sv;
-    }
-    ~ParameterV();
-};
-
-/****************************************************************/
-
-class UEnumParameter: public UIntParameter {
-private:
-    const value_pair* value_names;
-public:
-    virtual void writeJSON(gx_system::JsonWriter& jw);
-    virtual void readJSON_value(gx_system::JsonParser& jp);
-    virtual const value_pair *getValueNames() const;
-    const value_pair& get_pair() { return getValueNames()[get_value()]; }
-    UEnumParameter(const string& id, const string& name, const value_pair* vn, bool preset, unsigned int *v,
-                  unsigned int sv, bool ctrl);
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
 };
 
 /****************************************************************/
@@ -352,74 +340,53 @@ public:
 template<>
 class ParameterV<bool>: public Parameter {
 private:
+    virtual bool midi_set(float n, float high, float llimit, float ulimit); //RT
+    virtual void trigger_changed();
+protected:
     bool json_value;
     bool *value;
-public:
     bool std_value;
-    void set(bool val) const { *value = val; }
+    sigc::signal<void, bool> changed;
+    bool value_storage;
 public:
-    virtual void *zone();
+    bool set(bool val) const;
     virtual void stdJSON_value();
-    bool &get_value() const { return *value; }
+    bool get_value() const { return *value; }
     virtual bool on_off_value();
-    virtual void set(float n, float high, float llimit, float ulimit);
-    virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual void writeJSON(gx_system::JsonWriter& jw) const;
     virtual bool compareJSON_value();
     virtual void setJSON_value();
     virtual void readJSON_value(gx_system::JsonParser& jp);
     ParameterV(const string& id, const string& name, ctrl_type ctp, bool preset,
                   bool *v, bool sv, bool ctrl):
         Parameter(id, name, tp_bool, ctp, preset, ctrl),
-        value(v ? v : new bool()), std_value(sv) {
-	own_var = !v;
+        value(v ? v : &value_storage), std_value(sv) {
 	*value = sv;
     }
     ~ParameterV();
+    ParameterV(gx_system::JsonParser& jp);
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
+    sigc::signal<void, bool>& signal_changed() { return changed; }
 };
 
 /****************************************************************/
 
-class SwitchParameter: public Parameter {
-private:
-    bool value;
-    bool std_value;
-    bool json_value;
-    sigc::signal<void, bool> changed;
-public:
-    sigc::signal<void, bool>& signal_changed() { return changed; }
-    void set(bool val);
-    const bool& get() const { return value; }
-    virtual void *zone();
-    virtual void stdJSON_value();
-    virtual bool on_off_value();
-    virtual void set(float n, float high, float llimit, float ulimit);
-    virtual void writeJSON(gx_system::JsonWriter& jw);
-    virtual void readJSON_value(gx_system::JsonParser& jp);
-    virtual bool compareJSON_value();
-    virtual void setJSON_value();
-    SwitchParameter(const string& id, bool preset = false, bool sv = false):
-        Parameter(id, "", tp_switch, Switch, preset, false),
-        value(sv), std_value(sv), json_value(), changed() {}
-};
-
 /****************************************************************/
 
 class FileParameter: public Parameter {
-private:
+protected:
     Glib::RefPtr<Gio::File> value;
     Glib::RefPtr<Gio::File> std_value;
     Glib::RefPtr<Gio::File> json_value;
     sigc::signal<void> changed;
 public:
     sigc::signal<void>& signal_changed() { return changed; }
-    void set(const Glib::RefPtr<Gio::File>& val);
+    bool set(const Glib::RefPtr<Gio::File>& val);
     void set_path(const string& path);
     const Glib::RefPtr<Gio::File>& get() const { return value; }
-    virtual void *zone();
     virtual void stdJSON_value();
     virtual bool on_off_value();
-    virtual void set(float n, float high, float llimit, float ulimit);
-    virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual void writeJSON(gx_system::JsonWriter& jw) const;
     virtual void readJSON_value(gx_system::JsonParser& jp);
     virtual bool compareJSON_value();
     virtual void setJSON_value();
@@ -431,42 +398,45 @@ public:
         Parameter(id, "", tp_file, None, preset, false),
         value(0),
         std_value(0) {}
+    FileParameter(gx_system::JsonParser& jp);
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
     void set_standard(const string& filename);
     bool is_equal(const Glib::RefPtr<Gio::File>& v) const;
     bool is_standard() const { return is_equal(std_value); }
-    string get_path();
-    string get_directory_path();
-    string get_parse_name();
-    string get_display_name();
-    void copy(const string& destination);
+    string get_path() const;
+    string get_directory_path() const;
+    string get_parse_name() const;
+    string get_display_name() const;
+    void copy(const string& destination) const;
 };
 
 /****************************************************************/
 
 template<>
 class ParameterV<Glib::ustring>: public Parameter {
-private:
+protected:
     Glib::ustring json_value;
     Glib::ustring *value;
-public:
     Glib::ustring std_value;
+    sigc::signal<void, const Glib::ustring&> changed;
+    Glib::ustring value_storage;
 public:
-    void set(const Glib::ustring& val) const { *value = val; }
-    Glib::ustring &get_value() const { return *value; }
-    virtual void *zone();
+    bool set(const Glib::ustring& val) const;
+    const Glib::ustring& get_value() const { return *value; }
     virtual void stdJSON_value();
     virtual bool on_off_value();
-    virtual void set(float n, float high, float llimit, float ulimit);
-    virtual void writeJSON(gx_system::JsonWriter& jw);
+    virtual void writeJSON(gx_system::JsonWriter& jw) const;
     virtual bool compareJSON_value();
     virtual void setJSON_value();
     virtual void readJSON_value(gx_system::JsonParser& jp);
     ParameterV(const string& id, const string& name, Glib::ustring *v, const Glib::ustring& sv, bool preset = false)
 	: Parameter(id, name, tp_string, None, preset, false),
-	  value(v ? v : new Glib::ustring), std_value(sv) {
-	own_var = !v;
+	  value(v ? v : &value_storage), std_value(sv) {
     }
     ~ParameterV();
+    ParameterV(gx_system::JsonParser& jp);
+    virtual void serializeJSON(gx_system::JsonWriter& jw);
+    sigc::signal<void, const Glib::ustring&>& signal_changed() { return changed; }
 };
 
 
@@ -488,27 +458,11 @@ inline EnumParameter &Parameter::getEnum() {
     return *p;
 }
 
-inline UIntParameter &Parameter::getUInt() {
-    assert(isUInt());
-    return static_cast<UIntParameter&>(*this);
-}
-
-inline UEnumParameter &Parameter::getUEnum() {
-    UEnumParameter *p = dynamic_cast<UEnumParameter*>(this);
-    assert(p);
-    return *p;
-}
-
 inline BoolParameter &Parameter::getBool() {
     assert(isBool());
     return static_cast<BoolParameter&>(*this);
 }
 
-inline SwitchParameter &Parameter::getSwitch() {
-    assert(isSwitch());
-    return static_cast<SwitchParameter&>(*this);
-}
-
 inline FileParameter &Parameter::getFile() {
     assert(isFile());
     return static_cast<FileParameter&>(*this);
@@ -519,6 +473,31 @@ inline StringParameter &Parameter::getString() {
     return static_cast<StringParameter&>(*this);
 }
 
+inline sigc::signal<void, float>& Parameter::signal_changed_float() {
+    FloatParameter *p = dynamic_cast<FloatParameter*>(this);
+    assert(p);
+    return p->signal_changed();
+}
+
+inline sigc::signal<void, int>& Parameter::signal_changed_int() {
+    IntParameter *p = dynamic_cast<IntParameter*>(this);
+    assert(p);
+    return p->signal_changed();
+}
+
+inline sigc::signal<void, bool>& Parameter::signal_changed_bool() {
+    BoolParameter *p = dynamic_cast<BoolParameter*>(this);
+    assert(p);
+    return p->signal_changed();
+}
+
+inline sigc::signal<void, const Glib::ustring&>& Parameter::signal_changed_string() {
+    StringParameter *p = dynamic_cast<StringParameter*>(this);
+    assert(p);
+    return p->signal_changed();
+}
+
+
 /****************************************************************
  ** ParamMap
  */
@@ -526,30 +505,29 @@ inline StringParameter &Parameter::getString() {
 class ParamMap: boost::noncopyable {
  private:
     map<string, Parameter*> id_map;
-    map<const void*, Parameter*> addr_map;
+    bool replace_mode;
+    sigc::signal<void,Parameter*,bool> insert_remove;
 #ifndef NDEBUG
-    void unique_zone(Parameter* param);
     void unique_id(Parameter* param);
     void check_id(const string& id);
     void check_p(const char *p);
-    void check_addr(const void *p);
 #endif
-    void insert(Parameter* param); // private so we can make sure parameters are owned
+    Parameter *insert(Parameter* param); // private so we can make sure parameters are owned
 
  public:
     template<class T> friend class ParameterV;
     ParamMap();
     ~ParamMap();
+    void writeJSON(gx_system::JsonWriter& jw);
+    void readJSON(gx_system::JsonParser& jp);
+    Parameter *readJSON_one(gx_system::JsonParser& jp);
+    void writeJSON_one(gx_system::JsonWriter& jw, Parameter *p);
     typedef map<string, Parameter*>::const_iterator iterator;
     iterator begin() const { return id_map.begin(); }
     iterator end() const { return id_map.end(); }
-    bool hasZone(const void *p) const { return addr_map.find(p) != addr_map.end(); }
     bool hasId(const string& id) const { return id_map.find(id) != id_map.end(); }
     bool hasId(const char *p) const { return id_map.find(p) != id_map.end(); }
-    Parameter& operator[](const void *p) {
-        debug_check(check_addr, p);
-        return *addr_map[p];
-    }
+    void set_replace_mode(bool mode) { replace_mode = mode; }
     Parameter& operator[](const string& id) {
         debug_check(check_id, id);
         return *id_map[id];
@@ -559,22 +537,25 @@ class ParamMap: boost::noncopyable {
         return *id_map[p];
     }
     void set_init_values();
-    void reset_unit(Glib::ustring group_id) const;
-    bool unit_has_std_values(Glib::ustring group_id) const;
+    void reset_unit(const PluginDef *pdef) const;
+    bool unit_has_std_values(const PluginDef *pdef) const;
+    sigc::signal<void,Parameter*,bool> signal_insert_remove() { return insert_remove; }
+    void unregister(Parameter *p);
+    void unregister(const string& id);
     inline FloatParameter *reg_par(const string& id, const string& name, float *var, float std,
 				   float lower, float upper, float step) {
-	FloatParameter *p = new FloatParameter(id, name, Parameter::Continuous, true, var, std, lower, upper, step, true);
+	FloatParameter *p = new FloatParameter(id, name, Parameter::Continuous, true, var, std, lower, upper, step, true, replace_mode);
 	insert(p);
 	return p;
     }
     inline FloatParameter *reg_par_non_preset(
 	const string& id, const string& name, float *var, float std, float lower, float upper, float step) {
-	FloatParameter *p = new FloatParameter(id, name, Parameter::Continuous, false, var, std, lower, upper, step, false);
+	FloatParameter *p = new FloatParameter(id, name, Parameter::Continuous, false, var, std, lower, upper, step, false, replace_mode);
 	insert(p);
 	return p;
     }
     inline FloatParameter *reg_par(const string& id, const string& name, float *var, float std = 0) {
-	FloatParameter *p = new FloatParameter(id, name, Parameter::Switch, true, var, std, 0, 1, 1, true);
+	FloatParameter *p = new FloatParameter(id, name, Parameter::Switch, true, var, std, 0, 1, 1, true, replace_mode);
 	insert(p);
 	return p;
     }
@@ -598,14 +579,8 @@ class ParamMap: boost::noncopyable {
     }
     inline FloatEnumParameter *reg_enum_par(const string& id, const string& name,
 					    const value_pair *vl, float *var,
-					    int std = 0) {
-	FloatEnumParameter *p = new FloatEnumParameter(id, name, vl, true, var, std, true);
-	insert(p);
-	return p;
-    }
-    inline UEnumParameter *reg_uenum_par(const string& id, const string& name, const value_pair *vl,
-					 unsigned int *var, unsigned int std = 0) {
-	UEnumParameter *p = new UEnumParameter(id, name, vl, true, var, std, true);
+					    int std = 0, int low = 0) {
+	FloatEnumParameter *p = new FloatEnumParameter(id, name, vl, true, var, std, low, true, replace_mode);
 	insert(p);
 	return p;
     }
@@ -621,12 +596,7 @@ class ParamMap: boost::noncopyable {
     }
     inline FloatParameter *reg_non_midi_par(const string& id, float *val, bool preset,
 				 float std = 0, float lower = 0, float upper = 1, float step = 0) {
-	FloatParameter *p = new FloatParameter(id, "", Parameter::None, preset, val, std, lower, upper, step, false);
-	insert(p);
-	return p;
-    }
-    inline SwitchParameter *reg_switch(const string& id, bool preset = false, bool sv = false) {
-	SwitchParameter *p = new SwitchParameter(id, preset, sv);
+	FloatParameter *p = new FloatParameter(id, "", Parameter::None, preset, val, std, lower, upper, step, false, replace_mode);
 	insert(p);
 	return p;
     }
@@ -646,8 +616,6 @@ class ParamMap: boost::noncopyable {
 #endif
 };
 
-extern ParamMap parameter_map; // map id -> parameter, zone -> parameter
-
 /****************************************************************
  **
  ** Midi
@@ -690,9 +658,9 @@ extern MidiStandardControllers midi_std_ctr; // map ctrl num -> standard name
 
 class MidiController {
  private:
-    Parameter *param;
-    float _lower, _upper;
-    bool toggle;
+    Parameter *param; //RT
+    float _lower, _upper; //RT
+    bool toggle; //RT
  public:
     MidiController(Parameter& p, float l, float u, bool t=false):
         param(&p), _lower(l), _upper(u), toggle(t) {}
@@ -701,9 +669,10 @@ class MidiController {
     bool is_toggle() const { return toggle; }
     bool hasParameter(const Parameter& p) const { return *param == p; }
     Parameter& getParameter() const { return *param; }
-    static MidiController* readJSON(gx_system::JsonParser&, ParamMap& param);
-    void set_midi(int n, int last_value);
-    void set(float v, float high) { param->set(v, high, _lower, _upper); }
+    static MidiController *readJSON(gx_system::JsonParser& jp, ParamMap& param);
+    bool set_midi(int n, int last_value); //RT
+    void set(float v, float high) { param->midi_set(v, high, _lower, _upper); }
+    void trigger_changed() { param->trigger_changed(); }
     void writeJSON(gx_system::JsonWriter& jw) const;
 };
 
@@ -714,50 +683,60 @@ typedef list<MidiController> midi_controller_list;
 ** MidiControllerList
 **/
 
-class MidiControllerList {
- public:
-    typedef vector<midi_controller_list> controller_array;
-    enum { controller_array_size = 128 };
- private:
-    controller_array       map;
-    static int             last_midi_control_value[controller_array_size];
-    bool                   midi_config_mode;
-    int                    last_midi_control;
-    volatile gint          program_change;
+class ControllerArray: public vector<midi_controller_list> {
+public:
+    enum { array_size = 128 };
+    ControllerArray(): vector<midi_controller_list>(array_size) {}
+    ~ControllerArray() {}
+    void writeJSON(gx_system::JsonWriter& jw) const;
+    void readJSON(gx_system::JsonParser& jp, ParamMap& param);
+    int param2controller(Parameter& param, const MidiController** p);
+    bool deleteParameter(Parameter& p);
+};
+
+class MidiControllerList: public sigc::trackable {
+public:
+private:
+    ControllerArray        map; //RT
+    int                    last_midi_control_value[ControllerArray::array_size]; //RT
+    int                    last_midi_control; //RT
+    volatile gint          program_change; //RT
     Glib::Dispatcher       pgm_chg;
     sigc::signal<void>     changed;
     sigc::signal<void,int> new_program;
+    sigc::signal<void, int, int> midi_value_changed;
+private:
     void               on_pgm_chg();
- public:
+    bool check_midi_values();
+public:
     MidiControllerList();
     midi_controller_list& operator[](int n) { return map[n]; }
     int size() { return map.size(); }
     void set_config_mode(bool mode, int ctl=-1);
-    bool get_config_mode() { return midi_config_mode; }
+    bool get_config_mode() { return last_midi_control != -2; }
     int get_current_control() { return last_midi_control; }
     void set_current_control(int ctl) { last_midi_control = ctl; }
-    void set_ctr_val(int ctr, int val);
-    void deleteParameter(Parameter& param, bool quiet = false);
+    void set_ctr_val(int ctr, int val); //RT
+    void deleteParameter(Parameter& param);
     void modifyCurrent(Parameter& param, float lower, float upper, bool toggle);
-    int param2controller(Parameter& param, const MidiController** p);
-    void writeJSON(gx_system::JsonWriter& jw);
-    static void readJSON(gx_system::JsonParser& jp, ParamMap& param, controller_array& m);
-    static controller_array* create_controller_array() {
-	return new controller_array(controller_array_size); }
-    static int get_last_midi_control_value(unsigned int n) { assert(n < controller_array_size); return last_midi_control_value[n]; }
-    static void set_last_midi_control_value(unsigned int n, int v) { assert(n < controller_array_size); last_midi_control_value[n] = v; }
-    static void *get_midi_control_zone(unsigned int n) { assert(n < controller_array_size); return &last_midi_control_value[n]; }
-    void set_controller_array(const controller_array& m);
-    void remove_controlled_parameters(paramlist& plist, const controller_array *m);
+    int param2controller(Parameter& param, const MidiController** p) {
+	return map.param2controller(param, p); }
+    void writeJSON(gx_system::JsonWriter& jw) const { map.writeJSON(jw); }
+    int get_last_midi_control_value(unsigned int n) {
+	assert(n < ControllerArray::array_size); return last_midi_control_value[n]; } //RT
+    void set_last_midi_control_value(unsigned int n, int v) {
+	assert(n < ControllerArray::array_size); last_midi_control_value[n] = v; } //RT
+    void set_controller_array(const ControllerArray& m);
+    void remove_controlled_parameters(paramlist& plist, const ControllerArray *m);
     sigc::signal<void>& signal_changed() { return changed; }
     sigc::signal<void,int>& signal_new_program() { return new_program; }
-    void compute_midi_in(void* midi_input_port_buf);
+    void compute_midi_in(void* midi_input_port_buf);  //RT
     void update_from_controller(int ctr);
     void update_from_controllers();
+    sigc::signal<void, int, int>& signal_midi_value_changed() { return midi_value_changed; }
+    void request_midi_value_update();
 };
 
-extern MidiControllerList controller_map; // map ctrl num -> controlled parameters
-
 } // namespace gx_gui
 
 #endif  // SRC_HEADERS_GX_PARAMETER_H_
