@@ -25,6 +25,8 @@
 #ifndef SRC_HEADERS_GX_PITCH_TRACKER_H_
 #define SRC_HEADERS_GX_PITCH_TRACKER_H_
 
+#include <fftw3.h>
+
 namespace gx_engine {
 /* ------------- Pitch Tracker ------------- */
 
@@ -34,9 +36,11 @@ class PitchTracker {
     ~PitchTracker();
     void            init(int priority, int policy, unsigned int samplerate);
     void            add(int count, float *input);
-    float           get_estimated_freq() { return m_freq; }
+    float           get_estimated_freq() { return m_freq < 0 ? 0 : m_freq; }
     float           get_estimated_note();
     void            stop_thread();
+    void            reset();
+    void            set_fast_note_detection(bool v);
     Glib::Dispatcher new_freq;
  private:
     bool            setParameters(int priority, int policy, int sampleRate, int fftSize );
@@ -52,6 +56,14 @@ class PitchTracker {
     Resampler       resamp;
     int             m_sampleRate;
     float           m_freq;
+    // Value of the threshold above which
+    // the processing is activated.
+    float           signal_threshold_on;
+    // Value of the threshold below which
+    // the input audio signal is deactivated.
+    float           signal_threshold_off;
+    // Time between frequency estimates (in seconds)
+    float           tracker_period;
     // number of samples in input buffer
     int             m_buffersize;
     // Size of the FFT window.
