@@ -86,10 +86,26 @@ inline void atomic_set(volatile int* p, int v) {
     g_atomic_int_set(p, v);
 }
 
+inline void atomic_set(volatile unsigned int* p, unsigned int v) {
+    g_atomic_int_set(reinterpret_cast<volatile int*>(p), v);
+}
+
 inline int atomic_get(volatile int& p) {
     return g_atomic_int_get(&p);
 }
 
+inline unsigned int atomic_get(volatile unsigned int& p) {
+    return g_atomic_int_get(reinterpret_cast<volatile int*>(&p));
+}
+
+inline void atomic_inc(volatile int* p) {
+    g_atomic_int_inc(p);
+}
+
+inline void atomic_inc(volatile unsigned int* p) {
+    g_atomic_int_inc(reinterpret_cast<volatile int*>(p));
+}
+
 inline bool atomic_compare_and_exchange(volatile int *p, int oldv, int newv) {
     return g_atomic_int_compare_and_exchange(p, oldv, newv);
 }
@@ -140,7 +156,7 @@ class Accum {
     void add(int diff);
     int count() const { return n; }
     float mean() const { return sx / n; }
-    float stddev() const { return sqrt((n * sx2 - sx * sx) / (n * (n-1))); }
+    float stddev() const { return std::sqrt((n * sx2 - sx * sx) / (n * (n-1))); }
     float minimum() const { return mn; }
     float maximum() const { return mx; }
 };
@@ -149,8 +165,8 @@ inline void Accum::add(int diff) {
     n += 1;
     sx += diff;
     sx2 += static_cast<float>(diff) * diff;
-    mn = min(mn, diff);
-    mx = max(mx, diff);
+    mn = std::min(mn, diff);
+    mx = std::max(mx, diff);
 }
 
 
@@ -206,7 +222,7 @@ class MeasureThreadsafe {
 /* return time difference in ns, fail if > sec (doesn't fit int 32 bit int) */
 inline int MeasureThreadsafe::ts_diff(const timespec& ts1, const timespec& ts2) {
     time_t df = ts1.tv_sec - ts2.tv_sec;
-    if (abs(df) > 2) {
+    if (std::abs(df) > 2) {
         return -1; // failed
     }
     return df * 1000000000 + (ts1.tv_nsec - ts2.tv_nsec);
@@ -251,207 +267,239 @@ inline void measure_stop()  {}
 /****************************************************************/
 
 class SkinHandling {
+private:
+    Glib::ustring empty;
 public:
-    vector<string>   skin_list;
-    SkinHandling(const string& styledir)
+    std::vector<Glib::ustring>   skin_list;
+    SkinHandling(const std::string& styledir)
 	: skin_list() { set_styledir(styledir); }
-    void set_styledir(const string& styledir);
-    bool is_in_list(const string& name);
+    void set_styledir(const std::string& styledir);
+    bool is_in_list(const std::string& name);
+    const Glib::ustring& operator[](unsigned int idx);
+    unsigned int index(const Glib::ustring& name);
 };
 
 /****************************************************************/
 
 class PathList {
+public:
+    typedef std::list< Glib::RefPtr<Gio::File> > pathlist;
+    typedef std::list< Glib::RefPtr<Gio::File> >::const_iterator iterator;
 private:
-    typedef list< Glib::RefPtr<Gio::File> > pathlist;
     pathlist dirs;
 public:
-    PathList(): dirs() {}
-    void add(const string& d) { dirs.push_back(Gio::File::create_for_path(d)); }
-    bool contains(const string& d) const;
-    bool find_dir(string *d, const string& filename) const;
+    PathList(const char *env_name = 0);
+    void add(const std::string& d) { dirs.push_back(Gio::File::create_for_path(d)); }
+    bool contains(const std::string& d) const;
+    bool find_dir(std::string *d, const std::string& filename) const;
+    size_t size() { return dirs.size(); }
+    iterator begin() { return dirs.begin(); }
+    iterator end() { return dirs.end(); }
 };
 
-/****************************************************************
- ** CmdlineParser
+
+/****************************************************************/
+
+class PrefixConverter {
+public:
+    typedef std::map<char,std::string> symbol_path_map;
+private:
+    symbol_path_map dirs;
+public:
+    PrefixConverter(): dirs() {}
+    ~PrefixConverter() {}
+    std::string replace_symbol(const std::string& dir) const;
+    std::string replace_path(const std::string& dir) const;
+    void add(char s, const std::string& d);
+};
+
+
+/*****************************************************************
+ ** class DirectoryListing
  */
 
-class CmdlineOptions: public Glib::OptionContext, boost::noncopyable {
+class FileName {
+public:
+    std::string filename;
+    Glib::ustring displayname;
+    FileName(const std::string& filename_, const Glib::ustring& displayname_)
+	: filename(filename_), displayname(displayname_) {}
+};
+
+class IRFileListing {
+public:
+private:
+    std::vector<FileName> listing;
+public:
+    IRFileListing(const std::string& path);
+    ~IRFileListing() {}
+    std::vector<FileName>& get_listing() { return listing; }
+};
+
+void list_subdirs(PathList pl, std::vector<FileName>& dirs);
+
+/****************************************************************
+ ** class CmdlineOptions
+ */
+
+#define RPCPORT_DEFAULT (-2)
+#define RPCPORT_NONE (-1)
+
+class BasicOptions: boost::noncopyable {
+private:
+    std::string user_dir;
+    std::string user_IR_dir;
+    std::string sys_IR_dir;
+    PathList IR_pathlist;
+    PrefixConverter IR_prefixmap;
+    static BasicOptions *instance;
+protected:
+    std::string builder_dir;
+
+private:
+    friend BasicOptions& get_options();
+protected:
+    static void make_ending_slash(std::string& dirpath);
+public:
+    BasicOptions();
+    ~BasicOptions();
+    std::string get_user_filepath(const std::string& basename) const { return user_dir + basename; }
+    std::string get_user_ir_filepath(const std::string& basename) const { return user_IR_dir + basename; }
+    std::string get_builder_filepath(const std::string& basename) const { return builder_dir + basename; }
+    const std::string& get_user_dir() const { return user_dir; }
+    const std::string& get_user_IR_dir() const { return user_IR_dir; }
+    const std::string& get_sys_IR_dir() const { return sys_IR_dir; }
+    const PathList& get_IR_pathlist() const { return IR_pathlist; }
+    const PrefixConverter& get_IR_prefixmap() const { return IR_prefixmap; }
+};
+
+class CmdlineOptions: public BasicOptions, public Glib::OptionContext {
 private:
     Glib::OptionGroup main_group;
     Glib::OptionGroup optgroup_style;
     Glib::OptionGroup optgroup_jack;
+    Glib::OptionGroup optgroup_overload;
     Glib::OptionGroup optgroup_file;
     Glib::OptionGroup optgroup_debug;
-    string path_to_program;
+    std::string path_to_program;
     bool version;
     bool clear;
     Glib::ustring jack_input;
     Glib::ustring jack_midi;
     Glib::ustring jack_instance;
-    vector<Glib::ustring> jack_outputs;
+    std::vector<Glib::ustring> jack_outputs;
     Glib::ustring jack_uuid;
     Glib::ustring jack_uuid2;
-    string load_file;
-    string builder_dir;
-    string style_dir;
-    string factory_dir;
-    string pixmap_dir;
-    string user_dir;
-    string old_user_dir;
-    string preset_dir;
-    string pluginpreset_dir;
-    string plugin_dir;
-    string sys_IR_dir;
-    PathList IR_pathlist;
+    bool jack_noconnect;
+    Glib::ustring jack_servername;
+    std::string load_file;
+    std::string style_dir;
+    std::string factory_dir;
+    std::string pixmap_dir;
+    std::string old_user_dir;
+    std::string preset_dir;
+    std::string pluginpreset_dir;
+    std::string temp_dir;
+    std::string plugin_dir;
     Glib::ustring rcset;
+    bool nogui;
+    int rpcport;
+    Glib::ustring rpcaddress;
+    bool onlygui;
+    int sporadic_overload;
+    int idle_thread_timeout;
+    bool convolver_watchdog;
+    bool xrun_watchdog;
     bool lterminal;
-    static CmdlineOptions *instance;
-    void make_ending_slash(string& dirpath);
-    string get_opskin();
-    friend CmdlineOptions& get_options();
+    bool a_save;
+    bool auto_save;
+    std::string get_opskin();
+    void read_ui_vars();
+    void write_ui_vars();
 
 public:
 #ifndef NDEBUG
     bool dump_parameter;
 #endif
     SkinHandling skin;
+
+    // saved in ui_rc:
+    int mainwin_x;
+    int mainwin_y;
+    int mainwin_height;
+    int window_height;
+    int preset_window_height;
+    int mul_buffer;
+    Glib::ustring skin_name;
+    bool no_warn_latency;
+    bool system_order_rack_h;
+    bool system_show_value;
+    bool system_show_tooltips;
+    bool system_animations;
+    bool system_show_presets;
+    bool system_show_toolbar;
+    bool system_show_rack;
+
 public:
     CmdlineOptions();
     ~CmdlineOptions();
     void process(int argc, char** argv);
-    const string& get_path_to_program() const { return path_to_program; }
-    string get_style_filepath(const string& basename) const { return style_dir + basename; }
-    string get_pixmap_filepath(const string& basename) const { return pixmap_dir + basename; }
-    string get_builder_filepath(const string& basename) const { return builder_dir + basename; }
-    string get_user_filepath(const string& basename) const { return user_dir + basename; }
-    string get_preset_filepath(const string& basename) const { return preset_dir + basename; }
-    string get_factory_filepath(const string& basename) const { return factory_dir + basename; }
-    string get_pluginpreset_filepath(const string& id) const { return pluginpreset_dir + id; }
-    const string& get_user_dir() const { return user_dir; }
-    const string& get_old_user_dir() const { return old_user_dir; }
-    const string& get_plugin_dir() const { return plugin_dir; }
-    const string& get_preset_dir() const { return preset_dir; }
-    const string& get_pluginpreset_dir() const { return pluginpreset_dir; }
-    const string& get_factory_dir() const { return factory_dir; }
-    const string& get_sys_IR_dir() const { return sys_IR_dir; }
+    const std::string& get_path_to_program() const { return path_to_program; }
+    std::string get_style_filepath(const std::string& basename) const { return style_dir + basename; }
+    std::string get_pixmap_filepath(const std::string& basename) const { return pixmap_dir + basename; }
+    std::string get_preset_filepath(const std::string& basename) const { return preset_dir + basename; }
+    std::string get_plugin_filepath(const std::string& basename) const { return plugin_dir + basename; }
+    std::string get_factory_filepath(const std::string& basename) const { return factory_dir + basename; }
+    std::string get_temp_filepath(const std::string& basename) const { return temp_dir + basename; }
+    std::string get_pluginpreset_filepath(const std::string& id, bool factory) const {
+	return (factory ? factory_dir : pluginpreset_dir) + id; }
+    const std::string& get_old_user_dir() const { return old_user_dir; }
+    const std::string& get_plugin_dir() const { return plugin_dir; }
+    const std::string& get_preset_dir() const { return preset_dir; }
+    const std::string& get_pluginpreset_dir() const { return pluginpreset_dir; }
+    const std::string& get_temp_dir() const { return temp_dir; }
+    const std::string& get_factory_dir() const { return factory_dir; }
+    std::string get_ladspa_config_filename() const { return get_user_filepath("ladspa_defs.js"); }
     const Glib::ustring& get_rcset() const { return rcset; }
-    const string& get_loadfile() const { return load_file; }
+    bool get_clear_rc() const { return clear; }
+    bool get_nogui() const { return nogui; }
+    int get_rpcport() const { return rpcport; }
+    void set_rpcport(int port) { rpcport = port; }
+    const Glib::ustring& get_rpcaddress() { return rpcaddress; }
+    void set_rpcaddress(const Glib::ustring& address) { rpcaddress = address; }
+    const std::string& get_loadfile() const { return load_file; }
     const Glib::ustring& get_jack_instancename() const { return jack_instance; }
     const Glib::ustring& get_jack_uuid() const { return jack_uuid; }
     const Glib::ustring& get_jack_uuid2() const { return jack_uuid2; }
     const Glib::ustring& get_jack_midi() const { return jack_midi; }
     const Glib::ustring& get_jack_input() const { return jack_input; }
-    const PathList& get_IR_pathlist() const { return IR_pathlist; }
+    const Glib::ustring& get_jack_servername() const { return jack_servername; }
+    bool get_jack_noconnect() const { return jack_noconnect; }
+    bool get_opt_save_on_exit() const { return a_save; }
+    bool get_opt_autosave() const { return auto_save; }
     Glib::ustring get_jack_output(unsigned int n) const;
+    int get_idle_thread_timeout() const { return idle_thread_timeout; }
+    int get_sporadic_overload() const { return sporadic_overload; }
+    bool get_xrun_watchdog() const { return xrun_watchdog; }
+    bool get_convolver_watchdog() const { return convolver_watchdog; }
 };
 
-inline CmdlineOptions& get_options() {
-    assert(CmdlineOptions::instance);
-    return *CmdlineOptions::instance;
+inline BasicOptions& get_options() {
+    assert(BasicOptions::instance);
+    return *BasicOptions::instance;
 }
 
-/****************************************************************
- ** Logging
- */
-
-typedef enum {
-    kInfo,
-    kWarning,
-    kError,
-    kMessageTypeCount // just count, must be last
-} GxMsgType;
-
-class Logger: public sigc::trackable {
-private:
-    typedef sigc::signal<void, const string&, GxMsgType, bool> msg_signal;
-    struct logmsg {
-	string msg;
-	GxMsgType msgtype;
-	bool plugged;
-	logmsg(string m, GxMsgType t, bool p): msg(m), msgtype(t), plugged(p) {}
-    };
-    list<logmsg> msglist;
-    boost::mutex msgmutex;
-    Glib::Dispatcher* got_new_msg;
-    pthread_t ui_thread;
-    msg_signal handlers;
-    bool queue_all_msgs;
-    string format(const char* func, const string& msg);
-    void set_ui_thread();
-    Logger();
-    ~Logger();
-    void write_queued();
-    friend class LoggerGuard;
-public:
-    void unplug_queue();
-    msg_signal& signal_message();
-    void print(const char* func, const string& msg, GxMsgType msgtype);
-    static Logger& get_logger();
-    static void destroy();
-};
-
-void  gx_print_logmsg(const char*, const string&, GxMsgType);
-void  gx_print_warning(const char*, const string&);
-inline void gx_print_warning(const char* fnc, const boost::basic_format<char>& msg) {
-    gx_print_warning(fnc, msg.str());
-}
-void  gx_print_error(const char*, const string&);
-inline void gx_print_error(const char* fnc, const boost::basic_format<char>& msg) {
-    gx_print_error(fnc, msg.str());
-}
-void  gx_print_fatal(const char*, const string&);
-inline void gx_print_fatal(const char* fnc, const boost::basic_format<char>& msg) {
-    gx_print_fatal(fnc, msg.str());
-}
-void  gx_print_info(const char*, const string&);
-inline void gx_print_info(const char* fnc, const boost::basic_format<char>& msg) {
-    gx_print_info(fnc, msg.str());
-}
-
-class GxFatalError: public exception {
-private:
-    string msg;
-public:
-    virtual const char* what() const throw() {
-	return msg.c_str();
-    }
-    GxFatalError(string m): msg(m) {}
-    GxFatalError(boost::basic_format<char>& m): msg(m.str()) {}
-    ~GxFatalError() throw();
-};
-
-
-/****************************************************************
- ** class GxExit
- */
-
-class GxExit {
-private:
-    sigc::signal<void, bool> exit_sig;
-    pthread_t ui_thread;
-    sigc::signal<void,string> message;
-public:
-    GxExit();
-    ~GxExit();
-    void set_ui_thread() { ui_thread = pthread_self(); }
-    sigc::signal<void, bool>& signal_exit() { return exit_sig; }
-    sigc::signal<void,string>& signal_msg() { return message; }
-    void exit_program(string msg = "", int errcode = 1);
-    void fatal_msg(const string& msg) { message(msg); exit_program(msg); }
-    static GxExit& get_instance();
-};
-
 
 /****************************************************************
  ** misc function declarations
  */
 
-int  gx_system_call(const string&, bool devnull = false, bool escape = false);
+int  gx_system_call(const std::string&, bool devnull = false, bool escape = false);
+void strip(Glib::ustring& s);
 
 template <class T>
-inline string to_string(const T& t) {
-    stringstream ss;
+inline std::string to_string(const T& t) {
+    std::stringstream ss;
     ss << t;
     return ss.str();
 }
