@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2011, 2012 Andreas Degert, Hermann Meyer
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
+#define always_inline inline __attribute__((always_inline))
+
 #include <stdlib.h>
 #include <string.h>
 #include <ladspa.h>
@@ -23,13 +43,14 @@ using namespace gx_engine;
  *                       GENERAL CLASSES                        *
  ****************************************************************/
 
+
 /****************************************************************
  ** class ControlParameter
  */
 
 class ControlParameter {
 private:
-    MidiControllerList::controller_array *midi_control;
+    ControllerArray *midi_control;
     list<midi_controller_list*> ctlr;
     boost::mutex control_mutex;
     vector<LADSPA_Data*> parameter_port;
@@ -39,8 +60,8 @@ public:
     ControlParameter(int sz);
     ~ControlParameter();
     void set_port(unsigned int n, LADSPA_Data *p);
-    MidiControllerList::controller_array *readJSON(gx_system::JsonParser &jp, ParamMap& param);
-    void set_array(MidiControllerList::controller_array *m);
+    ControllerArray *readJSON(gx_system::JsonParser &jp, ParamMap& param);
+    void set_array(ControllerArray *m);
     void get_values();
 };
 
@@ -65,10 +86,10 @@ void ControlParameter::set_port(unsigned int n, LADSPA_Data *p) {
     }
 }
 
-MidiControllerList::controller_array *ControlParameter::readJSON(
+ControllerArray *ControlParameter::readJSON(
     gx_system::JsonParser &jp, ParamMap& param) {
-    MidiControllerList::controller_array *m = MidiControllerList::create_controller_array();
-    MidiControllerList::readJSON(jp, param, *m);
+    ControllerArray *m = new ControllerArray();
+    m->readJSON(jp, param);
     return m;
 }
 
@@ -81,12 +102,12 @@ void ControlParameter::log_assignment(int ctlr, int var, const midi_controller_l
 	}
 	s += p.l_group() + ": " + p.l_name();
     }
-    gx_system::gx_print_info(
+    gx_print_info(
 	_("assign parameter"),
 	boost::format(_("%1% -> controller %2% [%3%]")) % var % ctlr % s);
 }
 
-void ControlParameter::set_array(MidiControllerList::controller_array *m) {
+void ControlParameter::set_array(ControllerArray *m) {
     boost::mutex::scoped_lock lock(control_mutex);
     ctlr.clear();
     delete midi_control;
@@ -135,12 +156,13 @@ private:
     GxJConvSettings *jcset;
     ParamMap& param;
     paramlist plist;
-    MidiControllerList::controller_array *midi_list;
-    ConvolverAdapter* convolver;
+    ControllerArray *midi_list;
+    ConvolverStereoAdapter* stereo_convolver;
+    ConvolverMonoAdapter* mono_convolver;
     ControlParameter& control_parameter;
     friend class StateIO;
 public:
-    PresetIO(ParamMap& param, ConvolverAdapter* convolver, ControlParameter& cp);
+    PresetIO(ParamMap& param, ConvolverStereoAdapter* stereo_convolver,ConvolverMonoAdapter* mono_convolver, ControlParameter& cp);
     ~PresetIO();
     void read_preset(gx_system::JsonParser &jp, const gx_system::SettingsFileHeader&);
     void commit_preset();
@@ -150,12 +172,13 @@ public:
 };
 
 // cp not yet initialized, only use address!
-PresetIO::PresetIO(ParamMap& param_, ConvolverAdapter* convolver_, ControlParameter& cp)
+PresetIO::PresetIO(ParamMap& param_, ConvolverStereoAdapter* stereo_convolver_, ConvolverMonoAdapter* mono_convolver_, ControlParameter& cp)
     : jcset(0),
       param(param_),
       plist(),
       midi_list(0),
-      convolver(convolver_),
+      stereo_convolver(stereo_convolver_),
+      mono_convolver(mono_convolver_),
       control_parameter(cp) {
       }
 
@@ -182,7 +205,7 @@ void PresetIO::read_preset(gx_system::JsonParser &jp, const gx_system::SettingsF
 		jp.next(gx_system::JsonParser::value_key);
 		if (!param.hasId(jp.current_value())) {
 #if 0   // lots of warnings
-		    gx_system::gx_print_warning(
+		    gx_print_warning(
 			_("recall settings"),
 			_("unknown parameter: ")+jp.current_value());
 #endif
@@ -195,19 +218,19 @@ void PresetIO::read_preset(gx_system::JsonParser &jp, const gx_system::SettingsF
 	    } while (jp.peek() == gx_system::JsonParser::value_key);
 	    jp.next(gx_system::JsonParser::end_object);
         } else if (jp.current_value() == "jconv") {
-	    if (convolver) {
-		gx_system::PathList sl;
-		sl.add(Glib::build_filename(Glib::get_user_config_dir(), "guitarix")); //FIXME
-		sl.add(GX_SOUND_DIR);
+	    if (stereo_convolver) {
 		jcset = new GxJConvSettings();
-		jcset->readJSON(jp, sl);
+		jcset->readJSON(jp);
+	    } else if (mono_convolver) {
+		jcset = new GxJConvSettings();
+		jcset->readJSON(jp);
 	    } else {
 		jp.skip_object();
 	    }
         } else if (jp.current_value() == "midi_controller") {
 	    midi_list = control_parameter.readJSON(jp, param);
         } else {
-            gx_system::gx_print_warning(
+            gx_print_warning(
 		_("recall settings"),
 		_("unknown preset section: ") + jp.current_value());
 	    jp.skip_object();
@@ -217,8 +240,10 @@ void PresetIO::read_preset(gx_system::JsonParser &jp, const gx_system::SettingsF
 }
 
 void PresetIO::commit_preset() {
-    if (convolver && jcset) {
-	convolver->set(*jcset);
+    if (stereo_convolver && jcset) {
+	stereo_convolver->set(*jcset);
+    } else if (mono_convolver && jcset) {
+	mono_convolver->set(*jcset);
     }
     for (gx_engine::paramlist::iterator i = plist.begin(); i != plist.end(); ++i) {
         (*i)->setJSON_value();
@@ -243,7 +268,7 @@ void PresetIO::copy_preset(gx_system::JsonParser &jp, const gx_system::SettingsF
 
 class StateIO: public gx_system::AbstractStateIO, private PresetIO {
 public:
-    StateIO(ParamMap& param, ConvolverAdapter* convolver, ControlParameter& cp);
+    StateIO(ParamMap& param, ConvolverStereoAdapter* stereo_convolver, ConvolverMonoAdapter* mono_convolver, ControlParameter& cp);
     ~StateIO();
     void read_state(gx_system::JsonParser &jp, const gx_system::SettingsFileHeader&);
     void commit_state();
@@ -251,15 +276,15 @@ public:
 };
 
 // cp not yet initialized, only use address!
-StateIO::StateIO(ParamMap& param, ConvolverAdapter* convolver, ControlParameter& cp)
-    : PresetIO(param, convolver, cp) {
+StateIO::StateIO(ParamMap& param, ConvolverStereoAdapter* stereo_convolver, ConvolverMonoAdapter* mono_convolver, ControlParameter& cp)
+    : PresetIO(param, stereo_convolver, mono_convolver, cp) {
 }
 
 StateIO::~StateIO() {
 }
 
 void StateIO::read_state(gx_system::JsonParser &jp, const gx_system::SettingsFileHeader& head) {
-    MidiControllerList::controller_array *m = 0;
+    ControllerArray *m = 0;
     do {
 	jp.next(gx_system::JsonParser::value_string);
 	if (jp.current_value() == "current_preset") {
@@ -299,7 +324,7 @@ private:
 public:
     gx_system::PresetFile presetfile;
     LadspaSettings(string sfname, string presname,
-		   ParamMap& param, EngineControl&, ConvolverAdapter*, ControlParameter&);
+		   EngineControl&, ConvolverStereoAdapter*, ConvolverMonoAdapter*, ControlParameter&);
     ~LadspaSettings();
     void load(int num);
     bool idx_in_preset(int idx) { return idx >= 0 && idx < presetfile.size(); }
@@ -331,7 +356,6 @@ void LadspaSettings::load(Source src, const Glib::ustring& name) {
 	// might have changed because we read all state file sections
 	current_bank = current_name = "";
     }
-    gx_ui::GxUI::updateAllGuis();
     seq.clear_rack_changed();
     selection_changed();
 }
@@ -350,18 +374,19 @@ void LadspaSettings::change_preset_file(const std::string& newfile) {
 	}
 	presetlist_changed();
     } catch(gx_system::JsonException& e) {
-	gx_system::gx_print_error(
+	gx_print_error(
 	    newfile.c_str(), e.what());
     }
 }
 
 // seq and cp not yet initialized, only use address!
-LadspaSettings::LadspaSettings(string sfname, string presname, ParamMap& param,
-			       EngineControl& seq, ConvolverAdapter* convolver, ControlParameter& cp)
+LadspaSettings::LadspaSettings(string sfname, string presname, EngineControl& seq,
+			       ConvolverStereoAdapter* stereo_convolver, ConvolverMonoAdapter* mono_convolver,
+			       ControlParameter& cp)
     : GxSettingsBase(seq),
       current_source(state),
-      preset_io(param, convolver, cp),
-      state_io(param, convolver, cp) {
+      preset_io(seq.get_param(), stereo_convolver, mono_convolver, cp),
+      state_io(seq.get_param(), stereo_convolver, mono_convolver, cp) {
     set_io(&state_io, &preset_io);
     set_statefilename(sfname);
     change_preset_file(presname);
@@ -379,7 +404,7 @@ void LadspaSettings::load(int num) {
 	if (idx_in_preset(num-1)) {
 	    load_preset_by_idx(num-1);
 	} else {
-	    gx_system::gx_print_error("preset loader", boost::format("no preset number %1%") % num);
+	    gx_print_error("preset loader", boost::format("no preset number %1%") % num);
 	}
     }
 }
@@ -389,12 +414,12 @@ void LadspaSettings::load(int num) {
 ** misc. definitions
 */
 
-static void log_terminal(const string& msg, gx_system::GxMsgType tp, bool plugged) {
+static void log_terminal(const string& msg, GxLogger::MsgType tp, bool plugged) {
     const char *t;
     switch (tp) {
-    case gx_system::kInfo:    t = "I"; break;
-    case gx_system::kWarning: t = "W"; break;
-    case gx_system::kError:   t = "E"; break;
+    case GxLogger::kInfo:    t = "I"; break;
+    case GxLogger::kWarning: t = "W"; break;
+    case GxLogger::kError:   t = "E"; break;
     default:       t = "?"; break;
     }
     if (!plugged) {
@@ -408,7 +433,7 @@ static void init_logger() {
     if (logger_inited++) {
 	return;
     }
-    gx_system::Logger& log(gx_system::Logger::get_logger());
+    GxLogger& log(GxLogger::get_logger());
     if (log.signal_message().empty()) {
 	log.signal_message().connect(sigc::ptr_fun(log_terminal));
 	log.unplug_queue();
@@ -419,7 +444,7 @@ static void destroy_logger() {
     if (--logger_inited > 0) {
 	return;
     }
-    gx_system::Logger::destroy();
+    GxLogger::destroy();
 }
 
 
@@ -466,7 +491,6 @@ protected:
     LADSPA_Data * no_rt_mode_port;
     LADSPA_Data * priority_port;
     LADSPA_Data * latency_port;
-    ParamMap param;
     ControlParameter& control_parameter;
     LadspaSettings settings;
     void check_preset();
@@ -474,7 +498,7 @@ protected:
     void prepare_run();
     unsigned int activate(int *policy, int *prio);
     void load();
-    LadspaGuitarix(EngineControl& engine, ConvolverAdapter* convolver,
+    LadspaGuitarix(EngineControl& engine, ConvolverStereoAdapter* stereo_convolver, ConvolverMonoAdapter* mono_convolver,
 		   ControlParameter& cp, const char *envvar);
     ~LadspaGuitarix();
     static void start_presetloader() { PresetLoader::start_presetloader(); }
@@ -498,7 +522,7 @@ static string get_presetfile(const char *envvar) {
 
 // engine and cp not yet initialized, only use address!
 LadspaGuitarix::LadspaGuitarix(
-    EngineControl& engine, ConvolverAdapter* convolver, ControlParameter& cp, const char *envvar)
+    EngineControl& engine, ConvolverStereoAdapter* stereo_convolver, ConvolverMonoAdapter* mono_convolver, ControlParameter& cp, const char *envvar)
     : last_thread_id(),
       jack_bs(),
       jack_prio(),
@@ -510,9 +534,8 @@ LadspaGuitarix::LadspaGuitarix(
       no_rt_mode_port(),
       priority_port(),
       latency_port(),
-      param(),
       control_parameter(cp),
-      settings(get_statefile(), get_presetfile(envvar), param, engine, convolver, cp) {
+      settings(get_statefile(), get_presetfile(envvar), engine, stereo_convolver, mono_convolver, cp) {
     PresetLoader::add_instance(this);
 }
 
@@ -709,6 +732,8 @@ private:
     void load_static_plugins();
     void sr_changed(unsigned int);
     void bs_changed(unsigned int);
+    void overload(OverloadType tp, const char *reason);
+    virtual void set_rack_changed();
 public:
     MonoModuleChain mono_chain;  // active modules (amp chain, input to insert output)
     // ModuleSelector's
@@ -717,10 +742,12 @@ public:
     ModuleSelectorFromList ampstack;
     // internal audio modules
     NoiseGate noisegate;
+    ConvolverMonoAdapter mono_convolver;
     CabinetConvolver cabinet;
+    PreampConvolver preamp;
     ContrastConvolver contrast;
 public:
-    MonoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups);
+    MonoEngine(const string& plugin_dir, ParameterGroups& groups);
     ~MonoEngine();
     virtual void wait_ramp_down_finished();
     virtual bool update_module_lists();
@@ -740,7 +767,6 @@ bool MonoEngine::prepare_module_lists() {
 	(*i)->set_module();
     }
     list<Plugin*> modules;
-    rack_changed = false;
     pluginlist.ordered_mono_list(modules, PGN_MODE_NORMAL);
     bool ret_mono = mono_chain.set_plugin_list(modules);
     if (ret_mono) {
@@ -756,7 +782,7 @@ void MonoEngine::commit_module_lists() {
 	mono_chain.start_ramp_down();
 	mono_chain.wait_ramp_down_finished();
     }
-    mono_chain.commit(mono_chain.next_commit_needs_ramp);
+    mono_chain.commit(mono_chain.next_commit_needs_ramp, pmap);
     if (monoramp) {
 	mono_chain.start_ramp_up();
 	mono_chain.next_commit_needs_ramp = false;
@@ -771,6 +797,14 @@ bool MonoEngine::update_module_lists() {
     return false;
 }
 
+void MonoEngine::set_rack_changed() {
+    if (rack_changed.connected()) {
+	return;
+    }
+    rack_changed = Glib::signal_idle().connect(
+	sigc::bind_return(sigc::mem_fun(this, &MonoEngine::update_module_lists),false));
+}
+
 void MonoEngine::start_ramp_up() {
     mono_chain.start_ramp_up();
 }
@@ -817,6 +851,7 @@ static plugindef_creator builtin_tonestack_plugins[] = {
     gx_tonestacks::tonestack_fender_default::plugin,
     gx_tonestacks::tonestack_fender_deville::plugin,
     gx_tonestacks::tonestack_gibsen::plugin,
+    gx_tonestacks::tonestack_engl::plugin,
     0
 };
 
@@ -854,26 +889,30 @@ static const char* ampstack_groups[] = {
     0
 };
 
-MonoEngine::MonoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups)
+MonoEngine::MonoEngine(const string& plugin_dir, ParameterGroups& groups)
     : EngineControl(),
       resamp(),
       // ModuleSelector's
       crybaby(
-	  *this, ui, "crybaby", N_("Crybaby"), builtin_crybaby_plugins,
-	  "crybaby.autowah", _("select"), 0, PGN_POST_PRE),
+	  *this, "crybaby", N_("Crybaby"), "", builtin_crybaby_plugins,
+	  "crybaby.autowah", _("select"), 0, 0, PGN_POST_PRE),
       tonestack(
-	  *this, ui, "amp.tonestack", N_("Tonestack"),
+	  *this, "amp.tonestack", N_("Tonestack"), "",
 	  builtin_tonestack_plugins, "amp.tonestack.select",
-	  _("select"), 0, PGN_POST_PRE),
+	  _("select"), 0, 0, PGN_POST_PRE),
       ampstack(
-	  *this, ui, "ampstack", "?Tube", builtin_amp_plugins,
-	  "tube.select", _("select"), ampstack_groups),
+	  *this, "ampstack", "?Tube", "", builtin_amp_plugins,
+	  "tube.select", _("select"), 0, ampstack_groups),
       // internal audio modules
       noisegate(),
+      mono_convolver(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), get_param()),
       cabinet(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
+      preamp(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp),
       contrast(*this, sigc::mem_fun(mono_chain, &MonoModuleChain::sync), resamp) {
 
+    mono_convolver.set_sync(true);
     cabinet.set_sync(true);
+    preamp.set_sync(true);
     contrast.set_sync(true);
 
     load_static_plugins();
@@ -888,7 +927,7 @@ MonoEngine::MonoEngine(const string& plugin_dir, ParamMap& param, ParameterGroup
     add_selector(crybaby);
     add_selector(tonestack);
 
-    registerParameter(param, groups);
+    registerParameter(groups);
 
     signal_samplerate_change().connect(
 	sigc::mem_fun(*this, &MonoEngine::sr_changed));
@@ -911,6 +950,10 @@ void MonoEngine::bs_changed(unsigned int bs) {
     buffersize = bs;
 }
 
+void MonoEngine::overload(OverloadType tp, const char *reason) {
+    gx_print_error("overload", reason);
+}
+
 void MonoEngine::load_static_plugins() {
     PluginList& pl = pluginlist; // just a shortcut
 
@@ -939,9 +982,11 @@ void MonoEngine::load_static_plugins() {
 
     // mono
     pl.add(gx_effects::low_high_pass::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::highbooster::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::selecteq::plugin(),        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&crybaby.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::gx_distortion::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::ts9sim::plugin(),           PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::impulseresponse::plugin(), PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::compressor::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::overdrive::plugin(),       PLUGIN_POS_RACK, PGN_GUI);
@@ -953,11 +998,17 @@ void MonoEngine::load_static_plugins() {
     pl.add(gx_effects::phaser_mono::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::chorus_mono::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::flanger_mono::plugin(),    PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(pluginlib::flanger_gx::plugin(),       PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::gx_feedback::plugin(),     PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::peak_eq::plugin(),         PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::digital_delay::plugin(),   PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&tonestack.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&mono_convolver.plugin,                PLUGIN_POS_RACK, PGN_GUI);
     pl.add(&cabinet.plugin,                       PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&preamp.plugin,                        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::abgate::plugin(),           PLUGIN_POS_RACK);
     pl.add(pluginlib::vibe::plugin_mono(),        PLUGIN_POS_RACK);
+    pl.add(pluginlib::mbc::plugin(),              PLUGIN_POS_RACK);
 }
 
 
@@ -1042,15 +1093,15 @@ public:
 };
 
 LadspaGuitarixMono::LadspaGuitarixMono(unsigned long sr)
-    : LadspaGuitarix(engine, 0, control_parameter, "LADSPA_GUITARIX_MONO_PRESET"),
-      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), param, get_group_table()),
+    : LadspaGuitarix(engine, 0, &engine.mono_convolver, control_parameter, "LADSPA_GUITARIX_MONO_PRESET"),
+      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), get_group_table()),
       control_parameter(GUITARIX_PARAM_COUNT),
       rebuffer(),
       volume_port(),
-      volume_param(param["amp.out_ladspa"].getFloat()),
+      volume_param(engine.get_param()["amp.out_ladspa"].getFloat()),
       input_buffer(),
       output_buffer() {
-    param.set_init_values();
+    engine.get_param().set_init_values();
     engine.set_samplerate(sr);
 }
 
@@ -1068,7 +1119,7 @@ void LadspaGuitarixMono::activateGuitarix(LADSPA_Handle Instance) {
     bufsize = self.activate(&policy, &prio);
     self.rebuffer.set_bufsize(bufsize);
     self.engine.set_buffersize(bufsize);
-    gx_system::gx_print_info(
+    gx_print_info(
 	"amp activate",
 	boost::format("instance %1%, SR %2%, BS %3%, prio %4%")
 	% Instance % self.engine.get_samplerate() % bufsize % prio);
@@ -1327,11 +1378,13 @@ LadspaGuitarixMono::LADSPA::~LADSPA() {
 class StereoEngine: public EngineControl {
 private:
     void load_static_plugins();
+    void overload(OverloadType tp, const char *reason);
+    virtual void set_rack_changed();
 public:
     StereoModuleChain stereo_chain;
-    ConvolverAdapter convolver;
+    ConvolverStereoAdapter stereo_convolver;
 public:
-    StereoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups, const gx_system::PathList& pathlist);
+    StereoEngine(const string& plugin_dir, ParameterGroups& groups);
     ~StereoEngine();
     virtual void wait_ramp_down_finished();
     virtual bool update_module_lists();
@@ -1342,6 +1395,10 @@ public:
     void commit_module_lists();
 };
 
+void StereoEngine::overload(OverloadType tp, const char *reason) {
+    gx_print_error("overload", reason);
+}
+
 void StereoEngine::wait_ramp_down_finished() {
     stereo_chain.wait_ramp_down_finished();
 }
@@ -1351,7 +1408,6 @@ bool StereoEngine::prepare_module_lists() {
 	(*i)->set_module();
     }
     list<Plugin*> modules;
-    rack_changed = false;
     pluginlist.ordered_stereo_list(modules, PGN_MODE_NORMAL);
     bool ret_stereo = stereo_chain.set_plugin_list(modules);
     if (ret_stereo) {
@@ -1367,7 +1423,7 @@ void StereoEngine::commit_module_lists() {
 	stereo_chain.start_ramp_down();
 	stereo_chain.wait_ramp_down_finished();
     }
-    stereo_chain.commit(stereo_chain.next_commit_needs_ramp);
+    stereo_chain.commit(stereo_chain.next_commit_needs_ramp, pmap);
     if (stereoramp) {
 	stereo_chain.start_ramp_up();
 	stereo_chain.next_commit_needs_ramp = false;
@@ -1382,6 +1438,14 @@ bool StereoEngine::update_module_lists() {
     return false;
 }
 
+void StereoEngine::set_rack_changed() {
+    if (rack_changed.connected()) {
+	return;
+    }
+    rack_changed = Glib::signal_idle().connect(
+	sigc::bind_return(sigc::mem_fun(this, &StereoEngine::update_module_lists),false));
+}
+
 void StereoEngine::start_ramp_up() {
     stereo_chain.start_ramp_up();
 }
@@ -1395,12 +1459,12 @@ void StereoEngine::set_samplerate(unsigned int samplerate) {
     EngineControl::set_samplerate(samplerate);
 }
 
-StereoEngine::StereoEngine(const string& plugin_dir, ParamMap& param, ParameterGroups& groups, const gx_system::PathList& pathlist)
+StereoEngine::StereoEngine(const string& plugin_dir, ParameterGroups& groups)
     : EngineControl(),
       // internal audio modules
-      convolver(*this, sigc::mem_fun(stereo_chain, &StereoModuleChain::sync), param, pathlist, "") {
+      stereo_convolver(*this, sigc::mem_fun(stereo_chain, &StereoModuleChain::sync), get_param()) {
 
-    convolver.set_sync(true);
+    stereo_convolver.set_sync(true);
 
     load_static_plugins();
 
@@ -1409,7 +1473,7 @@ StereoEngine::StereoEngine(const string& plugin_dir, ParamMap& param, ParameterG
 	pluginlist.load_from_path(plugin_dir, PLUGIN_POS_RACK);
     }
 
-    registerParameter(param, groups);
+    registerParameter(groups);
 
 #ifndef NDEBUG
     pluginlist.printlist();
@@ -1442,10 +1506,12 @@ void StereoEngine::load_static_plugins() {
     pl.add(gx_effects::moog::plugin(),            PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_amps::gx_ampmodul::plugin(),        PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::tonecontroll::plugin(),    PLUGIN_POS_RACK, PGN_GUI);
-    pl.add(&convolver.plugin,                     PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(gx_effects::digital_delay_st::plugin(),PLUGIN_POS_RACK, PGN_GUI);
+    pl.add(&stereo_convolver.plugin,              PLUGIN_POS_RACK, PGN_GUI);
     pl.add(gx_effects::stereoverb::plugin(),      PLUGIN_POS_RACK, PGN_GUI);
     pl.add(pluginlib::zita_rev1::plugin(),        PLUGIN_POS_RACK);
     pl.add(pluginlib::vibe::plugin_stereo(),      PLUGIN_POS_RACK);
+    pl.add(pluginlib::mbcs::plugin(),             PLUGIN_POS_RACK);
 }
 
 
@@ -1453,14 +1519,6 @@ void StereoEngine::load_static_plugins() {
  ** class LadspaGuitarixStereo
  */
 
-class LadspaPathList: public gx_system::PathList {
-public:
-    inline LadspaPathList() {
-	add(Glib::build_filename(Glib::get_user_config_dir(), "guitarix")); //FIXME
-	add(GX_SOUND_DIR);
-    };
-};
-
 class LadspaGuitarixStereo: LadspaGuitarix {
 private:
 
@@ -1528,7 +1586,6 @@ private:
 		 LADSPA_Data *output_buffer1, LADSPA_Data *output_buffer2);
     };
 
-    LadspaPathList pathlist;
     StereoEngine engine;
     ControlParameter control_parameter;
     ReBuffer rebuffer;
@@ -1552,18 +1609,17 @@ public:
 };
 
 LadspaGuitarixStereo::LadspaGuitarixStereo(unsigned long sr)
-    : LadspaGuitarix(engine, &engine.convolver, control_parameter, "LADSPA_GUITARIX_STEREO_PRESET"),
-      pathlist(),
-      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), param, get_group_table(), pathlist),
+    : LadspaGuitarix(engine, &engine.stereo_convolver, 0, control_parameter, "LADSPA_GUITARIX_STEREO_PRESET"),
+      engine(Glib::build_filename(Glib::get_user_config_dir(), "guitarix/plugins/"), get_group_table()),
       control_parameter(GUITARIX_PARAM_COUNT),
       rebuffer(),
       volume_port(),
-      volume_param(param["amp.out_master_ladspa"].getFloat()),
+      volume_param(engine.get_param()["amp.out_master_ladspa"].getFloat()),
       input_buffer1(),
       input_buffer2(),
       output_buffer1(),
       output_buffer2() {
-    param.set_init_values();
+    engine.get_param().set_init_values();
     engine.set_samplerate(sr);
 }
 
@@ -1581,7 +1637,7 @@ void LadspaGuitarixStereo::activateGuitarix(LADSPA_Handle Instance) {
     bufsize = self.activate(&policy, &prio);
     self.rebuffer.set_bufsize(bufsize);
     self.engine.set_buffersize(bufsize);
-    gx_system::gx_print_info(
+    gx_print_info(
 	"fx activate",
 	boost::format("instance %1%, SR %2%, BS %3%, prio %4%")
 	% Instance % self.engine.get_samplerate() % bufsize % prio);
@@ -1873,6 +1929,10 @@ LadspaGuitarixStereo::LADSPA::~LADSPA() {
  ** function ladspa_descriptor
  */
 
+static void initBasicOptions() {
+    static gx_system::BasicOptions options;
+}
+
 /* Return a descriptor of the requested plugin type. */
 extern "C" __attribute__ ((visibility ("default")))
 const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index) {
@@ -1883,8 +1943,11 @@ const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index) {
     static bool inited = 0;
     if (!inited) {
 	inited = 1;
+#if ENABLE_NLS
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+#endif
+	initBasicOptions();
     }
     switch (Index) {
     case 0:
