@@ -123,7 +123,7 @@ public:
     static void process(int count, float * smpsl, float * smpsr, float * efxoutl, float * efxoutr, PluginDef *plugin);
     static void process_mono(int count, float *smps, float *efxout, PluginDef *plugin);
     static int registerparam(const ParamReg& reg);
-    static int uiloader(const UiBuilder& builder);
+    static int uiloader(const UiBuilder& builder, int form);
     static void del_instance(PluginDef *plugin);
 
 private:
@@ -154,10 +154,9 @@ private:
 	float n1;
 	float d0;
 	float d1; 
-    } vc[8], vcvo[8], ecvc[8], vevo[8], bootstrap[8];
+    } vc[8], vcvo[8], ecvc[8], vevo[8];
 
     NextValue lfo;
-    unsigned int PLFOtype;
 
     inline float vibefilter(float data, fparams *ftype, int stage);
     void init_vibes(unsigned int samplerate);
@@ -189,8 +188,10 @@ Vibe::Vibe(bool stereo)
     } else {
 	id = "univibe_mono";
 	name = N_("Vibe Mono");
+	shortname = N_("Vibe");
 	mono_audio = process_mono;
     }
+    category = N_("Modulation");
     set_samplerate = init;
     register_params = registerparam;
     load_ui = uiloader;
@@ -236,11 +237,14 @@ int Vibe::registerparam(const ParamReg& reg) {
     reg.registerVar(univibe_width,N_("Width"),"S",N_("LFO amplitude"),&self.Pwidth, 0.5, 0, 1, 0.01);
     reg.registerVar(univibe_depth,N_("Depth"),"S",N_("DC level in LFO"),&self.Pdepth,0.37,0,1,0.01);
     reg.registerVar(univibe_wet_dry,N_("Wet/Dry"),"S",N_("output mix (signal / effect)"),&self.wet_dry,1,0,1,0.01);
-    reg.registerVar(univibe_fb,N_("Fb"),"S",N_("sound modification by feedback"),&self.fb,-0.6,-1,1,0.01);
+    reg.registerVar(univibe_fb,NC_("Feedback", "Fb"),"S",N_("sound modification by feedback"),&self.fb,-0.6,-1,1,0.01);
     return 0;
 }
 
-int Vibe::uiloader(const UiBuilder& b) {
+int Vibe::uiloader(const UiBuilder& b, int form) {
+    if (!(form & UI_FORM_STACK)) {
+	return -1;
+    }
     Vibe& self = *static_cast<Vibe*>(b.plugin);
     const char *univibe_freq, *univibe_width, *univibe_depth, *univibe_wet_dry, *univibe_fb;
     if (self.Pstereo) {
@@ -261,6 +265,7 @@ int Vibe::uiloader(const UiBuilder& b) {
         b.create_master_slider(univibe_wet_dry,0);
     }
     b.closeBox();
+    b.openHorizontalBox("");
     if (self.Pstereo) {
 	b.openVerticalBox("");
 	b.openHorizontalBox("");
@@ -283,6 +288,7 @@ int Vibe::uiloader(const UiBuilder& b) {
 	b.closeBox();
 	b.closeBox();
     }
+    b.closeBox();
     return 0;
 }
 
